@@ -25,7 +25,7 @@
 #include "grts/structs.db.h"
 #include "grts/structs.workbench.physical.h"
 #include "base/string_utilities.h"
-#include "wb_context.h"
+#include "workbench/wb_context.h"
 
 #include <algorithm>
 
