@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,18 +21,20 @@
 
 #include <errno.h>
 #include "wb_component_basic.h"
+#include "wb_context_model.h"
 #include "model/wb_model_diagram_form.h"
-
+#include "workbench/wb_context.h"
 #include "grts/structs.workbench.model.h"
 #include "base/string_utilities.h"
 #include "base/file_functions.h"
+#include "grt/clipboard.h"
+
+using namespace std;
 
 using namespace wb;
 using namespace grt;
 using namespace bec;
 using namespace base;
-using namespace MySQL::Geometry;
-using namespace MySQL::Drawing;
 
 struct HandToolContext
 {
@@ -215,53 +217,44 @@ bool WBComponentBasic::handles_figure(const model_ObjectRef &object)
 }
 
 
-
-grt::ValueRef WBComponentBasic::place_layer_grt(grt::GRT *grt, ModelDiagramForm *form,
-                                             const Rect &rect)
-{
-  model_DiagramRef view(form->get_model_diagram());
-
-  std::string color;
-
-  if (form->get_tool_argument("workbench.model.Layer:Color").empty())
-    color= _wb->get_wb_options().get_string("workbench.model.Layer:Color");
-  else
-    color= form->get_tool_argument("workbench.model.Layer:Color");
-
-  model_LayerRef layer;
-
-  grt::AutoUndo undo(grt);
-  
-  layer= view->placeNewLayer(rect.left(), rect.top(), rect.width(), rect.height(), _("New Layer"));
-
-  if (layer.is_valid())
-    layer->color(color);
-
-  undo.end(_("Place New Layer"));
-  
-  return layer;
-}
-
-
-void WBComponentBasic::place_layer(ModelDiagramForm *view, const Rect &rect)
+grt::ValueRef WBComponentBasic::place_layer(ModelDiagramForm *form, const Rect &rect)
 {
   try
   {
-    _wb->execute_in_grt_thread("Place new layer in model",
-                        boost::bind(&WBComponentBasic::place_layer_grt, this, _1, view, rect));
+    model_DiagramRef view(form->get_model_diagram());
+
+    std::string color;
+
+    if (form->get_tool_argument("workbench.model.Layer:Color").empty())
+      color= _wb->get_wb_options().get_string("workbench.model.Layer:Color");
+    else
+      color= form->get_tool_argument("workbench.model.Layer:Color");
+
+    model_LayerRef layer;
+
+    grt::AutoUndo undo(get_grt());
+  
+    layer= view->placeNewLayer(rect.left(), rect.top(), rect.width(), rect.height(), _("New Layer"));
+
+    if (layer.is_valid())
+      layer->color(color);
+
+    undo.end(_("Place New Layer"));
+  
     _wb->show_status_text(_("Created new layer."));
+    return layer;
   }
   catch (std::exception &exc)
   {
     _wb->show_exception(_("Place new layer"), exc);
+    return grt::ValueRef();
   }
 }
 
-
 //-------------------------------------------------------------------------------------
 // Deletion
 
-
+/* unused
 void WBComponentBasic::delete_selection()
 {
   ModelDiagramForm *form;
@@ -278,25 +271,13 @@ void WBComponentBasic::delete_selection()
         objects.push_back(model_ObjectRef::cast_from(selection.get(i)));
     }
 
-    std::map<std::string, bool> options;
-
-    {
-      std::string option;
-
-      option= get_wb()->get_wb_options().get_string("workbench.physical:DeleteObjectConfirmation", "ask");
-      if (option == "delete")
-        options["deletedbobjects"]= true;
-      else if (option == "keep")
-        options["deletedbobjects"]= false;
-    }
-
     for (size_t c= objects.size(), i= 0; i < c; i++)
-      _wb->delete_object(objects[i], options);
+      _wb->get_model_context()->delete_object(objects[i]);
   }
-}
+}*/
 
 
-bool WBComponentBasic::delete_model_object(const model_ObjectRef &object, std::map<std::string,bool> &options)
+bool WBComponentBasic::delete_model_object(const model_ObjectRef &object, bool figure_only)
 {
   grt::AutoUndo undo(_wb->get_grt());
 
@@ -331,56 +312,65 @@ void WBComponentBasic::delete_object(ModelDiagramForm *view, const Point &pos)
 
   if (object.is_valid())
   {
-    std::map<std::string, bool> options;
-
-    if (_wb->delete_object(object, options))
-      _wb->show_status_text(strfmt(_("Deleted %s"), object.get_metaclass()->get_attribute("caption").c_str()));
+    if (_wb->get_model_context()->remove_figure(object))
+      _wb->show_status_text(strfmt(_("Removed %s"), object.get_metaclass()->get_attribute("caption").c_str()));
   }
 }
 
 
-void WBComponentBasic::activate_canvas_object(const model_ObjectRef &figure, bool newwindow)
+void WBComponentBasic::copy_object_to_clipboard(const grt::ObjectRef &object, grt::CopyContext &copy_context)
 {
-  if (figure.is_instance(workbench_model_NoteFigure::static_class_name()))
-    _wb->open_object_editor(figure, newwindow ? bec::ForceNewWindowFlag : bec::NoFlags);
-  else if (figure.is_instance(workbench_model_ImageFigure::static_class_name()))
-    _wb->open_object_editor(figure, newwindow ? bec::ForceNewWindowFlag : bec::NoFlags);
-  else if (figure.is_instance(model_Layer::static_class_name()))
-    _wb->open_object_editor(figure, newwindow ? bec::ForceNewWindowFlag : bec::NoFlags);
+  std::set<std::string> skip;
+  skip.insert("oldName");
+  //  skip.insert("filename");
+  
+  // copy table
+  grt::ObjectRef copy = copy_context.copy(object, skip);
+  
+  bec::Clipboard *clip= get_wb()->get_clipboard();
+  clip->append_data(copy);
 }
 
 
-
-model_ObjectRef WBComponentBasic::clone_object(const model_ObjectRef &object, const model_LayerRef &destlayer, grt::CopyContext &copy_context, bool copy_only)
+bool WBComponentBasic::can_paste_object(const grt::ObjectRef &object)
 {
-  grt::AutoUndo undo(_wb->get_grt());
+  if (object.is_instance(workbench_model_NoteFigure::static_class_name()) ||
+      object.is_instance(workbench_model_ImageFigure::static_class_name()) ||
+      object.is_instance(model_Layer::static_class_name()))
+    return true;
+  return false;
+}
+                              
+static void get_component_that_can_paste(WBComponent *compo,
+                                         const grt::ObjectRef &object,
+                                         WBComponent **result)
+{
+  if (compo->can_paste_object(object))
+    *result= compo;
+}
 
+model_ObjectRef WBComponentBasic::paste_object(ModelDiagramForm *view, const grt::ObjectRef &object, grt::CopyContext &copy_context)
+{
+  model_ObjectRef copy;
+  model_LayerRef destlayer(view->get_model_diagram()->rootLayer());
+  grt::AutoUndo undo(_wb->get_grt());
+  
   if (object.is_instance(model_Figure::static_class_name()))
   {
     std::set<std::string> skip;
+    model_FigureRef original(model_FigureRef::cast_from(object));
+    model_FigureRef figure(model_FigureRef::cast_from(grt::copy_object(object, skip)));
     
-    skip.insert("filename");    
-    model_FigureRef figure(model_FigureRef::cast_from(grt::copy_object(_wb->get_grt_manager()->get_grt(), object, skip)));
-
     if (destlayer.is_valid())
     {
       figure->layer(destlayer);
       figure->owner(destlayer->owner());
-
-      if (object.is_instance(workbench_model_ImageFigure::static_class_name()))
+  
       {
-        workbench_model_ImageFigureRef new_image(workbench_model_ImageFigureRef::cast_from(figure));
-        workbench_model_ImageFigureRef image(workbench_model_ImageFigureRef::cast_from(object));
-
-        new_image->setImageFile(image->filename());
-      }
-      
-      if (!copy_only)
-      {
-        if (grt::find_named_object_in_list(destlayer->owner()->figures(), object->name()).is_valid())
+        if (grt::find_named_object_in_list(destlayer->owner()->figures(), original->name()).is_valid())
           figure->name(grt::get_name_suggestion_for_list_object(destlayer->owner()->figures(),
-            *object->name()+"_copy"));
-
+                                                                *original->name()+"_copy"));
+        
         destlayer->owner()->addFigure(figure);
         undo.end("Duplicate Object");
       }
@@ -388,55 +378,75 @@ model_ObjectRef WBComponentBasic::clone_object(const model_ObjectRef &object, co
     else
     {
       figure->owner(model_DiagramRef());
-      figure->layer(model_LayerRef());    
+      figure->layer(model_LayerRef());
     }
-    return figure;
+    copy = figure;
   }
   else if (object.is_instance(model_Layer::static_class_name()))
   {
     std::set<std::string> skip;
-
+    
     skip.insert("figures");
-
+    
     model_LayerRef original(model_LayerRef::cast_from(object));
-
-    model_LayerRef layer_copy(model_LayerRef::cast_from(grt::copy_object(_wb->get_grt_manager()->get_grt(), object, skip)));
-
-    if (!copy_only && destlayer.is_valid())
+    
+    model_LayerRef layer_copy(model_LayerRef::cast_from(copy_context.copy(object, skip)));
+    
+    if (destlayer.is_valid())
       layer_copy->owner(destlayer->owner());
     
-    if (!copy_only)
-    {
-      if (grt::find_named_object_in_list(destlayer->owner()->layers(), object->name()).is_valid())
-        layer_copy->name(grt::get_name_suggestion_for_list_object(destlayer->owner()->layers(),
-          *object->name()+"_copy"));
-
-      destlayer->owner()->layers().insert(layer_copy);
-      undo.end("Duplicate Object");
-    }
-    else
-      layer_copy->owner(model_DiagramRef());
+    if (grt::find_named_object_in_list(destlayer->owner()->layers(), original->name()).is_valid())
+      layer_copy->name(grt::get_name_suggestion_for_list_object(destlayer->owner()->layers(),
+                                                                *original->name()+"_copy"));
 
+    destlayer->owner()->layers().insert(layer_copy);
+    undo.end("Duplicate Object");
+    
     // copy contained stuff that's to be duplicated
     for (size_t c= original->figures()->count(), i= 0; i < c; i++)
     {
       model_FigureRef figure(original->figures()[i]);
       
-      WBComponent *compo= _wb->get_component_handling(figure);
+      // paste the object
+      WBComponent *compo= 0;
+      _wb->foreach_component(boost::bind(get_component_that_can_paste, _1, figure, &compo));
       if (compo)
       {
-        model_FigureRef figure_copy= model_FigureRef::cast_from(compo->clone_object(figure, layer_copy, copy_context, copy_only));
-        
-        if (copy_only)
-          layer_copy->figures().insert(figure_copy);
+        model_ObjectRef tmp = compo->paste_object(view, figure, copy_context);
+        if (tmp.is_valid() && model_FigureRef::can_wrap(tmp))
+        {
+          model_FigureRef tmpf = model_FigureRef::cast_from(tmp);
+          
+          if (tmpf->layer() != layer_copy)
+          {
+            tmpf->layer()->figures().remove_value(tmpf);
+            tmpf->layer(layer_copy);
+            layer_copy->figures().insert(tmpf);
+            
+            tmpf->top(figure->top());
+            tmpf->left(figure->left());
+          }
+        }
       }
     }
     
     copy_context.update_references();
     
-    return layer_copy;
+    copy = layer_copy;
   }
-  return model_ObjectRef();
+  
+  return copy;
+}
+
+
+void WBComponentBasic::activate_canvas_object(const model_ObjectRef &figure, bool newwindow)
+{
+  if (figure.is_instance(workbench_model_NoteFigure::static_class_name()))
+    _wb->get_grt_manager()->open_object_editor(figure, newwindow ? bec::ForceNewWindowFlag : bec::NoFlags);
+  else if (figure.is_instance(workbench_model_ImageFigure::static_class_name()))
+    _wb->get_grt_manager()->open_object_editor(figure, newwindow ? bec::ForceNewWindowFlag : bec::NoFlags);
+  else if (figure.is_instance(model_Layer::static_class_name()))
+    _wb->get_grt_manager()->open_object_editor(figure, newwindow ? bec::ForceNewWindowFlag : bec::NoFlags);
 }
 
 
@@ -487,10 +497,10 @@ bool WBComponentBasic::handle_button_event(ModelDiagramForm *view, mdc::MouseBut
       }
       else
       {
-        rect.pos.x = std::max(rect.pos.x, 0.0);
-        rect.pos.y = std::max(rect.pos.y, 0.0);
-        rect.size.width = std::min(rect.size.width, max_size.width);
-        rect.size.height = std::min(rect.size.height, max_size.height);
+        rect.pos.x = max(rect.pos.x, 0.0);
+        rect.pos.y = max(rect.pos.y, 0.0);
+        rect.size.width = min(rect.size.width, max_size.width);
+        rect.size.height = min(rect.size.height, max_size.height);
         
         place_layer(view, rect);
         view->reset_tool(true);
@@ -590,7 +600,7 @@ bool WBComponentBasic::handle_motion_event(ModelDiagramForm *view, Point pos, md
       Point mouse_pos= hcontext->mouse_pos;
       Point viewport_pos= hcontext->viewport_pos;
 
-      Point newp= viewport_pos - wpos + mouse_pos;
+      Point newp= viewport_pos + Point((mouse_pos.x - wpos.x) / view->get_zoom(), (mouse_pos.y - wpos.y) / view->get_zoom());
 
       Rect r= cview->get_viewport();
       Size sz= cview->get_total_view_size();
