@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,9 +19,17 @@
 
 #include "stdafx.h"
 
+#include "base/file_utilities.h"
+#include "base/file_functions.h"
+#include "base/string_utilities.h"
+#include "base/log.h"
+
 #include "wb_context_model.h"
+
+#include "workbench/wb_context.h"
 #include "workbench/wb_context_ui.h"
 #include "workbench/wb_command_ui.h"
+
 #include "wb_component.h"
 #include "wb_component_basic.h"
 #include "wb_component_physical.h"
@@ -29,35 +37,45 @@
 #include "model/wb_model_diagram_form.h"
 #include "wb_overview_physical.h"
 #include "wb_overview_physical_schema.h"
+#include "objimpl/ui/mforms_ObjectReference_impl.h"
 #include "workbench/wb_model_file.h"
 #include "model/wb_overview_physical.h"
+#include "model/wb_user_datatypes.h"
+#include "model/wb_history_tree.h"
 #include "wbcanvas/model_diagram_impl.h"
 #include "wbcanvas/workbench_physical_model_impl.h"
 #include "workbench/wb_model_file.h"
-
+#include "grtdb/db_object_helpers.h"
 #include "grt/clipboard.h"
 #include "grt/common.h"
-#include "base/file_utilities.h"
-#include "base/file_functions.h"
-#include "base/string_utilities.h"
 
+#include "grtpp_notifications.h"
+
+#include "user_defined_type_editor.h"
+#include "wb_template_list.h"
+
+#include "mforms/tabview.h"
+#include "mforms/tabview_dock.h"
 #include "mforms/utilities.h"
 #include "mforms/menubar.h"
+#include "mforms/dockingpoint.h"
 
 using namespace base;
 using namespace bec;
 using namespace wb;
-using namespace MySQL::Geometry;
+using namespace base;
+
+DEFAULT_LOG_DOMAIN("ModelContext");
 
 static std::map<std::string, std::string> auto_save_files;
 
 
 WBContextModel::WBContextModel(WBContextUI *wbui)
-: _wbui(wbui), _file(0), _auto_save_point(0), _last_auto_save_time(0), _auto_save_timer(NULL)
+: _wbui(wbui), _file(0), _current_user_type_editor(0), _locked_view_for_plugin_exec(0), _auto_save_point(0), _last_auto_save_time(0), _auto_save_timer(NULL)
 
 {  
   _overview= new PhysicalOverviewBE(_wbui->get_wb());
-  
+
   scoped_connect(_wbui->get_wb()->get_grt_manager()->get_clipboard()->signal_changed(),boost::bind(&WBContextModel::selection_changed, this));
   scoped_connect(_overview->signal_selection_changed(),boost::bind(&WBContextModel::selection_changed, this)); // make edit menu captions to update
 
@@ -78,21 +96,49 @@ WBContextModel::WBContextModel(WBContextUI *wbui)
   cmdui->add_builtin_command("addModelRoutine",
                              boost::bind(&WBContextModel::add_model_rgroup, this),
                              validate);
+
+  cmdui->add_builtin_command("removeFigure",
+                             boost::bind(&WBContextModel::remove_figure, this),
+                             boost::bind(&WBContextModel::has_selected_figures, this));
   
   base::NotificationCenter::get()->add_observer(this, "GNMainFormChanged");
-  
+
   // setup auto-save for model
   int interval = _wbui->get_wb()->get_root()->options()->options().get_int("workbench:AutoSaveModelInterval", 60);
   if (interval > 0)
     _auto_save_timer = _wbui->get_wb()->get_grt_manager()->run_every(boost::bind(&WBContextModel::auto_save_document, this), interval);
   _auto_save_interval = interval;
-  
+
+  // DON'T set up any UI here. This is running on a background thread!
+  _secondary_sidebar = NULL;
+  _sidebar_dockpoint = NULL;
+  _template_panel = NULL;
+
+  // Default table templates list
+  grt::DictRef options(_wbui->get_wb()->get_root()->options()->options());
+  if (!options.has_key("TableTemplates"))
+  {
+    grt::ListRef<db_Table> templates = grt::ListRef<db_Table>::cast_from(_wbui->get_wb()->get_grt_manager()->get_grt()->unserialize(make_path(_wbui->get_wb()->get_datadir(), "data/table_templates.xml")));
+    options.set("TableTemplates", templates);
+  }
+
   scoped_connect(_wbui->get_wb()->get_root()->options()->signal_dict_changed(),boost::bind(&WBContextModel::option_changed, this, _1, _2, _3));
+
+  setup_secondary_sidebar();
 }
 
 
 WBContextModel::~WBContextModel()
 {
+  _grtmodel_panel.clear();
+
+  if (_secondary_sidebar != NULL)
+    _secondary_sidebar->release();
+  if (_sidebar_dockpoint != NULL)
+    _sidebar_dockpoint->release();
+
+  delete _template_panel;
+
   base::NotificationCenter::get()->remove_observer(this);
 
   if (_doc.is_valid() && _doc->physicalModels().is_valid() && _doc->physicalModels().count() > 0)
@@ -106,12 +152,49 @@ WBContextModel::~WBContextModel()
   cmdui->remove_builtin_command("addModelTable");
   cmdui->remove_builtin_command("addModelView");
   cmdui->remove_builtin_command("addModelRoutine");
-
+  cmdui->remove_builtin_command("removeFigure");
   _file= 0;
   delete _overview;
 }
 
 
+void WBContextModel::setup_secondary_sidebar()
+{
+  // Setup the secondary model sidebar, which should be shared between all model tabs
+  _secondary_sidebar = mforms::manage(new mforms::TabView(mforms::TabViewSelectorSecondary));
+  _template_panel = new TableTemplatePanel(_wbui->get_wb()->get_grt(), this);
+  _secondary_sidebar->add_page(_template_panel, _("Templates"));
+}
+
+
+mforms::TreeNodeView *WBContextModel::create_user_type_list()
+{
+  UserDatatypeList *type_list;
+
+  type_list = new UserDatatypeList(_wbui->get_wb());
+  type_list->set_catalog(_wbui->get_wb()->get_document()->physicalModels()[0]->catalog());
+  type_list->refresh();
+
+  type_list->scoped_connect(&_udt_list_changed, boost::bind(&UserDatatypeList::refresh, type_list));
+
+  return type_list;
+}
+
+
+mforms::TreeNodeView *WBContextModel::create_history_tree()
+{
+  HistoryTree *history_tree= new HistoryTree(_wbui->get_wb()->get_grt_manager(), _wbui->get_wb()->get_grt()->get_undo_manager());
+  history_tree->refresh();
+  return history_tree;
+}
+
+
+grt::GRT *WBContextModel::get_grt()
+{
+  return _wbui->get_wb()->get_grt();
+}
+
+
 void WBContextModel::option_changed(grt::internal::OwnedDict*dict, bool, const std::string&key)
 {
   if (key == "workbench:AutoSaveModelInterval" && dict == _wbui->get_wb()->get_wb_options().valueptr())
@@ -135,12 +218,10 @@ bool WBContextModel::auto_save_document()
       && _file
       && doc.is_valid() 
       && !wb->get_grt_manager()->get_dispatcher()->get_busy()
-      && wb->get_grt()->get_undo_manager()->get_latest_closed_undo_action() != _auto_save_point
-      && !wb->get_filename().empty())
+      && wb->get_grt()->get_undo_manager()->get_latest_closed_undo_action() != _auto_save_point)
   {
     _auto_save_point = wb->get_grt()->get_undo_manager()->get_latest_closed_undo_action();
     _last_auto_save_time = now;
-    
     try
     {
       // save the document in the same directory containing the expanded mwb file
@@ -183,22 +264,26 @@ void WBContextModel::detect_auto_save_files(const std::string &autosave_dir)
 
   for (std::list<std::string>::const_iterator d = autosaves.begin(); d != autosaves.end(); ++d)
   { 
+    if (!g_file_test(d->c_str(), G_FILE_TEST_IS_DIR))
+      continue;
+
     if (base::LockFile::check(bec::make_path(*d, ModelFile::lock_filename.c_str())) != base::LockFile::NotLocked)
       continue;
     
     if (g_file_test(bec::make_path(*d, MAIN_DOCUMENT_AUTOSAVE_NAME).c_str(), G_FILE_TEST_EXISTS))
     {
+      std::string path = bec::make_path(*d, "real_path");
       gchar *orig_path;
       gsize length;
-      if (g_file_get_contents(bec::make_path(*d, "real_path").c_str(),
-                              &orig_path, &length, NULL))
+      if (g_file_test(path.c_str(), (GFileTest)(G_FILE_TEST_IS_REGULAR|G_FILE_TEST_EXISTS))
+          && g_file_get_contents(path.c_str(), &orig_path, &length, NULL))
       {
         files[std::string(orig_path, length)] = *d;
         g_free(orig_path);
       }
       else
       {
-        std::string fname = g_basename(d->c_str());
+        std::string fname = base::basename(*d);
         fname = fname.substr(0, fname.rfind('.')).append(".mwb");
         // if no real_path file in the autosave, this could be an autosave from an older version
         files[fname] = *d;
@@ -206,7 +291,7 @@ void WBContextModel::detect_auto_save_files(const std::string &autosave_dir)
     }
     else
     {
-      g_message("Found model auto-save %s, but it is empty. Deleting it...", d->c_str());
+      log_info("Found model auto-save %s, but it is empty. Deleting it...\n", d->c_str());
       base_rmdir_recursively(d->c_str());
     }
   }
@@ -263,16 +348,31 @@ void WBContextModel::model_created(ModelFile *file, workbench_DocumentRef doc)
   _file= file;
   _doc= doc;
 
+  std::string target_version = _wbui->get_wb()->get_grt_manager()->get_app_option_string("DefaultTargetMySQLVersion");
+  if (target_version.empty())
+    target_version = "5.6.1";
+  
   _wbui->get_wb()->get_component<WBComponentLogical>()->setup_logical_model(doc.get_grt(), _doc);
-  _wbui->get_wb()->get_component<WBComponentPhysical>()->setup_physical_model(doc.get_grt(), _doc, "Mysql", "5.1.22");
+  _wbui->get_wb()->get_component<WBComponentPhysical>()->setup_physical_model(doc.get_grt(), _doc, "Mysql", target_version);
   
   _wbui->get_wb()->foreach_component(boost::bind(&WBComponent::reset_document, _1));
   
   _doc->physicalModels().get(0)->get_data()->set_delegate(this);
   
   _doc->physicalModels()[0]->get_data()->realize();
-  
+
   _wbui->get_wb()->request_refresh(RefreshNewModel, "", 0);
+
+  // setup GRT proxy object
+  _grtmodel_panel = ui_ModelPanelRef(_doc.get_grt());
+  _grtmodel_panel->model(_doc->physicalModels()[0]);
+
+  if (_sidebar_dockpoint == NULL)
+    _sidebar_dockpoint = mforms::manage(new mforms::DockingPoint(new mforms::TabViewDockingPoint(_secondary_sidebar, MODEL_DOCKING_POINT), true));
+  _grtmodel_panel->commonSidebar(mforms_to_grt(_doc.get_grt(), _sidebar_dockpoint));
+
+  grt::DictRef info(_doc.get_grt());
+  grt::GRTNotificationCenter::get()->send_grt("GRNModelCreated", _grtmodel_panel, info);
 }
 
 
@@ -290,16 +390,34 @@ void WBContextModel::model_loaded(ModelFile *file, workbench_DocumentRef doc)
   _wbui->get_wb()->request_refresh(RefreshNewModel, "", 0);
   
   std::string temp_dir = _file->get_tempdir_path();
-  for (std::map<std::string, std::string>::iterator iter= ::auto_save_files.begin();
+  for (std::map<std::string, std::string>::iterator iter = ::auto_save_files.begin();
        iter != ::auto_save_files.end(); ++iter)
   {
     if (iter->second == temp_dir)
     {
       ::auto_save_files.erase(iter);
-      _wbui->refresh_home_models();
+      _wbui->refresh_home_documents();
       break;
     }
   }
+
+  // setup GRT proxy object
+  _grtmodel_panel = ui_ModelPanelRef(get_grt());
+  _grtmodel_panel->model(_doc->physicalModels()[0]);
+
+  if (_sidebar_dockpoint == NULL)
+    _sidebar_dockpoint = mforms::manage(new mforms::DockingPoint(new mforms::TabViewDockingPoint(_secondary_sidebar, MODEL_DOCKING_POINT), true));
+  _grtmodel_panel->commonSidebar(mforms_to_grt(_doc.get_grt(), _sidebar_dockpoint));
+
+  grt::DictRef info(get_grt());
+  grt::GRTNotificationCenter::get()->send_grt("GRNModelOpened", _grtmodel_panel, info);
+}
+
+
+void WBContextModel::model_closed()
+{
+  grt::DictRef info(get_grt());
+  grt::GRTNotificationCenter::get()->send_grt("GRNModelClosed", _grtmodel_panel, info);
 }
 
 
@@ -506,7 +624,7 @@ ModelDiagramForm *WBContextModel::get_diagram_form(mdc::CanvasView *view)
 
 void WBContextModel::notify_diagram_created(ModelDiagramForm *view)
 {
-    scoped_connect(view->get_model_diagram()->signal_changed(),
+   view->scoped_connect(view->get_model_diagram()->signal_changed(),
         boost::bind(&WBContextModel::diagram_object_changed, this, _1, _2, view));
     
   // now called from wb_component_physical.cpp:model_list_changed
@@ -544,7 +662,7 @@ void WBContextModel::update_current_diagram(bec::UIForm *form)
     if (diagram.is_valid() && diagram->owner().is_valid())
       diagram->owner()->currentDiagram(diagram);    
     
-    (*_wbui->get_command_ui()->signal_validate_edit_menu_items())();
+    _wbui->get_command_ui()->revalidate_edit_menu_items();
   }
 }
 
@@ -555,7 +673,10 @@ void WBContextModel::diagram_object_changed(const std::string &member, const grt
   {
     if (view->get_model_diagram().is_valid())
     {
-      _wbui->get_wb()->request_refresh(RefreshViewName, view->get_model_diagram()->name(), (NativeHandle)view->get_view()->get_user_data());
+      base::NotificationInfo info;
+      info["form"] = view->form_id();
+      info["title"] = view->get_title();
+      base::NotificationCenter::get()->send("GNFormTitleDidChange", view, info);
       _wbui->get_physical_overview()->send_refresh_diagram(view->get_model_diagram());
     }
   }
@@ -585,6 +706,22 @@ bool WBContextModel::has_selected_schema()
   return false;
 }
 
+bool WBContextModel::has_selected_figures()
+{
+  ModelDiagramForm *view;
+  model_DiagramRef diagram(get_active_model_diagram(false));
+  if (!diagram.is_valid()) // in case an editor in a diagram tab is active
+  {
+    diagram = get_active_model_diagram(true);
+    view = dynamic_cast<ModelDiagramForm*>(_wbui->get_active_main_form());
+  }
+  else
+    view= dynamic_cast<ModelDiagramForm*>(_wbui->get_active_form());
+  if (view && view->has_selection())
+    return true;
+  return false;
+}
+
 bool WBContextModel::has_selected_model()
 {
   if (_wbui->get_active_main_form() == _overview)
@@ -599,24 +736,118 @@ void WBContextModel::add_model_schema()
 
 void WBContextModel::add_model_diagram()
 {
-  _wbui->get_wb()->add_new_diagram(get_active_model(true));
+  add_new_diagram(get_active_model(true));
 }
 
 void WBContextModel::add_model_table()
 {
-  _overview->get_active_schema_node()->add_new_db_table(_wbui->get_wb());
+  if (_overview->get_active_schema_node())
+    _overview->get_active_schema_node()->add_new_db_table(_wbui->get_wb());
 }
 
 
 void WBContextModel::add_model_view()
 {
-  _overview->get_active_schema_node()->add_new_db_view(_wbui->get_wb());
+  if (_overview->get_active_schema_node())
+    _overview->get_active_schema_node()->add_new_db_view(_wbui->get_wb());
 }
 
 
 void WBContextModel::add_model_rgroup()
 {
-  _overview->get_active_schema_node()->add_new_db_routine(_wbui->get_wb());
+  if (_overview->get_active_schema_node())
+    _overview->get_active_schema_node()->add_new_db_routine(_wbui->get_wb());
+}
+
+void WBContextModel::remove_figure()
+{
+  ModelDiagramForm *view;
+  model_DiagramRef diagram(get_active_model_diagram(false));
+  if (!diagram.is_valid()) // in case an editor in a diagram tab is active
+  {
+    diagram = get_active_model_diagram(true);
+    view = dynamic_cast<ModelDiagramForm*>(_wbui->get_active_main_form());
+  }
+  else
+    view= dynamic_cast<ModelDiagramForm*>(_wbui->get_active_form());
+  if (view)
+    view->remove_selection();
+}
+
+
+GrtObjectRef WBContextModel::duplicate_object(const db_DatabaseObjectRef &object, grt::CopyContext &copy_context)
+{
+  std::set<std::string> skip;
+  skip.insert("oldName");
+  
+  if (object.is_instance(db_Table::static_class_name()))
+  {
+    db_TableRef table(db_TableRef::cast_from(object));
+    
+    // copy table
+    db_TableRef dbtable(db_TableRef::cast_from(copy_context.copy(table, skip)));
+    
+    copy_context.update_references();
+    
+    // post-processing
+    // - uniquefy foreign key names
+    int max_fk_len = workbench_physical_ModelRef::cast_from(dbtable->owner()->owner()->owner())->rdbms()->maximumIdentifierLength();
+    grt::ListRef<db_ForeignKey> fks(dbtable->foreignKeys());
+    std::set<std::string> used_fk_names = bec::SchemaHelper::get_foreign_key_names(db_SchemaRef::cast_from(dbtable->owner()));
+    for (size_t c= fks.count(), i = 0; i < c; i++)
+    {
+      db_ForeignKeyRef fk(fks[i]);
+      
+      fk->name(bec::SchemaHelper::get_unique_foreign_key_name(used_fk_names, fk->name(), max_fk_len));
+    }
+    
+    if (grt::find_named_object_in_list(db_SchemaRef::cast_from(dbtable->owner())->tables(), dbtable->name()).is_valid())
+      dbtable->name(grt::get_name_suggestion_for_list_object(db_SchemaRef::cast_from(dbtable->owner())->tables(),
+                                                             *dbtable->name()+"_copy"));
+    grt::AutoUndo undo(get_grt());
+    
+    db_SchemaRef::cast_from(dbtable->owner())->tables().insert(dbtable);
+    
+    undo.end(strfmt(_("Duplicate Table '%s'"), dbtable->name().c_str()));
+    
+    return dbtable;
+  }
+  else if (object.is_instance(db_View::static_class_name()))
+  {
+    db_ViewRef view(db_ViewRef::cast_from(object));
+    
+    // copy view
+    db_ViewRef dbview(db_ViewRef::cast_from(copy_context.copy(view)));
+    
+    if (grt::find_named_object_in_list(db_SchemaRef::cast_from(dbview->owner())->views(), dbview->name()).is_valid())
+      dbview->name(grt::get_name_suggestion_for_list_object(db_SchemaRef::cast_from(dbview->owner())->views(),
+                                                            *dbview->name()+"_copy"));
+    grt::AutoUndo undo(get_grt());
+    
+    db_SchemaRef::cast_from(dbview->owner())->views().insert(dbview);
+    
+    undo.end(strfmt(_("Duplicate View '%s'"), dbview->name().c_str()));
+
+    return dbview;
+  }
+  else if (object.is_instance(db_RoutineGroup::static_class_name()))
+  {
+    db_RoutineGroupRef routineGroup(db_RoutineGroupRef::cast_from(object));
+    
+    // copy routineGroup
+    db_RoutineGroupRef dbroutineGroup(db_RoutineGroupRef::cast_from(copy_context.copy(routineGroup)));
+    
+    if (grt::find_named_object_in_list(db_SchemaRef::cast_from(dbroutineGroup->owner())->routineGroups(), dbroutineGroup->name()).is_valid())
+      dbroutineGroup->name(grt::get_name_suggestion_for_list_object(db_SchemaRef::cast_from(dbroutineGroup->owner())->routineGroups(),
+                                                                    *dbroutineGroup->name()+"_copy"));
+    grt::AutoUndo undo(get_grt());
+    
+    db_SchemaRef::cast_from(dbroutineGroup->owner())->routineGroups().insert(dbroutineGroup);
+    
+    undo.end(strfmt(_("Duplicate Routine Group '%s'"), dbroutineGroup->name().c_str()));
+    return dbroutineGroup;
+  }
+  return GrtObjectRef();
 }
 
 
@@ -714,6 +945,7 @@ int WBContextModel::get_object_list_popup_items(bec::UIForm *form,
   bec::MenuItem item;
   bool can_delete= objects->count() > 0;
   bool can_copy= can_delete;
+  bool can_remove= true;
   if (model != NULL)
     for (std::vector<bec::NodeId>::const_iterator iterator= nodes.begin(); iterator != nodes.end(); iterator++)
     {
@@ -722,29 +954,40 @@ int WBContextModel::get_object_list_popup_items(bec::UIForm *form,
       if (can_copy && !model->is_copyable(*iterator))
         can_copy= false;
     }
-  
-  item.checked= false;
-  item.enabled= can_copy && can_delete;
-  item.caption= label.empty() ? _("Cut") : strfmt(_("Cut %s"), label.c_str());
-  item.name= "builtin:cut";
-  item.type= MenuAction;
-  items.push_back(item);
-  
-  item.enabled= can_copy;
-  item.caption= label.empty() ? _("Copy") : strfmt(_("Copy %s"), label.c_str());
-  item.name= "builtin:copy";
-  item.type= MenuAction;
-  items.push_back(item);
-  
-  item.enabled= form->can_paste();
-  item.caption= strfmt(_("Paste %s"), wb->get_clipboard()->get_content_description().c_str());
-  item.name= "builtin:paste";
-  item.type= MenuAction;
-  items.push_back(item);
-  
-  item.type= MenuSeparator;
-  items.push_back(item);
-  
+
+  // can't copy/paste diagrams
+  if (nodes.empty() || nodes[0][0] != 0)
+  {
+    item.checked= false;
+    item.enabled= can_copy && can_delete;
+    item.caption= label.empty() ? _("Cut") : strfmt(_("Cut %s"), label.c_str());
+    item.name= "builtin:cut";
+    item.type= MenuAction;
+    items.push_back(item);
+    
+    item.enabled= can_copy;
+    item.caption= label.empty() ? _("Copy") : strfmt(_("Copy %s"), label.c_str());
+    item.name= "builtin:copy";
+    item.type= MenuAction;
+    items.push_back(item);
+    
+    item.enabled= form->can_paste();
+    item.caption= strfmt(_("Paste %s"), wb->get_clipboard()->get_content_description().c_str());
+    item.name= "builtin:paste";
+    item.type= MenuAction;
+    items.push_back(item);
+
+  // this is already implemented, but not yet exposed.. uncomment once the whole thing is working
+  //  item.enabled= form->can_paste();
+  //  item.caption= strfmt(_("Duplicate %s"), wb->get_clipboard()->get_content_description().c_str());
+  //  item.name= "builtin:duplicate";
+  //  item.type= MenuAction;
+  //  items.push_back(item);
+
+    item.type= MenuSeparator;
+    items.push_back(item);
+  }
+
   // Plugin dependent menu entries.
   size_t old_count= items.size();
   
@@ -752,11 +995,16 @@ int WBContextModel::get_object_list_popup_items(bec::UIForm *form,
   {
     add_object_plugins_to_popup_menu(objects, groups, items);
     
+    bool has_objects = false;
+    bool connections_only = true;
     grt::ListRef<GrtObject> model_objects(wb->get_grt());
     for (size_t c= objects.count(), i= 0; i < c; i++)
     {
+      if (!objects[i].is_instance(model_Connection::static_class_name()))
+        connections_only = false;
       if (objects[i].is_instance(model_Object::static_class_name()))
       {
+        has_objects = true;
         FOREACH_COMPONENT(wb->_components, compo)
         {
           GrtObjectRef model_object((*compo)->get_object_for_figure(model_ObjectRef::cast_from(objects[i])));
@@ -769,9 +1017,13 @@ int WBContextModel::get_object_list_popup_items(bec::UIForm *form,
         }
       }
     }
-    
+    if (connections_only || !has_objects)
+      can_remove = false;
+
     if (model_objects.count() > 0)
       add_object_plugins_to_popup_menu(model_objects, groups, items);
+    else
+      can_remove = false;
   }
   
   // At the end of the list there is the delete command.
@@ -788,7 +1040,15 @@ int WBContextModel::get_object_list_popup_items(bec::UIForm *form,
   item.name= "builtin:delete";
   item.type= MenuAction;
   items.push_back(item);
-  
+
+  if (nodes.empty() || nodes[0][0] != 0)
+  {
+    item.caption= label.empty() ? _("Remove Figure") : strfmt(_("Remove Figure %s"), label.c_str());
+    item.name= "builtin:removeFigure";
+    item.type= MenuAction;
+    item.enabled = can_remove;
+    items.push_back(item);
+  }
   return int(items.size() - initial_count);
 }
 
@@ -829,12 +1089,12 @@ int WBContextModel::add_object_plugins_to_popup_menu(const grt::ListRef<GrtObjec
       for (std::list<std::string>::const_iterator ctx= groups.begin();
            ctx != groups.end(); ++ctx)
       {
-        if ((*ctx)[ctx->size()-1] == '*' && g_strncasecmp(ctx->c_str(), str.c_str(), (guint) ctx->size() - 1) == 0)
+        if ((*ctx)[ctx->size()-1] == '*' && g_ascii_strncasecmp(ctx->c_str(), str.c_str(), (guint) ctx->size() - 1) == 0)
         {
           match= true;
           break;
         }
-        else if ((*ctx)[ctx->size() - 1] != '*' && g_strcasecmp(ctx->c_str(), str.c_str()) == 0)
+        else if ((*ctx)[ctx->size() - 1] != '*' && g_ascii_strcasecmp(ctx->c_str(), str.c_str()) == 0)
         {
           match= true;
           break;
@@ -861,8 +1121,15 @@ int WBContextModel::add_object_plugins_to_popup_menu(const grt::ListRef<GrtObjec
         (*iter)->groups().get_index("model/Editors") != grt::BaseListRef::npos)
     {
       app_PluginRef plugin(_wbui->get_wb()->get_plugin_manager()->get_plugin("wb.edit.editSelectedFigureInNewWindow"));
-      
-      item.caption= _("Edit in New Window...");
+
+      item.caption= _("Edit in New Tab...");
+      if (_wbui->get_active_form() && !_wbui->get_active_form()->get_edit_target_name().empty())
+      {
+        items.back().caption = base::strfmt(_("Edit %s..."), _wbui->get_active_form()->get_edit_target_name().c_str());
+
+        item.caption= base::strfmt(_("Edit %s in New Tab..."), _wbui->get_active_form()->get_edit_target_name().c_str());
+      }
+
       item.name= "plugin:"+*plugin->name();
       item.type= MenuAction;
       // state for this item will be the same as for the previous one
@@ -980,7 +1247,7 @@ void WBContextModel::selection_changed()
       }
     }
   }
-  (*_wbui->get_command_ui()->signal_validate_edit_menu_items())();
+  _wbui->get_command_ui()->revalidate_edit_menu_items();
 }
 
 
@@ -1096,3 +1363,172 @@ void WBContextModel::export_ps(const std::string &path)
 }
 
 #endif // BasicExport____
+
+
+#ifndef Diagrams_and_Canvas____
+
+//--------------------------------------------------------------------------------
+// Canvas Management
+
+
+void WBContextModel::add_new_diagram(const model_ModelRef &model)
+{
+  _wbui->get_wb()->show_status_text(_("Creating Diagram..."));
+  
+ // model_DiagramRef view(model_DiagramRef::cast_from(_wbui->get_wb()->execute_in_grt_thread("Create new diagram",
+   //                                                                       boost::bind(&model_Model::addNewDiagram, &model.content(), true))));
+  model_DiagramRef view = model->addNewDiagram(true);
+  if (view.is_valid())
+  {
+    model->currentDiagram(view);
+    view->get_data()->realize();
+  }
+  
+  _wbui->get_wb()->show_status_text(_("Diagram added."));
+}
+
+
+void WBContextModel::switch_diagram(const model_DiagramRef &view)
+{
+  _wbui->get_wb()->switched_view(view->get_data()->get_canvas_view());
+}
+
+#endif // Diagrams_and_Canvas____
+#ifndef Canvas_Objects____
+/** 
+ ****************************************************************************
+ * @brief Delete a canvas object from the canvas/model
+ * 
+ * Will remove the figure from the model and delete all known
+ * references to it. There may be some references left, which will cause
+ * the GRT object and its bridge to stay alive for some more time, so the
+ * bridge is marked as invalid to prevent the canvas item from appearing
+ * on screen.
+ *
+ * @param figure to be removed
+ ****************************************************************************
+ */
+bool WBContextModel::delete_object(model_ObjectRef object)
+{
+  model_DiagramRef view(model_DiagramRef::cast_from(object->owner()));
+  
+  FOREACH_COMPONENT(_wbui->get_wb()->_components, iter)
+  {
+    if ((*iter)->handles_figure(object))
+      return (*iter)->delete_model_object(object, false);
+  }
+  return false;
+}
+
+
+bool WBContextModel::remove_figure(model_ObjectRef object)
+{
+  model_DiagramRef view(model_DiagramRef::cast_from(object->owner()));
+  
+  FOREACH_COMPONENT(_wbui->get_wb()->_components, iter)
+  {
+    if ((*iter)->handles_figure(object))
+      return (*iter)->delete_model_object(object, true);
+  }
+  return false;
+}
+
+#endif // Canvas_Objects____
+
+model_DiagramRef WBContextModel::get_view_with_id(const std::string &id)
+{
+  return model_DiagramRef::cast_from(get_wbui()->get_wb()->get_grt()->find_object_by_id(id, "/wb/doc"));
+}
+
+bool WBContextModel::delete_diagram(const model_DiagramRef &view)
+{
+  grt::AutoUndo undo(get_wbui()->get_wb()->get_grt());
+  view->owner()->diagrams().remove_value(view);
+  undo.end(strfmt(_("Delete Diagram '%s'"), view->name().c_str()));
+  
+#ifdef _WIN32
+  // in windows, a diagram is released as soon as the tab is closed. That means
+  // if a user closes a diagram before deleting it, the diagram is released before 
+  // it's removed from the list of diagrams in the list. That will cause the overview
+  // to be refreshed too early and with outdated contents. We force another explicit refresh
+  // here to workaround that.
+  if (get_wbui()->get_physical_overview())
+    get_wbui()->get_physical_overview()->send_refresh_diagram(model_DiagramRef());
+#endif
+  return true;
+}
+
+
+void WBContextModel::begin_plugin_exec()
+{
+  // lock the canvas so that it doesn't keep refreshing all the time.
+  // XXX we have to find some way to allow plugins control refresh freezing
+  ModelDiagramForm *view= dynamic_cast<ModelDiagramForm*>(get_wbui()->get_active_main_form());
+  _locked_view_for_plugin_exec = 0;
+  
+  if (view) 
+  {
+    _locked_view_for_plugin_exec = view->get_view();
+    _locked_view_for_plugin_exec->lock_redraw();
+  }  
+}
+
+
+void WBContextModel::end_plugin_exec()
+{
+  // form can get destroyed after a plugin is executed
+  if (_locked_view_for_plugin_exec && get_diagram_form(_locked_view_for_plugin_exec))
+    _locked_view_for_plugin_exec->unlock_redraw();
+  _locked_view_for_plugin_exec = 0;
+}
+
+
+/**
+ * Called by the front end when the user data type editor has been closed. We can then remove our reference to it.
+ */
+static void userTypeEditorClosed(UserDefinedTypeEditor **editor_ptr)
+{
+  *editor_ptr = NULL;
+}
+
+void WBContextModel::show_user_type_editor(workbench_physical_ModelRef model)
+{
+  if (_current_user_type_editor == NULL)
+  {
+    _current_user_type_editor = new UserDefinedTypeEditor(get_wbui(), model);
+    scoped_connect(_current_user_type_editor->signal_closed(), boost::bind(userTypeEditorClosed, &_current_user_type_editor));
+  }
+  _current_user_type_editor->show_modal(NULL, NULL);
+}
+
+mforms::View *WBContextModel::shared_secondary_sidebar()
+{
+  return _secondary_sidebar;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+static struct RegisterNotifDocs
+{
+  RegisterNotifDocs()
+  {
+    base::NotificationCenter::get()->register_notification("GRNModelOpened",
+                                                           "modeling",
+                                                           "Sent when a model document finishes loading.",
+                                                           "ui.ModelPanel instance",
+                                                           "");
+
+    base::NotificationCenter::get()->register_notification("GRNModelCreated",
+                                                           "modeling",
+                                                           "Sent when a new model document is created.",
+                                                           "ui.ModelPanel instance",
+                                                           "");
+
+    base::NotificationCenter::get()->register_notification("GRNModelClosed",
+                                                           "modeling",
+                                                           "Sent when a model document is closed.",
+                                                           "ui.ModelPanel instance",
+                                                           "");
+  }
+} initdocs;
+
