@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -29,6 +29,7 @@
 #include "mdc_back_layer.h"
 #include "mforms/form.h"
 
+#include "model/wb_context_model.h"
 #include "model/wb_layer_tree.h"
 
 #include "grt/common.h"
@@ -43,17 +44,19 @@
 
 #include "wb_physical_model_diagram_features.h"
 #include "base/string_utilities.h"
-#include <boost/lambda/lambda.hpp>
+#include "base/log.h"
+#include <boost/lambda/bind.hpp>
 
 #include "mforms/menubar.h"
 
 #define DEFAULT_TOOL "basic/select"
 
+
+DEFAULT_LOG_DOMAIN("ModelDiagram");
+
 using namespace wb;
 using namespace bec;
 using namespace base;
-using namespace MySQL::Drawing;
-using namespace MySQL::Geometry;
 
 static const double zoom_steps[]= {
   2.0,
@@ -77,22 +80,17 @@ static const double zoom_steps[]= {
 
 
 ModelDiagramForm::ModelDiagramForm(WBComponent *owner, const model_DiagramRef &view)
-: _view(0), _owner(owner), _model_diagram(view), _layer_tree(owner->get_wb()), _mini_view(0), _menu(0), 
+: _view(0), _owner(owner), _model_diagram(view), _mini_view(0), _menu(0),
 _toolbar(0), _tools_toolbar(0), _options_toolbar(0)
 {
   _drag_panning= false;
   _space_panning= false;
+  _highlight_fks = false;
   _inline_edit_context= 0;
   _update_count = 0;
-
-  _layer_tree.set_activate_callback(boost::bind(&WBContextUI::activate_figure, owner->get_wb()->get_ui(), _1));
-
-  model_list_changed(0, true, grt::ValueRef());
-  
-  scoped_connect(view->owner()->signal_list_changed(), boost::bind(&ModelDiagramForm::model_list_changed, this, _1, _2, _3));
+  _layer_tree = 0;
 
   scoped_connect(_model_diagram->signal_refreshDisplay(), boost::bind(&ModelDiagramForm::diagram_changed, this, _1));
-  scoped_connect(_model_diagram->signal_list_changed(), boost::bind(&ModelDiagramForm::diagram_objects_changed, this, _1, _2, _3));
 
   _current_mouse_x= -1;
   _current_mouse_y= -1;
@@ -111,11 +109,16 @@ _toolbar(0), _tools_toolbar(0), _options_toolbar(0)
   _features= new PhysicalModelDiagramFeatures(this);
   
   _options_toolbar = new mforms::ToolBar(mforms::OptionsToolBar);
+  NotificationCenter::get()->add_observer(this, "GNColorsChanged");
 }
 
+//--------------------------------------------------------------------------------------------------
 
 ModelDiagramForm::~ModelDiagramForm()
 {
+  NotificationCenter::get()->remove_observer(this);
+
+  delete _layer_tree;
   delete _options_toolbar;
   delete _toolbar;
   delete _tools_toolbar;
@@ -124,15 +127,84 @@ ModelDiagramForm::~ModelDiagramForm()
   delete _features;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+void ModelDiagramForm::handle_notification(const std::string &name, void *sender, base::NotificationInfo &info)
+{
+  if (name == "GNColorsChanged")
+  {
+    // Single colors or the entire color scheme changed.
+    update_toolbar_icons();
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
+std::string ModelDiagramForm::get_form_context_name() const
+{
+  return WB_CONTEXT_MODEL; 
+}
+
+//--------------------------------------------------------------------------------------------------
+
 mforms::ToolBar *ModelDiagramForm::get_toolbar()
 {
   if (!_toolbar)
   {
     _toolbar = get_wb()->get_ui()->get_command_ui()->create_toolbar("data/model_diagram_toolbar.xml");
+    update_toolbar_icons();
   }
   return _toolbar;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+// Implemented in wb_sql_editor_form_ui.cpp.
+extern std::string find_icon_name(std::string icon_name, bool use_win8);
+
+void ModelDiagramForm::update_toolbar_icons()
+{
+  bool use_win8;
+
+  switch (base::Color::get_active_scheme())
+  {
+  case base::ColorSchemeStandardWin8:
+  case base::ColorSchemeStandardWin8Alternate:
+    use_win8 = true;
+    break;
+
+  default:
+    use_win8 = false;
+  }
+
+  mforms::ToolBarItem *item = _toolbar->find_item("wb.toggleSidebar");
+  if (item != NULL)
+  {
+    item->set_icon(find_icon_name(item->get_icon(), use_win8));
+    item->set_alt_icon(find_icon_name(item->get_alt_icon(), use_win8));
+  }
+
+  item = _toolbar->find_item("wb.toggleSecondarySidebar");
+  if (item != NULL)
+  {
+    item->set_icon(find_icon_name(item->get_icon(), use_win8));
+    item->set_alt_icon(find_icon_name(item->get_alt_icon(), use_win8));
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
+mforms::TreeNodeView *ModelDiagramForm::get_layer_tree()
+{
+  if (!_layer_tree)
+  {
+    _layer_tree = new LayerTree(get_wb()->get_ui(), this, _model_diagram);
+    _layer_tree->refresh();
+  }
+  return _layer_tree;
+}
+
+//--------------------------------------------------------------------------------------------------
 
 mforms::ToolBar *ModelDiagramForm::get_tools_toolbar()
 {
@@ -245,6 +317,7 @@ void ModelDiagramForm::revalidate_menu()
     _menu->set_item_checked("wb.edit.toggleGridAlign", get_diagram_options().get_int("AlignToGrid", 0) != 0);
     _menu->set_item_checked("wb.edit.toggleGrid", get_diagram_options().get_int("ShowGrid", 1) != 0);
     _menu->set_item_checked("wb.edit.togglePageGrid", get_diagram_options().get_int("ShowPageGrid", 1) != 0);
+    _menu->set_item_checked("wb.edit.toggleFKHighlight", get_diagram_options().get_int("ShowFKHighlight", 0) != 0);
     std::string notation = workbench_physical_ModelRef::cast_from(get_model_diagram()->owner())->figureNotation();
     for (int i= 0; figure_notations[i]; i++)
       _menu->set_item_checked(strfmt("wb.view.setFigureNotation:%s", figure_notations[i]), notation == figure_notations[i]);
@@ -271,30 +344,7 @@ void ModelDiagramForm::revalidate_menu()
   }
 }
 
-
-static ToolbarItemType get_toolbar_item_type(const std::string &type)
-{
-  if (type == "action")
-    return ToolbarAction;
-  else if (type == "separator")
-    return ToolbarSeparator;
-  else if (type == "toggle")
-    return ToolbarToggle;
-  else if (type == "radio")
-    return ToolbarRadio;
-  else if (type == "dropdown")
-    return ToolbarDropDown;
-  else if (type == "label")
-    return ToolbarLabel;
-  else if (type == "check")
-    return ToolbarCheck;
-  else if (type == "search")
-    return ToolbarSearch;
-  else
-    throw std::runtime_error("invalid toolbar item type "+type);
-  return ToolbarAction;
-}
-
+//--------------------------------------------------------------------------------------------------
 
 void ModelDiagramForm::update_options_toolbar()
 {
@@ -356,7 +406,6 @@ void ModelDiagramForm::update_options_toolbar()
 
     item->set_name(titem->name());
     item->set_tooltip(titem->tooltip()); 
-    //titem->command();
     
     _options_toolbar->add_item(item);
   }
@@ -430,31 +479,12 @@ void ModelDiagramForm::selection_changed()
     get_wb()->get_grt_manager()->run_once_when_idle(boost::bind(&ModelDiagramForm::revalidate_menu, this));
 }
 
-
-void ModelDiagramForm::model_list_changed(grt::internal::OwnedList *list, bool added, const grt::ValueRef &value)
-{
-  if (_update_count == 0)
-  {
-    if (!list || list == _model_diagram->owner()->diagrams().valueptr())
-    {
-      std::string path= _owner->get_wb()->get_ui()->get_diagram_path(_model_diagram);
-      if (path.empty())
-        throw std::logic_error("diagram is not in model yet");
-      _layer_tree.set_displayed_global_value(path, false);
-    }
-  }
-}
+//--------------------------------------------------------------------------------------------------
 
 void ModelDiagramForm::diagram_changed(const model_ObjectRef &object)
 {
-  if (_update_count == 0)
-    _layer_tree.tree_changed();
-}
-
-void ModelDiagramForm::diagram_objects_changed(grt::internal::OwnedList *list, bool added, const grt::ValueRef &value)
-{
-  if (_update_count == 0)
-    _layer_tree.tree_changed();
+//XXX  if (_update_count == 0)
+//XXX    _layer_tree->tree_changed();
 }
 
 void ModelDiagramForm::attach_canvas_view(mdc::CanvasView *cview)
@@ -824,8 +854,6 @@ void ModelDiagramForm::handle_mouse_double_click(mdc::MouseButton button, int x,
 {
   stop_editing();
   
-  Point pos(_view->window_to_canvas(x, y));
-  
   if (button != mdc::ButtonLeft && button != mdc::ButtonMiddle)
     return;
 
@@ -861,17 +889,14 @@ bool ModelDiagramForm::handle_key(const mdc::KeyInfo &key, bool press, mdc::Even
       {
         if (iter->command.find("tool:") == 0)
           set_tool(iter->command.substr(strlen("tool:")));
+        else if (iter->command == "zoomin")
+          zoom_in();
+        else if (iter->command == "zoomout")
+          zoom_out();
+        else if (iter->command == "zoomdefault")
+          set_zoom(1);
         else
-        {
-          if (iter->command == "zoomin")
-            zoom_in();
-          else
-            if (iter->command == "zoomout")
-              zoom_out();
-            else
-              if (iter->command == "zoomdefault")
-                set_zoom(1);
-        }
+          get_wb()->get_ui()->get_command_ui()->activate_command(iter->command);
         return true;
       }
     }
@@ -948,7 +973,9 @@ void ModelDiagramForm::set_tool(std::string tool)
         (*item)->set_checked(false);
     }
   }
-  _owner->get_wb()->tool_changed(_view);
+
+  if (_owner->get_wb()->tool_changed)
+    _owner->get_wb()->tool_changed(_view);
 }
 
 
@@ -972,13 +999,12 @@ void ModelDiagramForm::reset_tool(bool notify)
     _reset_tool(this);
 
   _cursor= "";
-  static const bool fval = false;
-  boost::function<bool ()> f = boost::lambda::var(fval);
+  boost::function<bool ()> f = boost::lambda::constant(false);
   _handle_button = (boost::bind(f));
   _handle_motion = (boost::bind(f));
   _reset_tool = (boost::bind(f));
 
-  if (notify)
+  if (notify && _owner->get_wb()->tool_changed)
     _owner->get_wb()->tool_changed(_view);
 }
 
@@ -1047,7 +1073,7 @@ bool ModelDiagramForm::can_select_all()
 
 static void check_if_can_paste(WBComponent *compo, const grt::ObjectRef &object, bool *result)
 {
-  if (compo->can_paste_non_model_object(object))
+  if (compo->can_paste_object(object))
     *result= true;
 }
 
@@ -1065,18 +1091,10 @@ bool ModelDiagramForm::can_paste()
       g_message("copy buffer has null value");
       return false;
     }
-    if (model_ObjectRef::can_wrap(*iter))
-    {
-      if (!wb->get_component_handling(model_ObjectRef::cast_from(*iter)))
-        return false;
-    }
-    else
-    {
-      bool result= false;
-      wb->foreach_component(boost::bind(check_if_can_paste, _1, *iter, &result));
-      if (!result)
-        return false;
-    }
+    bool result= false;
+    wb->foreach_component(boost::bind(check_if_can_paste, _1, *iter, &result));
+    if (!result)
+      return false;
   }
   return !get_clipboard()->empty();
 }
@@ -1128,38 +1146,40 @@ void ModelDiagramForm::cut()
 
   um->begin_undo_group();
   copy();
-  int count= delete_selection(true);
+  
+  int count= get_copiable_selection().count();
+  remove_selection();
   um->end_undo_group();
   um->set_action_description(strfmt(_("Cut %s"), edit_target_name.c_str()));
 
-  _owner->get_wb()->show_status_text(strfmt(_("%i object(s) cut."), count));
+  _owner->get_wb()->show_status_text(strfmt(_("%i figure(s) cut."), count));
 }
 
 
 void ModelDiagramForm::copy()
 {
+  // object must be duplicated on copy because if the object is edited after copy
+  // whatever is pasted should still be in the same state as it was on copy
   grt::ListRef<model_Object> selection(get_copiable_selection());
   bec::Clipboard *clip= get_clipboard();
-  model_LayerRef target_layer(get_model_diagram()->rootLayer());
   int count= 0;
   grt::CopyContext copy_context(_owner->get_grt());
-
+  
   clip->clear();
   for (size_t c= selection.count(), i= 0; i < c; i++)
   {
     WBComponent *compo= _owner->get_wb()->get_component_handling(selection.get(i));
     if (compo)
     {
-      model_ObjectRef copy(compo->clone_object(selection.get(i), target_layer, copy_context, true));
-      clip->append_data(copy);
+      compo->copy_object_to_clipboard(selection.get(i), copy_context);
       count++;
     }
   }
   clip->set_content_description(get_edit_target_name());
-
+  
   copy_context.finish();
   clip->changed();
-
+  
   _owner->get_wb()->show_status_text(strfmt(_("%i object(s) copied."), count));
 }
 
@@ -1174,7 +1194,7 @@ static void get_component_that_can_paste(WBComponent *compo,
                                          const grt::ObjectRef &object, 
                                          WBComponent **result)
 {
-  if (compo->can_paste_non_model_object(object))
+  if (compo->can_paste_object(object))
     *result= compo;
 }
 
@@ -1186,11 +1206,12 @@ void ModelDiagramForm::paste()
 
   WBContext *wb= _owner->get_wb();
   int count= 0;
+  int duplicated = 0;
 
   _paste_offset+= 20;
-
-  _owner->get_wb()->show_status_text(_("Pasting objects..."));
-
+  
+  _owner->get_wb()->show_status_text(_("Pasting figures..."));
+  
   grt::CopyContext context(wb->get_grt());
 
   grt::AutoUndo undo(wb->get_grt());
@@ -1198,55 +1219,47 @@ void ModelDiagramForm::paste()
   _model_diagram->unselectAll();
 
   std::list<grt::ObjectRef> data(get_clipboard()->get_data());
+
+  // find what objects are in layers so that if they are also selected individually, they don't
+  // get copied again
+  std::set<std::string> skip_objects;
   for (std::list<grt::ObjectRef>::iterator iter= data.begin();
-    iter != data.end(); ++iter)
+       iter != data.end(); ++iter)
   {
-    model_LayerRef layer= _model_diagram->rootLayer();
-    if (model_ObjectRef::can_wrap(*iter))
+    if (model_LayerRef::can_wrap(*iter))
     {
-      WBComponent *compo= wb->get_component_handling(model_ObjectRef::cast_from(*iter));
-
-      if (!compo) continue;
-
-      if (model_ObjectRef::can_wrap(*iter))
+      grt::ListRef<model_Figure> figures(model_LayerRef::cast_from(*iter)->figures());
+      GRTLIST_FOREACH(model_Figure, figures, fig)
       {
-        model_ObjectRef object(compo->clone_object(model_ObjectRef::cast_from(*iter), layer, context, false));
-        if (object.is_valid())
-        {
-          if (object.is_instance(model_Figure::static_class_name()))
-          {
-            model_FigureRef figure(model_FigureRef::cast_from(object));
-            figure->top(*figure->top() + _paste_offset);
-            figure->left(*figure->left() + _paste_offset);
-          }
-          else if (object.is_instance(model_Layer::static_class_name()))
-          {
-            model_LayerRef layer(model_LayerRef::cast_from(object));
-            layer->top(*layer->top() + _paste_offset);
-            layer->left(*layer->left() + _paste_offset);
-          }
-          count++;
-
-          _model_diagram->selectObject(object);
-        }
+        skip_objects.insert((*fig)->id());
       }
     }
-    else
+  }
+
+  for (std::list<grt::ObjectRef>::iterator iter= data.begin();
+    iter != data.end(); ++iter)
+  {    
+    // paste the object
+    WBComponent *compo= 0;
+    wb->foreach_component(boost::bind(get_component_that_can_paste, _1, *iter, &compo));
+    if (compo)
     {
-      WBComponent *compo= 0;
-      wb->foreach_component(
-        boost::bind(get_component_that_can_paste, _1, *iter, &compo)); 
-      if (compo)
+      if (skip_objects.find((*iter)->id()) == skip_objects.end())
       {
-        compo->paste_non_model_object(this, *iter);
+        compo->paste_object(this, *iter, context);
         count++;
       }
     }
+    else
+      log_warning("Don't know how to paste %s\n", iter->class_name().c_str());
   }
   context.finish();
   undo.end(strfmt(_("Paste %s"), get_clipboard()->get_content_description().c_str()));
 
-  _owner->get_wb()->show_status_text(strfmt(_("%i object(s) pasted."), count));
+  if (duplicated == 0)
+    _owner->get_wb()->show_status_text(strfmt(_("%i figure(s) pasted."), count));
+  else
+    _owner->get_wb()->show_status_text(strfmt(_("%i figure(s) pasted, %i duplicated."), count, duplicated));
 }
 
 
@@ -1260,14 +1273,34 @@ void ModelDiagramForm::select_all()
 }
 
 
-void ModelDiagramForm::delete_selection()
+void ModelDiagramForm::remove_selection()
 {
-  int count= delete_selection(false);
-  _owner->get_wb()->show_status_text(strfmt(_("%i object(s) deleted."), count));
+  grt::UndoManager *um= _owner->get_grt()->get_undo_manager();
+  grt::ListRef<model_Object> selection= get_selection();
+  
+  std::vector<model_ObjectRef> objects;
+  std::string edit_target_name= get_edit_target_name();
+  
+  um->begin_undo_group();
+  
+  for (size_t c= selection.count(), i= 0; i < c; i++)
+  {
+    if (selection.get(i).is_instance(model_Object::static_class_name()))
+      objects.push_back(model_ObjectRef::cast_from(selection.get(i)));
+  }
+  
+  for (size_t c= objects.size(), i= 0; i < c; i++)
+    _owner->get_wb()->get_model_context()->remove_figure(objects[i]);
+  
+  um->end_undo_group();
+  um->set_action_description(strfmt(_("Remove %s"), edit_target_name.c_str()));
+
+  _owner->get_wb()->show_status_text(strfmt(_("%i figure(s) removed. The corresponding DB objects were kept."), (int)objects.size()));
 }
 
 
-int ModelDiagramForm::delete_selection(bool noquestions)
+//XXX unused? but it's virtual, need to check if this is used anywhere...
+void ModelDiagramForm::delete_selection()
 {
   grt::UndoManager *um= _owner->get_grt()->get_undo_manager();
   grt::ListRef<model_Object> selection= get_selection();
@@ -1283,27 +1316,13 @@ int ModelDiagramForm::delete_selection(bool noquestions)
       objects.push_back(model_ObjectRef::cast_from(selection.get(i)));
   }
 
-  std::map<std::string, bool> options;
-
-  if (noquestions)
-    options["deletedbobjects"]= true;
-  else
-  {
-    std::string option;
-
-    option= _owner->get_wb()->get_wb_options().get_string("workbench.physical:DeleteObjectConfirmation", "ask");
-    if (option == "delete")
-      options["deletedbobjects"]= true;
-    else if (option == "keep")
-      options["deletedbobjects"]= false;
-  }
   for (size_t c= objects.size(), i= 0; i < c; i++)
-    _owner->get_wb()->delete_object(objects[i], options);
+    _owner->get_wb()->get_model_context()->delete_object(objects[i]);
 
   um->end_undo_group();
   um->set_action_description(strfmt(_("Delete %s"), edit_target_name.c_str()));
 
-  return objects.size();
+  _owner->get_wb()->show_status_text(strfmt(_("%i object(s) deleted."), (int)objects.size()));
 }
 
 
@@ -1820,3 +1839,18 @@ std::string ModelDiagramForm::get_title()
 }
 
 //--------------------------------------------------------------------------------------------------
+
+void ModelDiagramForm::set_highlight_fks(bool flag)
+{
+  _highlight_fks = flag;
+  _features->highlight_all_connections(flag);
+}
+
+//--------------------------------------------------------------------------------------------------
+ModelDiagramForm::UpdateLock::~UpdateLock()
+{
+  if (_form->_update_count > 0)
+    _form->_update_count--;
+  if (_form->_update_count == 0)
+    _form->_layer_tree->refresh();
+}
