@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,7 +22,7 @@
 #include "workbench/wb_context.h"
 #include "wb_context_model.h"
 #include "wb_user_datatypes.h"
-#include "mforms/menu.h"
+#include "mforms/menubar.h"
 
 using namespace wb;
 using namespace base;
@@ -35,9 +35,8 @@ UserDatatypeList::UserDatatypeList(WBContext *wb)
   add_column(mforms::StringColumnType, "Flags", 80);
   end_columns();
 
-  _menu = new mforms::Menu();
-  _menu->set_handler(boost::bind(&UserDatatypeList::handle_menu_action, this, _1));
-  _menu->add_item("Edit User Types...", "edit");
+  _menu = new mforms::ContextMenu();
+  _menu->add_item_with_title("Edit User Types...", boost::bind(&UserDatatypeList::handle_menu_action, this, "edit"));
   set_context_menu(_menu);
 }
 
