@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,23 +20,43 @@
 
 #include "stdafx.h"
 
-#include "workbench/wb_context_ui.h"
-#include "sqlide/wb_context_sqlide.h"
-#include "sqlide/wb_sql_editor_form.h"
-#include "sqlide/wb_sql_editor_snippets.h"
-#include "grt/common.h"
 #include "base/file_utilities.h"
 #include "base/string_utilities.h"
 #include "base/sqlstring.h"
+#include "base/trackable.h"
+#include "base/log.h"
+#include "base/notifications.h"
+#include "base/ui_form.h"
+
+#include "grtpp.h"
+
+#include "grts/structs.h"
+#include "grts/structs.app.h"
+
+#include "grt/editor_base.h"
+
+#include "workbench/wb_context.h"
+#include "workbench/wb_context_ui.h"
+#include "workbench/wb_command_ui.h"
+
+#include "sqlide/wb_context_sqlide.h"
+#include "sqlide/wb_sql_editor_form.h"
+#include "sqlide/wb_sql_editor_snippets.h"
+#include "sqlide/wb_sql_editor_help.h"
+#include "sqlide/wb_sql_editor_tree_controller.h"
+#include "grt/common.h"
 
 #include "objimpl/db.query/db_query_Editor.h"
 #include "objimpl/db.query/db_query_Resultset.h"
 #include "objimpl/db.query/db_query_EditableResultset.h"
 #include "objimpl/db.query/db_query_QueryBuffer.h"
 #include "grts/structs.db.query.h"
-#include "base/trackable.h"
 
-#include "base/log.h"
+#include "grtdb/db_object_helpers.h"
+
+#include "mforms/code_editor.h"
+#include "mforms/menubar.h"
+#include "mforms/toolbar.h"
 
 using namespace wb;
 using namespace bec;
@@ -59,11 +79,11 @@ class MYSQLWBBACKEND_PUBLIC_FUNC db_query_EditorConcreteImplData : public db_que
       if (added)
       {
         editor->grtobj()->owner(_self);
-        _self->queryBuffers().insert(editor->grtobj());
+        _self->queryEditors().insert(db_query_QueryEditorRef::cast_from(editor->grtobj()));
       }
       else
       {
-        _self->queryBuffers().remove_value(editor->grtobj());
+        _self->queryEditors().remove_value(db_query_QueryEditorRef::cast_from(editor->grtobj()));
         editor->grtobj()->reset_references();
       }
     }
@@ -74,22 +94,36 @@ class MYSQLWBBACKEND_PUBLIC_FUNC db_query_EditorConcreteImplData : public db_que
     boost::shared_ptr<SqlEditorForm> ref(_editor);
     if (ref)
     {
-      if (added)
+      db_query_QueryEditorRef editor;
+      
+      boost::shared_ptr<Sql_editor> sqleditor(ref->sql_editor(editor_index));
+      if (sqleditor)
       {
-        if (!rset->is_readonly())
-          _self->resultsets().insert(grtwrap_editablerecordset(_self, rset));
-        else
-          _self->resultsets().insert(grtwrap_recordset(_self, rset));
+        editor = db_query_QueryEditorRef::cast_from(sqleditor->grtobj());
       }
       else
       {
-        for (size_t c= _self->resultsets().count(), i= 0; i < c; i++)
+        log_warning("recordset_list_changed received for invalid editor index (%i). Internal state of db_query_QueryEditor might have become inconsistent\n",
+                    editor_index);
+        return;
+      }
+
+      if (added)
+      {
+        if (!rset->is_readonly())
+          editor->resultsets().insert(grtwrap_editablerecordset(editor, rset));
+        else
+          editor->resultsets().insert(grtwrap_recordset(editor, rset));
+      }
+      else
+      {
+        for (size_t c= editor->resultsets().count(), i= 0; i < c; i++)
         {
-          db_query_ResultsetRef e(_self->resultsets()[i]);
+          db_query_ResultsetRef e(editor->resultsets()[i]);
           
-          if (e->get_data()->recordset == rset)
+          if (dynamic_cast<WBRecordsetResultset*>(e->get_data())->recordset == rset)
           {
-            _self->resultsets().remove(i);
+            editor->resultsets().remove(i);
             e->reset_references();
             break;
           }
@@ -106,13 +140,12 @@ class MYSQLWBBACKEND_PUBLIC_FUNC db_query_EditorConcreteImplData : public db_que
     {
       int oindex = ref->active_sql_editor_index();
       
-      for (size_t c= _self->queryBuffers().count(), i= 0; i < c; i++)
+      for (size_t c= _self->queryEditors().count(), i= 0; i < c; i++)
       {
-        db_query_QueryBufferRef e(_self->queryBuffers()[i]);
+        db_query_QueryBufferRef e(_self->queryEditors()[i]);
         if (e == editor->grtobj())
         {
           ref->active_sql_editor_index(i);
-          ref->do_partial_ui_refresh(SqlEditorForm::RefreshEditor);      
           break;
         }
       }
@@ -123,13 +156,13 @@ class MYSQLWBBACKEND_PUBLIC_FUNC db_query_EditorConcreteImplData : public db_que
 public:
   db_query_EditorConcreteImplData(boost::shared_ptr<SqlEditorForm> editor,
                                   const db_query_EditorRef &self)
-  : _editor(editor), _self(dynamic_cast<db_query_Editor*>(self.valueptr()))
+  : _self(dynamic_cast<db_query_Editor*>(self.valueptr())), _editor(editor)
   {
     for (size_t i= 0; i < (size_t) editor->sql_editor_count(); i++)
     {
-      db_query_QueryBufferRef qb(editor->sql_editor(i)->grtobj());
+      db_query_QueryEditorRef qb(db_query_QueryEditorRef::cast_from(editor->sql_editor(i)->grtobj()));
       qb->owner(self);
-      _self->queryBuffers().insert(qb);
+      _self->queryEditors().insert(qb);
     }
     
     editor->recordset_list_changed.connect(boost::bind(&db_query_EditorConcreteImplData::recordset_list_changed, this, _1, _2, _3));
@@ -145,17 +178,25 @@ public:
       return _editor->connection_descriptor();
     return db_mgmt_ConnectionRef();
   }
-  
-  virtual db_query_QueryBufferRef addQueryBuffer()
+
+  virtual grt::IntegerRef isConnected() const
+  {
+    boost::shared_ptr<SqlEditorForm> ref(_editor);
+    if (ref)
+      return grt::IntegerRef(_editor->connected() ? 1 : 0);
+    return grt::IntegerRef(0);
+  }
+
+  virtual db_query_QueryEditorRef addQueryEditor()
   {
     boost::shared_ptr<SqlEditorForm> ref(_editor);
     if (ref)
     {
       _editor->new_sql_script_file();
       
-      return _editor->active_sql_editor()->grtobj();
+      return db_query_QueryEditorRef::cast_from(_editor->active_sql_editor()->grtobj());
     }
-    return db_query_QueryBufferRef();
+    return db_query_QueryEditorRef();
   }
   
   virtual grt::IntegerRef addToOutput(const std::string &text, long bringToFront)
@@ -193,7 +234,24 @@ public:
 
     return grt::IntegerRef(0);
   }
-  
+
+  virtual db_query_ResultsetRef executeManagementQuery(const std::string &sql, bool log)
+  {
+    boost::shared_ptr<SqlEditorForm> ref(_editor);
+    if (ref)
+    {
+      return ref->exec_management_query(sql, log);
+    }
+    return db_query_ResultsetRef();
+  }
+
+  virtual void executeManagementCommand(const std::string &sql, bool log)
+  {
+    boost::shared_ptr<SqlEditorForm> ref(_editor);
+    if (ref)
+      ref->exec_management_sql(sql, log);
+  }
+
   virtual db_query_EditableResultsetRef createTableEditResultset(const std::string &schema, const std::string &table, const std::string &where, bool showGrid)
   {
     boost::shared_ptr<SqlEditorForm> ref(_editor);
@@ -235,39 +293,57 @@ public:
     return "";
   }
   
-  virtual db_query_QueryBufferRef activeQueryBuffer()
+  virtual db_query_QueryEditorRef activeQueryEditor()
   {
     boost::shared_ptr<SqlEditorForm> ref(_editor);
     if (ref)
     {
       Sql_editor::Ref editor(ref->active_sql_editor());
       if (editor)
-        return editor->grtobj();
+        return db_query_QueryEditorRef::cast_from(editor->grtobj());
     }
-    return db_query_QueryBufferRef();
+    return db_query_QueryEditorRef();
   }
-  
-  virtual db_query_ResultsetRef activeResultset()
+
+  virtual void editLiveObject(const db_DatabaseObjectRef &object, const db_CatalogRef &catalog)
   {
     boost::shared_ptr<SqlEditorForm> ref(_editor);
     if (ref)
     {
-      Recordset::Ref rset(ref->active_recordset(ref->active_sql_editor_index()));
-      
-      if (rset)
-      {
-        for (size_t c= _self->resultsets().count(), i= 0; i < c; i++)
-        {
-          db_query_ResultsetRef e(_self->resultsets()[i]);
-          
-          if (e->get_data()->recordset == rset)
-            return e;
-        }
-      }
+      ref->get_live_tree()->open_alter_object_editor(object, catalog);
     }
-    return db_query_ResultsetRef();
   }
-  
+
+  virtual void alterLiveObject(const std::string &type, const std::string &schemaName, const std::string &objectName)
+  {
+    boost::shared_ptr<SqlEditorForm> ref(_editor);
+    if (ref)
+    {
+      wb::LiveSchemaTree::ObjectType otype;
+      if (type == "db.Schema")
+        otype = wb::LiveSchemaTree::Schema;
+      else if (type == "db.Table")
+        otype = wb::LiveSchemaTree::Table;
+      else if (type == "db.View")
+        otype = wb::LiveSchemaTree::View;
+      else if (type == "db.StoredProcedure")
+        otype = wb::LiveSchemaTree::Procedure;
+      else if (type == "db.Function")
+        otype = wb::LiveSchemaTree::Function;
+      else
+        return;
+      ref->get_live_tree()->do_alter_live_object(otype, schemaName, objectName);
+    }
+  }
+
+  virtual grt::ListRef<db_query_LiveDBObject> schemaTreeSelection() const
+  {
+    boost::shared_ptr<SqlEditorForm> ref(_editor);
+    if (ref)
+      return grt::ListRef<db_query_LiveDBObject>::cast_from(ref->get_live_tree()->get_schema_tree()->get_selected_objects());
+    return grt::ListRef<db_query_LiveDBObject>();
+  }
+
   void detach()
   {
     _editor.reset();
@@ -283,42 +359,42 @@ protected:
 
 void WBContextSQLIDE::call_in_editor(void (SqlEditorForm::*method)())
 {
-  SqlEditorForm *db_sql_editor= get_active_sql_editor();
-  if (db_sql_editor)
-    (db_sql_editor->*method)();
+  SqlEditorForm *form= get_active_sql_editor();
+  if (form)
+    (form->*method)();
 }
 
 
 void WBContextSQLIDE::call_in_editor_str(void (SqlEditorForm::*method)(const std::string &arg), const std::string &arg)
 {
-  SqlEditorForm *db_sql_editor= get_active_sql_editor();
-  if (db_sql_editor)
-    (db_sql_editor->*method)(arg);
+  SqlEditorForm *form= get_active_sql_editor();
+  if (form)
+    (form->*method)(arg);
 }
 
 
 void WBContextSQLIDE::call_in_editor_bool(void (SqlEditorForm::*method)(bool arg), bool arg)
 {
-  SqlEditorForm *db_sql_editor= get_active_sql_editor();
-  if (db_sql_editor)
-    (db_sql_editor->*method)(arg);
+  SqlEditorForm *form= get_active_sql_editor();
+  if (form)
+    (form->*method)(arg);
 }
 
 static void call_export(wb::WBContextSQLIDE *sqlide)
 {
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  if (db_sql_editor)
+  SqlEditorForm *form= sqlide->get_active_sql_editor();
+  if (form)
   {
-    int editor = db_sql_editor->active_sql_editor_index();
+    int editor = form->active_sql_editor_index();
     if (editor >= 0)
-      db_sql_editor->show_export_recordset(editor, db_sql_editor->active_recordset(editor));
+      form->show_export_recordset(editor, form->active_recordset(editor));
   }
 }
 
 static bool validate_export(wb::WBContextSQLIDE *sqlide)
 {
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  return db_sql_editor && db_sql_editor->active_sql_editor_index() >= 0 && db_sql_editor->active_recordset(db_sql_editor->active_sql_editor_index());
+  SqlEditorForm *form = sqlide->get_active_sql_editor();
+  return form && form->active_sql_editor_index() >= 0 && form->active_recordset(form->active_sql_editor_index());
 }
 
 static void call_run_file(wb::WBContextSQLIDE *sqlide)
@@ -350,7 +426,7 @@ static void call_revert(wb::WBContextSQLIDE *sqlide)
     int i = editor->active_sql_editor_index();
     if (i >= 0)
     {
-      if (editor->sql_editor_dirty(i))
+      if (editor->sql_editor(i)->get_editor_control()->is_dirty())
       {
         int rc = mforms::Utilities::show_message(_("Revert to Saved"),
                             base::strfmt(_("Do you want to revert to the most recently saved version of '%s'?\nAny changes since them will be lost."),
@@ -380,14 +456,11 @@ static bool validate_revert(wb::WBContextSQLIDE *sqlide)
 
 static void call_save_snippet(wb::WBContextSQLIDE *sqlide)
 {
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  if (db_sql_editor)
+  SqlEditorForm *form = sqlide->get_active_sql_editor();
+  if (form)
   {
-    int start, end;
-    std::string text = db_sql_editor->active_sql_editor()->sql();
-    if (db_sql_editor->active_sql_editor()->selected_range(start, end))
-      text = text.substr(start, end - start);
-    if (db_sql_editor->save_snippet(text))
+    std::string text = form->active_sql_editor()->selected_text();
+    if (form->save_snippet(text))
       sqlide->get_grt_manager()->replace_status_text("SQL saved to snippets list.");
   }
 }
@@ -395,30 +468,26 @@ static void call_save_snippet(wb::WBContextSQLIDE *sqlide)
 
 static void call_continue_on_error(wb::WBContextSQLIDE *sqlide)
 {
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  if (db_sql_editor)
-    db_sql_editor->continue_on_error(!db_sql_editor->continue_on_error());
+  SqlEditorForm *form = sqlide->get_active_sql_editor();
+  if (form)
+    form->continue_on_error(!form->continue_on_error());
 }
 
 
 static void call_reconnect(wb::WBContextSQLIDE *sqlide)
 {
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  if (db_sql_editor && !db_sql_editor->is_running_query())
-  {
-    db_sql_editor->grt_manager()->replace_status_text("Reconnecting...");
-    db_sql_editor->connect();
-    db_sql_editor->grt_manager()->replace_status_text("Connection reopened");
-  }
+  SqlEditorForm *form= sqlide->get_active_sql_editor();
+
+  sqlide->reconnect_editor(form);
 }
 
 
 static void call_new_connection(wb::WBContextSQLIDE *sqlide)
 {
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  if (db_sql_editor)
+  SqlEditorForm *form = sqlide->get_active_sql_editor();
+  if (form)
   {
-    db_mgmt_ConnectionRef conn(db_sql_editor->connection_descriptor());
+    db_mgmt_ConnectionRef conn(form->connection_descriptor());
     sqlide->get_wbui()->get_wb()->add_new_query_window(conn);
   }
 }
@@ -426,37 +495,30 @@ static void call_new_connection(wb::WBContextSQLIDE *sqlide)
 
 static void call_exec_sql(wb::WBContextSQLIDE *sqlide, bool current_statement_only)
 {
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  if (db_sql_editor)
-    db_sql_editor->do_partial_ui_refresh(current_statement_only ? SqlEditorForm::RunCurrentStatement : SqlEditorForm::RunCurrentScript);
+  SqlEditorForm *form= sqlide->get_active_sql_editor();
+  if (form)
+    form->run_editor_contents(current_statement_only);
 }  
 
 static bool validate_exec_sql(wb::WBContextSQLIDE *sqlide)
 {
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  return (db_sql_editor && !db_sql_editor->is_running_query());
-}
-
-
-static bool validate_exec_current_sql_statement(wb::WBContextSQLIDE *sqlide)
-{
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  return (db_sql_editor && !db_sql_editor->is_running_query());
+  SqlEditorForm *form = sqlide->get_active_sql_editor();
+  return (form && !form->is_running_query() && form->connected());
 }
 
 
 static void call_save_edits(wb::WBContextSQLIDE *sqlide)
 {
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  if (db_sql_editor)
+  SqlEditorForm *form = sqlide->get_active_sql_editor();
+  if (form)
   {
-    int i = db_sql_editor->active_sql_editor_index();
+    int i = form->active_sql_editor_index();
     if (i >= 0)
     {
-      Recordset::Ref rs= db_sql_editor->active_recordset(i);
+      Recordset::Ref rs= form->active_recordset(i);
       if (rs)
       {
-        db_sql_editor->do_partial_ui_refresh(SqlEditorForm::SaveRecordsetChanges);
+        form->do_partial_ui_refresh(SqlEditorForm::SaveRecordsetChanges);
         rs->apply_changes();
       }
     }
@@ -465,36 +527,104 @@ static void call_save_edits(wb::WBContextSQLIDE *sqlide)
 
 static void call_discard_edits(wb::WBContextSQLIDE *sqlide)
 {
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  if (db_sql_editor)
+  SqlEditorForm *form = sqlide->get_active_sql_editor();
+  if (form)
   {
-    int i = db_sql_editor->active_sql_editor_index();
+    int i = form->active_sql_editor_index();
     if (i >= 0)
     {
-      Recordset::Ref rs= db_sql_editor->active_recordset(i);
+      Recordset::Ref rs= form->active_recordset(i);
       if (rs)
       {
-        db_sql_editor->do_partial_ui_refresh(SqlEditorForm::DiscardRecordsetChanges);
+        form->do_partial_ui_refresh(SqlEditorForm::DiscardRecordsetChanges);
         rs->rollback();
       }
     }
   }
 }
 
+//--------------------------------------------------------------------------------------------------
 
 static bool validate_save_edits(wb::WBContextSQLIDE *sqlide)
 {
-  SqlEditorForm *db_sql_editor= sqlide->get_active_sql_editor();
-  if (!db_sql_editor)
+  SqlEditorForm *form = sqlide->get_active_sql_editor();
+  if (!form)
     return false;
-  int i = db_sql_editor->active_sql_editor_index();
+  int i = form->active_sql_editor_index();
   if (i < 0)
     return false;
-  Recordset::Ref rs= db_sql_editor->active_recordset(i);
+  Recordset::Ref rs= form->active_recordset(i);
   return (rs && rs->has_pending_changes());
 }
 
 
+//--------------------------------------------------------------------------------------------------
+
+static bool validate_list_members(wb::WBContextSQLIDE *sqlide)
+{
+  return sqlide->get_grt_manager()->get_app_option_int("DbSqlEditor:CodeCompletionEnabled") != 0;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+static void new_script_tab(wb::WBContextSQLIDE *sqlide)
+{
+  SqlEditorForm *form= sqlide->get_active_sql_editor();
+  if (form)
+  {
+    if (sqlide->get_grt_manager()->get_app_option_int("DbSqlEditor:DiscardUnsavedQueryTabs", 0))
+      form->new_sql_scratch_area();
+    else
+      form->new_sql_script_file();
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
+static bool validate_toolbar_alias_toggle(wb::WBContextSQLIDE *sqlide, const std::string &item_name);
+
+static void call_toolbar_alias_toggle(wb::WBContextSQLIDE *sqlide, const std::string &item_name)
+{
+  SqlEditorForm *form= sqlide->get_active_sql_editor();
+  if (form)
+  {
+    mforms::ToolBarItem *item = form->get_toolbar()->find_item(item_name);
+    if (item)
+    {
+      item->set_checked(!item->get_checked());
+      item->callback();
+      validate_toolbar_alias_toggle(sqlide, item_name); // update menu item title
+    }
+  }
+}
+
+static bool validate_toolbar_alias_toggle(wb::WBContextSQLIDE *sqlide, const std::string &item_name)
+{
+  SqlEditorForm *form= sqlide->get_active_sql_editor();
+  if (form)
+  {
+    mforms::ToolBarItem *item = form->get_toolbar()->find_item(item_name);
+    mforms::MenuItem *mitem = NULL;
+
+    mitem = form->get_menubar()->find_item("view");
+    if (mitem)
+      mitem = mitem->find_item("sidebars");
+    if (mitem)
+      mitem = mitem->find_item("alias."+item_name);
+    if (item && mitem)
+    {
+      std::string title = mitem->get_title();
+      if (item->get_checked())
+        base::replace(title, "Show", "Hide");
+      else
+        base::replace(title, "Hide", "Show");
+      mitem->set_title(title);
+    }
+  }
+  return true;
+}
+
+
 //--------------------------------------------------------------------------------------------------
 
 WBContextSQLIDE::WBContextSQLIDE(WBContextUI *wbui)
@@ -539,7 +669,7 @@ bool WBContextSQLIDE::auto_save_workspaces()
     }
     catch (const std::exception &exception)
     {
-      g_warning("Exception during auto-save of SQL Editors: %s", exception.what());
+      log_warning("Exception during auto-save of SQL Editors: %s\n", exception.what());
       wb->get_grt_manager()->replace_status_text(base::strfmt("Error during auto-save of SQL Editors: %s", exception.what()));
     }
   }
@@ -599,12 +729,20 @@ bec::GRTManager *WBContextSQLIDE::get_grt_manager()
   return _wbui->get_wb()->get_grt_manager();
 }
 
+
+CommandUI *WBContextSQLIDE::get_cmdui()
+{
+  return _wbui->get_command_ui();
+}
+
+
 void WBContextSQLIDE::handle_notification(const std::string &name, void *sender, base::NotificationInfo &info)
 {
   if (name == "GNAppClosing")
     finalize();
 }
 
+
 void WBContextSQLIDE::init()
 {
   DbSqlEditorSnippets::setup(this, bec::make_path(get_grt_manager()->get_user_datadir(), "snippets"));
@@ -614,7 +752,11 @@ void WBContextSQLIDE::init()
   
   // setup some builtin commands handled by ourselves for the SQL IDE
   wb::CommandUI *cmdui = _wbui->get_command_ui();
-  
+
+  cmdui->add_builtin_command("alias.wb.toggleSidebar", boost::bind(call_toolbar_alias_toggle, this, "wb.toggleSidebar"), boost::bind(validate_toolbar_alias_toggle, this, "wb.toggleSidebar"));
+  cmdui->add_builtin_command("alias.wb.toggleSecondarySidebar", boost::bind(call_toolbar_alias_toggle, this, "wb.toggleSecondarySidebar"), boost::bind(validate_toolbar_alias_toggle, this, "wb.toggleSecondarySidebar"));
+  cmdui->add_builtin_command("alias.wb.toggleOutputArea", boost::bind(call_toolbar_alias_toggle, this, "wb.toggleOutputArea"), boost::bind(validate_toolbar_alias_toggle, this, "wb.toggleOutputArea"));
+
   cmdui->add_builtin_command("query.execute", boost::bind(call_exec_sql, this, false), boost::bind(validate_exec_sql, this));
   cmdui->add_builtin_command("query.execute_current_statement", boost::bind(call_exec_sql, this, true), boost::bind(validate_exec_sql, this));
   
@@ -624,17 +766,21 @@ void WBContextSQLIDE::init()
   cmdui->add_builtin_command("query.commit", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::commit));
   cmdui->add_builtin_command("query.rollback", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::rollback));
   cmdui->add_builtin_command("query.autocommit", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::toggle_autocommit));
+  cmdui->add_builtin_command("query.showFieldInfo", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::toggle_show_field_info));
 
   cmdui->add_builtin_command("query.new_schema", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, &SqlEditorForm::toolbar_command, "query.new_schema"));
+  cmdui->add_builtin_command("query.show_schema_inspector", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, &SqlEditorForm::toolbar_command, "query.show_schema_inspector"));
   cmdui->add_builtin_command("query.new_table", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, &SqlEditorForm::toolbar_command, "query.new_table")); 
   cmdui->add_builtin_command("query.new_view", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, &SqlEditorForm::toolbar_command, "query.new_view"));
   cmdui->add_builtin_command("query.new_routine", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, &SqlEditorForm::toolbar_command, "query.new_routine"));
+  cmdui->add_builtin_command("query.new_function", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, &SqlEditorForm::toolbar_command, "query.new_function"));
   
   cmdui->add_builtin_command("query.savesnippet", boost::bind(call_save_snippet, this));
   cmdui->add_builtin_command("query.new_connection", boost::bind(call_new_connection, this));
   
   cmdui->add_builtin_command("query.newQuery", boost::bind(&WBContextSQLIDE::call_in_editor_bool, this, &SqlEditorForm::new_sql_scratch_area, false));
-  cmdui->add_builtin_command("query.newFile", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::new_sql_script_file));
+  //cmdui->add_builtin_command("query.newFile", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::new_sql_script_file));
+  cmdui->add_builtin_command("query.newFile", boost::bind(new_script_tab, this));
   cmdui->add_builtin_command("query.openFile", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, (void(SqlEditorForm::*)(const std::string&))&SqlEditorForm::open_file, ""));
   cmdui->add_builtin_command("query.saveFile", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::save_file));
   cmdui->add_builtin_command("query.saveFileAs", boost::bind(call_save_file_as, this));
@@ -653,6 +799,9 @@ void WBContextSQLIDE::init()
   cmdui->add_builtin_command("query.explain", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::explain_sql));
   cmdui->add_builtin_command("query.explain_current_statement",
                                                boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::explain_current_statement));
+
+  cmdui->add_builtin_command("list-members", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::list_members),
+                             boost::bind(validate_list_members, this));
 }
 
 
@@ -672,41 +821,106 @@ void WBContextSQLIDE::finalize()
 }
 
 
-static void *connect_editor(SqlEditorForm::Ref editor)
+void WBContextSQLIDE::reconnect_editor(SqlEditorForm *editor)
 {
+  boost::shared_ptr<sql::TunnelConnection> tunnel;
+
+  // open tunnel, if needed
   try
   {
-    editor->connect();
+    tunnel = sql::DriverManager::getDriverManager()->getTunnel(editor->connection_descriptor());
   }
-  catch (sql::AuthenticationError &exc)
+  catch (grt::user_cancelled)
   {
-    return new sql::AuthenticationError(exc);
+    editor->grt_manager()->replace_status_text("Tunnel connection cancelled.");
+    return;
+  }
+  try
+  {
+    if (editor && !editor->is_running_query())
+    {
+      editor->grt_manager()->replace_status_text("Reconnecting...");
+      if (editor->connect(tunnel))
+        editor->grt_manager()->replace_status_text("Connection reopened.");
+      else
+      {
+        editor->grt_manager()->replace_status_text("Could not reconnect.");
+        if (tunnel.get())
+        {
+          // check whether this was a tunnel related error
+          std::string type, message;
+          while (tunnel->get_message(type, message))
+          {
+            log_debug("From tunnel %s: %s\n", type.c_str(), message.c_str());
+            if (type == "ERROR")
+              mforms::Utilities::show_error("Reconnect", "Tunnel error: "+message, "OK");
+          }
+        }
+      }
+    }
   }
   catch (std::exception &exc)
   {
+    SqlEditorForm::report_connection_failure(exc.what(), editor->connection_descriptor());
+    return;
+  }
+}
+
+static void *connect_editor(SqlEditorForm::Ref editor, boost::shared_ptr<sql::TunnelConnection> tunnel)
+{
+  try
+  {
+    log_debug3("Connecting SQL editor...\n");
+    editor->connect(tunnel);
+  }
+  catch (sql::AuthenticationError &exc)
+  {
+    log_error("Got an authentication error during connection: %s\n", exc.what());
     return new std::string(exc.what());
   }
+  catch (std::exception &exc)
+  {
+    if (tunnel.get())
+    {
+      // check whether this was a tunnel related error
+      std::string type, message;
+      while (tunnel->get_message(type, message))
+      {
+        log_debug("From tunnel %s: %s\n", type.c_str(), message.c_str());
+        if (type == "ERROR")
+          return new std::string("Tunnel error: "+message);
+      }
+    }
+    log_error("Got an exception during connection: %s\n", exc.what());
+    return new std::string(exc.what());
+  }
+  log_debug3("Connection to SQL editor succeeded\n");
   return new std::string();
 }
 
 static bool cancel_connect_editor()
 {
+  log_debug3("Cancelling connection...\n");
   return true;
 }
 
 SqlEditorForm::Ref WBContextSQLIDE::create_connected_editor(const db_mgmt_ConnectionRef &conn)
 {
+  // start by opening the tunnel, if needed
+  boost::shared_ptr<sql::TunnelConnection> tunnel = sql::DriverManager::getDriverManager()->getTunnel(conn);
+
   SqlEditorForm::Ref editor(SqlEditorForm::create(this, conn));
 
   void *result_ptr = 0;
   if (!mforms::Utilities::run_cancelable_task(_("Opening SQL Editor"), 
                                               strfmt(_("An SQL editor instance for '%s' is opening and should be available in a "
                                                        "moment.\n\nPlease stand by..."), conn->name().c_str()),
-                                              boost::bind(connect_editor, editor),
+                                              boost::bind(connect_editor, editor, tunnel),
                                               boost::bind(cancel_connect_editor),
-                                              result_ptr)
-      || !result_ptr)
+                                              result_ptr))
     throw grt::user_cancelled("canceled");
+  if (!result_ptr)
+    throw grt::user_cancelled("connection error");
   
   std::string *result = (std::string*)result_ptr;
   if (result->empty())
@@ -715,24 +929,48 @@ SqlEditorForm::Ref WBContextSQLIDE::create_connected_editor(const db_mgmt_Connec
   {
     std::string tmp(*result);
     delete result;
+    
+    if (tmp == ":PASSWORD_EXPIRED")
+    {
+      grt::BaseListRef args(conn->get_grt(), grt::AnyType);
+      args.ginsert(conn);
+      int result = *grt::IntegerRef::cast_from(conn->get_grt()->call_module_function("WbAdmin", "handleExpiredPassword", args));
+      if (result)
+        return create_connected_editor(conn);
+      throw grt::user_cancelled("password reset cancelled by user");
+    }
+    
     throw std::runtime_error(tmp);
   }
-  
-  editor->finish_startup();
 
   {
-    // create entry for grt tree
+    // Create entry for grt tree and update volatile data in the connection.
     db_query_EditorRef object(_wbui->get_wb()->get_grt());
     object->owner(_wbui->get_wb()->get_root());
     object->name(conn->name());
     
     object->set_data(new db_query_EditorConcreteImplData(editor, object));
 
+    {
+      std::map<std::string, std::string> details(editor->connection_details());
+      grt::DictRef parameter_values = conn->parameterValues();
+      parameter_values.gset("serverVersion", details["dbmsProductVersion"]);
+      std::time_t time = std::time(0);
+      parameter_values.gset("lastConnected", (long)time);
+
+      GrtVersionRef version = CatalogHelper::parse_version(object.get_grt(), details["dbmsProductVersion"]);
+      version->owner(object);
+      version->name(grt::StringRef(details["dbmsProductName"]));
+      object->serverVersion(version);
+    }
+
     _wbui->get_wb()->get_root()->sqlEditors().insert(object);
   }
-    
+
   _open_editors.push_back(editor);
 
+  editor->finish_startup();
+
   // setup auto-save for model
   if (!_auto_save_active)
   {
@@ -772,25 +1010,7 @@ bool WBContextSQLIDE::activate_live_object(GrtObjectRef object)
   SqlEditorForm *editor= get_active_sql_editor();
   if (!editor)
     return false;
-  return editor->activate_live_object(object);
-}
-
-
-bool WBContextSQLIDE::create_live_object(GrtObjectRef object_type, std::string owner_name, std::string obj_name)
-{
-  SqlEditorForm *editor= get_active_sql_editor();
-  if (!editor)
-    return false;
-  return editor->create_live_object(object_type, owner_name, obj_name);
-}
-
-
-bool WBContextSQLIDE::drop_live_object(GrtObjectRef object_type, std::string owner_name, std::string obj_name)
-{
-  SqlEditorForm *editor= get_active_sql_editor();
-  if (!editor)
-    return false;
-  return editor->drop_live_object(object_type, owner_name, obj_name);
+  return editor->get_live_tree()->activate_live_object(object);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -838,19 +1058,11 @@ void WBContextSQLIDE::editor_will_close(SqlEditorForm* editor)
     
     for (size_t c= editors.count(), i= 0; i < c; i++)
     {
-      if (dynamic_cast<db_query_EditorConcreteImplData*>(editors[i]->get_data())->editor_object().get() == editor)
+      db_query_EditorRef current_editor(editors[i]);
+      if (dynamic_cast<db_query_EditorConcreteImplData*>(current_editor->get_data())->editor_object().get() == editor)
       {
-        db_query_EditorRef editor(editors[i]);
-        
-        // remove all recordsets
-        while (editor->resultsets().count() > 0)
-        {
-          editor->resultsets()[0]->owner(GrtObjectRef());
-          editor->resultsets().remove(0);
-        }
-        
-        editor->reset_references();
-        dynamic_cast<db_query_EditorConcreteImplData*>(editors[i]->get_data())->detach();
+        current_editor->reset_references();
+        dynamic_cast<db_query_EditorConcreteImplData*>(current_editor->get_data())->detach();
         
         editors.remove(i);
         break;
@@ -884,14 +1096,20 @@ void WBContextSQLIDE::update_plugin_arguments_pool(bec::ArgumentPool &args)
     db_query_EditorRef editor(get_grt_editor_object(editor_ptr));
     if (editor.is_valid())
     {
-      db_query_ResultsetRef rset(editor->activeResultset());
+      db_query_QueryEditorRef qeditor(editor->activeQueryEditor());
+      if (qeditor.is_valid())
+      {
+        db_query_ResultsetRef rset(qeditor->activeResultset());
       
-      args.add_entries_for_object("activeSQLEditor", editor);
-      args.add_entries_for_object("activeQueryBuffer", editor->activeQueryBuffer());
-      args.add_entries_for_object("", editor->activeQueryBuffer());
-      
-      if (rset.is_valid())
-        args.add_entries_for_object("activeResultset", rset, "db.query.Resultset");
+        args.add_entries_for_object("activeSQLEditor", editor);
+        args.add_entries_for_object("activeQueryBuffer", qeditor);
+        args.add_entries_for_object("activeQueryEditor", qeditor);
+        args.add_entries_for_object("", qeditor);
+        if (rset.is_valid())
+          args.add_entries_for_object("activeResultset", rset, "db.query.Resultset");
+      }
+      else
+        args.add_entries_for_object("activeSQLEditor", editor);
     }
   }
 }
@@ -912,3 +1130,56 @@ db_query_EditorRef WBContextSQLIDE::get_grt_editor_object(SqlEditorForm *editor)
 }
 
 
+
+
+
+
+//--------------------------------------------------------------------------------------------------
+
+static struct RegisterNotifDocs
+{
+  RegisterNotifDocs()
+  {
+    base::NotificationCenter::get()->register_notification("GRNSQLEditorOpened",
+                                                           "sqlide",
+                                                           "Sent when a connection tab finishes initializing and is about to be shown on screen.",
+                                                           "db.query.Editor instance",
+                                                           "");
+
+    base::NotificationCenter::get()->register_notification("GRNSQLEditorReconnected",
+                                                           "sqlide",
+                                                           "Sent when the connection state of a SQL editor changes (reconnect, disconnect) it's DB connection. Resent whenever the user clicks Reconnect.",
+                                                           "db.query.Editor instance",
+                                                           "connected - whether the connection is open");
+
+    base::NotificationCenter::get()->register_notification("GRNServerStateChanged",
+                                                           "sqlide",
+                                                           "Sent by the Admin module when it is detected that the server state (running or stopped) changed.",
+                                                           "db.query.Editor instance",
+                                                           "state - running or stopped\n"
+                                                           "connection - the connection object for the server");
+
+    base::NotificationCenter::get()->register_notification("GRNLiveDBObjectMenuWillShow",
+                                                           "sqlide",
+                                                           "Sent when the context menu is about to be shown for a live DB object (eg. from the live schema tree in the SQL IDE sidebar).",
+                                                           "db.query.Editor instance",
+                                                           "menu - mforms.ObjectReference of a mforms.Menu object which is being shown\n"
+                                                           "selection - a list of db.query.LiveDBObject for the selected objects");
+
+    base::NotificationCenter::get()->register_notification("GRNLiveDBObjectSelectionDidChange",
+                                                           "sqlide",
+                                                           "Sent when the selection in the schema tree changes. Avoid hooking slow actions to this as it will make object selection less smooth.",
+                                                           "db.query.Editor instance",
+                                                           "selection-size - number of selected items in the tree");
+
+    base::NotificationCenter::get()->register_notification("GRNSQLResultSetMenuWillShow",
+                                                         "sqlide",
+                                                         "Sent when the context menu is about to be shown for a resultset in the SQL IDE.",
+                                                         "db.query.Resultset instance",
+                                                         "menu - mforms.ObjectReference of a mforms.Menu object which is being shown\n"
+                                                         "selected_rows - a list of int values of the selected rows\n"
+                                                         "selected_column - if this value is set, the selection is a single cell at selected_rows[0], selected_column");
+  }
+} initdocs;
+
+
