@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,7 +21,6 @@
 #include "wb_sql_editor_form.h"
 #include "wb_sql_editor_buffer.h"
 
-#include "grtui/eol_format_dialog.h"
 #include "grtui/file_charset_dialog.h"
 
 #include <boost/foreach.hpp>
@@ -32,11 +31,16 @@
 #include "base/util_functions.h"
 #include "base/file_functions.h"
 #include "base/string_utilities.h"
+#include "base/log.h"
+
 #include "mforms/utilities.h"
 #include "mforms/filechooser.h"
+#include "mforms/code_editor.h"
 
-// 5M max file size for auto-restoring
-#define MAX_FILE_SIZE_FOR_AUTO_RESTORE 5000000
+#include "query_side_palette.h"
+
+// 20 MB max file size for auto-restoring
+#define MAX_FILE_SIZE_FOR_AUTO_RESTORE 20000000
 
 DEFAULT_LOG_DOMAIN("SqlEditor")
 
@@ -71,7 +75,7 @@ void SqlEditorForm::save_workspace(const std::string &workspace_name, bool is_au
 {
   GMutexLock sql_editors_mutex(_sql_editors_mutex);
   
-  if (_sql_editors.size() < 1 || (_sql_editors.size() == 1 && _sql_editors[0]->is_scratch && _sql_editors[0]->editor->sql().empty()))
+  if (_sql_editors.size() < 1 || (_sql_editors.size() == 1 && _sql_editors[0]->is_scratch && _sql_editors[0]->editor->empty()))
     return;
   
   std::string path;
@@ -144,7 +148,7 @@ void SqlEditorForm::save_workspace(const std::string &workspace_name, bool is_au
     }
     else
     {
-      if (sql_editor_info->dirty)
+      if (sql_editor_info->editor->get_editor_control()->is_dirty())
       {
         if (extension(sql_editor_info->autosave_filename) == ".scratch")
         {
@@ -165,16 +169,18 @@ void SqlEditorForm::save_workspace(const std::string &workspace_name, bool is_au
     // only save editor contents for scratch areas and unsaved editors
     if (!filename.empty())
     {
-      const std::string &sql(sql_editor_info->editor->sql());
+      // We don't need to lock the editor as we are in the main thread here
+      // and directly set the file content without detouring to anything that could change the text.
       GError *error = 0;
       std::string fn = make_path(path, filename);
       
-      if (!g_file_set_contents(fn.c_str(), sql.data(), sql.size(), &error))
+      std::pair<const char*, size_t> text = sql_editor_info->editor->text_ptr();
+      if (!g_file_set_contents(fn.c_str(), text.first, text.second, &error))
       {
         std::string msg(strfmt("Could not save snapshot of editor contents to %s: %s", fn.c_str(), error->message));
         g_error_free(error);
         base_rmdir_recursively(path.c_str());
-        throw msg;
+        throw std::runtime_error(msg);
       }
     }
     else
@@ -202,13 +208,11 @@ static bool check_if_file_too_big_to_restore(const std::string &path, const std:
   if ((length = get_file_size(path.c_str())) > MAX_FILE_SIZE_FOR_AUTO_RESTORE)
   {
   again:
-    std::string max_size = sizefmt(MAX_FILE_SIZE_FOR_AUTO_RESTORE, false);
     std::string size = sizefmt(length, false);
     int rc = mforms::Utilities::show_warning("Restore Workspace",
-                                             strfmt("%s is bigger than %s (%s), would you like to restore it anyway?",
-                                                    file_caption.c_str(), max_size.c_str(), size.c_str()
-                                                    ),
-                                             "Restore", "Skip", allow_save_as ? "Save As..." : ""
+      strfmt("The file %s has a size of %s. Are you sure you want to restore this file?",
+      file_caption.c_str(), size.c_str()),
+      "Restore", "Skip", allow_save_as ? "Save As..." : ""
                                              );
     if (rc == mforms::ResultCancel)
       return false;
@@ -374,10 +378,10 @@ bool SqlEditorForm::load_workspace(const std::string &workspace_name)
       }
       
       int i = add_sql_editor(true);
-      Sql_editor_info::Ref editor(_sql_editors[i]);
-      sql_editor_new_ui(_active_sql_editor_index);
-      editor->editor->sql(std::string(data, length));
-      editor->dirty = true;
+      Sql_editor_info::Ref info(_sql_editors[i]);
+      sql_editor_new_ui(i);
+      info->editor->sql(data ? data : "");
+      info->editor->get_editor_control()->reset_dirty();
       
       _active_sql_editor_index = i;
       g_free(data);
@@ -461,24 +465,25 @@ bool SqlEditorForm::load_workspace(const std::string &workspace_name)
           else
           {
             int i = add_sql_editor(false);
-            Sql_editor_info::Ref editor(_sql_editors[i]);
+            Sql_editor_info::Ref info(_sql_editors[i]);
             
             // set active editor now so that any callbacks triggered now go to the right editor
             _active_sql_editor_index = i;
             
             sql_editor_new_ui(_active_sql_editor_index);
-            editor->editor->sql(std::string(data, length));
-            editor->filename = filename;
-            editor->dirty = true;
+            info->editor->sql(data ? data : "");
+            info->editor->get_editor_control()->reset_dirty();
+            info->filename = filename;
             if (!filename.empty())
-              editor->caption = base::strip_extension(base::basename(filename));
+              info->caption = base::strip_extension(base::basename(filename));
             
             g_free(data);
           }
         }
         else if (filename.empty()) // An empty sql file.
         {
-          add_sql_editor(false, false);
+          int index = add_sql_editor(false, false);
+          sql_editor_new_ui(index);
         }
       }
     }
@@ -500,22 +505,21 @@ bool SqlEditorForm::load_workspace(const std::string &workspace_name)
   return true;
 }
 
-
-
 int SqlEditorForm::add_sql_editor(bool scratch, bool start_collapsed)
 {
-  Sql_editor::Ref sql_editor= Sql_editor::create(rdbms());
-  sql_editor->eol(EolHelpers::eol());
+  db_query_QueryEditorRef grtobj(grt_manager()->get_grt());
+  Sql_editor::Ref sql_editor= Sql_editor::create(rdbms(), wbsql()->get_grt_editor_object(this)->serverVersion(), grtobj);
   sql_editor->sql_check_progress_msg_throttle(_progress_status_update_interval);
-  sql_editor->text_change_signal.connect(boost::bind(&SqlEditorForm::on_sql_editor_text_change, this));
-  //sql_editor->text_selection_change_signal.connect(boost::bind(&SqlEditorForm::on_sql_editor_text_selection_change, this));
+  sql_editor->set_auto_completion_cache(_auto_completion_cache);
   sql_editor->sql_mode(_sql_mode);
-  
+  sql_editor->set_current_schema(active_schema());
+  scoped_connect(sql_editor->text_change_signal(),
+    boost::bind(&SqlEditorForm::do_partial_ui_refresh, this, (int)RefreshEditorTitle));
   int sql_editor_index;
   {
     GMutexLock sql_editors_mutex(_sql_editors_mutex);
     Sql_editor_info::Ref info(new Sql_editor_info());
-    info->toolbar = setup_editor_toolbar();
+    info->toolbar = setup_editor_toolbar(sql_editor);
     info->editor = sql_editor;
     info->is_scratch = scratch;
     info->recordset_mutex = g_mutex_new();
@@ -584,7 +588,12 @@ void SqlEditorForm::rename_autosave_files(int from, int to)
 
 void SqlEditorForm::remove_sql_editor(int editor_index)
 {
+  //if we're removing editor, just cancel query side timer cause there is only one timer
+  if (_side_palette)
+    _side_palette->cancel_timer();
+
   Sql_editor_info::Ref info(_sql_editors.at(editor_index));
+  info->editor->stop_processing();
   {
     // Notify the UI that recordsets must be closed.
     GMutexLock  lock(info->recordset_mutex);
@@ -595,9 +604,7 @@ void SqlEditorForm::remove_sql_editor(int editor_index)
       while (rsets->size() > 0)
       {
         recordset_list_changed(editor_index, *rsets->begin(), false);
-#ifndef _WIN32
         rsets->erase(rsets->begin());
-#endif
       }
     }
   }
@@ -648,6 +655,7 @@ int SqlEditorForm::sql_editor_count()
 void SqlEditorForm::active_sql_editor_index(int val) 
 { 
   _active_sql_editor_index= val; 
+
   validate_menubar();
 }
 
@@ -670,20 +678,12 @@ Sql_editor::Ref SqlEditorForm::active_sql_editor_or_new_scratch()
 Sql_editor::Ref SqlEditorForm::sql_editor(int new_index)
 {
   GMutexLock sql_editors_mutex(_sql_editors_mutex);
-  return _sql_editors.at(new_index)->editor;
+  if (new_index >= 0 && new_index < (int)_sql_editors.size())
+    return _sql_editors.at(new_index)->editor;
+  return Sql_editor::Ref();
 }
 
 
-void SqlEditorForm::sql_editor_dirty(int new_index, bool flag)
-{
-  if (flag != _sql_editors.at(new_index)->dirty)
-  {
-    _sql_editors.at(new_index)->dirty = flag;
-    
-    do_partial_ui_refresh(RefreshEditorTitle);
-  }
-}
-
 bool SqlEditorForm::sql_editor_reorder(Sql_editor::Ref editor, int new_index)
 {
   int old_index = sql_editor_index(editor);
@@ -719,6 +719,24 @@ bool SqlEditorForm::sql_editor_reorder(Sql_editor::Ref editor, int new_index)
   return false;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Returns true if this is an editor created from context menu (so it is supposed to maximize the resultset).
+ */
+bool SqlEditorForm::sql_editor_start_collapsed(int index)
+{
+  bool result = _sql_editors[index]->start_collapsed;
+
+  // Reset the collapsed flag in case it is queried multiple times even though the editor has already
+  // been set up. This is to avoid frequently resetting the collapsed state when rerunning the query
+  // in that particular sql editor.
+  _sql_editors[index]->start_collapsed = false;
+  return result;
+}
+
+//--------------------------------------------------------------------------------------------------
+
 bool SqlEditorForm::sql_editor_will_close(int new_index)
 {
   int edited_recordsets = 0;
@@ -742,23 +760,33 @@ bool SqlEditorForm::sql_editor_will_close(int new_index)
     }
   }
   
-  if (sql_editor_dirty(new_index))
+  bool check_scratch_editors = true;
+
+  // if Save of workspace on close is enabled, we don't need to check whether there are unsaved scratch 
+  // SQL editors but other stuff should be checked.
+  grt::ValueRef option(_grtm->get_app_option("workbench:SaveSQLWorkspaceOnClose"));
+  if (option.is_valid() && *grt::IntegerRef::cast_from(option))
+    check_scratch_editors = false;
+
+  if (!sql_editor_is_scratch(new_index) || check_scratch_editors)
   {
-    int r;
-    r = mforms::Utilities::show_warning(_("Close SQL Tab"), 
-                                        strfmt(_("SQL script %s has unsaved changes.\n"
-                                                 "Would you like to Save these changes, discard them or cancel closing the page?"),
-                                               sql_editor_path(new_index).c_str()), _("Save"), _("Cancel"), _("Discard Changes"));
-    
-    if (r == mforms::ResultCancel)
-      return false;
-    else if (r == mforms::ResultOk)
+    if (sql_editor(new_index)->get_editor_control()->is_dirty())
     {
-      if (!save_sql_script_file(sql_editor_path(new_index), new_index))
+      int result = mforms::Utilities::show_warning(_("Close SQL Tab"), 
+        strfmt(_("SQL script %s has unsaved changes.\n"
+        "Would you like to Save these changes, discard them or cancel closing the page?"),
+        sql_editor_path(new_index).c_str()), _("Save"), _("Cancel"), _("Don't Save"));
+
+      if (result == mforms::ResultCancel)
         return false;
+      else if (result == mforms::ResultOk)
+      {
+        if (!save_sql_script_file(sql_editor_path(new_index), new_index))
+          return false;
+      }
+      else
+        sql_editor(new_index)->get_editor_control()->reset_dirty();
     }
-    else
-      _sql_editors[new_index]->dirty = false;
   }
   
   int r = -1;
@@ -767,14 +795,14 @@ bool SqlEditorForm::sql_editor_will_close(int new_index)
                                         strfmt(_("An edited recordset has unsaved changes in SQL tab %s.\n"
                                                  "Would you like to save these changes, discard them or cancel closing the page?"),
                                                sql_editor_caption(new_index).c_str()),
-                                        _("Save Changes"), _("Cancel"), _("Discard Changes"));
+                                        _("Save Changes"), _("Cancel"), _("Don't Save"));
   else
     if (edited_recordsets > 0)
       r = mforms::Utilities::show_warning(_("Close SQL Tab"),
                                           strfmt(_("There are %i recordsets with unsaved changes in SQL tab %s.\n"
                                                    "Would you like to save these changes, discard them or cancel closing to review them manually?"),
                                                  edited_recordsets, sql_editor_caption(new_index).c_str()),
-                                          _("Save All"), _("Cancel"), _("Discard Changes"));
+                                          _("Save All"), _("Cancel"), _("Don't Save"));
   
   bool success = true;
   if (r == mforms::ResultCancel)
@@ -820,7 +848,7 @@ std::string SqlEditorForm::sql_editor_caption(int new_index)
   
   if (new_index >= 0 && new_index < (int) _sql_editors.size())
   {
-    if (_sql_editors.at(new_index)->dirty && !_sql_editors.at(new_index)->is_scratch)
+    if (_sql_editors.at(new_index)->editor->get_editor_control()->is_dirty() && !_sql_editors.at(new_index)->is_scratch)
       return _sql_editors.at(new_index)->caption+"*";
     else
       return _sql_editors.at(new_index)->caption;
@@ -860,8 +888,6 @@ int SqlEditorForm::sql_editor_index_for_recordset(long long rset)
   return -1;
 }
 
-
-
 RecordsetsRef SqlEditorForm::sql_editor_recordsets(const int index)
 {
   Sql_editor_info::Ref editor_info = _sql_editors[index];
@@ -889,34 +915,15 @@ int SqlEditorForm::sql_editor_index(Sql_editor::Ref editor)
 }
 
 
-void SqlEditorForm::set_sql_editor_text(const std::string &sql)
+void SqlEditorForm::set_sql_editor_text(const char *sql)
 {
   if (active_sql_editor())
   {
-    active_sql_editor()->is_refresh_enabled(true);
+    active_sql_editor()->set_refresh_enabled(true);
     active_sql_editor()->sql(sql);
-    do_partial_ui_refresh(SqlEditorForm::RefreshEditor);
   }
 }
 
-
-void SqlEditorForm::on_sql_editor_text_change()
-{
-  if (_updating_sql_editor == 0)
-  {
-    sql_editor_dirty(_active_sql_editor_index, true);
-  }
-}
-
-
-void SqlEditorForm::on_sql_editor_text_selection_change()
-{
-  // not necessary in mac, maybe in windows or linux.. if useless, just remove it
-  //  _context_ui->signal_validate _edit_menu_items().emit();
-}
-
-
-
 void SqlEditorForm::new_sql_script_file()
 {
   _active_sql_editor_index= add_sql_editor(false);
@@ -967,7 +974,7 @@ void SqlEditorForm::open_file(const std::string &path, bool in_new_tab)
   if (in_new_tab)
     new_sql_script_file();
   
-  if (sql_editor_dirty(_active_sql_editor_index))
+  if (sql_editor(_active_sql_editor_index)->get_editor_control()->is_dirty())
   {
     int r = mforms::Utilities::show_warning(_("Open File"), 
                                             strfmt(_("SQL script %s has unsaved changes.\n"
@@ -983,6 +990,7 @@ void SqlEditorForm::open_file(const std::string &path, bool in_new_tab)
   sql_editor_open_file(_active_sql_editor_index, file_path, "");
 }
 
+#define EDITOR_TEXT_LIMIT 100 * 1024 * 1024
 
 void SqlEditorForm::sql_editor_open_file(int index, const std::string &file_path, const std::string &encoding)
 {
@@ -992,10 +1000,23 @@ void SqlEditorForm::sql_editor_open_file(int index, const std::string &file_path
   if (index < 0 || index >= (int)_sql_editors.size())
     return;
   
-  gchar *contents= NULL;
-  gsize length= 0;
-  GError *error= NULL;
+  gchar *contents = NULL;
+  gsize length = base_get_file_size(file_path.c_str());
+  GError *error = NULL;
   
+  if (length == 0)
+    return;
+
+  if (length > EDITOR_TEXT_LIMIT)
+  {
+    // File is larger than 100 MB. Tell the user we are going to switch off code folding and
+    // auto completion.
+    int result = mforms::Utilities::show_warning(_("Large File"), strfmt(_("The file \"%s\" has a size "
+      "of %.2f MB. Are you sure you want to load this large file?\n\nNote: code folding "
+      "will be disabled for this file."), file_path.c_str(), length / 1024.0 / 1024.0), _("OK, load it"), _("No, better not"));
+    if (result != mforms::ResultOk)
+      return;
+  }
   _grtm->replace_status_text(strfmt(_("Loading SQL script file '%s'..."), file_path.c_str()));
   
   if (!g_file_get_contents(file_path.c_str(), &contents, &length, &error))
@@ -1006,8 +1027,8 @@ void SqlEditorForm::sql_editor_open_file(int index, const std::string &file_path
     return;
   }
   
+  // XXX: the handling here is ineffective. Switch to const char* instead copying around data.
   std::string utf8_contents;
-  std::string utf8_contents_fixed;
   std::string original_encoding;
   if (!FileCharsetDialog::ensure_filedata_utf8(contents, length, encoding,
                                                file_path, utf8_contents, &original_encoding))
@@ -1037,37 +1058,18 @@ void SqlEditorForm::sql_editor_open_file(int index, const std::string &file_path
     return;
   }  
   
-  std::string *sql= NULL;
-  if (EolHelpers::check(utf8_contents))
-  {
-    sql= &utf8_contents;
-  }
-  else
-  {
-    EolFormatDialog dialog(_("Inconsistent Line Endings"),
-                           strfmt(_("File: \"%s\"\nLine endings are inconsistent.\nSelect line ending format to apply.\nResult of conversion will replace original file contents if file is saved afterwards."),
-                                  file_path.c_str()));
-    if (dialog.run())
-    {
-      EolHelpers::fix(utf8_contents, utf8_contents_fixed, dialog.eol_format());
-      sql= &utf8_contents_fixed;
-    }
-  }
-  
-  if (!sql) // line endings fix was cancelled by user
-    return;
-  
   _updating_sql_editor++;
-  set_sql_editor_text(*sql);
+  set_sql_editor_text(utf8_contents.c_str());
   _updating_sql_editor--;
   
   Sql_editor_info::Ref editor(_sql_editors[index]);
-  
+  editor->editor->get_editor_control()->reset_dirty();
+
   editor->filename = file_path;
   editor->orig_encoding = original_encoding;
-  editor->dirty = false;
   editor->is_scratch = false;
   editor->caption = base::strip_extension(base::basename(file_path));
+  base::file_mtime(file_path, editor->file_timestamp);
   //_context_ui->get_wb()->add_recent_file(file_path);
   {
     NotificationInfo info;
@@ -1117,14 +1119,10 @@ bool SqlEditorForm::save_sql_script_file(const std::string &file_name, int edito
     //    if (!g_str_has_suffix(path.c_str(), ".sql"))
     //      path.append(".sql");
     
-    // tell UI that editor data changed
-    do_partial_ui_refresh(SqlEditorForm::RefreshEditorBackend);
-    
-    std::string contents= sql_editor(editor_index)->sql();
-    
     _grtm->replace_status_text(strfmt(_("Saving SQL script to '%s'..."), path.c_str()));
     
-    if (!g_file_set_contents(path.c_str(), contents.data(), contents.length(), &error))
+    std::pair<const char*, size_t> text = sql_editor(editor_index)->text_ptr();
+    if (!g_file_set_contents(path.c_str(), text.first, text.second, &error))
     {
       _grtm->replace_status_text(strfmt(_("Error saving SQL script to '%s'."), path.c_str()));
       
@@ -1135,11 +1133,12 @@ bool SqlEditorForm::save_sql_script_file(const std::string &file_name, int edito
     }
     
     Sql_editor_info::Ref editor(_sql_editors[editor_index]);
+    editor->editor->get_editor_control()->reset_dirty();
     editor->filename = path;
-    editor->dirty = false;
     editor->is_scratch = false;
     editor->caption = base::strip_extension(base::basename(path));
-    
+    base::file_mtime(path, editor->file_timestamp);
+
     do_partial_ui_refresh(RefreshEditorTitle);
     
     //_context_ui->get_wb()->add_recent_file(path);
@@ -1164,3 +1163,4 @@ std::string SqlEditorForm::restore_sql_from_history(int entry_index, std::list<i
   return _history->restore_sql_from_history(entry_index, detail_indexes);
 }
 
+
