@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2013 Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,7 +20,7 @@
 #include "tut_stdafx.h"
 
 #include "wb_helpers.h"
-#include "workbench/wb_history_tree.h"
+#include "model/wb_history_tree.h"
 #include "grtdb/db_object_helpers.h"
 #include "grt_test_utility.h"
 
@@ -31,7 +31,7 @@ using namespace wb;
 
 BEGIN_TEST_DATA_CLASS(wb_undo_editors)
 public:
-  WBTester             wb;
+  WBTester             tester;
   WBContextUI         *wbui;
   UndoManager         *um;
   OverviewBE          *overview;
@@ -43,8 +43,8 @@ public:
 
 TEST_DATA_CONSTRUCTOR(wb_undo_editors)
 {
-  wbui     = wb.wb->get_ui();
-  um       = wb.wb->get_grt()->get_undo_manager();
+  wbui     = tester.wb->get_ui();
+  um       = tester.wb->get_grt()->get_undo_manager();
   overview = wbui->get_physical_overview();
   
   wbui->set_active_form(overview);
@@ -71,14 +71,14 @@ TEST_MODULE(wb_undo_editors, "undo tests for editors in Workbench");
 // setup
 TEST_FUNCTION(1)
 {
-  bool flag= wb.wb->open_document("data/workbench/undo_test_model2.mwb");
+  bool flag= tester.wb->open_document("data/workbench/undo_test_model2.mwb");
   ensure("open_document", flag);
   
-  ensure_equals("schemas", wb.get_catalog()->schemata().count(), 1U);
+  ensure_equals("schemas", tester.get_catalog()->schemata().count(), 1U);
   
-  schema = wb.get_catalog()->schemata()[0];
+  schema = tester.get_catalog()->schemata()[0];
 
-  _rdbms = db_mgmt_RdbmsRef::cast_from(wb.grt->unserialize("../../modules/db.mysql/res/mysql_rdbms_info.xml"));
+  _rdbms = db_mgmt_RdbmsRef::cast_from(tester.grt->unserialize("../../modules/db.mysql/res/mysql_rdbms_info.xml"));
   
   // make sure the loaded model contains expected number of things
   ensure_equals("tables", schema->tables().count(), 2U);
@@ -87,29 +87,13 @@ TEST_FUNCTION(1)
   ensure_equals("undo stack is empty", um->get_undo_stack().size(), 0U);
 }
 
-// final check
-TEST_FUNCTION(100)
-{
-  ensure_equals("tables", schema->tables()[0]->columns()->count(), 2U);
-  WBTester other;
-
-  wb.get_pview()->unselectAll();
-  
-  other.wb->open_document("data/workbench/undo_test_model2.mwb");
-
-  // check if the document matches the one originally loaded
-  grt_ensure_equals("compare model",
-                    wb.wb->get_root(),
-                    other.wb->get_root());
-}
-
 // Editors
 //----------------------------------------------------------------------------------------
 TEST_FUNCTION(2) // General
 {
   db_mysql_TableRef table(db_mysql_TableRef::cast_from(schema->tables()[0]));
   const std::string old_name = table->name();
-  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(wb.wb->get_grt_manager(), table, _rdbms));
+  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(tester.wb->get_grt_manager(), table, _rdbms));
   
   // Rename
   be->set_name("new_name");
@@ -130,7 +114,7 @@ TEST_FUNCTION(2) // General
 TEST_FUNCTION(3) // Table (general)
 {
   db_mysql_TableRef table(db_mysql_TableRef::cast_from(schema->tables()[0]));
-  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(wb.wb->get_grt_manager(), table, _rdbms));
+  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(tester.wb->get_grt_manager(), table, _rdbms));
   
   // Change Collation
   std::string old_value = be->get_table_option_by_name("CHARACTER SET - COLLATE");
@@ -177,7 +161,7 @@ TEST_FUNCTION(3) // Table (general)
 TEST_FUNCTION(4) // Table (add column)
 {   
   db_mysql_TableRef table(db_mysql_TableRef::cast_from(schema->tables()[0]));
-  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(wb.wb->get_grt_manager(), table, _rdbms));
+  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(tester.wb->get_grt_manager(), table, _rdbms));
 
   // Add 
   const size_t existing_cols_nr = table->columns().count();
@@ -196,7 +180,7 @@ TEST_FUNCTION(4) // Table (add column)
 TEST_FUNCTION(5) // Table (rename column)
 {   
   db_mysql_TableRef table(db_mysql_TableRef::cast_from(schema->tables()[0]));
-  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(wb.wb->get_grt_manager(), table, _rdbms));
+  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(tester.wb->get_grt_manager(), table, _rdbms));
   
   const std::string old_name = table->columns()[0]->name();
   // Rename 
@@ -212,7 +196,7 @@ TEST_FUNCTION(5) // Table (rename column)
 TEST_FUNCTION(6) // Table (Column type change)
 {
   db_mysql_TableRef table(db_mysql_TableRef::cast_from(schema->tables()[0]));
-  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(wb.wb->get_grt_manager(), table, _rdbms));
+  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(tester.wb->get_grt_manager(), table, _rdbms));
 
   bec::TableColumnsListBE* cols = be->get_columns();
 
@@ -243,25 +227,25 @@ TEST_FUNCTION(6) // Table (Column type change)
 TEST_FUNCTION(7) // Table (Column flag change)
 {
   db_mysql_TableRef table(db_mysql_TableRef::cast_from(schema->tables()[0]));
-  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(wb.wb->get_grt_manager(), table, _rdbms));
+  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(tester.wb->get_grt_manager(), table, _rdbms));
 
   bec::TableColumnsListBE* cols = be->get_columns();
 
   // Change Flag 
-  int flag = cols->get_column_flag(bec::NodeId(0), "PRIMARY KEY");
-  cols->set_column_flag(bec::NodeId(0), "PRIMARY KEY", !flag);
+  int flag = cols->get_column_flag(bec::NodeId(0), "UNSIGNED");
+  cols->set_column_flag(bec::NodeId(0), "UNSIGNED", !flag);
   check_only_one_undo_added();
-  ensure("Column flag was not changed", flag != cols->get_column_flag(bec::NodeId(0), "PRIMARY KEY"));
+  ensure("Column flag was not changed", flag != cols->get_column_flag(bec::NodeId(0), "UNSIGNED"));
 
   check_undo();
-  ensure("Column flag change undo failed", flag == cols->get_column_flag(bec::NodeId(0), "PRIMARY KEY"));
+  ensure("Column flag change undo failed", flag == cols->get_column_flag(bec::NodeId(0), "UNSIGNED"));
 }
 
 //------------------------------------------------------------------------------
 TEST_FUNCTION(8) // Table (column NN change)
 {
   db_mysql_TableRef table(db_mysql_TableRef::cast_from(schema->tables()[0]));
-  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(wb.wb->get_grt_manager(), table, _rdbms));
+  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(tester.wb->get_grt_manager(), table, _rdbms));
 
   bec::TableColumnsListBE* cols = be->get_columns();
 
@@ -285,7 +269,7 @@ TEST_FUNCTION(8) // Table (column NN change)
 TEST_FUNCTION(9) // Table (column default change)
 {
   db_mysql_TableRef table(db_mysql_TableRef::cast_from(schema->tables()[0]));
-  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(wb.wb->get_grt_manager(), table, _rdbms));
+  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(tester.wb->get_grt_manager(), table, _rdbms));
 
   bec::TableColumnsListBE* cols = be->get_columns();
   
@@ -305,7 +289,7 @@ TEST_FUNCTION(9) // Table (column default change)
 TEST_FUNCTION(10) // Table (column remove)
 {
   db_mysql_TableRef table(db_mysql_TableRef::cast_from(schema->tables()[0]));
-  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(wb.wb->get_grt_manager(), table, _rdbms));
+  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(tester.wb->get_grt_manager(), table, _rdbms));
 
   const size_t ncols = table->columns().count();
   ensure_equals("tables", ncols, 2U);
@@ -321,7 +305,7 @@ TEST_FUNCTION(10) // Table (column remove)
 TEST_FUNCTION(11) // Table (index add)
 {
   db_mysql_TableRef table(db_mysql_TableRef::cast_from(schema->tables()[0]));
-  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(wb.wb->get_grt_manager(), table, _rdbms));
+  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(tester.wb->get_grt_manager(), table, _rdbms));
 
   const size_t nidx = table->indices().count();
   // Add
@@ -340,7 +324,7 @@ TEST_FUNCTION(11) // Table (index add)
 TEST_FUNCTION(12) // Table (index rename)
 {
   db_mysql_TableRef table(db_mysql_TableRef::cast_from(schema->tables()[0]));
-  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(wb.wb->get_grt_manager(), table, _rdbms));
+  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(tester.wb->get_grt_manager(), table, _rdbms));
 
   const size_t nidx = table->indices().count();
   be->add_index("idx");
@@ -360,7 +344,7 @@ TEST_FUNCTION(12) // Table (index rename)
 TEST_FUNCTION(13) // Table (index remove)
 {
   db_mysql_TableRef table(db_mysql_TableRef::cast_from(schema->tables()[0]));
-  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(wb.wb->get_grt_manager(), table, _rdbms));
+  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(tester.wb->get_grt_manager(), table, _rdbms));
 
   const size_t nidx = table->indices().count();
   be->add_index("idx");
@@ -379,7 +363,10 @@ TEST_FUNCTION(13) // Table (index remove)
 TEST_FUNCTION(14) // Table (index type change)
 {
   db_mysql_TableRef table(db_mysql_TableRef::cast_from(schema->tables()[0]));
-  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(wb.wb->get_grt_manager(), table, _rdbms));
+  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(tester.wb->get_grt_manager(), table, _rdbms));
+
+  // set engine to something that supports FULLTEXT and RTREE index options
+  table->tableEngine("MyISAM");
 
   const size_t nidx = table->indices().count();
   be->add_index("idx");
@@ -401,7 +388,7 @@ TEST_FUNCTION(14) // Table (index type change)
 TEST_FUNCTION(15) // Table (index storage change)
 {
   db_mysql_TableRef table(db_mysql_TableRef::cast_from(schema->tables()[0]));
-  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(wb.wb->get_grt_manager(), table, _rdbms));
+  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(tester.wb->get_grt_manager(), table, _rdbms));
 
   const size_t nidx = table->indices().count();
   be->add_index("idx");
@@ -424,7 +411,7 @@ TEST_FUNCTION(15) // Table (index storage change)
 TEST_FUNCTION(16) // Table (index column add)
 {
   db_mysql_TableRef table(db_mysql_TableRef::cast_from(schema->tables()[0]));
-  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(wb.wb->get_grt_manager(), table, _rdbms));
+  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(tester.wb->get_grt_manager(), table, _rdbms));
 
   const size_t nidx = table->indices().count();
   be->add_index("idx");
@@ -453,7 +440,7 @@ TEST_FUNCTION(16) // Table (index column add)
 TEST_FUNCTION(17) // Table (fk add)
 {
   db_mysql_TableRef table(db_mysql_TableRef::cast_from(schema->tables()[0]));
-  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(wb.wb->get_grt_manager(), table, _rdbms));
+  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(tester.wb->get_grt_manager(), table, _rdbms));
   const size_t nfks = table->foreignKeys().count();
 
   be->add_fk("fk1");
@@ -469,17 +456,17 @@ TEST_FUNCTION(17) // Table (fk add)
 TEST_FUNCTION(18) // Table (fk remove)
 {
   db_mysql_TableRef table(db_mysql_TableRef::cast_from(schema->tables()[0]));
-  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(wb.wb->get_grt_manager(), table, _rdbms));
+  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(tester.wb->get_grt_manager(), table, _rdbms));
   const size_t nfks = table->foreignKeys().count();
 
   be->add_fk("fk1");
   check_only_one_undo_added();
-  
+  ensure_equals("FK was added", nfks+1, table->foreignKeys().count());
   be->remove_fk(bec::NodeId(nfks));
   check_only_one_undo_added();
-  ensure("FK was not removed", nfks == table->foreignKeys().count());
+  ensure_equals("FK was removed", nfks, table->foreignKeys().count());
   check_undo();  
-  ensure("FK remove undo failed", nfks+1 != table->foreignKeys().count());
+  ensure_equals("FK remove undo", nfks+1, table->foreignKeys().count());
 
   check_undo();  
 }
@@ -488,7 +475,7 @@ TEST_FUNCTION(18) // Table (fk remove)
 TEST_FUNCTION(19) // Table (fk rename)
 {
   db_mysql_TableRef table(db_mysql_TableRef::cast_from(schema->tables()[0]));
-  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(wb.wb->get_grt_manager(), table, _rdbms));
+  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(tester.wb->get_grt_manager(), table, _rdbms));
   const size_t              nfks = table->foreignKeys().count();
   bec::FKConstraintListBE*  fks  = be->get_fks();
 
@@ -510,7 +497,7 @@ TEST_FUNCTION(19) // Table (fk rename)
 TEST_FUNCTION(20) // Table (fk set ref table)
 {
   db_mysql_TableRef table(db_mysql_TableRef::cast_from(schema->tables()[0]));
-  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(wb.wb->get_grt_manager(), table, _rdbms));
+  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(tester.wb->get_grt_manager(), table, _rdbms));
   const size_t              nfks = table->foreignKeys().count();
   bec::FKConstraintListBE*  fks  = be->get_fks();
 
@@ -531,7 +518,7 @@ TEST_FUNCTION(21) // Table (fk set ref column)
 {
   db_mysql_TableRef table(db_mysql_TableRef::cast_from(schema->tables()[0]));
   db_mysql_TableRef table2(db_mysql_TableRef::cast_from(schema->tables()[1]));
-  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(wb.wb->get_grt_manager(), table, _rdbms));
+  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(tester.wb->get_grt_manager(), table, _rdbms));
   const size_t              nfks = table->foreignKeys().count();
   bec::FKConstraintListBE*  fks  = be->get_fks();
 
@@ -565,14 +552,17 @@ TEST_FUNCTION(21) // Table (fk set ref column)
 TEST_FUNCTION(22) // Table (Triggers)
 {
   db_mysql_TableRef table(db_mysql_TableRef::cast_from(schema->tables()[0]));
-  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(wb.wb->get_grt_manager(), table, _rdbms));
+  std::auto_ptr<MySQLTableEditorBE> be(new MySQLTableEditorBE(tester.wb->get_grt_manager(), table, _rdbms));
 
   const std::string trig_sql = be->get_all_triggers_sql();
-  be->set_triggers_sql("create trigger tr1 before insert on table1 for each row set new.idtable1 = 1", true);//dunno if this should be true or fasle
+  be->set_triggers_sql("create trigger tr1 before insert on table1 for each row set new.idtable1 = 1", true);
   check_only_one_undo_added();
-  ensure("Triggers sql was not set", be->get_all_triggers_sql() == "create trigger tr1 before insert on table1 for each row set new.idtable1 = 1");
+  std::string expected = "-- Full Trigger DDL Statements\n-- Note: Only CREATE TRIGGER statements are allowed\n"
+    "DELIMITER $$\n\nUSE `mydb`$$\n\ncreate trigger tr1 before insert on table1 for each row set new.idtable1 = 1$$\n\n";
+
+  ensure_equals("Triggers sql set", be->get_all_triggers_sql(), expected);
   check_undo();
-  ensure("Triggers sql undo failed", be->get_all_triggers_sql() == trig_sql);
+  ensure_equals("Triggers sql undo", be->get_all_triggers_sql(), trig_sql);
 }
 
 /*
