@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,13 +17,9 @@
  * 02110-1301  USA
  */
 
-
-
 #ifndef _WB_CONTEXT_H_
 #define _WB_CONTEXT_H_
 
-#include "wb_config.h"
-
 #ifndef _WIN32
 #include <vector>
 #endif
@@ -31,41 +27,38 @@
 #include "base/ui_form.h"
 #include "grt/grt_manager.h"
 #include "base/notifications.h"
-#include "grt/plugin_manager.h"
-#include "grt/icon_manager.h"
+
+#include "wb_context_names.h"
 
 #include "grts/structs.workbench.h"
 #include "wb_backend_public_interface.h"
 
-#include "sqlide/wb_context_sqlide.h"
-
-#include "model/wb_context_model.h"
-#include "sqlide/wb_context_sqlide.h"
-
 #include <grtpp_undo_manager.h>
 
-#include "mdc.h"
 #include "mforms/utilities.h"
 #include "base/trackable.h"
 
 
-#define APP_MAJOR_NUMBER 5
-#define APP_MINOR_NUMBER 2
-#define APP_RELEASE_NUMBER 40
-#define APP_REVISION_NUMBER 8790
-#define APP_RELEASE_TYPE ""
-
+#include "wb_version.h"
 
 #define WBContext_VERSION 5
 
+namespace mdc {
+  class CanvasView;
+  class CanvasItem;
+};
 
 namespace bec {
   class Clipboard;
+  class IconManager;
 };
 
+class SqlEditorForm;
+
 namespace wb {
   class WBContextUI;
-
+  class WBContextModel;
+  class WBContextSQLIDE;
   class WorkbenchImpl;
   class WBComponent;
 
@@ -78,20 +71,16 @@ namespace wb {
     RefreshNeeded, // Front end should schedule a refresh flush asap (usually called in worker thread).
     RefreshNothing,
     RefreshSchemaList,
-    RefreshSchema,
     RefreshSchemaNoReload,
     RefreshNewDiagram,
     RefreshSelection,
     RefreshCloseEditor, // argument: object-id (close all if "")
-    RefreshFindOutput,
     
     RefreshNewModel,
 
     RefreshOverviewNodeInfo,    // argument: node-id, overview ptr (bec::UIForm*)
     RefreshOverviewNodeChildren,// argument: node-id, overview ptr 
 
-    RefreshCatalogTreeNode,
-    RefreshViewName,
     RefreshDocument,
     RefreshCloseDocument,
     RefreshZoom,
@@ -100,16 +89,6 @@ namespace wb {
     RefreshFinishEdits // Force all ongoing edit operations (eg in TreeView cells) to be commited
   };
 
-  struct WBShortcut
-  {
-    std::string shortcut;
-    mdc::KeyInfo key;
-    mdc::EventState modifiers;
-
-    std::string name;
-    std::string command;
-  };
-
   struct MYSQLWBBACKEND_PUBLIC_FUNC WBPaperSize
   {
     std::string name;
@@ -124,18 +103,6 @@ namespace wb {
     std::string description;
   };
 
-  // context names
-  #define WB_CONTEXT_GLOBAL  "*global"
-  #define WB_CONTEXT_HOME_GLOBAL "home"
-  #define WB_CONTEXT_MODEL_GLOBAL  "*model"
-  #define WB_CONTEXT_QUERY_GLOBAL  "*query"
-  #define WB_CONTEXT_ADMIN_GLOBAL  "*admin"
-
-  #define WB_CONTEXT_MODEL   "model"
-  #define WB_CONTEXT_EDITOR "editor" // object editors
-  #define WB_CONTEXT_PHYSICAL_OVERVIEW "overview.physical"
-  #define WB_CONTEXT_QUERY "dbquery"
-  
   // basic toolbar names
   #define WB_TOOLBAR_OPTIONS "options"
 
@@ -166,10 +133,6 @@ namespace wb {
 
     // Show some text in the application's status bar: must be thread-safe
     boost::function<void (std::string)> show_status_text;
-    // Displays a progress bar
-    // Args: title, status, percentage completed
-    // Must return false if the action must be cancelled
-    boost::function<bool (std::string,std::string,float)> show_progress;
 
     // ok/cancel, title, flags, returnval
     boost::function<bool (std::string, int, std::string&)> request_input;
@@ -205,7 +168,7 @@ namespace wb {
     boost::function<void (bool)> lock_gui;
 
     // Closes the application
-    boost::function<void ()> quit_application;
+    boost::function<bool ()> quit_application;
   };
 
 
@@ -220,14 +183,18 @@ namespace wb {
     std::string user_data_dir;
     std::string open_at_startup_type; // model, query, admin, script
     std::string open_at_startup;
+    std::string open_connection;
     std::string run_at_startup; // script to be executed when started
     std::string run_language; // language of the script in run_at_startup
     bool force_sw_rendering;
     bool force_opengl_rendering;
     bool verbose;
     bool quit_when_done;
+    bool testing;     // True if we are currently running unit tests.
+    bool init_python; // True by default. Can be switched off for testing.
 
     WBOptions();
+    bool parse_args(char **argv, int argc, int *retval = NULL);
   };
 
   
@@ -241,7 +208,7 @@ namespace wb {
 
    public:
     WBContext(WBContextUI *ui, bool verbose = false);
-    ~WBContext();
+    virtual ~WBContext();
 
     bool software_rendering_enforced();
     bool opengl_rendering_enforced();
@@ -250,18 +217,19 @@ namespace wb {
     void init_finish_(WBOptions *options);
     void finalize();
     
+    bool is_commercial();
     WBContextUI *get_ui() { return _uicontext; }
 
     bec::UIForm *get_active_form();
     bec::UIForm *get_active_main_form();
 
     WBContextModel *get_model_context() { return _model_context; }
-    WBContextSQLIDE *get_sqlide_context() { return &_sqlide_context; }
+    WBContextSQLIDE *get_sqlide_context() { return _sqlide_context; }
     
     // Document handling.
     void new_document();
-    bool can_close_document();
-    bool close_document(); // returns false for cancelled
+    bool can_close_document(); // returns false for cancelled
+    bool close_document(); 
     
     void close_document_finish();
     void new_model_finish();
@@ -280,31 +248,26 @@ namespace wb {
 
     void update_plugin_arguments_pool(bec::ArgumentPool &args);
 
-    // Diagrams
-    void add_new_diagram(const model_ModelRef &model);
-    bool delete_diagram(const model_DiagramRef &view);
-
-    void switch_diagram(const model_DiagramRef &view);
-    
     // DB Querying
     boost::shared_ptr<SqlEditorForm> add_new_query_window(const db_mgmt_ConnectionRef &target);
     
     // Admin
-    void add_new_admin_window(const db_mgmt_ServerInstanceRef &target);
+    void add_new_admin_window(const db_mgmt_ConnectionRef &target);
+
+    // Generic plugin tabs
+    void add_new_plugin_window(const std::string &plugin_id, const std::string &caption);
 
     // GUI Plugin
     void register_builtin_plugins(grt::ListRef<app_Plugin> plugins);
 
     void close_gui_plugin(NativeHandle handle);
 
-    bool delete_object(model_ObjectRef object, std::map<std::string, bool>& options);
-
 
     //
     void request_refresh(RefreshType type, const std::string &str, NativeHandle ptr= (NativeHandle)0);
 
     const std::string &get_user_datadir() const { return _user_datadir; }
-    //TODO: Temporarily solution need to make ModelFile grt class
+    // TODO: Temporary solution need to make ModelFile grt class
     workbench_DocumentRef openModelFile(const std::string &file);
     std::string getTempDir();
     int closeModelFile();
@@ -332,7 +295,7 @@ namespace wb {
 
     bec::GRTManager *get_grt_manager() const { return _manager; }
     grt::GRT *get_grt() const { return _manager->get_grt(); }
-    WorkbenchImpl *get_workbench() { return _workbench; };
+    boost::shared_ptr<WorkbenchImpl> get_workbench() { return _workbench; };
 
     bec::Clipboard *get_clipboard() const { return _clipboard; }
 
@@ -342,14 +305,7 @@ namespace wb {
 
     std::string get_datadir() const { return _datadir; }
     
-    template<class C> grt::Ref<C> get_parent_for_object(const GrtObjectRef &object)
-    {
-      GrtObjectRef obj= object;
-      while (obj.is_valid() && !obj.is_instance(C::static_class_name()))
-        obj= obj->owner();
-      return grt::Ref<C>::cast_from(obj);
-    }
-
+    bool cancel_idle_tasks();
     void flush_idle_tasks();
 
  // utilities for error reporting
@@ -372,12 +328,7 @@ namespace wb {
     void execute_async_in_grt_thread(const std::string &name, 
                                      const boost::function<grt::ValueRef (grt::GRT*)> &function) THROW (grt::grt_runtime_error);
 
-    void open_object_editor(const GrtObjectRef &object, bec::GUIPluginFlags flags=bec::NoFlags);
     bool activate_live_object(const GrtObjectRef &object);
-    bool create_live_object(GrtObjectRef object_type, std::string owner_name, std::string obj_name);
-    bool drop_live_object(GrtObjectRef object_type, std::string owner_name, std::string obj_name);
-
-    model_DiagramRef get_view_with_id(const std::string &id);
 
     std::string create_attached_file(const std::string &group, const std::string &tmpl);
     void save_attached_file_contents(const std::string &name, const char *data, size_t size);
@@ -404,7 +355,6 @@ namespace wb {
   protected:
     friend class WBContextModel; // to access _components
     
-    ModelFile* _model_file;
     bec::GRTManager *_manager;
     bec::PluginManager *_plugin_manager;
 
@@ -414,6 +364,7 @@ namespace wb {
     bool _send_messages_to_shell;
     bool _asked_for_saving;
     bool _initialization_finished;
+    bool _attachments_changed;
     
     std::string _datadir;
     std::string _user_datadir;
@@ -431,11 +382,11 @@ namespace wb {
 
     WBContextUI *_uicontext;
     WBContextModel *_model_context;
-    WBContextSQLIDE _sqlide_context;
+    WBContextSQLIDE *_sqlide_context;
     
     std::vector<WBComponent*> _components;
 
-    WorkbenchImpl *_workbench;
+    boost::shared_ptr<WorkbenchImpl> _workbench;
 
     bec::Clipboard *_clipboard;
 
@@ -445,23 +396,22 @@ namespace wb {
     grt::UndoAction *_save_point;
     
     TunnelManager *_tunnel_manager;
+
+    ModelFile* _model_import_file;
     
     bool _force_sw_rendering;     // Command line switch.
     bool _force_opengl_rendering; // Command line switch.
- 
-    //boost::signals2::signal<void ()> _signal_app_closing;
-    
-    void delete_canvas_item(mdc::CanvasItem *item);
 
-    grt::ListRef<app_PaperType> get_paper_types(grt::GRT *grt);
+    grt::ListRef<app_PaperType> get_paper_types(grt::GRT *grt, boost::shared_ptr<grt::internal::Unserializer> unserializer);
 
     // setup
-    void init_grt_tree(grt::GRT *grt, WBOptions *options);
+    void init_grt_tree(grt::GRT *grt, WBOptions *options, boost::shared_ptr<grt::internal::Unserializer> unserializer);
     void run_init_scripts_grt(grt::GRT *grt, WBOptions *options);
     void init_plugins_grt(grt::GRT *grt, WBOptions *options);
     void init_plugin_groups_grt(grt::GRT *grt, WBOptions *options);
     void init_object_listeners_grt(grt::GRT *grt);
     void init_properties_grt(workbench_DocumentRef &doc);
+    void init_rdbms_modules(grt::GRT *grt);
 
     void do_close_document(bool destroying);
 
@@ -473,26 +423,28 @@ namespace wb {
     
     bool auto_save_document();
     std::string get_auto_save_dir();
+
+    void cleanup_options();
   public:
     void save_app_options();
+    void save_connections();
+    void save_instances();
 
-//    boost::signals2::signal<void ()>* signal_app_closing() { return &_signal_app_closing; }
-    
   protected:
     void add_recent_file(const std::string &file);
 
-    void load_app_state();
+    void load_app_state(boost::shared_ptr<grt::internal::Unserializer> unserializer);
     void save_app_state();
-    void load_starters();
+    void load_starters(boost::shared_ptr<grt::internal::Unserializer> unserializer);
     void save_starters();
 
-    grt::ValueRef new_document_grt(grt::GRT *grt);
     grt::ValueRef save_grt(grt::GRT *grt);
     
     grt::ValueRef execute_plugin_grt(grt::GRT *grt, const app_PluginRef &plugin, const grt::BaseListRef &args);
     void plugin_finished(const grt::ValueRef &result, const app_PluginRef &plugin);
 
-    void handle_message(const grt::Message &msg);
+    bool handle_message(const grt::Message &msg);
+    void handle_grt_message(bec::MessageListStorage::MessageEntryRef message);
 
     void reset_document();
     void reset_listeners();
@@ -508,6 +460,7 @@ namespace wb {
 
     bool install_module_file(const std::string &path);
     bool uninstall_module(grt::Module *module);
+    void run_script_file(const std::string &path);
     
   private:
     bool find_connection_password(const db_mgmt_ConnectionRef &conn, std::string &password);
@@ -545,7 +498,7 @@ namespace wb {
 
     boost::function<void (std::string)> perform_command;
     
-    boost::function<void ()> quit_application;
+    boost::function<bool ()> quit_application;
   };
 
   struct GUILock
