@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -51,7 +51,9 @@
 // updated to 1.4.0 in 5.2.0
 // updated to 1.4.1 in 5.2.19
 // updated to 1.4.2 in 5.2.33
-#define DOCUMENT_VERSION "1.4.2"
+// updated to 1.4.3 in 5.2.40
+// updated to 1.4.4 in 5.2.42
+#define DOCUMENT_VERSION "1.4.4"
 
 #define ZIP_FILE_FORMAT "1.0"
 
@@ -279,12 +281,11 @@ void ModelFile::open(const std::string &path, GRTManager *grtm)
     }
   }
 
-  gchar *basename= g_path_get_basename(path.c_str());
+  std::string basename = base::basename(path);
   
   // do some sanity checks
-  if (strlen(basename) == 0)
+  if (basename.empty())
   {
-    g_free(basename);
     throw std::runtime_error("Invalid path "+path);
   }
 
@@ -317,6 +318,22 @@ void ModelFile::open(const std::string &path, GRTManager *grtm)
   bool recover = false;
   if (!auto_save_dir.empty() && g_file_test(auto_save_dir.c_str(), G_FILE_TEST_EXISTS))
   {
+    // See if we can acquire the auto save lock. If that fails then another instance of WB has the lock already
+    // so we don't do anything with this file.
+    try
+    {
+      base::LockFile test_lock(bec::make_path(auto_save_dir, lock_filename.c_str()));
+    }
+    catch (const base::file_locked_error)
+    {
+      mforms::Utilities::show_warning(_("Opening Document"),
+        base::strfmt(_("Could not lock the document %s.\n"
+        "The file is probably already opened in another instance of the application."),
+        path.c_str()), _("OK"));
+
+      return;
+    }
+
     time_t file_ts = get_file_timestamp(path);
     time_t autosave_ts = get_file_timestamp(bec::make_path(auto_save_dir, MAIN_DOCUMENT_NAME));
     if (autosave_ts == 0)
@@ -373,7 +390,6 @@ void ModelFile::open(const std::string &path, GRTManager *grtm)
   if (!recover)
   {
     _content_dir= create_document_dir(_temp_dir, basename);
-    g_free(basename);
 
     if (file_is_zip)
     {
@@ -395,7 +411,6 @@ void ModelFile::open(const std::string &path, GRTManager *grtm)
   }
   else
   {
-    g_free(basename);
     _dirty= true;
 
     // re-lock it for ourselves
@@ -403,6 +418,45 @@ void ModelFile::open(const std::string &path, GRTManager *grtm)
   }
 }
 
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Returns a previously set zip file comment (if any) without unzipping the file.
+ */
+std::string ModelFile::read_comment(const std::string &path)
+{
+  std::string schemas;
+  int err;
+  zip *z = zip_open(path.c_str(), 0, &err);
+  if (z != NULL)
+  {
+    int length;
+    const char *value = zip_get_archive_comment(z, &length, 0);
+    if (value && length > 0)
+    {
+      std::string comment(value, length);
+      if (length > -1)
+      {
+        std::size_t found = comment.find("model-schemas:");
+        if (found != std::string::npos)
+        {
+
+          const char *ptr = comment.c_str() + found + 15;
+          while(*ptr)
+          {
+            if (*ptr != '\n')
+              schemas += *ptr;
+            ptr++;
+          }
+        }
+      }
+    }
+    zip_close(z);
+  }
+  return schemas;
+}
+
+//--------------------------------------------------------------------------------------------------
 
 void ModelFile::create(GRTManager *grtm)
 {
@@ -522,32 +576,31 @@ std::list<std::string> ModelFile::unpack_zip(const std::string &zipfile, const s
       zip_fclose(file);
       continue;
     }
-    gchar *dirname= g_path_get_dirname(zname);
-    gchar *basename= g_path_get_basename(zname);
+    std::string dirname = base::dirname(zname);
+    std::string basename = base::basename(zname);
 
     //skip lock file as it is already locked and inaccessible
-    if (!strcmp(basename,lock_filename.c_str()))
+    if (basename == lock_filename)
+    {
+      zip_fclose(file);
       continue;
+    }
 
     std::string outpath= destdir;
 
-    if (dirname && *dirname)
+    if (!dirname.empty())
     {
       outpath.append("/");
       outpath.append(dirname);
       if (g_mkdir_with_parents(outpath.c_str(), 0700) < 0)
       {
-        g_free(dirname);
-        g_free(basename);
         zip_fclose(file);
         zip_close(z);
         throw grt::os_error(_("Error creating temporary directory while opending document."), errno);
       }
     }
     outpath.append("/");
-    outpath.append(basename ? basename : "");
-    g_free(dirname);
-    g_free(basename);
+    outpath.append(basename);
 
     FILE *outfile= base_fopen(outpath.c_str(), "wb+");
     if (!outfile)
@@ -652,7 +705,7 @@ static void zip_dir_contents(zip *z, const std::string &destdir, const std::stri
 }
 
 
-void ModelFile::pack_zip(const std::string &zipfile, const std::string &destdir)
+void ModelFile::pack_zip(const std::string &zipfile, const std::string &destdir, const std::string &comment)
 {
   int err= 0;
   std::string curdir;
@@ -682,7 +735,14 @@ void ModelFile::pack_zip(const std::string &zipfile, const std::string &destdir)
       throw grt::os_error("Cannot create file.");
   }
 
-  zip_set_archive_comment(z, ZIP_FILE_COMMENT, sizeof(ZIP_FILE_COMMENT));
+  std::string zip_comment = ZIP_FILE_COMMENT;
+  if (!comment.empty())
+  {
+    zip_comment += '\n';
+    zip_comment += comment;
+  }
+
+  zip_set_archive_comment(z, zip_comment.c_str(), zip_comment.size());
 
   try
   {
@@ -758,7 +818,7 @@ workbench_DocumentRef ModelFile::unserialize_document(grt::GRT *grt, xmlDocPtr x
  * (if there is one). Checks are performed to ensure existing backup files can be removed and existing
  * model files can be renamed to .bak.
  */
-bool ModelFile::save_to(const std::string &path)
+bool ModelFile::save_to(const std::string &path, const std::string &comment)
 {
   AutoLock lock(_mutex);
 #ifdef _WIN32
@@ -824,11 +884,11 @@ bool ModelFile::save_to(const std::string &path)
   g_remove(get_path_for("real_path").c_str());
 
   if (g_path_is_absolute(path.c_str()))
-    pack_zip(path, _content_dir);
+    pack_zip(path, _content_dir, comment);
   else
   {
     char *prefix= g_get_current_dir();
-    pack_zip(std::string(prefix).append("/").append(path), _content_dir);
+    pack_zip(std::string(prefix).append("/").append(path), _content_dir, comment);
     g_free(prefix);
   }
 
@@ -882,13 +942,10 @@ std::string ModelFile::get_db_file_path()
 */
 std::string ModelFile::add_attachment_file(const std::string &destdir, const std::string &path)
 {
-  gchar *base;
   std::string prefix= destdir + "/";
   if (!path.empty())
   {
-    base= g_path_get_basename(path.c_str());
-    prefix += base;
-    g_free(base);
+    prefix += base::basename(path);
   }
 
   int i= 1;
@@ -926,13 +983,8 @@ std::string ModelFile::add_attachment_file(const std::string &destdir, const std
       throw std::runtime_error(std::string("Error adding file to document: ").append(exc.what()));
     }
   }
-  base= g_path_get_basename(destfile.c_str());
-  destfile= base;
-  g_free(base);
 
-  base= g_path_get_basename(destdir.c_str());
-  destfile= std::string(base).append("/").append(destfile);
-  g_free(base);
+  destfile= base::basename(destdir).append("/").append(base::basename(destfile));
 
   return destfile;
 }
