@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -26,8 +26,13 @@
 #include "grtdb/db_object_helpers.h"
 #include "grts/structs.db.mysql.h"
 #include "sqlide/table_inserts_loader_be.h"
+#include "base/log.h"
 
 #include "base/string_utilities.h"
+#include "mforms/utilities.h"
+#include "grtsqlparser/sql_facade.h"
+
+DEFAULT_LOG_DOMAIN("ModelUpgrader");
 
 using namespace wb;
 using namespace grt;
@@ -535,7 +540,7 @@ workbench_DocumentRef ModelFile::attempt_document_upgrade(const workbench_Docume
 
   // in 1.4.1 (after 5.2.19) a new index field was added to db.ForeignKey pointing to the
   // matching index created for the FK and the FOREIGN index type was dropped (just uses INDEX now)
-  if (major <= 1 && (minor < 4 || (minor == 4 && revision == 0)))
+  if (major <= 1 && (minor < 4 || (minor == 4 && revision < 2)))
   {
     ListRef<db_Schema> schemata(doc->physicalModels()[0]->catalog()->schemata());
     for (size_t sc= schemata.count(), si= 0; si < sc; si++)
@@ -587,7 +592,7 @@ workbench_DocumentRef ModelFile::attempt_document_upgrade(const workbench_Docume
           {
             fk->index(found_index);
 
-            g_message("Document Upgrade: ForeignKey %s.%s was assigned the index %s by column matching", 
+            log_info("Document Upgrade: ForeignKey %s.%s was assigned the index %s by column matching\n", 
                       table->name().c_str(), fk->name().c_str(), 
                       found_index->name().c_str());
           }
@@ -598,18 +603,143 @@ workbench_DocumentRef ModelFile::attempt_document_upgrade(const workbench_Docume
             fk->index(new_index);
             table->indices().insert(new_index);
 
-            g_warning("ForeignKey %s.%s has no matching index, created one", 
+            log_warning("ForeignKey %s.%s has no matching index, created one\n", 
                       table->name().c_str(), fk->name().c_str());            
           }
         }
       }
     }
   }
-    
+
+
+  bec::GRTManager *grtm= bec::GRTManager::get_instance_for(doc.get_grt());
+  if (major <= 1 && (minor < 4 || (minor == 4 && revision < 3)))
+  {
+      bool ask_confirmation = true;
+      bool aborted = false;
+      ListRef<db_Schema> schemata(doc->physicalModels()[0]->catalog()->schemata());
+      for (size_t sc= schemata.count(), si= 0; (si < sc) && !aborted; si++)
+      {
+          ListRef<db_Table> tables= schemata[si]->tables();
+          for (size_t c= tables.count(), i= 0; (i < c) && !aborted; i++)
+          {
+              ListRef<db_ForeignKey> fks(tables[i]->foreignKeys());
+              ListRef<db_Index> indexes(tables[i]->indices());
+              for (size_t ic= indexes.count(), ii= 0; (ii < ic) && !aborted; ii++)
+                  for (size_t fc= fks.count(), fi= 0; (fi < fc) && !aborted; fi++)
+                      if(fks[fi]->name() == indexes[ii]->name())
+                      {
+                          if (ask_confirmation &&
+                              mforms::Utilities::show_message("Fix Index Names", "Index names identical to FK names were found in the model, which is not allowed for MySQL 5.5 and later. \r\nWould you like to rename the indexes?",
+                              "Rename", "Ignore") != mforms::ResultOk)
+                          {
+                              aborted = true;
+                              break;
+                          }
+                          ask_confirmation = false;
+                          std::string idx_name = indexes[ii]->name();
+                          if (idx_name.length() > 59)
+                              idx_name.resize(59);
+                          indexes[ii]->name(grt::get_name_suggestion_for_list_object(indexes,idx_name.append("_idx"),false));
+                          log_info("Document Upgrade: Index %s.%s was renamed to %s.%s to avoid name conflict with FK\n",
+                              tables[i]->name().c_str(), fks[fi]->name().c_str(),
+                              tables[i]->name().c_str(), indexes[ii]->name().c_str());
+                          continue;
+                      }
+          }
+      }
+      if (!ask_confirmation)
+        grtm->has_unsaved_changes(true);
+  }
+
+  if (major <= 1 && (minor < 4 || (minor == 4 && revision < 4)))
+  {
+      ListRef<db_Schema> schemata(doc->physicalModels()[0]->catalog()->schemata());
+
+      for (size_t sc= schemata.count(), si= 0; si < sc; si++)
+      {
+          db_SchemaRef schema(schemata[si]);
+
+          db_mgmt_RdbmsRef rdbms = get_rdbms_for_db_object(schema);
+          SqlFacade::Ref sql_facade = SqlFacade::instance_for_rdbms(rdbms);
+          Invalid_sql_parser::Ref sql_parser= sql_facade->invalidSqlParser();
+          Sql_specifics::Ref sql_specifics = sql_facade ->sqlSpecifics();
+          std::string non_std_sql_delimiter = sql_specifics->non_std_sql_delimiter();
+
+
+          ListRef<db_Table> tables= schema->tables();
+          for (size_t c= tables.count(), i= 0; i < c; i++)
+          {
+              db_TableRef table= tables[i];
+
+              std::string triggers_sql;
+
+              triggers_sql.append(strfmt("DELIMITER %s\n\n", non_std_sql_delimiter.c_str())).append(
+                  "USE `").append(table->owner()->name()).append("`").
+                  append(non_std_sql_delimiter.c_str()).append("\n\n");
+
+              grt::ListRef<db_Trigger> triggers= table->triggers();
+              size_t triggers_count= triggers.count();
+              typedef std::map<int, db_TriggerRef> OrderedTriggers;
+              typedef std::list<db_TriggerRef> UnorderedTriggers;
+              OrderedTriggers ordered_triggers;
+              UnorderedTriggers unordered_triggers;
+
+              for (size_t i= 0; i < triggers_count; ++i)
+              {
+                  db_TriggerRef trigger= triggers.get(i);
+                  int sequenceNumber= trigger->sequenceNumber();
+                  if (ordered_triggers.find(sequenceNumber) == ordered_triggers.end())
+                      ordered_triggers[sequenceNumber]= trigger;
+                  else
+                      unordered_triggers.push_back(trigger);
+              }
+
+              for (OrderedTriggers::iterator i= ordered_triggers.begin(), i_end= ordered_triggers.end(); i != i_end; ++i)
+                  sql_parser->parse_trigger(i->second, i->second->sqlDefinition());
+
+              for (UnorderedTriggers::iterator i= unordered_triggers.begin(), i_end= unordered_triggers.end(); i != i_end; ++i)
+                  sql_parser->parse_trigger(*i, (*i)->sqlDefinition());
+          }
+
+          ListRef<db_View> views = schema->views();
+          for (size_t c= views.count(), i= 0; i < c; i++)
+          {
+              db_ViewRef view = views[i];
+              sql_parser->parse_view(view, view->sqlDefinition());      
+          }
+
+          ListRef<db_Routine> routines = schema->routines();
+          for (size_t c= routines.count(), i= 0; i < c; i++)
+          {
+              db_RoutineRef routine = routines[i];
+              std::string routine_sql;
+
+              routine_sql.append(strfmt("DELIMITER %s\n\n", non_std_sql_delimiter.c_str())).append(
+                  "USE `").append(routine->owner()->name()).append("`").
+                  append(non_std_sql_delimiter.c_str()).append("\n\n");
+              routine_sql.append(routine->sqlDefinition());
+              sql_parser->parse_routine(routine, routine_sql);
+          }
+
+      }
+  }
+  if (major <= 1 && (minor < 4 || (minor == 4 && revision <= 4)))
+  {
+      if (!doc->physicalModels()[0]->catalog()->version().is_valid())
+      {
+          GrtVersionRef version(doc->get_grt());
+          version->name("Version");
+          version->majorNumber(5);
+          version->minorNumber(5);
+          version->releaseNumber(-1);
+          version->buildNumber(-1);
+          doc->physicalModels()[0]->catalog()->version(version);
+      };
+  }
   // Version * -> X.X.X //!
   if (!has_file(get_rel_db_file_path()))
   {
-    bec::GRTManager *grtm= bec::GRTManager::get_instance_for(doc.get_grt());
     add_db_file(grtm, _content_dir);
 
     // finish moving table inserts sql scripts from xml to sqlite db
@@ -813,7 +943,7 @@ static int fix_duplicate_uuid_bug(xmlNodePtr node,
           std::string remapped = strfmt("%s%i", id.c_str(), (int)remapped_ids.size());
           if (remapped_ids.find(id) != remapped_ids.end()
               || remapped_ids[id].find(sname) != remapped_ids[id].end())
-            g_warning("Found more than 1 duplicate entry for same UUID, will be unable to correctly fix file (%s %s)",
+            log_warning("Found more than 1 duplicate entry for same UUID, will be unable to correctly fix file (%s %s)\n",
                       sname.c_str(), id.c_str());
           if (remapped_ids[id].size() == 0)
           {
@@ -860,11 +990,11 @@ static void fix_duplicate_uuid_bug_references(grt::GRT *grt, xmlNodePtr node,
             if (XMLTraverser::node_prop(node, "type") == "list" && XMLTraverser::node_prop(node, "content-type") == "object"
                 && !(sname = XMLTraverser::node_prop(node, "content-struct-name")).empty())
             {
-              g_message("link %s is in list of %s", id.c_str(), sname.c_str());
+              log_info("link %s is in list of %s\n", id.c_str(), sname.c_str());
             }
             else
             {
-              g_message("link %s has no struct-name", id.c_str());
+              log_info("link %s has no struct-name\n", id.c_str());
               continue;
             }
           }
@@ -906,7 +1036,7 @@ static void fix_duplicate_uuid_bug_references(grt::GRT *grt, xmlNodePtr node,
                 if (mc->name() == sname)
                 {
                   count++;
-                  g_message("found match %i %s for %s:%s (%s -> %s)", count, opt->first.c_str(), sname.c_str(), target_key.c_str(),
+                  log_info("found match %i %s for %s:%s (%s -> %s)\n", count, opt->first.c_str(), sname.c_str(), target_key.c_str(),
                             id.c_str(), opt->second.c_str());
                   xmlNodeSetContent(n, (xmlChar*)opt->second.c_str());
                   ok = true;
@@ -917,7 +1047,7 @@ static void fix_duplicate_uuid_bug_references(grt::GRT *grt, xmlNodePtr node,
             }
 
             if (!ok)
-              g_message("WARNING: Could not resolve link for %s %s (%i) key='%s'. Report to the developers to get a fix.",
+              log_warning("WARNING: Could not resolve link for %s %s (%i) key='%s'. Report to the developers to get a fix.\n",
                         id.c_str(), sname.c_str(), (int)structs.size(),
                         XMLTraverser::node_prop(n, "key").c_str());
           }
