@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,14 +21,13 @@
 
 #include <stack>
 
-#include "wb_config.h"
-
 #include "wb_overview.h"
 #include "model/wb_overview_physical.h"
 #include "grt/icon_manager.h"
 #include "grt/clipboard.h"
 #include "grt/exceptions.h"
 
+#include "model/wb_context_model.h"
 #include "workbench/wb_context.h"
 #include "workbench/wb_context_ui.h"
 #include "grt/validation_manager.h"
@@ -53,7 +52,7 @@ int OverviewBE::Node::get_popup_menu_items(WBContext *wb, bec::MenuItemList &ite
 
 bool OverviewBE::ObjectNode::activate(WBContext *wb)
 {
-  wb->open_object_editor(object);
+  wb->get_grt_manager()->open_object_editor(object, bec::NoFlags);
 
   return true;
 }
@@ -1014,6 +1013,8 @@ bec::MenuItemList OverviewBE::get_popup_items_for_nodes(const std::vector<bec::N
     {
       groups.push_back("Catalog/*");
       groups.push_back("Overview/*");
+      groups.push_back("Menu/Catalog");
+      groups.push_back("Menu/Overview");
     }
 
     // model_Model will put lots of unrelated items
@@ -1071,12 +1072,12 @@ bool OverviewBE::activate_popup_item_for_nodes(const std::string &name, const st
   else if (name == "builtin:delete")
   {
     grt::AutoUndo undo(_wb->get_grt());
-    
+
     for (std::vector<bec::NodeId>::const_iterator end= nodes.end(), iter= nodes.begin();
          iter != end; ++iter)
     {
       Node *n= get_node(*iter);
-      if (n)        
+      if (n)
         n->delete_object(_wb);
     }
     
