@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,12 +23,18 @@
 #include <sstream>
 
 #include "base/string_utilities.h"
+#include "base/threading.h"
+#include "base/log.h"
+
 #include "wmi.h"
-#include "grt/common.h"
+
+#pragma comment(lib, "comsuppw.lib")
 
 using namespace wmi;
 using namespace grt;
 
+DEFAULT_LOG_DOMAIN("wmi")
+
 //--------------------------------------------------------------------------------------------------
 
 /**
@@ -92,14 +98,20 @@ std::string wmiResultToString(HRESULT wmiResult)
     hres = status->GetErrorCodeText(wmiResult, 0, 0, &errorString);
 
     if (FAILED(hres))
+    {
+      log_error("Converting result code to string failed with error: %d\n", hres);
       result = "Internal error: WMI error description retrieval failed.";
+    }
 
     CW2A converted_text(errorString, CP_UTF8);
     result = converted_text;
     status->Release();
   }
   else
+  {
+    log_error("Could not instatiate a status code text converter. Error code: %d\n", hres);
     result = "Internal error: WMI status code text creation failed.";
+  }
 
   return result;
 }
@@ -149,28 +161,36 @@ std::string serviceResultToString(unsigned int result)
 WmiMonitor::WmiMonitor(IWbemServices* services, const std::string& parameter)
   : _services(services), _propertyHandle(0), _namePropertyHandle(0), _findTotal(false)
 {
+  log_debug("Creating new wmi monitor for parameter: %s\n", parameter.c_str());
+  
+  const std::vector<std::string> args = base::split(parameter, ".");
+
+  if (args.size() < 2)
+  {
+    log_error("Invalid parameter format - cannot continue\n");
+    throw std::runtime_error(_("Wrong monitor format. Got '") + parameter + _("', expected '<WMIClass.Name>'"));
+  }
+
   CComBSTR instancePath;
 
   HRESULT hr = CoCreateInstance(CLSID_WbemRefresher, NULL, CLSCTX_INPROC_SERVER, IID_IWbemRefresher, 
     (void**) &_refresher);
   if (FAILED(hr))
+  {
+    log_error("Could not create a wbem refresher instance. Error: %d\n", hr);
     throw std::runtime_error(_("WMI - Could not create monitor object.\n\n") + wmiResultToString(hr));
+  }
 
   CComPtr<IWbemConfigureRefresher> config;
   hr = _refresher->QueryInterface(IID_IWbemConfigureRefresher, (void**) &config);
   if (FAILED(hr))
   {
+    log_error("QueryInterface for wbem configure refersher failed with error: %d\n", hr);
+
     // _refersher is a smart pointer and is automatically freed.
     throw std::runtime_error(_("WMI - Could not create monitor object.\n\n") + wmiResultToString(hr));
   }
 
-  const std::vector<std::string> args = base::split(parameter, ".");
-  
-  if (args.size() < 2)
-  {
-	throw std::runtime_error(_("Wrong monitor format. Got '") + parameter + _("', expected '<WMIClass.Name>'"));
-  }
-
   CComBSTR path = args[0].c_str();
   _propertyName = args[1].c_str();
 
@@ -181,7 +201,9 @@ WmiMonitor::WmiMonitor(IWbemServices* services, const std::string& parameter)
   hr = config->AddEnum(_services, path, 0, NULL, &_enumerator, &_enumeratorId);
   if (FAILED(hr))
   {
-	throw std::runtime_error(_("WMI - Could not register monitoring enumerator.\n\n") + wmiResultToString(hr));
+    std::string result = wmiResultToString(hr);
+    log_error("Could not register monitoring enumerator. Error: %s\n", result.c_str());
+    throw std::runtime_error(_("WMI - Could not register monitoring enumerator.\n"));
   }
 }
 
@@ -189,20 +211,25 @@ WmiMonitor::WmiMonitor(IWbemServices* services, const std::string& parameter)
 
 WmiMonitor::~WmiMonitor()
 {
+  log_debug("Destroying monitor\n");
+  
   CComPtr<IWbemConfigureRefresher> config;
   HRESULT hr = _refresher->QueryInterface(IID_IWbemConfigureRefresher, (void**) &config);
   if (SUCCEEDED(hr))
     config->Remove(_enumeratorId, WBEM_FLAG_REFRESH_NO_AUTO_RECONNECT);
-  // Ignore query interface errors here, as we cannot do anything about them anymore anyway.
+  else
+  {
+    std::string result = wmiResultToString(hr);
+    log_error("Could not remove enumerator from wbem config refresher. Error: %s\n", result.c_str());
+  }
 }
 
 //--------------------------------------------------------------------------------------------------
 
 std::string WmiMonitor::readValue()
 {
-  // TODO: error processing is very rudimentary here.
-  //       Though raising exceptions for errors is not a good option here,
-  //       especially when considering border crossings like C++ <-> Python, C++ <-> Lua.
+  log_debug("Reading next monitoring value\n");
+
   // Refresh the enumerator so we actually get values.
   _refresher->Refresh(0L);
 
@@ -220,6 +247,9 @@ std::string WmiMonitor::readValue()
     hr = _enumerator->GetObjects(0L, returnCount, accessors,  &returnCount);
     if (FAILED(hr))
     {
+      std::string result = wmiResultToString(hr);
+      log_error("Cannot get value object from enumerator. Error: %s\n", result.c_str());
+
       delete [] accessors;
       return "0";
     }
@@ -233,10 +263,22 @@ std::string WmiMonitor::readValue()
   if (_propertyHandle == 0)
     hr = accessors[0]->GetPropertyHandle(_propertyName, &propertyType, &_propertyHandle);
 
+  if (FAILED(hr))
+  {
+    std::string result = wmiResultToString(hr);
+    log_error("Cannot get property handle from wbem accessor. Error: %s\n", result.c_str());
+  }
+
   CIMTYPE namePropType;
   if (_findTotal && _namePropertyHandle == 0)
     hr = accessors[0]->GetPropertyHandle(L"Name", &namePropType, &_namePropertyHandle);
 
+  if (FAILED(hr))
+  {
+    std::string result = wmiResultToString(hr);
+    log_error("Cannot get name property handle from wbem accessor. Error: %s\n", result.c_str());
+  }
+
   if (_propertyHandle != 0)
   {
     DWORD value = 0;
@@ -253,6 +295,12 @@ std::string WmiMonitor::readValue()
         if (StrCmpW((LPCWSTR) buffer, L"_Total") == 0)
         {
           hr = accessors[i]->ReadDWORD(_propertyHandle, &value);
+          if (FAILED(hr))
+          {
+            std::string result = wmiResultToString(hr);
+            log_error("Cannot read DWORD value from wbem accessor. Error: %s\n", result.c_str());
+          }
+
           break;
         }
       }
@@ -262,6 +310,12 @@ std::string WmiMonitor::readValue()
       // No need to search for a specific value. Simply return the first (and mostly only) property value
       // we got.
       hr = accessors[0]->ReadDWORD(_propertyHandle, &value);
+
+      if (FAILED(hr))
+      {
+        std::string result = wmiResultToString(hr);
+        log_error("Cannot read DWORD value from wbem accessor. Error: %s\n", result.c_str());
+      }
     }
 
     std::stringstream ss;
@@ -278,12 +332,17 @@ std::string WmiMonitor::readValue()
 
 //----------------- WmiServices --------------------------------------------------------------------
 
-static GStaticMutex _locatorMutex_ = G_STATIC_MUTEX_INIT;
-static IWbemLocator* _locator_ = NULL;
-static int _locator_refcount_ = 0;
+static GStaticMutex _locator_mutex = G_STATIC_MUTEX_INIT;
+static IWbemLocator* _locator = NULL;
+static int _locator_refcount = 0;
 
 WmiServices::WmiServices(const std::string& server, const std::string& user, const std::string& password)
 {
+  if (server.empty())
+    log_debug("Creating WmiServices for local server (user: %s)\n", user.c_str());
+  else
+    log_debug("Creating WmiServices for remote server: %s (user: %s)\n", server.c_str(), user.c_str());
+
   allocate_locator();
 
   // Connect to the given target node using the provided credentials.
@@ -297,11 +356,11 @@ WmiServices::WmiServices(const std::string& server, const std::string& user, con
     CComBSTR nodeString = string2Bstr(unc);
     CComBSTR userString = string2Bstr(user);
     CComBSTR passwordString = string2Bstr(password);
-    hr = _locator_->ConnectServer(nodeString, userString, passwordString, NULL,
+    hr = _locator->ConnectServer(nodeString, userString, passwordString, NULL,
       WBEM_FLAG_CONNECT_USE_MAX_WAIT, NULL, NULL, &_services);
   }
   else
-    hr = _locator_->ConnectServer(L"root\\cimv2", NULL, NULL, NULL, WBEM_FLAG_CONNECT_USE_MAX_WAIT,
+    hr = _locator->ConnectServer(L"root\\cimv2", NULL, NULL, NULL, WBEM_FLAG_CONNECT_USE_MAX_WAIT,
       NULL, NULL, &_services);
 
   if (SUCCEEDED(hr))
@@ -319,6 +378,7 @@ WmiServices::WmiServices(const std::string& server, const std::string& user, con
     cID.Flags          = SEC_WINNT_AUTH_IDENTITY_UNICODE;
 */
     // Set a blanket to our service proxy to establish a security context.
+    // Values as recommended (http://msdn.microsoft.com/en-us/library/windows/desktop/aa393620%28v=vs.85%29.aspx).
     hr = CoSetProxyBlanket(
       _services,                   // Indicates the proxy to set
       RPC_C_AUTHN_DEFAULT,         // Authentication service
@@ -326,24 +386,30 @@ WmiServices::WmiServices(const std::string& server, const std::string& user, con
       COLE_DEFAULT_PRINCIPAL,      // Server principal name
       RPC_C_AUTHN_LEVEL_DEFAULT,   // Authentication level
       RPC_C_IMP_LEVEL_IMPERSONATE, // Impersonation level
-      NULL,                        // client identity
-      //&cID,
-      EOAC_NONE                    // proxy capabilities
+      COLE_DEFAULT_AUTHINFO,       // client identity
+      EOAC_DEFAULT                 // proxy capabilities
     );
 
     if (FAILED(hr))
     {
-      throw std::runtime_error(_("WMI setting security blanket failed.\n\n") + wmiResultToString(hr));
+      std::string result = wmiResultToString(hr);
+      log_error("Could not set proxy blanket for our wmi services. Error: %s\n", result.c_str());
+      throw std::runtime_error(_("WMI setting security blanket failed.\n"));
     }
   }
   else
-    throw std::runtime_error(_("Could not connect to target machine.\n\n") + wmiResultToString(hr));
+  {
+    std::string result = wmiResultToString(hr);
+    log_error("Could not connect to target machine. Error: %s\n", result.c_str());
+    throw std::runtime_error(_("Could not connect to target machine.\n"));
+  }
 }
 
 //--------------------------------------------------------------------------------------------------
 
 WmiServices::~WmiServices()
 {
+  log_debug("Destroying services\n");
   deallocate_locator();
 }
 
@@ -351,31 +417,39 @@ WmiServices::~WmiServices()
 
 void WmiServices::allocate_locator()
 {
-  bec::GStaticMutexLock lock(_locatorMutex_);
-  if (_locator_refcount_ == 0)
+  log_debug("Allocating wbem locator\n");
+
+  base::GStaticMutexLock lock(_locator_mutex);
+  if (_locator_refcount == 0)
   {
     HRESULT hr = CoCreateInstance(CLSID_WbemLocator, NULL, CLSCTX_INPROC_SERVER, IID_IWbemLocator,
-      reinterpret_cast<void**>(&_locator_));
+      reinterpret_cast<void**>(&_locator));
 
     if (FAILED(hr))
-      throw std::runtime_error("Internal error: Instantiation of IWbemLocator failed.\n\n" + wmiResultToString(hr));
+    {
+      std::string result = wmiResultToString(hr);
+      log_error("Could not create wbem locator. Error: %s\n", result.c_str());
+      throw std::runtime_error("Internal error: Instantiation of IWbemLocator failed.\n");
+    }
   }
 
-  _locator_refcount_++;
+  _locator_refcount++;
 }
 
 //--------------------------------------------------------------------------------------------------
 
 void WmiServices::deallocate_locator()
 {
-  bec::GStaticMutexLock lock(_locatorMutex_);
-  if (_locator_refcount_ > 0)
+  log_debug("Deallocating wbem locator\n");
+
+  base::GStaticMutexLock lock(_locator_mutex);
+  if (_locator_refcount > 0)
   {
-    _locator_refcount_--;
-    if (_locator_refcount_ == 0)
+    _locator_refcount--;
+    if (_locator_refcount == 0)
     {
-      _locator_->Release();
-      _locator_ = NULL;
+      _locator->Release();
+      _locator = NULL;
     }
   }
 }
@@ -395,10 +469,12 @@ void WmiServices::deallocate_locator()
  */
 grt::DictListRef WmiServices::query(grt::GRT* grt, const std::string& query)
 {
+  log_debug3("Running wmi query: %s\n", query.c_str());
+
   // Making this function explicitly thread-safe might be unnecessary as we don't have
   // any data which is allocated/deallocated concurrently. But since we know we will be called
   // from different threads we play safe here, as it does not harm either.
-  bec::GStaticMutexLock lock(_locatorMutex_);
+  base::GStaticMutexLock lock(_locator_mutex);
 
   grt::DictListRef queryResult(grt);
 
@@ -451,6 +527,14 @@ grt::DictListRef WmiServices::query(grt::GRT* grt, const std::string& query)
 
           VariantClear(&value);
         }
+        else
+        {
+          char *name_ = _com_util::ConvertBSTRToString(name);
+
+          std::string result = wmiResultToString(hr);
+          log_error("Couldn't get the value for %s. Error: %s\n", name_, result.c_str());
+          delete [] name_;
+        }
       }
       SafeArrayDestroy(names);
 
@@ -459,7 +543,9 @@ grt::DictListRef WmiServices::query(grt::GRT* grt, const std::string& query)
   }
   else
   {
-    throw std::runtime_error("WMI query execution failed.\n\n" + wmiResultToString(hr));
+    std::string result = wmiResultToString(hr);
+    log_error("Query execution failed. Error: %s\n", result.c_str());
+    throw std::runtime_error("WMI query execution failed");
   }
 
   return queryResult;
@@ -482,19 +568,29 @@ grt::DictListRef WmiServices::query(grt::GRT* grt, const std::string& query)
  */
 std::string WmiServices::serviceControl(const std::string& service, const std::string& action)
 {
-  bec::GStaticMutexLock lock(_locatorMutex_);
+  log_debug3("Running wmi service control query for service: %s (action: %s)\n", service.c_str(), action.c_str());
+
+  base::GStaticMutexLock lock(_locator_mutex);
 
   CComBSTR instancePath = string2Bstr("Win32_Service.Name='" + service + "'");
 
   CComPtr<IWbemClassObject> serviceInstance;
   HRESULT hr = _services->GetObject(instancePath, 0, NULL, &serviceInstance, NULL);
   if (FAILED(hr))
-    return "error: " + wmiResultToString(hr);
+  {
+    std::string result = wmiResultToString(hr);
+    log_error("Query execution failed. Error: %s\n", result.c_str());
+    return "error, see log";
+  }
 
   VARIANT value;
   hr = serviceInstance->Get(L"State", 0, &value, NULL, NULL);
   if (FAILED(hr))
+  {
+    std::string result = wmiResultToString(hr);
+    log_error("Could not get state value for the service. Error: %s\n", result.c_str());
     return "unknown";
+  }
 
   std::string state = base::tolower(variant2string(value));
   VariantClear(&value);
@@ -547,18 +643,47 @@ std::string WmiServices::serviceControl(const std::string& service, const std::s
       CComPtr<IWbemClassObject> inInstance;
       CComPtr<IWbemClassObject> outClass;
 
-      if (SUCCEEDED(hr))
-        hr = _services->GetObject(serviceClassPath, 0, NULL, &serviceClass, NULL);
+      hr = _services->GetObject(serviceClassPath, 0, NULL, &serviceClass, NULL);
       if (SUCCEEDED(hr))
         hr = serviceClass->GetMethod(methodName, 0, &inClass, &outClass);
+      else
+      {
+        char *serviceClassPath_ = _com_util::ConvertBSTRToString(serviceClassPath);
+        std::string result = wmiResultToString(hr);
+        log_error("Could not get object for service class path: %s. Error: %s\n", serviceClassPath_, result.c_str());
+        delete [] serviceClassPath_;
+      }
+
       if (SUCCEEDED(hr) && inClass)
         hr = inClass->SpawnInstance(0, &inInstance);
+      else
+      {
+        char *methodName_ = _com_util::ConvertBSTRToString(methodName);
+        std::string result = wmiResultToString(hr);
+        log_error("Could not get in/out class for method: %s. Error: %s\n", methodName_, result.c_str());
+        delete [] methodName_;
+      }
 
       if (SUCCEEDED(hr))
         hr = _services->ExecMethod(instancePath, methodName, 0, NULL, inInstance, &outInstance, NULL);
+      else
+      {
+        std::string result = wmiResultToString(hr);
+        log_error("Could not spawn in-class instance. Error: %s\n", result.c_str());
+      }
 
       if (FAILED(hr))
-        return "error: " + wmiResultToString(hr);
+      {
+        char *serviceClassPath_ = _com_util::ConvertBSTRToString(serviceClassPath);
+        char *methodName_ = _com_util::ConvertBSTRToString(methodName);
+        std::string result = wmiResultToString(hr);
+        log_error("Could not execute method %d at path %s. Error: %s\n", methodName_,
+          serviceClassPath_, result.c_str());
+        delete [] methodName_;
+        delete [] serviceClassPath_;
+
+        return "error, see log";
+      }
 
       // Try to figure out what happened if the call as such succeeded but returned an error code.
       // Service methods return uint32 error codes.
@@ -566,14 +691,24 @@ std::string WmiServices::serviceControl(const std::string& service, const std::s
       hr = outInstance->Get(L"ReturnValue", 0, &returnValue, NULL, NULL);
 
       if (FAILED(hr))
-        return "error: " + wmiResultToString(hr);
+      {
+        std::string result = wmiResultToString(hr);
+        log_error("Could not get the return value of the query. Error: %s\n", result.c_str());
+
+        return "error, see log";
+      }
 
       if (returnValue.vt != VT_EMPTY)
       {
         unsigned int result = V_UI4(&returnValue);
         VariantClear(&returnValue);
         if (result != 0)
-          return "error: " + serviceResultToString(result);
+        {
+          std::string text =  serviceResultToString(result);
+          log_error("Variant conversion for return value failed. Error: %s\n", text.c_str());
+          
+          return "error, see log";
+        }
       }
 
       // So far everything worked fine. Now check the service if it changed as we intended.
@@ -585,14 +720,15 @@ std::string WmiServices::serviceControl(const std::string& service, const std::s
       {
         Sleep(sleepTime);
 
-        HRESULT hr = _services->GetObject(instancePath, 0, NULL, &serviceInstance, NULL);
-        if (FAILED(hr))
-          return "error: " + wmiResultToString(hr);
-
         VARIANT value;
         hr = serviceInstance->Get(L"State", 0, &value, NULL, NULL);
         if (FAILED(hr))
-          return "error: " + wmiResultToString(hr);
+        {
+          std::string result = wmiResultToString(hr);
+          log_error("Could not get state value for the service. Error: %s\n", result.c_str());
+
+          return "unknown";
+        }
 
         state = base::tolower(variant2string(value));
         VariantClear(&value);
@@ -610,7 +746,10 @@ std::string WmiServices::serviceControl(const std::string& service, const std::s
       } while (true);
 
       if (state != expectedState)
+      {
+        log_error("Timeout waiting for the service to change status. Returning error to caller.\n");
         return "error";
+      }
 
       return "completed";
     }
@@ -629,7 +768,9 @@ std::string WmiServices::serviceControl(const std::string& service, const std::s
  */
 std::string WmiServices::systemStat(const std::string& what)
 {
-  bec::GStaticMutexLock lock(_locatorMutex_);
+  log_debug3("Running wmi system stat call (what: %s)\n", what.c_str());
+
+  base::GStaticMutexLock lock(_locator_mutex);
 
   CComBSTR instancePath = string2Bstr("Win32_OperatingSystem");
 
@@ -656,8 +797,22 @@ std::string WmiServices::systemStat(const std::string& what)
     {
       result = base::tolower(variant2string(value));
       VariantClear(&value);
-    };
-  };
+    }
+    else
+    {
+      char *propertyName_ = _com_util::ConvertBSTRToString(propertyName);
+
+      std::string result = wmiResultToString(hr);
+      log_error("Could not get the value for property %s. Error: %s\n", propertyName_, result.c_str());
+      delete [] propertyName_;
+    }
+  }
+  else
+  {
+    std::string result = wmiResultToString(hr);
+    log_error("Could not run the system stat query. Error: %s\n", result.c_str());
+  }
+
 
   return result;
 }
