@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -45,6 +45,7 @@ ShellBE::ShellBE(GRTManager *grtm, GRTDispatcher *dispatcher)
   _shell= 0;
 
   _save_history_size= 0;
+  _skip_history = 0;
 
   g_static_mutex_init(&_text_queue_mutex);
 
@@ -185,6 +186,7 @@ bool ShellBE::setup(const std::string &lang)
 
 void ShellBE::start()
 {
+  _skip_history = 1;
   // trigger a command that will in turn make the GUI receive a ready
   // callback with the prompt that should be shown
   process_line_async("print(\"Ready.\\n\")");
@@ -194,6 +196,11 @@ void ShellBE::start()
 void ShellBE::save_history_line(const std::string &line)
 {
   if (line.empty()) return;
+  if (_skip_history > 0)
+  {
+    --_skip_history;
+    return;
+  }
   
   //remove empty cmd if it is on top of the stack
   if (!_history.empty() && (*_history.begin()).empty())
