@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -259,6 +259,7 @@ void PluginManagerImpl::rescan_plugins()
       
       if (!check_plugin_validity(plugin, *pm))
       {
+        plugin->reset_references();
         continue;
       }
       _plugin_source_module[plugin->name()]= (*pm)->name();
@@ -271,6 +272,7 @@ void PluginManagerImpl::rescan_plugins()
       {
         if (grt->verbose())
           grt->send_info("Plugin "+*plugin->name()+" is disabled, skipping...", "");
+        plugin->reset_references();
         continue;
       }
 
@@ -804,6 +806,9 @@ std::string PluginManagerImpl::open_gui_plugin(const app_PluginRef &plugin, cons
   }
   else // a normal plugin implemented by a GRT module
   {
+    // Opening a normal plugin is usually done in the context of the grt thread and we want to
+    // continue that way. But if we are currently in the main thread switch here to the grt thread
+    // for opening the plugin.
     if (_grtm->in_main_thread())
     {
       _grtm->get_dispatcher()->execute_simple_function("Open normal plugin",
