@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,6 +33,7 @@
 #include "grt/common.h"
 #include "base/ui_form.h" // for menu stuff
 #include "base/trackable.h"
+#include "base/threading.h"
 
 #include "wbpublic_public_interface.h"
 #include <boost/shared_ptr.hpp>
@@ -84,7 +85,7 @@ namespace bec
       ~Pool()
       {
         {
-          GMutexLock lock(_sync);
+          base::GMutexLock lock(_sync);
           std::for_each(_pool.begin(), _pool.end(), Pool::delete_object);
         }
         
@@ -96,7 +97,7 @@ namespace bec
         T* item = 0;
         try
         {
-          GMutexLock lock(_sync);
+          base::GMutexLock lock(_sync);
           if ( _pool.size() > 0 )
           { 
             item = _pool.back();
@@ -118,7 +119,7 @@ namespace bec
     
       void put(T* item)
       {
-         GMutexLock lock(_sync);
+        base::GMutexLock lock(_sync);
         _pool.push_back(item);
       }
   
