@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,21 +16,29 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
+
 #include "stdafx.h"
 
 #include "editor_dbobject.h"
 #include "grtsqlparser/sql_facade.h"
 #include "grt/validation_manager.h"
-#include <sstream>
+#include "grtpp_notifications.h"
 
 #include "base/string_utilities.h"
+#include "base/notifications.h"
 
 #include "mforms/utilities.h"
+#include "mforms/code_editor.h"
+
+#include "sqlide/sql_editor_be.h"
+#include "db_object_helpers.h"
 
 #define DEFAULT_COLLATION_CAPTION "default collation"
 
 using namespace bec;
 
+//--------------------------------------------------------------------------------------------------
+
 DBObjectEditorBE::DBObjectEditorBE(GRTManager *grtm, 
                                    const db_DatabaseObjectRef &object,
                                    const db_mgmt_RdbmsRef &rdbms)
@@ -39,17 +47,7 @@ DBObjectEditorBE::DBObjectEditorBE(GRTManager *grtm,
   _ignored_object_fields_for_ui_refresh.insert("lastChangeDate");
 
   if (!_rdbms.is_valid())
-  {
-    _rdbms= db_mgmt_RdbmsRef::cast_from(object->customData().get("liveRdbms"));
-    std::istringstream iss(object->customData().get_string("contextDbSqlEditor"));
-    void *ptr= NULL;
-    iss >> ptr;
-    if (ptr)
-    {
-      UIForm *ui_form= reinterpret_cast <UIForm *> (ptr);
-      ui_form->notify_of_ui_form_creation(this);
-    }
-  }
+    _rdbms= db_mgmt_RdbmsRef::cast_from(object->customData().get("liveRdbms"));    
 
   if (_rdbms.is_valid())
   {
@@ -63,58 +61,81 @@ DBObjectEditorBE::DBObjectEditorBE(GRTManager *grtm,
   }
   
   _val_notify_conn = ValidationManager::signal_notify()->connect(boost::bind(&DBObjectEditorBE::notify_from_validation, this, _1, _2, _3, _4));
+  
+  
+  grt::DictRef info(grtm->get_grt());
+  info.gset("form", form_id());
+  info.set("object", object);
+  // must be delayed, because observer will probably need the form to be finished constructing
+  grt::GRTNotificationCenter::get()->send_grt("GRNDBObjectEditorCreated", grt::ObjectRef(), info);    
 }
 
+//--------------------------------------------------------------------------------------------------
 
 bool DBObjectEditorBE::is_editing_live_object()
 {
   return get_dbobject()->customData().get("liveRdbms").is_valid();
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void DBObjectEditorBE::apply_changes_to_live_object()
 {
+  BaseEditor::apply_changes_to_live_object();
+
   on_apply_changes_to_live_object(this, false);
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void DBObjectEditorBE::refresh_live_object()
 {
+  BaseEditor::refresh_live_object();
+
   on_refresh_live_object(this);
 }
 
+//--------------------------------------------------------------------------------------------------
 
 bool DBObjectEditorBE::can_close()
 {
-  bool res= BaseEditor::can_close();
+  // Editing in a model always allows to close the editor. Save checks are done when the model
+  // is closed.
+  if (!is_editing_live_object())
+    return true;
 
-  if (res && is_editing_live_object() && !on_apply_changes_to_live_object.empty())
+  bool res = BaseEditor::can_close();
+
+  // Note: the result of the BaseEditor::can_close() is only used if there's no apply callback set.
+  //       Otherwise we always use the callback for checks (because there can be other changes than
+  //       just the code editor which is checked in the BaseEditor).
+  if (!on_apply_changes_to_live_object.empty())
   {
-    bool is_object_modified= on_apply_changes_to_live_object(this, true);
+    bool is_object_modified = on_apply_changes_to_live_object(this, true);
     if (is_object_modified)
     {
       int user_choice= mforms::Utilities::show_warning(
-        _("Object was changed"),
-        _("Do you want to apply changes made to this object?"),
-        _("Apply"),
+        base::strfmt(_("Object %s was changed"), get_name().c_str()),
+        base::strfmt(_("Do you want to save changes made to %s?"), get_name().c_str()),
+        _("Save"),
         _("Cancel"),
-        _("Don't Apply")
+        _("Don't Save")
         );
 
       if (mforms::ResultOk == user_choice)
-      {
-        on_apply_changes_to_live_object(this, false);
-      }
+        res = on_apply_changes_to_live_object(this, false);
       else if (mforms::ResultCancel == user_choice)
-      {
-        res= false;
-      }
+        res = false;
+      else
+        res = true;
     }
+    else
+      res = true;
   }
-
   return res;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 bool DBObjectEditorBE::should_close_on_delete_of(const std::string &oid)
 { 
@@ -128,24 +149,65 @@ bool DBObjectEditorBE::should_close_on_delete_of(const std::string &oid)
   return false;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void DBObjectEditorBE::update_change_date()
 {
   get_object().set_member("lastChangeDate", grt::StringRef(fmttime(0, DATETIME_FMT)));
 }
 
+//--------------------------------------------------------------------------------------------------
 
 std::string DBObjectEditorBE::get_object_type()
 {
   return get_object().get_metaclass()->get_attribute("caption");
 }
 
+//--------------------------------------------------------------------------------------------------
+
+GrtVersionRef DBObjectEditorBE::get_rdbms_target_version()
+{
+  if (is_editing_live_object())
+    return get_catalog()->version();
+  else
+  {
+    if (get_catalog()->version().is_valid())
+      return get_catalog()->version();
+    
+    std::string target_version = get_grt_manager()->get_app_option_string("DefaultTargetMySQLVersion");
+    if (target_version.empty())
+      target_version = "5.5";
+    
+    GrtVersionRef version = CatalogHelper::parse_version(get_grt_manager()->get_grt(), target_version);
+
+    return version;
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
+bool DBObjectEditorBE::is_server_version_at_least(int major, int minor)
+{
+  GrtVersionRef version = get_rdbms_target_version();
+
+  if (version.is_valid())
+  {
+    if (*version->majorNumber() > major ||
+        (*version->majorNumber() == major && *version->minorNumber() >= minor))
+      return true;
+    return false;
+  }
+  return true;
+}
+
+//--------------------------------------------------------------------------------------------------
 
 std::string DBObjectEditorBE::get_name()
 {
   return get_dbobject()->name();
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void DBObjectEditorBE::set_name(const std::string &name)
 {
@@ -163,12 +225,14 @@ void DBObjectEditorBE::set_name(const std::string &name)
   }
 }
 
+//--------------------------------------------------------------------------------------------------
 
 std::string DBObjectEditorBE::get_comment()
 {
   return get_dbobject()->comment();
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void DBObjectEditorBE::set_comment(const std::string &descr)
 {
@@ -185,7 +249,7 @@ void DBObjectEditorBE::set_comment(const std::string &descr)
   }
 }
 
-
+//--------------------------------------------------------------------------------------------------
 
 std::string DBObjectEditorBE::get_sql()
 {
@@ -193,12 +257,14 @@ std::string DBObjectEditorBE::get_sql()
   return "";// get_dbobject().sql();
 }
 
+//--------------------------------------------------------------------------------------------------
 
 bool DBObjectEditorBE::is_sql_commented()
 {
   return (*get_dbobject()->commentedOut() != 0);
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void DBObjectEditorBE::set_sql_commented(bool flag)
 {
@@ -213,51 +279,7 @@ void DBObjectEditorBE::set_sql_commented(bool flag)
   undo.end(_("Comment Out SQL"));
 }
 
-
-void DBObjectEditorBE::set_custom_data(const std::string &name, const std::string &value)
-{
-  if (get_dbobject()->customData().get_string(name) != value)
-  {
-    //grt::AutoUndo undo(get_grt(), new UndoObjectChangeGroup(get_dbobject().id(), "custom:"+name));
-    AutoUndoEdit undo(this, get_dbobject(), "custom:"+name);
-
-    get_dbobject()->customData().set(name, grt::StringRef(value));
-    update_change_date();
-
-    undo.end(_("Set Custom Data ")+name);
-  }
-}
-
-
-void DBObjectEditorBE::delete_custom_data(const std::string &name)
-{
-  //grt::AutoUndo undo(get_grt());
-  AutoUndoEdit undo(this);
-
-  get_dbobject()->customData().remove(name);
-  update_change_date();
-
-  undo.end(_("Unset Custom Data ")+name);
-}
-
-
-size_t DBObjectEditorBE::count_custom_data()
-{
-  return get_dbobject()->customData().count();
-}
-
-
-void DBObjectEditorBE::get_custom_data(size_t index, std::string &name, std::string &value)
-{
-  /*QQQ
-  grt::ValueRef v;
-
-  get_dbobject()->customData().get_by_index(index, name, v);
-
-  value= *grt::StringRef::cast_from(v);
-   */
-}
-
+//--------------------------------------------------------------------------------------------------
 
 db_CatalogRef DBObjectEditorBE::get_catalog()
 {
@@ -270,6 +292,7 @@ db_CatalogRef DBObjectEditorBE::get_catalog()
   return db_CatalogRef::cast_from(object);
 }
 
+//--------------------------------------------------------------------------------------------------
 
 db_SchemaRef DBObjectEditorBE::get_schema()
 {
@@ -282,11 +305,14 @@ db_SchemaRef DBObjectEditorBE::get_schema()
   return db_SchemaRef::cast_from(object);
 }
 
+//--------------------------------------------------------------------------------------------------
+
 std::string DBObjectEditorBE::get_schema_name()
 {
   return get_schema()->name();
 }
 
+//--------------------------------------------------------------------------------------------------
 
 db_SchemaRef DBObjectEditorBE::get_schema_with_name(const std::string &schema_name)
 {
@@ -295,6 +321,7 @@ db_SchemaRef DBObjectEditorBE::get_schema_with_name(const std::string &schema_na
   return grt::find_named_object_in_list(catalog->schemata(), schema_name);
 }
 
+//--------------------------------------------------------------------------------------------------
 
 std::vector<std::string> DBObjectEditorBE::get_all_schema_names()
 {
@@ -312,6 +339,31 @@ std::vector<std::string> DBObjectEditorBE::get_all_schema_names()
   return names;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Function: custom_string_compare
+ * Details: performs identifier comparison removing the quoting
+ *          This function will be used when sorting the dbobjects
+ */
+bool DBObjectEditorBE::custom_string_compare(const std::string &first, const std::string &second)
+{
+  // This will remove the quoting if present on all the identifier items
+  std::vector<std::string> first_lst = base::split_qualified_identifier(first);
+  std::vector<std::string> second_lst = base::split_qualified_identifier(second);
+  
+  std::string first_str = first_lst[0];
+  for(size_t index = 1; index < first_lst.size();index++)
+    first_str = first_str + "." + first_lst[index];
+    
+  std::string second_str = second_lst[0];
+  for(size_t index = 1; index < second_lst.size();index++)
+    second_str = second_str + "." + second_lst[index];
+  
+  return base::stl_string_compare(first_str, second_str, false);
+}
+
+//--------------------------------------------------------------------------------------------------
 
 std::vector<std::string> DBObjectEditorBE::get_all_table_names()
 {
@@ -337,11 +389,12 @@ std::vector<std::string> DBObjectEditorBE::get_all_table_names()
       table_list.push_back("`" + schema_name + "`.`" + *schema->tables()[i]->name() + "`");
   }
 
-  std::sort(table_list.begin(), table_list.end());
+  std::sort(table_list.begin(), table_list.end(), boost::bind(&DBObjectEditorBE::custom_string_compare, this, _1, _2));
   table_list.push_back("Specify Manually...");
   return table_list;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 std::vector<std::string> DBObjectEditorBE::get_schema_table_names()
 {
@@ -355,10 +408,11 @@ std::vector<std::string> DBObjectEditorBE::get_schema_table_names()
       table_list.push_back("`" + schema_name + "`.`" + *schema->tables()[i]->name() + "`");
   }
 
-  std::sort(table_list.begin(), table_list.end());
+  std::sort(table_list.begin(), table_list.end(), boost::bind(&DBObjectEditorBE::custom_string_compare, this, _1, _2));
   return table_list;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 std::vector<std::string> DBObjectEditorBE::get_table_column_names(const std::string &fq_table_name)
 {
@@ -376,7 +430,7 @@ std::vector<std::string> DBObjectEditorBE::get_table_column_names(const std::str
     table_name= parts[0];
     schema= get_schema();
   }
-  else
+  else if (!parts.empty())
   {
     schema= get_schema_with_name(parts[0]);
     table_name= parts[1];
@@ -396,6 +450,7 @@ std::vector<std::string> DBObjectEditorBE::get_table_column_names(const std::str
   return columns;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 std::vector<std::string> DBObjectEditorBE::get_table_column_names(const db_TableRef &table)
 {
@@ -408,6 +463,8 @@ std::vector<std::string> DBObjectEditorBE::get_table_column_names(const db_Table
   return columns;
 }
 
+//--------------------------------------------------------------------------------------------------
+
 std::vector<std::string> DBObjectEditorBE::get_charset_collation_list()
 {
   std::vector<std::string> collation_list;
@@ -430,6 +487,7 @@ std::vector<std::string> DBObjectEditorBE::get_charset_collation_list()
   return collation_list;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 std::string DBObjectEditorBE::format_charset_collation(const std::string &charset, const std::string &collation)
 {
@@ -444,6 +502,7 @@ std::string DBObjectEditorBE::format_charset_collation(const std::string &charse
     return charset + " - " + collation;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 bool DBObjectEditorBE::parse_charset_collation(const std::string &str, std::string &charset, std::string &collation)
 {
@@ -462,20 +521,33 @@ bool DBObjectEditorBE::parse_charset_collation(const std::string &str, std::stri
   return false;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 Sql_editor::Ref DBObjectEditorBE::get_sql_editor()
 {
   if (!_sql_editor)
   {
-    _sql_editor= Sql_editor::create(get_rdbms());
+    _sql_editor = Sql_editor::create(get_rdbms(), GrtVersionRef());
+    _sql_editor->set_sql_check_enabled(false); // Disable automatic parsing. We have our own here.
     grt::DictRef obj_options= get_dbobject()->customData();
     if (obj_options.has_key("sqlMode"))
       _sql_editor->sql_mode(obj_options.get_string("sqlMode"));
+    _sql_editor->set_sql_check_enabled(true);
   }
   return _sql_editor;
 }
 
+//--------------------------------------------------------------------------------------------------
 
+void bec::DBObjectEditorBE::reset_editor_undo_stack()
+{
+  get_sql_editor()->get_editor_control()->reset_dirty();
+}
+
+//--------------------------------------------------------------------------------------------------
+
+// XXX: this function should go.
+//      It doesn't actually set any sql but parses the text for errors (which is normally done by the code editor).
 void DBObjectEditorBE::set_sql(const std::string &sql, bool sync, const db_DatabaseObjectRef &template_obj, const std::string &comment)
 {
   _sql_parser_log.clear();
@@ -497,6 +569,7 @@ void DBObjectEditorBE::set_sql(const std::string &sql, bool sync, const db_Datab
     dispatcher->add_task(task);
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void DBObjectEditorBE::sql_parser_msg_cb(const grt::Message &msg)
 {
@@ -504,6 +577,7 @@ void DBObjectEditorBE::sql_parser_msg_cb(const grt::Message &msg)
     _sql_parser_log.push_back(msg.format());
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void DBObjectEditorBE::notify_from_validation(const grt::Validator::Tag& tag, const grt::ObjectRef& obj, const std::string& msg, const int level)
 {
@@ -546,45 +620,60 @@ void DBObjectEditorBE::notify_from_validation(const grt::Validator::Tag& tag, co
   }
 }
 
+//--------------------------------------------------------------------------------------------------
+
 void DBObjectEditorBE::sql_parser_task_finished_cb(grt::ValueRef value)
 {
   if (_sql_parser_log_cb)
     _sql_parser_log_cb(_sql_parser_log);
   _sql_parser_log.clear();
-  do_ui_refresh();
+
+  if (_ignore_object_changes_for_ui_refresh == 0)
+  {
+    if (_grtm->in_main_thread())
+      do_ui_refresh();
+    else
+      _ui_refresh_conn = _grtm->run_once_when_idle(boost::bind(&RefreshUI::do_ui_refresh, this));
+  }
+  else
+    _ignored_object_changes_for_ui_refresh++;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void DBObjectEditorBE::set_sql_parser_task_cb(const Sql_parser_task_cb &cb)
 {
   _sql_parser_task_cb= cb;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void DBObjectEditorBE::set_sql_parser_log_cb(const Sql_parser_log_cb &cb)
 {
   _sql_parser_log_cb= cb;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void DBObjectEditorBE::set_sql_parser_err_cb(const Sql_parser_err_cb &cb)
 {
   _sql_parser_err_cb = cb;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void DBObjectEditorBE::check_sql()
 {
   Sql_editor::Ref sql_editor= get_sql_editor();
   if (sql_editor)
   {
-    sql_editor->is_sql_check_enabled(true);
     // provoke database object to refresh FE control contents
     db_DatabaseObjectRef db_object= get_dbobject();
     (*db_object->signal_changed())("", grt::ValueRef());
   }
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void DBObjectEditorBE::sql_mode(const std::string &value)
 {
@@ -592,3 +681,5 @@ void DBObjectEditorBE::sql_mode(const std::string &value)
   if (sql_editor)
     sql_editor->sql_mode(value);
 }
+
+//--------------------------------------------------------------------------------------------------
