@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,16 +19,10 @@
 #ifndef _EDITOR_DBOBJECT_H_
 #define _EDITOR_DBOBJECT_H_
 
-
 #include "wbpublic_public_interface.h"
 #include "grt/editor_base.h"
 
-#include "grt/tree_model.h"
 #include "grtsqlparser/invalid_sql_parser.h"
-#include "grts/structs.db.h"
-#include "grts/structs.db.mgmt.h"
-
-#include <memory>
 
 namespace bec {
 
@@ -52,14 +46,8 @@ namespace bec {
     virtual bool is_sql_commented();
     virtual void set_sql_commented(bool flag);
 
-    virtual void set_custom_data(const std::string &name, const std::string &value);
-    virtual size_t count_custom_data();
-    virtual void get_custom_data(size_t index, std::string &name, std::string &value);
-    virtual void delete_custom_data(const std::string &name);
-
     virtual Sql_editor::Ref get_sql_editor();
-
-  public: // helper functions
+    virtual void reset_editor_undo_stack();
 
     db_SchemaRef get_schema();
     virtual std::string get_schema_name();
@@ -80,8 +68,6 @@ namespace bec {
     std::string format_charset_collation(const std::string &charset, const std::string &collation);
 
     void update_change_date();
-
-  public:
     void sql_mode(const std::string &value);
 
   protected:
@@ -123,6 +109,8 @@ namespace bec {
     virtual bool is_editing_live_object();
     virtual void apply_changes_to_live_object();
     virtual void refresh_live_object();
+    bool is_server_version_at_least(int major, int minor);
+
     boost::function<bool (DBObjectEditorBE*, bool)> on_apply_changes_to_live_object;
     boost::function<void (DBObjectEditorBE*)> on_refresh_live_object;
     boost::function<void (DBObjectEditorBE*)> on_create_live_table_stubs;
@@ -137,6 +125,8 @@ namespace bec {
     // Real-time validation part
     grt::MessageType    _last_validation_check_status;
     std::string         _last_validation_message;
+    
+    bool custom_string_compare(const std::string &first, const std::string &second);
   };
 };
 
