@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,14 +25,14 @@
 
 #include "db_object_helpers.h"
 #include "grtpp_util.h"
+#include "grt/clipboard.h"
 #include "grt/validation_manager.h"
 
 #include "base/string_utilities.h"
 #include "base/file_utilities.h"
+#include "objimpl/db.query/db_query_EditableResultset.h"
 
-#ifdef __APPLE__
 #include "base/log.h"
-#endif
 
 #include "mforms/utilities.h"
 
@@ -40,7 +40,7 @@ using namespace bec;
 using namespace grt;
 using namespace base;
 
-DEFAULT_LOG_DOMAIN("Table Editor backend")
+DEFAULT_LOG_DOMAIN("TableEditorBE")
 
 /*
  The big mess of user datatypes, synonyms and column flags.
@@ -98,11 +98,17 @@ std::vector<std::string> TableColumnsListBE::get_datatype_names()
   types.push_back("DATETIME");       
   types.push_back("BLOB");
   
+  GrtVersionRef target_version(_owner->get_rdbms_target_version());
   std::vector<db_SimpleDatatypeRef> sorted_types;
   grt::ListRef<db_SimpleDatatype> stypes(_owner->get_catalog()->simpleDatatypes());
   for (grt::ListRef<db_SimpleDatatype>::const_iterator iter= stypes.begin(); iter != stypes.end(); ++iter)
+  {
+    if (target_version.is_valid() && !bec::CatalogHelper::is_type_valid_for_version(*iter, target_version))
+      continue;
+    
     sorted_types.push_back(*iter);
-  
+  }
+
   // sort by group and name
   std::sort(sorted_types.begin(), sorted_types.end(), sort_simple_type);
   
@@ -115,13 +121,19 @@ std::vector<std::string> TableColumnsListBE::get_datatype_names()
       group= (*iter)->group();
       types.push_back("-");
     }
+
+    std::string tmp;
+
     if ((*iter)->parameterFormatType() == 1 ||
         (*iter)->parameterFormatType() == 3 ||
         (*iter)->parameterFormatType() == 4 ||
         (*iter)->parameterFormatType() == 10)
-      types.push_back(*(*iter)->name()+"()");
+      tmp = *(*iter)->name()+"()";
     else
-      types.push_back((*iter)->name());
+      tmp = (*iter)->name();
+
+    if (types.empty() || types.back() != tmp)
+      types.push_back(tmp);
   }
   
   // insert UDTs
@@ -251,11 +263,16 @@ void TableColumnsListBE::reorder(const NodeId &node, int nindex)
     update_primary_index_order();
   
     _owner->update_change_date();
-    
+
+    _owner->freeze_refresh_on_object_change();
     (*_owner->get_table()->signal_refreshDisplay())("column");
+    _owner->thaw_refresh_on_object_change(true);
     undo.end(strfmt(_("Reorder Column '%s.%s'"), _owner->get_name().c_str(), _owner->get_table()->columns()[node[0]]->name().c_str()));
 
-    _owner->do_partial_ui_refresh(::bec::TableEditorBE::RefreshColumnList);
+    if (nindex < node[0])
+      _owner->do_partial_ui_refresh(::bec::TableEditorBE::RefreshColumnMoveUp);
+    else
+      _owner->do_partial_ui_refresh(::bec::TableEditorBE::RefreshColumnMoveDown);
   }
 }
 
@@ -744,6 +761,7 @@ bool TableColumnsListBE::set_field(const NodeId &node, int column, int value)
     return true;
   case IsNotNull:
     {
+      FreezeRefresh frz(_owner);
       AutoUndoEdit undo(_owner);
       col->isNotNull(value != 0);
 
@@ -759,13 +777,25 @@ bool TableColumnsListBE::set_field(const NodeId &node, int column, int value)
     }
     return true;
   case IsBinary:
-    return set_column_flag(node, "BINARY", value!=0);
+    {
+      FreezeRefresh frz(_owner);
+      return set_column_flag(node, "BINARY", value!=0);
+    }
   case IsUnsigned:
-    return set_column_flag(node, "UNSIGNED", value!=0);
+    {
+      FreezeRefresh frz(_owner);
+      return set_column_flag(node, "UNSIGNED", value!=0);
+    }
   case IsZerofill:
-    return set_column_flag(node, "ZEROFILL", value!=0);
+    {
+      FreezeRefresh frz(_owner);
+      return set_column_flag(node, "ZEROFILL", value!=0);
+    }
   case IsUnique:
-    return make_unique_index(col, value!=0);
+    {
+      FreezeRefresh frz(_owner);
+      return make_unique_index(col, value!=0);
+    }
   case Flags:
   case Default:
     return false;
@@ -871,7 +901,10 @@ bool TableColumnsListBE::get_field_grt(const NodeId &node, int column, grt::Valu
 
     }
     else
-      value= grt::StringRef("");
+    {
+      value = grt::StringRef("");
+      return false;
+    }
     return true;
   }
   return false;
@@ -1001,7 +1034,7 @@ int TableColumnsListBE::get_column_flag(const ::bec::NodeId &node, const std::st
       for(size_t i= 0; i < flags_count; i++)
       {
         grt::StringRef existing_flag= col_flags.get(i);
-        if(g_strcasecmp(flag_name.c_str(), existing_flag.c_str()) == 0)
+        if(g_ascii_strcasecmp(flag_name.c_str(), existing_flag.c_str()) == 0)
           return 1;
       }
     }else if (col->userType().is_valid())
@@ -1028,10 +1061,12 @@ std::string TableColumnsListBE::quote_value_if_needed(const db_ColumnRef &column
   // Non-text types (like datetime) shouldn't be auto-quoted.
   // Using the "text" and "string" groups here includes more than we'd need (e.g. TEXT blobs), but as long
   // as this function is only used for default values (which blobs cannot have) we are fine.
-  bool isChar = (g_strcasecmp(datatypeGroupName.c_str(), "string") == 0) || 
-                (g_strcasecmp(datatypeGroupName.c_str(), "text") == 0);
+  bool isChar = (g_ascii_strcasecmp(datatypeGroupName.c_str(), "string") == 0) || 
+                (g_ascii_strcasecmp(datatypeGroupName.c_str(), "text") == 0) ||
+                (g_ascii_strcasecmp(datatypeName.c_str(), "ENUM") == 0) ||
+                (g_ascii_strcasecmp(datatypeName.c_str(), "SET") == 0);
 
-  if (isChar && (g_strcasecmp(value.c_str(), "NULL") != 0) && (value != "0") && (value[0] != '\''))
+  if (isChar && (value != "NULL") && (value != "0") && (value[0] != '\''))
     return std::string("'").append(escape_sql_string(value)).append("'");
   return value;
 }
@@ -1056,6 +1091,7 @@ MenuItemList TableColumnsListBE::get_popup_items_for_nodes(const std::vector<Nod
   sep.type = MenuSeparator;
 
   MenuItem item;
+
   item.caption = "Move Up";
   item.name    = "moveUpToolStripMenuItem";
   item.enabled = nodes.size() == 1;
@@ -1066,8 +1102,36 @@ MenuItemList TableColumnsListBE::get_popup_items_for_nodes(const std::vector<Nod
   item.enabled = nodes.size() == 1;
   items.push_back(item);
   items.push_back(sep);
+  
+  item.caption = "Copy";
+  item.name    = "copyColumnToolStripMenuItem";
+  item.enabled = nodes.size() > 0;
+  items.push_back(item);
+  
+  item.caption = "Cut";
+  item.name    = "cutColumnToolStripMenuItem";
+  item.enabled = nodes.size() > 0;
+  items.push_back(item);
+  
+  item.caption = "Paste";
+  item.name    = "pasteColumnToolStripMenuItem";
+  item.enabled = false;
+  bec::Clipboard *clip = _owner->get_grt_manager()->get_clipboard();
+  if (clip->is_data())
+  {
+    std::list<grt::ObjectRef> data = clip->get_data();
+    bool ok = true;
+    for (std::list<grt::ObjectRef>::iterator i = data.begin(); i != data.end(); ++i)
+    {
+      if (!db_ColumnRef::can_wrap(*i))
+        ok = false;
+    }
+    item.enabled = ok;
+  }
+  
+  items.push_back(item);
 
-  item.caption = "Delete Selected Columns";
+  item.caption = "Delete Selected";
   item.name    = "deleteSelectedColumnsToolStripMenuItem";
   item.enabled = nodes.size() > 0;
   items.push_back(item);
@@ -1114,6 +1178,62 @@ bool TableColumnsListBE::activate_popup_item_for_nodes(const std::string &name,
       reorder(nodes.front(), row+1);
     }    
   }
+  else if (name == "copyColumnToolStripMenuItem")
+  {
+    bec::Clipboard *clip = _owner->get_grt_manager()->get_clipboard();
+    clip->clear();
+    for (std::vector<NodeId>::const_iterator iter= nodes.begin();
+         iter != nodes.end(); ++iter)
+    {
+      if ((*iter)[0] < real_count())
+      {
+        db_ColumnRef col(_owner->get_table()->columns().get((*iter)[0]));
+        clip->append_data(col);
+      }
+    }
+    clip->set_content_description("Table Column");
+  }
+  else if (name == "cutColumnToolStripMenuItem")
+  {
+    bec::Clipboard *clip = _owner->get_grt_manager()->get_clipboard();
+    clip->clear();
+    for (std::vector<NodeId>::const_iterator iter= nodes.begin();
+         iter != nodes.end(); ++iter)
+    {
+      if ((*iter)[0] < real_count())
+      {
+        db_ColumnRef col(_owner->get_table()->columns().get((*iter)[0]));
+        clip->append_data(col);
+      }
+    }
+    clip->set_content_description("Table Column");
+    for (std::vector<NodeId>::reverse_iterator iter= nodes.rbegin();
+         iter != nodes.rend(); ++iter)
+      delete_node(*iter);
+  }
+  else if (name == "pasteColumnToolStripMenuItem")
+  {
+    bec::Clipboard *clip = _owner->get_grt_manager()->get_clipboard();
+    if (clip->is_data())
+    {
+      AutoUndoEdit undo(_owner);
+      std::list<grt::ObjectRef> data = clip->get_data();
+      int insert_after = nodes.empty() ? -1 : nodes[0][0];
+
+      if (insert_after >= real_count())
+        insert_after = -1;
+
+      for (std::list<grt::ObjectRef>::const_iterator i = data.begin(); i != data.end(); ++i)
+      {
+        db_ColumnRef column(db_ColumnRef::cast_from(*i));
+        _owner->duplicate_column(column, insert_after < 0 ? -1 : insert_after++);
+      }
+ 
+      refresh();
+      _owner->update_change_date();
+      undo.end(base::strfmt("Paste columns to table %s", _owner->get_name().c_str()));
+    }
+  }
   else if (name == "deleteSelectedColumnsToolStripMenuItem")
   {
     for (std::vector<NodeId>::reverse_iterator iter= nodes.rbegin();
@@ -1312,11 +1432,13 @@ bool IndexColumnsListBE::set_field(const NodeId &node, int column, int value)
   if (node[0] >= count())
     return false;
 
-  if (!_owner->index_editable(_owner->get_selected_index())
-      && column != OrderIndex) // index order can be changed for PKs
-        return false;
-
   icolumn= get_index_column(_owner->get_owner()->get_table()->columns()[node[0]]);
+  if (!_owner->index_editable(_owner->get_selected_index())
+      && column != OrderIndex && !(icolumn->referencedColumn()->simpleType().is_valid() &&  column == Length && icolumn->referencedColumn()->simpleType()->group()->name() == "string")) // index order can be changed for PKs
+        return false;                                                                                                           // index length can be changed for PK if it's text
+
+
+
 
   switch (column)
   {
@@ -1489,7 +1611,7 @@ bool IndexListBE::index_editable(const db_IndexRef &index)
 }
 
 
-bool IndexListBE::index_belongs_to_fk(const db_IndexRef &index)
+db_ForeignKeyRef IndexListBE::index_belongs_to_fk(const db_IndexRef &index)
 {
   // check if the index belongs to a FK, if so it cannot be edited
   if (index.is_valid())
@@ -1497,21 +1619,24 @@ bool IndexListBE::index_belongs_to_fk(const db_IndexRef &index)
     ListRef<db_ForeignKey> fks(db_TableRef::cast_from(index->owner())->foreignKeys());
     for (size_t c= fks.count(), i= 0; i < c; i++)
       if (fks[i]->index() == index)
-        return true;
+        return fks[i];
   }
-  return false;
+  return db_ForeignKeyRef();
 }
 
 
 MenuItemList IndexListBE::get_popup_items_for_nodes(const std::vector<NodeId> &nodes)
 {
-  db_IndexRef index(get_selected_index());
+  db_IndexRef index;
+
+  if (!nodes.empty() && nodes[0][0] < (int)_owner->get_table()->indices().count())
+    index = _owner->get_table()->indices()[nodes[0][0]];
   
   MenuItemList items;
   MenuItem item;
-  item.caption = _("Delete Selected Indices");
+  item.caption = _("Delete Selected");
   item.name    = "deleteIndices";
-  item.enabled = nodes.size() > 0 && !index_belongs_to_fk(index) && index_editable(index);
+  item.enabled = index.is_valid() && nodes.size() > 0 && /*!index_belongs_to_fk(index) && */index_editable(index);
   items.push_back(item);
 
   return items;
@@ -1525,7 +1650,21 @@ bool IndexListBE::activate_popup_item_for_nodes(const std::string &name, const s
   if (name == "deleteIndices")
   {
     for (std::vector<NodeId>::reverse_iterator node= nodes.rbegin(); node != nodes.rend(); ++node)
+    {
+      if ((*node)[0] < (int)_owner->get_table()->indices().count())
+      {
+        db_IndexRef index(_owner->get_table()->indices().get((*node)[0]));
+        db_ForeignKeyRef fk;
+        if (index.is_valid() && (fk = index_belongs_to_fk(index)).is_valid())
+        {
+          mforms::Utilities::show_message("Cannot Delete Index",
+                                          base::strfmt("The index '%s' belongs to the Foreign Key '%s'.\nYou must delete the Foreign Key to delete the index.", index->name().c_str(), fk->name().c_str()),
+                                          "OK", "", "");
+          continue;
+        }
+      }
       _owner->remove_index(*node);
+    }
   }
   else
     return false;
@@ -1558,7 +1697,7 @@ NodeId IndexListBE::add_column(const db_ColumnRef &column, const db_IndexRef &aI
   if (!index.is_valid())
     return NodeId();
 
-  if (!index_editable(index) || index_belongs_to_fk(index))
+  if (!index_editable(index) || index_belongs_to_fk(index).is_valid())
     return NodeId();
 
   std::string column_struct= index.get_metaclass()->get_member_type("columns").content.object_class;
@@ -1586,7 +1725,7 @@ void IndexListBE::remove_column(const NodeId &node)
   RefreshUI::Blocker __centry(*_owner);
 
   db_IndexRef index= get_selected_index();
-  if (!index_editable(index) || index_belongs_to_fk(index))
+  if (!index_editable(index) || index_belongs_to_fk(index).is_valid())
     return;
 
   db_ColumnRef column(_owner->get_table()->columns().get(node[0]));
@@ -1650,7 +1789,7 @@ bool IndexListBE::set_field(const NodeId &node, int column, const std::string &v
 
   index= _owner->get_table()->indices().get(node[0]);
 
-  if (!index_editable(index))
+  if (!index_editable(index) && column != Comment) // Allow editing comments on PKs.
     return false;
 
   switch (column)
@@ -1797,7 +1936,7 @@ static std::set<std::string> get_indexed_column_ids(const db_TableRef &table)
  * @param node of the source column in the FK column list.
  * @return List of possible column names.
  */
-std::vector<std::string> FKConstraintColumnsListBE::get_ref_columns_list(const NodeId &node)
+std::vector<std::string> FKConstraintColumnsListBE::get_ref_columns_list(const NodeId &node, bool filtered)
 {
   db_ForeignKeyRef fk(_owner->get_selected_fk());
   db_ColumnRef srccolumn(_owner->get_owner()->get_table()->columns()[node[0]]);
@@ -1810,8 +1949,10 @@ std::vector<std::string> FKConstraintColumnsListBE::get_ref_columns_list(const N
     std::set<std::string> indexed_column_ids(get_indexed_column_ids(table));
 
     for (size_t c= columns.count(), i= 0; i < c; i++)
-    {      
-      if (_owner->get_owner()->check_column_referenceable_by_fk(columns[i], srccolumn))
+    {     
+      if (!filtered)
+        names.push_back(columns[i]->name());
+      else if (_owner->get_owner()->check_column_referenceable_by_fk(columns[i], srccolumn))
       {
         // column must have index to be in a Fk
         if (table->isPrimaryKeyColumn(columns[i]))
@@ -1822,6 +1963,9 @@ std::vector<std::string> FKConstraintColumnsListBE::get_ref_columns_list(const N
     }
     names.insert(names.end(), names2.begin(), names2.end());
 
+    if (fk->referencedTable()->isStub())
+      names.push_back("Specify Column...");
+
     return names;
   }
 
@@ -1879,6 +2023,8 @@ bool FKConstraintColumnsListBE::set_fk_column_pair(const db_ColumnRef &column, c
   
   if (changed_real_fk)
   {
+    TableHelper::update_foreign_key_index(fk);
+
     _owner->get_owner()->update_change_date();
     undo.end(strfmt(_("Set Ref. Column for FK '%s.%s'"), 
                     _owner->get_owner()->get_name().c_str(), fk->name().c_str()));
@@ -2004,16 +2150,37 @@ bool FKConstraintColumnsListBE::set_field(const NodeId &node, int column, const
   case RefColumn:
     if (fk.is_valid() && fk->referencedTable().is_valid())
     {
-      int index;
-      
-      // check if column is acceptable
-      std::vector<std::string> allowed_list = get_ref_columns_list(node);
-      if (std::find(allowed_list.begin(), allowed_list.end(), value) == allowed_list.end())
-        return false;
+      std::auto_ptr<AutoUndoEdit> undo;
 
       tcolumn= grt::find_named_object_in_list(fk->referencedTable()->columns(), value);
 
-      index= get_fk_column_index(node);
+      // special handling for stub tables
+      if (fk->referencedTable()->isStub() && !tcolumn.is_valid() && !_owner->get_owner()->is_editing_live_object())
+      {
+        std::string column_name;
+        // ask for a column name and add it to the stub table
+        if (mforms::Utilities::request_input(_("Specify Referenced Column"), _("Referenced Column Name:"),
+                                             "", column_name))
+        {
+          tcolumn = grt::find_named_object_in_list(fk->referencedTable()->columns(), column_name);
+          if (!tcolumn.is_valid())
+          {
+            undo = std::auto_ptr<AutoUndoEdit>(new AutoUndoEdit(_owner->get_owner()));
+
+            // create the column
+            tcolumn = grt::copy_object(_owner->get_owner()->get_table()->columns()[node[0]]);
+            fk->referencedTable()->columns().insert(tcolumn);
+            tcolumn->owner(fk->referencedTable());
+            tcolumn->name(column_name);
+
+            fk->referencedTable()->addPrimaryKeyColumn(tcolumn);
+          }
+        }
+        else
+          return false;
+      }
+
+      int index= get_fk_column_index(node);
 
       // if columns is not enabled yet, enable it
       if (index < 0)
@@ -2024,7 +2191,44 @@ bool FKConstraintColumnsListBE::set_field(const NodeId &node, int column, const
           return false;
       }
 
+      // check if column is acceptable
+      std::vector<std::string> allowed_list = get_ref_columns_list(node);
+      if (std::find(allowed_list.begin(), allowed_list.end(), value) == allowed_list.end())
+      {
+        std::set<std::string> indexed_column_ids(get_indexed_column_ids(fk->referencedTable()));
+
+        if (indexed_column_ids.find(tcolumn->id()) == indexed_column_ids.end())
+        {
+          mforms::Utilities::show_message("Create Foreign Key",
+                              base::strfmt("Selected column %s must be indexed and be of a compatible type for a Foreign Key to be created.", tcolumn->name().c_str()),
+                              "OK");
+        }
+        else
+        {
+          db_TableRef reftable = fk->referencedTable();
+          std::string hint;
+          hint = base::strfmt("\nHint: source column has type %s%s,\nColumn %s of referenced table is %s%s.",
+                              _owner->get_owner()->get_table()->columns()[node[0]]->flags().get_index("UNSIGNED") != grt::BaseListRef::npos ? "UNSIGNED " : "",
+                              _owner->get_owner()->get_table()->columns()[node[0]]->formattedType().c_str(),
+                              value.c_str(),
+                              tcolumn->flags().get_index("UNSIGNED") != grt::BaseListRef::npos ? "UNSIGNED " : "",
+                              tcolumn->formattedType().c_str());
+          // show a warning
+          mforms::Utilities::show_message("Create Foreign Key",
+                                          base::strfmt("The selected columns do not have compatible types.\n%s",
+                                                       hint.c_str()),
+                                          "OK");
+        }
+        return false;
+      }
+
       set_fk_column_pair(fk->columns()[index], tcolumn);
+
+      if (undo.get())
+      {
+        undo->end(base::strfmt("Set Column of Foreign Key %s.%s.%s", _owner->get_owner()->get_schema()->name().c_str(),
+                               _owner->get_owner()->get_name().c_str(), fk->name().c_str()));
+      }
     }
     return true;
   }
@@ -2035,7 +2239,6 @@ bool FKConstraintColumnsListBE::set_field(const NodeId &node, int column, const
 bool FKConstraintColumnsListBE::set_field(const NodeId &node, int column, int value)
 {
   db_ForeignKeyRef fk(_owner->get_selected_fk());
-  db_ColumnRef tcolumn;
 
   switch (column)
   {
@@ -2051,7 +2254,33 @@ bool FKConstraintColumnsListBE::set_field(const NodeId &node, int column, int va
                  strfmt(_("Remove Column from FK '%s.%s'"), _owner->get_owner()->get_name().c_str(), fk->name().c_str()));
       }
       else
+      {
         undo.cancel();
+
+        db_ColumnRef tcolumn;
+        if (node[0] < (int)_owner->get_owner()->get_table()->columns().count())
+          tcolumn = fk->owner()->columns()[node[0]];
+        db_TableRef reftable = fk->referencedTable();
+
+        if (reftable.is_valid() && tcolumn.is_valid() && get_ref_columns_list(node).empty())
+        {
+          std::string hint;
+          if (reftable->primaryKey().is_valid() && reftable->primaryKey()->columns().count() > 0)
+            hint = base::strfmt("\nHint: source column has type %s%s,\nPK of referenced table is %s%s.",
+                                tcolumn->flags().get_index("UNSIGNED") != grt::BaseListRef::npos ? "UNSIGNED " : "",
+                                tcolumn->formattedType().c_str(),
+                                reftable->primaryKey()->columns()[0]->referencedColumn()->flags().get_index("UNSIGNED") != grt::BaseListRef::npos ? "UNSIGNED " : "",
+                                reftable->primaryKey()->columns()[0]->referencedColumn()->formattedType().c_str());
+
+          // show a warning
+          mforms::Utilities::show_message("Create Foreign Key",
+                                          base::strfmt("Referenced table has no candidate columns with a compatible type for %s.%s.%s",
+                                                       tcolumn->owner()->name().c_str(),
+                                                       tcolumn->name().c_str(),
+                                                       hint.c_str()),
+                                          "OK");
+        }
+      }
       return true;
     }
     return false;
@@ -2132,7 +2361,9 @@ NodeId FKConstraintListBE::add_column(const db_ColumnRef &column, const db_Colum
     fk->columns().insert(column);
     fk->referencedColumns().insert(refcolumn);
 
-    TableHelper::update_foreign_key_index(fk.get_grt(), fk); //!
+    // make sure that there's an index that matches the columns in the FK or
+    // that the one created for it is up to date
+    TableHelper::update_foreign_key_index(fk);
 
     _owner->update_change_date();
 
@@ -2173,7 +2404,7 @@ void FKConstraintListBE::remove_column(const NodeId &node)
   {
     fk->referencedColumns().remove(i);
   }
-  TableHelper::update_foreign_key_index(fk.get_grt(), fk);//!
+  TableHelper::update_foreign_key_index(fk);
 
   _owner->update_change_date();
   undo.end(
@@ -2357,15 +2588,26 @@ bool FKConstraintListBE::set_field(const NodeId &node, int column, const std::st
           schema= _owner->get_schema_with_name(parts[0]);
         }
 
-        if (!schema.is_valid())
-          return false;
+        db_TableRef dbtable;
+        if (schema.is_valid())
+        {
+          dbtable= grt::find_named_object_in_list(schema->tables(), table, "name");
+          if (!dbtable.is_valid())
+            return false;
+        }
 
-        db_TableRef dbtable= grt::find_named_object_in_list(schema->tables(), table, "name");
-
-        if (dbtable.is_valid() && dbtable != fk->referencedTable())
+        if (!dbtable.is_valid() || dbtable != fk->referencedTable())
         {
           AutoUndoEdit undo(_owner);
 
+          {
+            std::string schema_name = schema.is_valid() ? *schema->name() : parts[0];
+            log_info("Creating stub schema and table %s.%s for foreign key\n", schema_name.c_str(), table.c_str());
+            dbtable = _owner->create_stub_table(schema_name, table);
+            if (!dbtable.is_valid())
+              return false;
+          }
+
           fk->referencedTable(dbtable);
           
           // reset the currently set columns
@@ -2374,7 +2616,7 @@ bool FKConstraintListBE::set_field(const NodeId &node, int column, const std::st
           
           get_columns()->refresh();
 
-          TableHelper::update_foreign_key_index(fk.get_grt(), fk); //!
+          TableHelper::update_foreign_key_index(fk);
 
           _owner->update_change_date();
           bec::ValidationManager::validate_instance(_owner->get_table(), "chk_fk_lgc");
@@ -2508,7 +2750,7 @@ MenuItemList FKConstraintListBE::get_popup_items_for_nodes(const std::vector<Nod
   MenuItemList items;
 
   MenuItem item;
-  item.caption = _("Delete selected FKs");
+  item.caption = _("Delete selected");
   item.name    = "deleteSelectedFKs";
   item.enabled = (nodes.size() >= 1);
   items.push_back(item);
@@ -2599,6 +2841,32 @@ NodeId TableEditorBE::add_column(const std::string &name)
 }
 
 
+NodeId TableEditorBE::duplicate_column(const db_ColumnRef &column, int insert_after)
+{
+  db_ColumnRef new_column = shallow_copy_object(column);
+
+  std::string name = new_column->name();
+  std::string new_name = name;
+  for (int i = 1;; i++)
+  {
+    if (!find_named_object_in_list(get_table()->columns(), new_name).is_valid())
+      break;
+    new_name = strfmt("%s_copy%i", name.c_str(), i);
+  }
+  if (new_name != *new_column->name())
+    new_column->name(new_name);
+  new_column->owner(get_table());
+  get_table()->addColumn(new_column);
+  if (insert_after >= 0)
+    get_table()->columns()->reorder(get_table()->columns()->get_index(new_column), insert_after);
+
+  bec::ValidationManager::validate_instance(new_column, CHECK_NAME);
+  bec::ValidationManager::validate_instance(_table, "columns-count");
+  
+  return NodeId(get_table()->columns().count()-1);
+}
+
+
 db_ColumnRef TableEditorBE::get_column_with_name(const std::string &name)
 {
   return grt::find_named_object_in_list<db_Column>(get_table()->columns(), name, "name");
@@ -2640,6 +2908,12 @@ void TableEditorBE::remove_column(const NodeId &node)
 
 NodeId TableEditorBE::add_fk(const std::string &name)
 {
+
+   if(!get_table()->columns().count())
+   {
+      mforms::Utilities::show_warning("FK Creation", "Cannot add FK on empty table, add some columns first", "OK");
+      return NodeId();
+   }
   ListRef<db_ForeignKey> fklist= get_table()->foreignKeys();
   db_ForeignKeyRef fk;
 
@@ -2694,6 +2968,12 @@ bool TableEditorBE::remove_fk(const NodeId &fk)
 
 NodeId TableEditorBE::add_index(const std::string &name)
 {
+   if(!get_table()->columns().count())
+   {
+      mforms::Utilities::show_warning("Index Creation", "Cannot add Index on empty table, add some columns first", "OK");
+      return NodeId();
+   }
+
   ListRef<db_Index> indices= get_table()->indices();
   db_IndexRef index;
 
@@ -2733,7 +3013,7 @@ bool TableEditorBE::remove_index(const NodeId &index)
 
   // do not allow removal of PRIMARY or FOREIGN indices by user
   if (!get_indexes()->index_editable(indexobj) ||
-      get_indexes()->index_belongs_to_fk(indexobj))
+      get_indexes()->index_belongs_to_fk(indexobj).is_valid())
     return false;
 
   AutoUndoEdit undo(this);
@@ -2831,6 +3111,7 @@ std::string TableEditorBE::format_column_type(db_ColumnRef &column)
 }
 
 
+// TODO: probably obsolete, as we no longer set sql for multiple triggers at once.
 void TableEditorBE::set_triggers_sql(const std::string &sql, bool sync)
 {
   set_sql_parser_task_cb(boost::bind(&TableEditorBE::parse_triggers_sql, this, _1, _2));
@@ -2840,12 +3121,12 @@ void TableEditorBE::set_triggers_sql(const std::string &sql, bool sync)
 
 grt::ValueRef TableEditorBE::parse_triggers_sql(grt::GRT* grt, grt::StringRef sql)
 {
-  //grt::AutoUndo undo(grt);
   AutoUndoEdit undo(this);
 
   if (!_sql_parser.get())
     throw std::logic_error("SQL parser is not initialzed");
 
+  _table->customData().set("NonTriggerSQLFound", grt::IntegerRef(0));
   grt::IntegerRef res= _sql_parser->parse_triggers(_table, sql.c_str());
 
   undo.end(strfmt(_("Edit triggers of table `%s`.`%s`"), get_schema_name().c_str(), get_name().c_str()));
@@ -2870,8 +3151,13 @@ Recordset::Ref TableEditorBE::get_inserts_model()
     _inserts_model= Recordset::create(_grtm);
     _inserts_model->show_apply_buttons(true);
     _inserts_model->data_storage(_inserts_storage);
-    _inserts_model->export_wizard = boost::bind(&TableEditorBE::show_export_wizard, this, (mforms::Form*)0);
     _inserts_model->refresh();
+    
+    _inserts_model->action_list().register_action("record_export",
+                                                  boost::bind(&TableEditorBE::show_export_wizard, this, (mforms::Form*)0));
+    _inserts_model->action_list().register_action("record_import",
+                                                  boost::bind(&TableEditorBE::show_import_wizard, this));
+
   }
   return _inserts_model;
 }
@@ -2908,6 +3194,38 @@ void TableEditorBE::show_export_wizard(mforms::Form *owner)
 
 //--------------------------------------------------------------------------------------------------
 
+void TableEditorBE::show_import_wizard()
+{
+  grt::BaseListRef args(_grtm->get_grt());
+  
+  db_TableRef table(get_table());
+  if (table.is_valid())
+  {
+    args.ginsert(grtwrap_editablerecordset(table, _inserts_model));
+
+    grt::Module *module = _grtm->get_grt()->get_module("SQLIDEUtils");
+    if (module)
+    {
+      try
+      {
+        module->call_function("importRecordsetDataFromFile", args);
+      }
+      catch (grt::module_error &exc)
+      {
+        mforms::Utilities::show_error("Import Standard INSERTs", "Error during data import:\n" + exc.inner, "OK");
+      }
+      catch (std::exception &exc)
+      {
+        mforms::Utilities::show_error("Import Standard INSERTs", "Error during data import:\n" + std::string(exc.what()), "OK");
+      }
+    }
+    else
+      log_error("Can't find module SQLIDEUtils for record importer");
+  }
+}
+                                                  
+//--------------------------------------------------------------------------------------------------
+
 Sql_editor::Ref TableEditorBE::get_sql_editor()
 {
   Sql_editor::Ref sql_editor= DBObjectEditorBE::get_sql_editor();
@@ -2922,3 +3240,9 @@ Sql_editor::Ref TableEditorBE::get_sql_editor()
   }
   return sql_editor;
 }
+
+std::string TableEditorBE::get_title()
+{
+  return base::strfmt("%s - Table", get_name().c_str()); 
+}
+
