@@ -7,8 +7,10 @@
 class ConnectionPage : public grtui::WizardPage
 {
 public:
-  ConnectionPage(grtui::WizardForm *form, const char *name= "connect")
-    : WizardPage(form, name), _dbconn(0)
+  ConnectionPage(grtui::WizardForm *form, const char *name= "connect", const std::string &selection_save_name = "")
+  : WizardPage(form, name), _dbconn(0),
+  _connect(grtui::DbConnectPanelDefaults|(selection_save_name.empty() ? 0 : grtui::DbConnectPanelDontSetDefaultConnection)),
+  _selection_save_name(selection_save_name)
   {
     set_title(_("Set Parameters for Connecting to a DBMS"));
     set_short_title(_("Connection Options"));
@@ -30,12 +32,26 @@ protected:
   {
     if (!_dbconn)
       throw std::logic_error("must call set_db_connection() 1st");
-    
+
+    if (!_selection_save_name.empty())
+    {
+      std::string name = wizard()->grtm()->get_app_option_string(_selection_save_name);
+      if (!name.empty())
+        _connect.set_active_stored_conn(name);
+    }
+
     return true;
   }
 
   virtual bool advance()
-  { 
+  {
+    if (!_selection_save_name.empty())
+    {
+      db_mgmt_ConnectionRef conn(_connect.get_connection());
+      if (conn.is_valid() && conn->name() != "")
+        wizard()->grtm()->set_app_option(_selection_save_name, conn->name());
+    }
+
     return WizardPage::advance();
   }
 
@@ -50,6 +66,7 @@ protected:
 protected:
   DbConnection *_dbconn;
   grtui::DbConnectPanel _connect;
+  std::string _selection_save_name;
 };
 
 #endif // _CONNECTION_PAGE_H_
