@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,11 +19,9 @@
 
 #include "stdafx.h"
 
-#include "wb_config.h"
-
 #include "base/geometry.h"
 
-using namespace MySQL::Geometry;
+using namespace base;
 
 #include "grtdb/db_helpers.h"
 #include "db_conn_be.h"
@@ -31,6 +29,7 @@ using namespace MySQL::Geometry;
 #include "grt/grt_manager.h"
 #include "grtdb/charset_utils.h"
 #include "base/string_utilities.h"
+#include "base/log.h"
 
 #include <boost/shared_ptr.hpp>
 #include <boost/scoped_array.hpp>
@@ -41,6 +40,7 @@ using namespace MySQL::Geometry;
 
 #undef max
 
+DEFAULT_LOG_DOMAIN("DbConnectPanel");
 
 static const std::string control_name_prefix= "ctrl__";
 
@@ -73,6 +73,10 @@ DbDriverParam::ParamType DbDriverParam::decode_param_type(std::string type_name)
     result= ptPassword;
   else if (0 == type_name.compare("keychain"))
     result= ptKeychainPassword;
+  else if (0 == type_name.compare("enum"))
+    result= ptEnum;
+  else if (0 == type_name.compare("text"))
+    result= ptText;
   else
     g_warning("Unkown DB driver parameter type '%s'", type_name.c_str());
 
@@ -86,15 +90,23 @@ _type(ptUnknown)
 {
   _type= decode_param_type(_inner->paramType());
 
-  if (_type == ptKeychainPassword)
-    set_value(driver_param->name());
-  else if (stored_conn.is_valid())
+  if (stored_conn.is_valid() && !(*stored_conn->name()).empty())
     set_value(stored_conn->parameterValues().get(driver_param->name(), driver_param->defaultValue()));
   else
     set_value(driver_param->defaultValue());
 }
 
 
+DbDriverParam::DbDriverParam(const db_mgmt_DriverParameterRef &driver_param, const grt::ValueRef &value)
+:
+_inner(driver_param),
+_type(ptUnknown)
+{
+  _type= decode_param_type(_inner->paramType());
+  set_value(value);
+}
+
+
 ControlType DbDriverParam::get_control_type() const
 {
   switch (get_type())
@@ -108,6 +120,10 @@ ControlType DbDriverParam::get_control_type() const
       return ctFileSelector;
     case DbDriverParam::ptKeychainPassword:
       return ctKeychainPassword;
+    case DbDriverParam::ptEnum:
+      return ctEnumSelector;
+    case DbDriverParam::ptText:
+      return ctText;
     case DbDriverParam::ptInt:
     case DbDriverParam::ptString:
     case DbDriverParam::ptPassword:
@@ -125,6 +141,7 @@ void DbDriverParam::set_value(const grt::ValueRef &value)
   case ptPassword:
   case ptDir:
   case ptFile:
+  case ptText:
   case ptKeychainPassword: // this only keeps the storage key format
     {
       _value= grt::StringRef::cast_from(value);
@@ -146,11 +163,17 @@ void DbDriverParam::set_value(const grt::ValueRef &value)
           _value= grt::IntegerRef(n);
         }
         else
-          _value= grt::IntegerRef();
+          _value= grt::ValueRef();
       }
 
       break;
     }
+      
+  case ptEnum:
+    {
+      _value= grt::StringRef::cast_from(value);
+      break;
+    }
 
   case ptUnknown:
   default:
@@ -161,6 +184,39 @@ void DbDriverParam::set_value(const grt::ValueRef &value)
 }
 
 
+std::vector<std::pair<std::string, std::string> > DbDriverParam::get_enum_options()
+{
+  std::vector<std::pair<std::string, std::string> > options;
+  grt::Module *module = _inner.get_grt()->get_module(*_inner->lookupValueModule());
+  if (module)
+  {
+    grt::BaseListRef args(_inner.get_grt());
+    grt::ValueRef result = module->call_function(*_inner->lookupValueMethod(), args);
+    if (result.is_valid() && grt::StringListRef::can_wrap(result))
+    {
+      grt::StringListRef list = grt::StringListRef::cast_from(result);
+      for (int i= 0; i < (int)list.count(); i++)
+      {
+        std::string s = list[i];
+        std::string::size_type pos;
+        if ((pos = s.find('|')) != std::string::npos)
+          options.push_back(std::make_pair(s.substr(0, pos), s.substr(pos+1)));
+        else
+          options.push_back(std::make_pair(s, s));
+      }
+    }
+    else
+      log_warning("Error calling enum value lookup method %s.%s for DriverParameter %s",
+                  _inner->lookupValueModule().c_str(), _inner->lookupValueMethod().c_str(),
+                  _inner->name().c_str());
+  }
+  else
+    log_warning("Error searching module for enum value lookup method %s.%s for DriverParameter %s",
+                _inner->lookupValueModule().c_str(), _inner->lookupValueMethod().c_str(),
+                _inner->name().c_str());  
+  return options;
+}
+
 //----------------------------------------------------------------------
 
 
@@ -244,8 +300,17 @@ void DbDriverParams::init(
   int hmargin,
   int vmargin)
 {
-  begin_layout();
+  if (begin_layout)
+    begin_layout();
 
+  // check what options are in the connections file that are not known, so we can lump them and show in the other options textbox
+  std::vector<std::string> unknown_options;
+  if (stored_conn.is_valid())
+  {
+    grt::DictRef vals(stored_conn->parameterValues());
+    for (grt::DictRef::const_iterator i = vals.begin(); i != vals.end(); ++i)
+      unknown_options.push_back(i->first);
+  }
   free_dyn_mem();
 
   if (suspend_layout)
@@ -259,22 +324,53 @@ void DbDriverParams::init(
   _control_name_index.clear();
   DbDriverParam *param_handle;
   bool trim_schema = false;
+  db_mgmt_DriverParameterRef others_option;
   // create param handles
   for (size_t n= 0; n < param_count; ++n)
   {
-    db_mgmt_DriverParameterRef param= params.get(n);    
+    db_mgmt_DriverParameterRef param= params.get(n);
+
+    // remove known options
+    std::vector<std::string>::iterator it = std::find(unknown_options.begin(), unknown_options.end(), *param->name());
+    if (it != unknown_options.end())
+      unknown_options.erase(it);
+
     if (skip_schema && param->name() == "schema")
     {
       trim_schema = true;
       continue;
     }
+    if (param->name() == "$others")
+    {
+      others_option = param;
+      continue;
+    }
     param_handle= new DbDriverParam(param, stored_conn);
     _collection[_control_name_index.size()]= param_handle;
     _control_name_index[control_name_prefix + *param->name()]= param_handle;
   }
+
+  if (others_option.is_valid())
+  {
+    std::string unknown_options_text;
+    for (std::vector<std::string>::const_iterator k = unknown_options.begin(); k != unknown_options.end(); ++k)
+    {
+      if (!k->empty())
+      {
+        unknown_options_text.append(*k);
+        unknown_options_text.append("=");
+        unknown_options_text.append(stored_conn->parameterValues().get(*k).repr());
+        unknown_options_text.append("\n");
+      }
+    }
+    param_handle= new DbDriverParam(others_option, grt::StringRef(unknown_options_text));
+    _collection[_control_name_index.size()]= param_handle;
+    _control_name_index[control_name_prefix + *others_option->name()]= param_handle;
+  }
+
   if (trim_schema)
     _collection.resize(param_count-1);
-  for (int layout_type= 0; layout_type < 2; ++layout_type) // separate cycle for controls tagged as advanced layout
+  for (int layout_type= 0; layout_type < 3; ++layout_type) // separate cycle for controls tagged as advanced layout
   {
     LayoutRows rows;
     LayoutRow row(0, hmargin);
@@ -327,7 +423,7 @@ void DbDriverParams::init(
           row.insert(ctrl);
         }
 
-        // add param description to the row desc
+       // add param description to the row desc
         row.add_desc(param->description());
       }
     }
@@ -354,15 +450,18 @@ void DbDriverParams::init(
         LayoutControl ctrl= r->desc_control();
         ctrl.bounds.top = row_index;
 
-        if(create_control)
+        if (create_control)
             create_control(ctrl.param_handle, ctrl.type, ctrl.bounds, ctrl.caption);
       }
       row_index++;
     }
   }
-  suspend_layout(false);
+
+  if (suspend_layout)
+    suspend_layout(false);
   
-  end_layout();
+  if (end_layout)
+    end_layout();
 }
 
 
@@ -382,7 +481,36 @@ grt::DictRef DbDriverParams::get_params() const
     {
       DbDriverParam *param_handle= *i;
       if (param_handle->get_value().is_valid())
-        params.set(param_handle->object()->name(), param_handle->get_value());
+      {
+        if (param_handle->object()->name() == "$others")
+        {
+          std::vector<std::string> options(base::split(param_handle->get_value().repr(), "\n"));
+          for (std::vector<std::string>::const_iterator op = options.begin(); op != options.end(); ++op)
+          {
+            std::string name, value;
+            base::partition(*op, "=", name, value);
+            if (value.empty())
+              params.set(name, grt::StringRef(value));
+            else if (value[0] == '\'' && value[value.size()-1] == '\'')
+              params.set(name, grt::StringRef(base::unescape_sql_string(value, '\\')));
+            else if (value[0] == '"' && value[value.size()-1] == '"')
+              params.set(name, grt::StringRef(base::unescape_sql_string(value, '\\')));
+            else
+            {
+              bool isnum = true;
+              for (int i = value[0] == '-' ? 1 : 0; isnum && i < (int)value.size(); i++)
+                if (!isdigit(value[i]))
+                  isnum= false;
+              if (isnum)
+                params.set(name, grt::IntegerRef(atoi(value.c_str())));
+              else
+                params.set(name, grt::StringRef(value));
+            }
+          }
+        }
+        else
+          params.set(param_handle->object()->name(), param_handle->get_value());
+      }
     }
     return params;
   }
@@ -422,16 +550,15 @@ std::string DbDriverParams::validate() const
 
 //----------------------------------------------------------------------
 
+// TODO: can this be removed?
+//static const char *DEFAULT_RDBMS_ID= "com.mysql.rdbms.mysql";
 
-static const char *DEFAULT_RDBMS_ID= "com.mysql.rdbms.mysql";
-
-
-void DbConnection::init(const db_mgmt_ManagementRef &mgmt, bool skip_schema)
+DbConnection::DbConnection(const db_mgmt_ManagementRef &mgmt, const db_mgmt_DriverRef &driver, bool skip_schema)
+: _mgmt(mgmt), _active_driver(driver), _skip_schema(skip_schema) 
 {
-  _mgmt= mgmt;
-  _skip_schema = skip_schema;
 }
 
+
 void DbConnection::set_control_callbacks(
   const boost::function<void (bool)> &suspend_layout,
   const boost::function<void ()> &begin_layout,
@@ -447,38 +574,37 @@ void DbConnection::set_control_callbacks(
 
 db_mgmt_ConnectionRef DbConnection::get_connection()
 {
+  save_changes();
   return _connection;
 }
 
 
 void DbConnection::save_changes()
 {
-  _connection->driver(get_active_driver());
+  _connection->driver(_active_driver);
   
-  grt::merge_contents(_connection->parameterValues(), _db_driver_param_handles.get_params(), true);
+  grt::replace_contents(_connection->parameterValues(), _db_driver_param_handles.get_params());
   _connection->hostIdentifier(bec::get_host_identifier_for_connection(_connection));
 }
 
 DbConnection::~DbConnection()
 {
-};
+}
 
-void DbConnection::set_connection(const db_mgmt_ConnectionRef &connection)
+void DbConnection::set_connection_and_update(const db_mgmt_ConnectionRef &connection)
 {
   if (_connection != connection)
   {
     _connection= connection;
-    if (_connection.is_valid() && !_connection->driver().is_valid())
-      _connection->driver(get_active_driver());
+    _active_driver = connection->driver();
 
-    /*
-    _db_driver_param_handles.init(connection->driver(),
+    _db_driver_param_handles.init(_active_driver,
                                   _connection,
                                   _suspend_layout,
-                                  _clear_param_controls,
+                                  _begin_layout,
                                   _create_control,
+                                  _end_layout,
                                   _skip_schema);
-     */
   }
 }
 
@@ -488,7 +614,7 @@ void DbConnection::set_connection_keeping_parameters(const db_mgmt_ConnectionRef
   if (_connection != connection)
   {
     _connection= connection;
-    _connection->driver(get_active_driver());
+    _connection->driver(_active_driver);
     grt::DictRef curparams(_db_driver_param_handles.get_params());
     if (curparams.is_valid())
       grt::merge_contents(_connection->parameterValues(), curparams, true);
@@ -496,14 +622,13 @@ void DbConnection::set_connection_keeping_parameters(const db_mgmt_ConnectionRef
 }
 
 
-void DbConnection::set_active_driver(int driver_index)
+void DbConnection::set_driver_and_update(db_mgmt_DriverRef driver)
 {
-  _active_db_driver_index= driver_index;
-  
+  _active_driver = driver;
   if (_connection.is_valid())
-    _connection->driver(get_active_driver());
+    _connection->driver(driver);
   
-  _db_driver_param_handles.init(get_active_driver(),
+  _db_driver_param_handles.init(_active_driver,
                                 _connection,
                                 _suspend_layout,
                                 _begin_layout,
@@ -549,12 +674,13 @@ sql::ConnectionWrapper DbConnection::get_dbc_connection()
     get_connection(),
     boost::bind(&DbConnection::init_dbc_connection, this, _1, _2));
 
+  /* This is not used by modeling
   if (dbc_conn.get() != NULL)
   {
-    if (_rdbms.is_valid() && (_rdbms.id() == DEFAULT_RDBMS_ID))
+    if (_active_driver.is_valid() && _active_driver->owner().is_valid() && (_active_driver->owner().id() == DEFAULT_RDBMS_ID))
     {
       // set SQL_MODE variable to be consistent with SQL stored in the model
-      bec::GRTManager *grtm= bec::GRTManager::get_instance_for(_rdbms->get_grt());
+      bec::GRTManager *grtm= bec::GRTManager::get_instance_for(_active_driver->get_grt());
       grt::ValueRef sql_mode_value= grtm->get_app_option("SqlMode");
       if (sql_mode_value.is_valid() && grt::StringRef::can_wrap(sql_mode_value))
       {
@@ -563,7 +689,7 @@ sql::ConnectionWrapper DbConnection::get_dbc_connection()
         stmt->execute(base::strfmt("SET @DEFAULT_SQL_MODE=@@SQL_MODE, SQL_MODE='%s'", sql_mode_string.c_str()));
       }
     }
-  }
+  }*/
 
   return dbc_conn;
 }
@@ -575,38 +701,3 @@ std::string DbConnection::validate_driver_params() const
 }
 
 
-void DbConnection::set_active_rdbms(int index)
-{
-  _rdbms= _mgmt->rdbms().get(index);
-}
-
-
-grt::ListRef<db_mgmt_Driver> DbConnection::get_driver_list()
-{
-  return _rdbms->drivers();
-}
-
-
-int DbConnection::get_default_rdbms_index() const
-{
-  size_t index= find_object_index_in_list(_mgmt->rdbms(), DEFAULT_RDBMS_ID);
-  return (int)index;
-}
-
-
-int DbConnection::get_rdbms_default_driver_index() const
-{
-  size_t result= -1;
-  if (_rdbms.is_valid())
-  {
-    db_mgmt_DriverRef defaultDriver= _rdbms->defaultDriver();
-    result= find_object_index_in_list(_rdbms->drivers(), defaultDriver.id());
-  }
-  return (int)result;
-}
-
-
-grt::ListRef<db_mgmt_Rdbms> DbConnection::get_rdbms_list()
-{
-  return _mgmt->rdbms();
-}
