@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,6 +27,7 @@
 #include "cppdbc.h"
 #include "grtui/binary_data_editor.h"
 
+#include "mforms/menubar.h"
 #include "mforms/utilities.h"
 #include "mforms/filechooser.h"
 
@@ -41,20 +42,20 @@
 using namespace bec;
 using namespace base;
 
-#define CATCH_AND_DISPATCH_EXCEPTION(rethrow, res) \
+#define CATCH_AND_DISPATCH_EXCEPTION(rethrow, context, res) \
 catch (sql::SQLException &e)\
 {\
-  rethrow ? throw : task->send_msg(grt::ErrorMsg, strfmt("Error Code: %i\n%s", e.getErrorCode(), e.what()));\
+  rethrow ? throw : task->send_msg(grt::ErrorMsg, strfmt("Error Code: %i\n%s", e.getErrorCode(), e.what()), context);\
   res= false;\
 }\
 catch (sqlite::database_exception &e)\
 {\
-  rethrow ? throw : task->send_msg(grt::ErrorMsg, e.what());\
+  rethrow ? throw : task->send_msg(grt::ErrorMsg, e.what(), context);\
   res= false;\
 }\
 catch (std::exception &e)\
 {\
-  rethrow ? throw : task->send_msg(grt::ErrorMsg, e.what());\
+  rethrow ? throw : task->send_msg(grt::ErrorMsg, e.what(), context);\
   res= false;\
 }
 
@@ -81,6 +82,7 @@ static gint next_id = 0;
 Recordset::Recordset(GRTManager *grtm)
   : VarGridModel(grtm), _show_apply_buttons(false), task(GrtThreadedTask::create(grtm))
 {
+  _context_menu = 0;
   _id = g_atomic_int_get(&next_id);
   g_atomic_int_inc(&next_id);
 
@@ -94,6 +96,7 @@ Recordset::Recordset(GRTManager *grtm)
 Recordset::Recordset(GrtThreadedTask::Ref parent_task)
   : VarGridModel(parent_task->grtm()), _show_apply_buttons(false), task(GrtThreadedTask::create(parent_task))
 {
+  _context_menu = 0;
   _id = g_atomic_int_get(&next_id);
   g_atomic_int_inc(&next_id);
 
@@ -105,6 +108,7 @@ Recordset::Recordset(GrtThreadedTask::Ref parent_task)
 
 Recordset::~Recordset()
 {
+  delete _context_menu;
 }
 
 
@@ -165,7 +169,7 @@ bool Recordset::reset(Recordset_data_storage::Ptr data_storage_ptr, bool rethrow
 
       _readonly_reason = data_storage->readonly_reason();
     }
-    CATCH_AND_DISPATCH_EXCEPTION(rethrow, res)
+    CATCH_AND_DISPATCH_EXCEPTION(rethrow, "Reset recordset", res)
   }
 
   refresh_ui_status_bar();
@@ -231,19 +235,25 @@ void Recordset::refresh()
 {
   if (has_pending_changes())
   {
-    task->send_msg(grt::ErrorMsg, ERRMSG_PENDING_CHANGES);
+    task->send_msg(grt::ErrorMsg, ERRMSG_PENDING_CHANGES, _("Refresh Recordset"));
     return;
   }
 
+  std::string data_search_string = _data_search_string;
+
   VarGridModel::refresh();
   reset();
+
+  // reapply filter, if needed
+  if (!data_search_string.empty())
+    set_data_search_string(data_search_string);
 }
 
 
 void Recordset::rollback()
 {
   if (!reset(false))
-    task->send_msg(grt::ErrorMsg, "Rollback failed");
+    task->send_msg(grt::ErrorMsg, _("Rollback failed"), _("Rollback recordset changes"));
 }
 
 
@@ -303,7 +313,7 @@ Recordset::Cell Recordset::cell(RowId row, ColumnId column)
 
       // insert new empty data record
       {
-        std::list<sqlite::Variant> bind_vars;
+        std::list<sqlite::variant_t> bind_vars;
         bind_vars.push_back((int)rowid);
         emit_partition_commands(data_swap_db.get(), data_swap_db_partition_count(), "insert into `data%s` (id) values (?)", bind_vars);
       }
@@ -334,7 +344,7 @@ Recordset::Cell Recordset::cell(RowId row, ColumnId column)
     // init new row fields with null-values
     Cell new_cell= _data.begin() + (_data.size() - _column_count);
     for (ColumnId col= 0; _column_count > col; ++col, ++new_cell)
-      *(new_cell)= sqlite::Null();
+      *(new_cell)= sqlite::null_t();
     _data[_data.size() - _column_count + _rowid_column]= (int)rowid;
   }
 
@@ -342,15 +352,16 @@ Recordset::Cell Recordset::cell(RowId row, ColumnId column)
 }
 
 
-void Recordset::after_set_field(const NodeId &node, int column, const sqlite::Variant &value)
+void Recordset::after_set_field(const NodeId &node, int column, const sqlite::variant_t &value)
 {
   VarGridModel::after_set_field(node, column, value);
   mark_dirty(node[0], column, value);
   refresh_ui_status_bar();
+  tree_changed();
 }
 
 
-void Recordset::mark_dirty(RowId row, ColumnId column, const sqlite::Variant &new_value)
+void Recordset::mark_dirty(RowId row, ColumnId column, const sqlite::variant_t &new_value)
 {
   GStaticRecMutexLock data_mutex(_data_mutex);
 
@@ -422,8 +433,8 @@ bool Recordset::delete_nodes(std::vector<bec::NodeId> &nodes)
       node[0]-= processed_node_count;
       RowId row= node[0];
 
-      RowId rowid;
-      if (get_field_(node, _rowid_column, (int&)rowid))
+      int rowid;
+      if (get_field_(node, _rowid_column, rowid))
       {
         boost::shared_ptr<sqlite::connection> data_swap_db= this->data_swap_db();
         sqlide::Sqlite_transaction_guarder transaction_guarder(data_swap_db.get());
@@ -434,28 +445,28 @@ bool Recordset::delete_nodes(std::vector<bec::NodeId> &nodes)
           std::string partition_suffix= data_swap_db_partition_suffix(partition);
           sqlite::command save_deleted_data_record_statement(*data_swap_db,
             strfmt("insert into `deleted_rows%s` select * from `data%s` where id=?", partition_suffix.c_str(), partition_suffix.c_str()));
-          save_deleted_data_record_statement % (int)rowid;
+          save_deleted_data_record_statement % rowid;
           save_deleted_data_record_statement.emit();
         }
 
         // delete data record
         {
-          std::list<sqlite::Variant> bind_vars;
-          bind_vars.push_back((int)rowid);
+          std::list<sqlite::variant_t> bind_vars;
+          bind_vars.push_back(rowid);
           emit_partition_commands(data_swap_db.get(), data_swap_db_partition_count(), "delete from `data%s` where id=?", bind_vars);
         }
 
         // delete data index record
         {
           sqlite::command delete_data_index_record_statement(*data_swap_db, "delete from `data_index` where id=?");
-          delete_data_index_record_statement % (int)rowid;
+          delete_data_index_record_statement % rowid;
           delete_data_index_record_statement.emit();
         }
 
         // log delete action
         {
           sqlite::command add_change_record_statement(*data_swap_db, _add_change_record_statement);
-          add_change_record_statement % (int)rowid;
+          add_change_record_statement % rowid;
           add_change_record_statement % -1;
           static sqlite::null_type null_obj;
           add_change_record_statement % null_obj;
@@ -548,10 +559,10 @@ grt::StringRef Recordset::do_apply_changes(grt::GRT *grt, Ptr self_ptr, Recordse
   try
   {
     data_storage->apply_changes(self_ptr);
-    task->send_msg(grt::InfoMsg, "Commit complete");
+    task->send_msg(grt::InfoMsg, _("Commit complete"), _("Commit recordset changes"));
     reset(data_storage_ptr, false);
   }
-  CATCH_AND_DISPATCH_EXCEPTION(false, res)
+  CATCH_AND_DISPATCH_EXCEPTION(false, "Commit recordset changes", res)
 
   return grt::StringRef("");
 }
@@ -630,7 +641,7 @@ void Recordset::limit_rows(bool value)
 {
   if (has_pending_changes())
   {
-    task->send_msg(grt::ErrorMsg, ERRMSG_PENDING_CHANGES);
+    task->send_msg(grt::ErrorMsg, ERRMSG_PENDING_CHANGES, _("Limit Rows"));
     return;
   }
 
@@ -883,8 +894,8 @@ void Recordset::rebuild_data_index(sqlite::connection *data_swap_db, bool do_cac
         qv.store_unknown_as_string= true;
         qv.allow_func_escaping= false;
       }
-      sqlite::Variant var_string_type= std::string();
-      sqlite::Variant var_string;
+      sqlite::variant_t var_string_type= std::string();
+      sqlite::variant_t var_string;
       std::string sql_string;
 
       // column filters subclause
@@ -997,7 +1008,12 @@ void Recordset::rebuild_data_index(sqlite::connection *data_swap_db, bool do_cac
 void Recordset::paste_rows_from_clipboard(int dest_row)
 {
   std::string text = mforms::Utilities::get_clipboard_text();
-  std::vector<std::string> rows = base::split(text, "\n");
+  std::vector<std::string> rows;
+  
+  if (text.find("\r\n") != std::string::npos)
+    rows = base::split(text, "\r\n");
+  else
+    rows = base::split(text, "\n");
 
   if (rows.empty())
     return;
@@ -1020,13 +1036,17 @@ void Recordset::paste_rows_from_clipboard(int dest_row)
     }
   }
   
+  int separator = ',';
+  if (text.find('\t') != std::string::npos)
+    separator = '\t';
+  
   for (std::vector<std::string>::const_iterator row = rows.begin(); row != rows.end(); ++row)
   {
     if (!row->empty())
     {
-      std::vector<std::string> parts = base::split_token_list(*row, ',');
+      std::vector<std::string> parts = base::split_token_list(*row, separator);
       
-      if (parts.size() != get_column_count())
+      if ((int)parts.size() != get_column_count())
       {
         mforms::Utilities::show_error("Cannot Paste Row",
                                       strfmt("Number of fields in pasted data doesn't match the columns in the table (%i vs %i).\n"
@@ -1045,7 +1065,7 @@ void Recordset::paste_rows_from_clipboard(int dest_row)
         {
           if (!token.empty() && token[0] == '\'' && token[token.size()-1] == '\'')
             token = token.substr(1, token.size()-2);
-          set_field(dest_row, i, token);
+          set_field(dest_row, i, base::unescape_sql_string(token, '\''));
         }
       }
       dest_row++;
@@ -1054,130 +1074,134 @@ void Recordset::paste_rows_from_clipboard(int dest_row)
 }
 
 
-bec::MenuItemList Recordset::get_popup_menu_items(const std::vector<int> &rows, int clicked_column)
+mforms::ContextMenu *Recordset::get_context_menu()
 {
-  bec::MenuItemList items;
-  bec::MenuItem item;
-  bool ro = is_readonly();
-
-  bec::MenuItemList plugin_items;
-  if (popup_menu_items_slot)
-    plugin_items= popup_menu_items_slot(rows, clicked_column);
-
-  item.name = "edit_cell";
-  if (ro)
-    item.caption = "Open Value in Viewer";
-  else
-    item.caption = "Open Value in Editor";
-  item.enabled = (rows.size() > 0) && (clicked_column >= 0);
-  if (item.enabled)
-  {
-    switch (get_real_column_type(clicked_column))
-    {
-    case StringType:
-    case BlobType:
-      break;
-    default:
-      item.enabled= false;
-      break;
-    }
-  }
-  items.push_back(item);
-  
-  item.name = "";
-  item.type = bec::MenuSeparator;
-  item.caption = "";
-  items.push_back(item);
-  item.type = bec::MenuAction;
-  
-  item.name = "set_to_null";
-  item.caption = "Set Field(s) to NULL";
-#ifdef _WIN32
-  item.enabled = clicked_column >= 0 && !ro;
-#else
-  item.enabled = clicked_column >= 0 && rows.size() == 1 && !ro;
-#endif
-  items.push_back(item);
-
-  item.name = "set_to_function";
-  item.caption = "Mark Field Value as a Function";
-#ifdef _WIN32
-  item.enabled = clicked_column >= 0 && !ro;
-#else
-  item.enabled = clicked_column >= 0 && rows.size() == 1 && !ro;
-#endif
-  items.push_back(item);
-  
-  item.name = "delete_row";
-  item.caption = "Delete Row(s)";
-  item.enabled = rows.size() > 0 && !ro;
-  items.push_back(item);
-  
-  item.name = "";
-  item.type = bec::MenuSeparator;
-  item.caption = "";
-  items.push_back(item);
-  item.type = bec::MenuAction;
-  
-  item.name = "load_from_file";
-  item.caption = "Load Value From File";
-  item.enabled = clicked_column >= 0 && rows.size() == 1 && !ro;
-  items.push_back(item);
-
-  item.name = "save_to_file";
-  item.caption = "Save Value To File";
-  item.enabled = clicked_column >= 0 && rows.size() == 1 && !ro;
-  items.push_back(item);
-  
-  item.name = "";
-  item.type = bec::MenuSeparator;
-  item.caption = "";
-  items.push_back(item);
-  item.type = bec::MenuAction;
-
-  item.name = "copy_row";
-  item.caption = "Copy Row Content";
-  item.enabled = rows.size() > 0;
-  items.push_back(item);
-
-  item.name = "copy_row_unquoted";
-  item.caption = "Copy Row Content (unquoted)";
-  item.enabled = rows.size() > 0;
-  items.push_back(item);
-  
-  item.name = "copy_field";
-  item.caption = "Copy Field Content";
-  item.enabled = clicked_column >= 0 && rows.size() == 1;
-  items.push_back(item);
-
-  item.name = "copy_field_unquoted";
-  item.caption = "Copy Field Content (unquoted)";
-  item.enabled = clicked_column >= 0 && rows.size() == 1;
-  items.push_back(item);
-
-  item.name = "paste_row";
-  item.caption = "Paste Row";
-  item.enabled = rows.size() <= 1 && !mforms::Utilities::get_clipboard_text().empty() && !ro;
-  items.push_back(item);
-    
-  if (!plugin_items.empty())
-  {
-    item.name = "";
-    item.type = bec::MenuSeparator;
-    item.caption = "";
-    items.push_back(item);
-    
-    items.insert(items.end(), plugin_items.begin(), plugin_items.end());
-  }
-  return items;
+  if (!_context_menu)
+    _context_menu = new mforms::ContextMenu();
+  return _context_menu;
 }
 
 
-bool Recordset::activate_popup_menu_item(const std::string &action, const std::vector<int> &rows, int clicked_column)
+void Recordset::update_selection_for_menu(const std::vector<int> &rows, int clicked_column)
+{
+  // TODO: lift the restriction to a single column.
+  //       We need to support multiple cells (in multiple columns) on all platforms.
+  _selected_rows = rows;
+  _selected_column = clicked_column;
+
+  if (_context_menu)
+  {
+    _context_menu->remove_all();
+
+    bool ro = is_readonly();
+
+    mforms::MenuItem *item;
+
+    item = _context_menu->add_item_with_title(ro ? "Open Value in Viewer" : "Open Value in Editor",
+                                              boost::bind(&Recordset::activate_menu_item, this, "edit_cell", rows, clicked_column),
+                                              "edit_cell");
+    item->set_enabled((rows.size() == 1) && (clicked_column >= 0));
+    if (item->get_enabled())
+    {
+      switch (get_real_column_type(clicked_column))
+      {
+      case StringType:
+      case BlobType:
+        break;
+      default:
+        item->set_enabled(false);
+        break;
+      }
+    }
+
+    _context_menu->add_separator();
+
+    item = _context_menu->add_item_with_title("Set Field(s) to NULL",
+                                       boost::bind(&Recordset::activate_menu_item, this, "set_to_null", rows, clicked_column),
+                                       "set_to_null");
+
+    // On Windows we can select individual cells, so it is perfectly ok to allow acting on multiple
+    // cells. The other platforms only select entire rows in the multi-selection case.
+    // So we to have disallow acting on them to avoid changing unrelated entries.
+  #ifdef _WIN32
+    item->set_enabled(clicked_column >= 0 && !ro);
+  #else
+    item->set_enabled(clicked_column >= 0 && rows.size() == 1 && !ro);
+  #endif
+
+    item = _context_menu->add_item_with_title("Mark Field Value as a Function",
+                                      boost::bind(&Recordset::activate_menu_item, this, "set_to_function", rows, clicked_column),
+                                      "set_to_function");
+  #ifdef _WIN32
+    item->set_enabled(clicked_column >= 0 && !ro);
+  #else
+    item->set_enabled(clicked_column >= 0 && rows.size() == 1 && !ro);
+  #endif
+    
+    item = _context_menu->add_item_with_title("Delete Row(s)",
+                                              boost::bind(&Recordset::activate_menu_item, this, "delete_row", rows, clicked_column),
+                                              "delete_row");
+    item->set_enabled(rows.size() > 0 && !ro);
+
+    _context_menu->add_separator();
+
+    item = _context_menu->add_item_with_title("Load Value From File...",
+                                              boost::bind(&Recordset::activate_menu_item, this, "load_from_file", rows, clicked_column),
+                                              "load_from_file");
+    item->set_enabled(clicked_column >= 0 && rows.size() == 1 && !ro);
+
+    item = _context_menu->add_item_with_title("Save Value To File...",
+                                              boost::bind(&Recordset::activate_menu_item, this, "save_to_file", rows, clicked_column),
+                                              "save_to_file");
+    item->set_enabled(clicked_column >= 0 && rows.size() == 1 && !ro);
+
+    _context_menu->add_separator();
+
+    item = _context_menu->add_item_with_title("Copy Row",
+                                              boost::bind(&Recordset::activate_menu_item, this, "copy_row", rows, clicked_column),
+                                              "copy_row");
+    item->set_enabled(rows.size() > 0);
+    item = _context_menu->add_item_with_title("Copy Row (with names)",
+                                              boost::bind(&Recordset::activate_menu_item, this, "copy_row_with_names", rows, clicked_column),
+                                              "copy_row_with_names");
+
+    item = _context_menu->add_item_with_title("Copy Row (unquoted)",
+                                              boost::bind(&Recordset::activate_menu_item, this, "copy_row_unquoted", rows, clicked_column),
+                                              "copy_row_unquoted");
+    item->set_enabled(rows.size() > 0);
+    item = _context_menu->add_item_with_title("Copy Row (with names, unquoted)",
+                                              boost::bind(&Recordset::activate_menu_item, this, "copy_row_unquoted_with_names", rows, clicked_column),
+                                              "copy_row_unquoted_with_names");
+
+    item = _context_menu->add_item_with_title("Copy Row (tab separated)",
+                                              boost::bind(&Recordset::activate_menu_item, this, "copy_row_tabsep", rows, clicked_column),
+                                              "copy_row_tabsep");
+    item->set_enabled(rows.size() > 0);
+
+    item = _context_menu->add_item_with_title("Copy Field",
+                                              boost::bind(&Recordset::activate_menu_item, this, "copy_field", rows, clicked_column),
+                                              "copy_field");
+    item->set_enabled(clicked_column >= 0 && rows.size() == 1);
+
+    item = _context_menu->add_item_with_title("Copy Field (unquoted)",
+                                              boost::bind(&Recordset::activate_menu_item, this, "copy_field_unquoted", rows, clicked_column),
+                                              "copy_field_unquoted");
+    item->set_enabled(clicked_column >= 0 && rows.size() == 1);
+
+    item = _context_menu->add_item_with_title("Paste Row",
+                                              boost::bind(&Recordset::activate_menu_item, this, "paste_row", rows, clicked_column),
+                                              "paste_row");
+    item->set_enabled(rows.size() <= 1 && !mforms::Utilities::get_clipboard_text().empty() && !ro);
+  }
+}
+
+
+void Recordset::activate_menu_item(const std::string &action, const std::vector<int> &rows, int clicked_column)
 {
   bool result = false;
   bool need_ui_refresh = false;
 
+  // TODO: the tests here for rows count and clicked_column are all unnecessary. This has already be done.
   if (action == "edit_cell")
   {
     if (rows.size() == 1 && clicked_column >= 0)
@@ -1188,22 +1212,22 @@ bool Recordset::activate_popup_menu_item(const std::string &action, const std::v
   }
   else if (action == "set_to_null")
   {
-    if (rows.size() == 1 && clicked_column >= 0)
+    for (size_t i = 0; i < rows.size(); ++i)
     {
       bec::NodeId node;
-      node.append(rows[0]);
+      node.append(rows[i]);
       set_field_null(node, clicked_column);
-      result = true;
     }
+    result = true;
   }
   else if (action == "set_to_function")
   {
-    if (rows.size() == 1 && clicked_column >= 0)
+    for (size_t i = 0; i < rows.size(); ++i)
     {
       bec::NodeId node;
       Cell cell;
       
-      node.append(rows[0]);
+      node.append(rows[i]);
       std::string function;
       if (!get_cell(cell, node, clicked_column, false))
         function = "";
@@ -1218,9 +1242,12 @@ bool Recordset::activate_popup_menu_item(const std::string &action, const std::v
   {
     if (rows.size() > 0)
     {
-      bec::NodeId node;
-      node.append(rows[0]);
-      delete_node(node);
+      std::vector<int> sorted_rows(rows);
+      std::sort(sorted_rows.begin(), sorted_rows.end());
+      std::vector<bec::NodeId> nodes;
+      for (std::vector<int>::reverse_iterator iter = sorted_rows.rbegin(); iter != sorted_rows.rend(); ++iter)
+        nodes.push_back(bec::NodeId(*iter));
+      delete_nodes(nodes);
       need_ui_refresh = true;
       result = true;
     }
@@ -1249,18 +1276,36 @@ bool Recordset::activate_popup_menu_item(const std::string &action, const std::v
   {
     if (rows.size() > 0)
     {
-      copy_rows_to_clipboard(rows);
+      copy_rows_to_clipboard(rows, ", ");
       result = true;
     }
   }
+  else if (action == "copy_row_with_names")
+  {
+    copy_rows_to_clipboard(rows, ", ", true, true);
+    result = true;
+  }
   else if (action == "copy_row_unquoted")
   {
     if (rows.size() > 0)
     {
-      copy_rows_to_clipboard(rows, false);
+      copy_rows_to_clipboard(rows, ", ", false);
       result = true;
     }
   }
+  else if (action == "copy_row_unquoted_with_names")
+  {
+    copy_rows_to_clipboard(rows, ", ", false, true);
+    result = true;
+  }
+  else if (action == "copy_row_tabsep")
+  {
+    if (rows.size() > 0)
+    {
+      copy_rows_to_clipboard(rows, "\t", false);
+      result = true;
+    }
+  }  
   else if (action == "copy_field")
   {
     if (rows.size() == 1 && clicked_column >= 0)
@@ -1282,18 +1327,13 @@ bool Recordset::activate_popup_menu_item(const std::string &action, const std::v
     paste_rows_from_clipboard(rows.empty() ? -1 : rows[0]);
     result = true;
   }
-  else
-    if (call_popup_menu_item_slot(action, rows, clicked_column))
-      result = true;
 
   if (need_ui_refresh)
     refresh_ui();
-
-  return result;
 }
 
 
-void Recordset::copy_rows_to_clipboard(const std::vector<int> &indeces, bool quoted)
+void Recordset::copy_rows_to_clipboard(const std::vector<int> &indeces, std::string sep, bool quoted, bool with_header)
 {
   ColumnId editable_col_count= get_column_count();
   if (!editable_col_count)
@@ -1301,32 +1341,43 @@ void Recordset::copy_rows_to_clipboard(const std::vector<int> &indeces, bool quo
 
   sqlide::QuoteVar qv;
   {
-    qv.escape_string= boost::bind(sqlide::QuoteVar::escape_ansi_sql_string, _1);
+    qv.escape_string= boost::bind(base::escape_sql_string, _1, false);
     qv.store_unknown_as_string= true;
     qv.allow_func_escaping= true;
   }
 
   Cell cell;
   std::string text;
+
+  if (with_header)
+  {
+    text = "# ";
+    for (ColumnId col= 0; editable_col_count > col; ++col)
+    {
+      if (col > 0)
+        text.append(sep);
+      text.append(get_column_caption(col));
+    }
+    text.append("\n");
+  }
+
   BOOST_FOREACH (RowId row, indeces)
   {
+    std::string line;
     for (ColumnId col= 0; editable_col_count > col; ++col)
     {
       bec::NodeId node(row);
       if (!get_cell(cell, node, col, false))
         continue;
+      if (col > 0)
+        line+= sep;
       if (quoted)
-        text+= strfmt("%s, ",
-                      boost::apply_visitor(qv, _column_types[col], *cell).c_str());
+        line+= boost::apply_visitor(qv, _column_types[col], *cell);
       else
-        text+= strfmt("%s, ",
-                      boost::apply_visitor(_var_to_str, *cell).c_str());
-    }
-    if (!text.empty())
-    {
-      text.resize(text.size()-2);
-      text+= "\n";
+        line+= boost::apply_visitor(_var_to_str, *cell);
     }
+    if (!line.empty())
+      text+= line+"\n";
   }
   mforms::Utilities::set_clipboard_text(text);
 }
@@ -1353,13 +1404,6 @@ void Recordset::copy_field_to_clipboard(int row, int column, bool quoted)
   mforms::Utilities::set_clipboard_text(text);
 }
 
-
-void Recordset::invoke_export()
-{
-  export_wizard();
-}
-
-
 static void add_toolbar_action_item(bec::ToolbarItemList &items, bec::IconManager *im, const std::string &item_icon, const std::string &item_name, const std::string &item_tooltip)
 {
   bec::ToolbarItem item;
@@ -1434,7 +1478,7 @@ bec::ToolbarItemList Recordset::get_toolbar_items()
   bec::ToolbarItem item;
   bec::IconManager *im = bec::IconManager::get_instance();
 
-  add_toolbar_label_item(items, "Filter:");
+  add_toolbar_label_item(items, "Result Set Filter:");
   {
     bec::ToolbarItem item;
     item.command = "filter";
@@ -1442,7 +1486,7 @@ bec::ToolbarItemList Recordset::get_toolbar_items()
     items.push_back(item);
   }
   if (!_data_storage || _data_storage->reloadable())
-    add_toolbar_action_item(items, im, "tiny_refresh.png", "record_refresh", "Refresh data from data source");
+    add_toolbar_action_item(items, im, "record_refresh.png", "record_refresh", "Refresh data from data source");
 
   if (!is_readonly())
   {
@@ -1460,13 +1504,17 @@ bec::ToolbarItemList Recordset::get_toolbar_items()
     }
   }
   add_toolbar_separator_item(items);
-  add_toolbar_label_item(items, "Export:");
+  if (!is_readonly())
+    add_toolbar_label_item(items, "Export/Import:");
+  else
+    add_toolbar_label_item(items, "Export:");
   add_toolbar_action_item(items, im, "record_export", "Export recordset to an external file");
-  
+  if (!is_readonly())
+    add_toolbar_action_item(items, im, "record_import", "Import records from an external file");
+
 #ifndef __APPLE__
   add_toolbar_separator_item(items);
-  add_toolbar_label_item(items, "Autosize:");
-  //  add_toolbar_action_item(items, im, "record_autosize", "Resize columns to fit contents");  
+  add_toolbar_label_item(items, "Wrap Cell Content:");
   add_toolbar_action_item(items, im, "record_wrap_vertical", "Toggle wrapping of cell contents");
 #endif
   
@@ -1474,9 +1522,8 @@ bec::ToolbarItemList Recordset::get_toolbar_items()
   {
     add_toolbar_separator_item(items);
     add_toolbar_label_item(items, "Fetch rows:");
-    add_toolbar_action_item(items, im, "record_fetch_next.png", "scroll_rows_frame_forward", "Fetch next frame of records from the data source");
     add_toolbar_action_item(items, im, "record_fetch_prev.png", "scroll_rows_frame_backward", "Fetch previous frame of records from the data source");
-    add_toolbar_action_item(items, im, "record_fetch_all", "Toggle limitation of the records number");
+    add_toolbar_action_item(items, im, "record_fetch_next.png", "scroll_rows_frame_forward", "Fetch next frame of records from the data source");
   }
   return items;
 }
@@ -1504,9 +1551,6 @@ void Recordset::register_default_actions()
 
   _action_list.register_action("record_refresh",
     boost::bind(&Recordset::refresh, this));
-
-  _action_list.register_action("record_export",
-    boost::bind(&Recordset::invoke_export, this));
 }
 
 
@@ -1522,9 +1566,9 @@ private:
   std::string _encoding;
   bool _read_only;
 public:
-  result_type operator()(const sqlite::Null &v) { return new BinaryDataEditor(_grtm, NULL, 0, _encoding, _read_only); }
+  result_type operator()(const sqlite::null_t &v) { return new BinaryDataEditor(_grtm, NULL, 0, _encoding, _read_only); }
   result_type operator()(const std::string &v) { return new BinaryDataEditor(_grtm, v.c_str(), v.length(), _encoding, _read_only); }
-  result_type operator()(const sqlite::BlobRef &v) { return new BinaryDataEditor(_grtm, ((!v || v->empty()) ? NULL : (const char*)&(*v)[0]), v->size(), _encoding, _read_only); }
+  result_type operator()(const sqlite::blob_ref_t &v) { return new BinaryDataEditor(_grtm, ((!v || v->empty()) ? NULL : (const char*)&(*v)[0]), v->size(), _encoding, _read_only); }
   template<typename V> result_type operator()(const V &v) { return NULL; }
 };
 class DataEditorSelector2 : public boost::static_visitor<BinaryDataEditor*>
@@ -1536,7 +1580,7 @@ private:
   bool _read_only;
 public:
   template<typename V> result_type operator()(const std::string &t, const V &v) { return DataEditorSelector(_grtm, _read_only, "UTF-8")(v); }
-  template<typename V> result_type operator()(const sqlite::BlobRef &t, const V &v) { return DataEditorSelector(_grtm, _read_only, "LATIN1")(v); }
+  template<typename V> result_type operator()(const sqlite::blob_ref_t &t, const V &v) { return DataEditorSelector(_grtm, _read_only, "LATIN1")(v); }
   template<typename T, typename V> result_type operator()(const T &r, const V &v) {
     //return NULL;
     // For unknown types treat them for now as string values. Since we have a binary editor pane there that should work
@@ -1551,8 +1595,8 @@ void Recordset::open_field_data_editor(RowId row, ColumnId column)
   bool res= true;
   try
   {
-    sqlite::Variant blob_value;
-    sqlite::Variant *value;
+    sqlite::variant_t blob_value;
+    sqlite::variant_t *value;
 
     if (sqlide::is_var_blob(_real_column_types[column]))
     {
@@ -1585,11 +1629,11 @@ void Recordset::open_field_data_editor(RowId row, ColumnId column)
     data_editor->signal_saved.connect(boost::bind(&Recordset::set_field_value,this, row, column, data_editor));
     data_editor->show(true);
   }
-  CATCH_AND_DISPATCH_EXCEPTION(false, res)
+  CATCH_AND_DISPATCH_EXCEPTION(false, "Open field editor", res)
 }
 
 
-class DataValueConv : public boost::static_visitor<sqlite::Variant>
+class DataValueConv : public boost::static_visitor<sqlite::variant_t>
 {
 public:
   DataValueConv() : _data(NULL), _length(0) {}
@@ -1599,15 +1643,15 @@ private:
   const char *_data;
   size_t _length;
 public:
-  result_type operator()(const sqlite::BlobRef &t)
+  result_type operator()(const sqlite::blob_ref_t &t)
   {
-    sqlite::BlobRef val= sqlite::BlobRef(new sqlite::Blob());
+    sqlite::blob_ref_t val= sqlite::blob_ref_t(new sqlite::blob_t());
     val->resize(_length);
     memcpy(&(*val)[0], _data, _length);
     return val;
   }
   result_type operator()(const std::string &t) { return std::string(_data, _length); }
-  template<typename T> result_type operator()(const T &t) { return sqlite::Unknown(); }
+  template<typename T> result_type operator()(const T &t) { return sqlite::unknown_t(); }
 };
 
 
@@ -1620,7 +1664,7 @@ void Recordset::set_field_value(RowId row, ColumnId column, BinaryDataEditor *da
 void Recordset::set_field_raw_data(RowId row, ColumnId column, const char *data, size_t data_length)
 {
   DataValueConv data_value_conv(data, data_length);
-  sqlite::Variant value= boost::apply_visitor(data_value_conv, _real_column_types[column]);
+  sqlite::variant_t value= boost::apply_visitor(data_value_conv, _real_column_types[column]);
   if (sqlide::is_var_unknown(value))
     throw std::logic_error("Can't save value of this data type.");
   bec::NodeId node(row);
@@ -1670,7 +1714,7 @@ public:
   DataValueDump(const char *filename) : os(filename, std::ios_base::out|std::ios_base::binary) {}
   std::ofstream os;
 public:
-  result_type operator()(const sqlite::BlobRef &v)
+  result_type operator()(const sqlite::blob_ref_t &v)
   {
     std::copy(v->begin(), v->end(), std::ostreambuf_iterator<char>(os));
   }
@@ -1683,8 +1727,8 @@ void Recordset::save_to_file(const bec::NodeId &node, int column, const std::str
 {
   GStaticRecMutexLock data_mutex(_data_mutex);
       
-  sqlite::Variant blob_value;
-  sqlite::Variant *value;
+  sqlite::variant_t blob_value;
+  sqlite::variant_t *value;
   
   if (sqlide::is_var_blob(_real_column_types[column]))
   {
@@ -1726,6 +1770,6 @@ void Recordset::save_to_file(const bec::NodeId &node, int column)
     {      
       save_to_file(node, column, chooser.get_path());
     }
-    CATCH_AND_DISPATCH_EXCEPTION(false, res)
+    CATCH_AND_DISPATCH_EXCEPTION(false, "Save field to file", res)
   }
 }
