@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -40,7 +40,8 @@ using namespace base;
 Recordset_cdbc_storage::Recordset_cdbc_storage(GRTManager *grtm)
 :
 Recordset_sql_storage(grtm),
-_reloadable(true)
+_reloadable(true),
+_gather_field_info(false)
 {
 }
 
@@ -58,21 +59,21 @@ sql::Dbc_connection_handler::ConnectionRef Recordset_cdbc_storage::dbms_conn_ref
 }
 
 
-class FetchVar : public boost::static_visitor<sqlite::Variant>
+class FetchVar : public boost::static_visitor<sqlite::variant_t>
 {
 public:
   FetchVar(sql::ResultSet *rs) : _rs(rs), _foreknown_blob_size(-1) {}
-  result_type operator()(const sqlite::Unknown &v, const sqlite::Variant &index) const { return _rs->getString(boost::get<int>(index)); }
-  result_type operator()(const sqlite::Null &v, const sqlite::Variant &index) const { return sqlite::Null(); }
-  result_type operator()(const std::string &v, const sqlite::Variant &index) const { return _rs->getString(boost::get<int>(index)); }
-  result_type operator()(const int &v, const sqlite::Variant &index) const { return _rs->getInt(boost::get<int>(index)); }
-  result_type operator()(const boost::int64_t &v, const sqlite::Variant &index) const { return (boost::int64_t)_rs->getInt64(boost::get<int>(index)); }
-  result_type operator()(const long double &v, const sqlite::Variant &index) const { return _rs->getDouble(boost::get<int>(index)); }
-  result_type operator()(const sqlite::BlobRef &v, const sqlite::Variant &index)
+  result_type operator()(const sqlite::unknown_t &v, const sqlite::variant_t &index) const { return _rs->getString(boost::get<int>(index)); }
+  result_type operator()(const sqlite::null_t &v, const sqlite::variant_t &index) const { return sqlite::null_t(); }
+  result_type operator()(const std::string &v, const sqlite::variant_t &index) const { return _rs->getString(boost::get<int>(index)); }
+  result_type operator()(const int &v, const sqlite::variant_t &index) const { return _rs->getInt(boost::get<int>(index)); }
+  result_type operator()(const boost::int64_t &v, const sqlite::variant_t &index) const { return (boost::int64_t)_rs->getInt64(boost::get<int>(index)); }
+  result_type operator()(const long double &v, const sqlite::variant_t &index) const { return _rs->getDouble(boost::get<int>(index)); }
+  result_type operator()(const sqlite::blob_ref_t &v, const sqlite::variant_t &index)
   {
-    sqlite::BlobRef blob_ref;
+    sqlite::blob_ref_t blob_ref;
     std::auto_ptr<std::istream> is(_rs->getBlob(boost::get<int>(index)));
-    if (-1 == _foreknown_blob_size)
+    if ((size_t)-1 == _foreknown_blob_size)
     {
       const size_t BUFF_SIZE= 4096;
       std::list<std::vector<char> > chunks;
@@ -85,8 +86,8 @@ public:
         is->read(&chunk[0], BUFF_SIZE);
         blob_size+= is->gcount();
       }
-      blob_ref.reset(new sqlite::Blob(chunks.size() * BUFF_SIZE));
-      sqlite::Blob *blob= blob_ref.get();
+      blob_ref.reset(new sqlite::blob_t(chunks.size() * BUFF_SIZE));
+      sqlite::blob_t *blob= blob_ref.get();
       int n= 0;
       BOOST_FOREACH (const std::vector<char> &chunk, chunks)
       {
@@ -97,11 +98,11 @@ public:
     }
     else
     {
-      blob_ref.reset(new sqlite::Blob(_foreknown_blob_size));
-      sqlite::Blob *blob= blob_ref.get();
+      blob_ref.reset(new sqlite::blob_t(_foreknown_blob_size));
+      sqlite::blob_t *blob= blob_ref.get();
       is->read((char*)&(*blob)[0], _foreknown_blob_size);
-      if (is->gcount() != _foreknown_blob_size)
-        throw std::runtime_error(strfmt("BLOB size mismatch: server reports %i bytes, fetched %i bytes", (int)_foreknown_blob_size, is->gcount()));
+      if ((size_t)is->gcount() != _foreknown_blob_size)
+        throw std::runtime_error(strfmt("BLOB size mismatch: server reports %i bytes, fetched %i bytes", (int)_foreknown_blob_size, (int)is->gcount()));
       _foreknown_blob_size= -1;
     }
     return blob_ref;
@@ -159,8 +160,8 @@ void Recordset_cdbc_storage::do_unserialize(Recordset *recordset, sqlite::connec
   ColumnId editable_col_count= rs_meta->getColumnCount();
 
   // column types
-  static std::map<std::string, sqlite::Variant> known_types;
-  static std::map<std::string, sqlite::Variant> known_real_types;
+  static std::map<std::string, sqlite::variant_t> known_types;
+  static std::map<std::string, sqlite::variant_t> known_real_types;
   {
     struct Known_type_initializer
     {
@@ -169,7 +170,7 @@ void Recordset_cdbc_storage::do_unserialize(Recordset *recordset, sqlite::connec
       //! TODO: unify value range constraints
       Known_type_initializer(GRT *grt)
       {
-        known_types["BIT"]= sqlite::Unknown();
+        known_types["BIT"]= sqlite::unknown_t();
 
         known_types["ENUM"]= std::string();
         known_types["SET"]= std::string();
@@ -185,7 +186,7 @@ void Recordset_cdbc_storage::do_unserialize(Recordset *recordset, sqlite::connec
         known_types["FLOAT"]= std::string();//!ld;
         known_types["DOUBLE"]= std::string();//!ld;
 
-        known_types["NULL"]= sqlite::Null();
+        known_types["NULL"]= sqlite::null_t();
 
         known_types["TIMESTAMP"]= std::string();
         known_types["DATE"]= std::string();
@@ -193,10 +194,10 @@ void Recordset_cdbc_storage::do_unserialize(Recordset *recordset, sqlite::connec
         known_types["DATETIME"]= std::string();
         known_types["YEAR"]= int();
 
-        known_types["TINYBLOB"]= sqlite::BlobRef();
-        known_types["BLOB"]= sqlite::BlobRef();
-        known_types["MEDIUMBLOB"]= sqlite::BlobRef();
-        known_types["LONGBLOB"]= sqlite::BlobRef();
+        known_types["TINYBLOB"]= sqlite::blob_ref_t();
+        known_types["BLOB"]= sqlite::blob_ref_t();
+        known_types["MEDIUMBLOB"]= sqlite::blob_ref_t();
+        known_types["LONGBLOB"]= sqlite::blob_ref_t();
 
         known_types["TINYTEXT"]= std::string();
         known_types["TEXT"]= std::string();
@@ -206,22 +207,9 @@ void Recordset_cdbc_storage::do_unserialize(Recordset *recordset, sqlite::connec
         known_types["VARCHAR"]= std::string();
         known_types["CHAR"]= std::string();
 
-        known_types["VARBINARY"]= sqlite::BlobRef();
-        known_types["BINARY"]= sqlite::BlobRef();
-        DictRef options= DictRef::cast_from(grt->get("/wb/options/options"));
-        if (options.is_valid())
-        {
-          bool treat_binary_as_text= (options.get_int("DbSqlEditor:MySQL:TreatBinaryAsText", 0) != 0);
-          if (treat_binary_as_text)
-          {
-            known_types["VARBINARY"]= std::string();
-            known_types["BINARY"]= std::string();
-          }
-        }
+        known_types["GEOMETRY"]= sqlite::unknown_t();
 
-        known_types["GEOMETRY"]= sqlite::Unknown();
-
-        known_types["UNKNOWN"]= sqlite::Unknown();
+        known_types["UNKNOWN"]= sqlite::unknown_t();
 
         known_real_types= known_types;
         {
@@ -236,10 +224,46 @@ void Recordset_cdbc_storage::do_unserialize(Recordset *recordset, sqlite::connec
 
           known_real_types["FLOAT"]= ld;
           known_real_types["DOUBLE"]= ld;
+          
+          known_real_types["VARBINARY"]= sqlite::blob_ref_t();
+          known_real_types["BINARY"]= sqlite::blob_ref_t();
         }
       }
     };
     static Known_type_initializer known_type_initializer(_grtm->get_grt());
+
+    // assign these here since they can change
+    known_types["VARBINARY"]= sqlite::blob_ref_t();
+    known_types["BINARY"]= sqlite::blob_ref_t();
+    DictRef options= DictRef::cast_from(_grtm->get_grt()->get("/wb/options/options"));
+    if (options.is_valid())
+    {
+      bool treat_binary_as_text= (options.get_int("DbSqlEditor:MySQL:TreatBinaryAsText", 0) != 0);
+      if (treat_binary_as_text)
+      {
+        known_types["VARBINARY"]= std::string();
+        known_types["BINARY"]= std::string();
+      }
+    }
+  }
+
+  if (_gather_field_info)
+  {
+    std::stringstream ss;
+    for (ColumnId i = 0; i < editable_col_count; ++i)
+    {
+      ss << "Field " << i+1 << ": " << rs_meta->getColumnLabel(i+1) << "\n";
+      ss << "Catalog: " << rs_meta->getCatalogName(i+1) << "\n";
+      ss << "Schema: " << rs_meta->getSchemaName(i+1) << "\n";
+      ss << "Table: " << rs_meta->getTableName(i+1) << "\n";
+      ss << "Type: " << rs_meta->getColumnTypeName(i+1) << "\n";
+//      ss << "Collation:" << "\n";
+      ss << "Display size: " << rs_meta->getColumnDisplaySize(i+1) << "\n";
+      ss << "Precision: " << rs_meta->getPrecision(i+1) << "\n";
+      ss << "Scale: " << rs_meta->getScale(i+1) << "\n";
+      ss << "\n";
+    }
+    _field_info = ss.str();
   }
 
   column_types.reserve(editable_col_count);
@@ -273,17 +297,18 @@ void Recordset_cdbc_storage::do_unserialize(Recordset *recordset, sqlite::connec
     _pkey_columns.reserve(rowid_col_count);
     column_names.reserve(editable_col_count + rowid_col_count);
     column_types.reserve(editable_col_count + rowid_col_count);
+    real_column_types.reserve(editable_col_count + rowid_col_count);
     while (rs->next())
     {
       Recordset::Column_names::const_iterator i=
         std::find(column_names.begin(), column_names.end(), rs->getString("COLUMN_NAME"));
       if (i != column_names.end())
       {
-      ColumnId col= std::distance((Recordset::Column_names::const_iterator)column_names.begin(), i);
-      column_names.push_back(column_names[col]);
-      column_types.push_back(column_types[col]);
-      real_column_types.push_back(real_column_types[col]);
-      _pkey_columns.push_back(col); // copy original value of pk field(s)
+        ColumnId col= std::distance((Recordset::Column_names::const_iterator)column_names.begin(), i);
+        column_names.push_back(column_names[col]);
+        column_types.push_back(column_types[col]);
+        real_column_types.push_back(real_column_types[col]);
+        _pkey_columns.push_back(col); // copy original value of pk field(s)
       }
       else
         rowid_col_count--;
@@ -315,11 +340,11 @@ void Recordset_cdbc_storage::do_unserialize(Recordset *recordset, sqlite::connec
       {
         if (rs->isNull(n+1) || null_value_columns[n])
         {
-          row_values[n]= sqlite::Null();
+          row_values[n]= sqlite::null_t();
         }
         else
         {
-          sqlite::Variant index= (int)n+1;
+          sqlite::variant_t index= (int)n+1;
           row_values[n]= boost::apply_visitor(fetch_var, column_types[n], index);
         }
       }
@@ -340,7 +365,7 @@ void Recordset_cdbc_storage::do_unserialize(Recordset *recordset, sqlite::connec
 }
 
 
-void Recordset_cdbc_storage::do_fetch_blob_value(Recordset *recordset, sqlite::connection *data_swap_db, RowId rowid, ColumnId column, sqlite::Variant &blob_value)
+void Recordset_cdbc_storage::do_fetch_blob_value(Recordset *recordset, sqlite::connection *data_swap_db, RowId rowid, ColumnId column, sqlite::variant_t &blob_value)
 {
   sql::Dbc_connection_handler::ConnectionRef dbms_conn= dbms_conn_ref();
 
@@ -357,7 +382,7 @@ void Recordset_cdbc_storage::do_fetch_blob_value(Recordset *recordset, sqlite::c
     if (pkey_predicate.empty())
     {
       // no pk for recordset - blob fields are fetched along with other fields - hence this is a true NULL value rather than a placeholder
-      blob_value= sqlite::Null();
+      blob_value= sqlite::null_t();
       return;
     }
     sql_query= strfmt("select `%s`, length(`%s`) from (%s) t where %s",
@@ -378,15 +403,15 @@ void Recordset_cdbc_storage::do_fetch_blob_value(Recordset *recordset, sqlite::c
     FetchVar fetch_var(rs.get());
     while (rs->next())
     {
-      sqlite::Variant val;
+      sqlite::variant_t val;
       if (rs->isNull(1))
       {
-        blob_value= sqlite::Null();
+        blob_value= sqlite::null_t();
       }
       else
       {
         fetch_var.foreknown_blob_size(rs->getInt(2));
-        sqlite::Variant index= 1;
+        sqlite::variant_t index= 1;
         blob_value= boost::apply_visitor(fetch_var, column_types[column], index);
       }
     }
@@ -397,7 +422,7 @@ void Recordset_cdbc_storage::do_fetch_blob_value(Recordset *recordset, sqlite::c
 class BlobVarToStream : public boost::static_visitor<boost::shared_ptr<std::stringstream> >
 {
 public:
-  result_type operator()(const sqlite::BlobRef &v)
+  result_type operator()(const sqlite::blob_ref_t &v)
   {
     size_t data_length= v->size();
     std::string data((const char*)&(*v)[0], data_length);
@@ -427,7 +452,7 @@ void Recordset_cdbc_storage::run_sql_script(const Sql_script &sql_script)
       if (sql_script.statements_bindings.end() != sql_bindings)
       {
         int bind_var_index= 1;
-        BOOST_FOREACH (const sqlite::Variant &bind_var, *sql_bindings)
+        BOOST_FOREACH (const sqlite::variant_t &bind_var, *sql_bindings)
         {
           if (sqlide::is_var_null(bind_var))
           {
@@ -450,8 +475,8 @@ void Recordset_cdbc_storage::run_sql_script(const Sql_script &sql_script)
     catch (sql::SQLException &e)
     {
       ++err_count;
-      msg+= strfmt("%s\n%i: %s\n\n", sql.c_str(), e.getErrorCode(), e.what());
-      on_sql_script_run_error(e.getErrorCode(), e.what(), sql);
+      msg = strfmt("%i: %s", e.getErrorCode(), e.what());
+      on_sql_script_run_error(e.getErrorCode(), msg, sql);
     }
     ++processed_statement_count;
     progress_state+= progress_state_inc;
@@ -461,7 +486,7 @@ void Recordset_cdbc_storage::run_sql_script(const Sql_script &sql_script)
   if (err_count)
   {
     dbms_conn->rollback();
-    msg= strfmt("%i error(s) saving changes to table %s:\n\n%sRollback complete", err_count, full_table_name().c_str(), msg.c_str());
+    msg= strfmt("%i error(s) saving changes to table %s", err_count, full_table_name().c_str());
     on_sql_script_run_statistics((processed_statement_count - err_count), err_count);
     throw std::runtime_error(msg.c_str());
   }
