@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,6 +24,7 @@
 #include "recordset_be.h"
 #include "grtsqlparser/sql_facade.h"
 #include "base/string_utilities.h"
+#include "base/sqlstring.h"
 #include "base/boost_smart_ptr_helpers.h"
 #include <sqlite/query.hpp>
 #include <boost/foreach.hpp>
@@ -52,14 +53,14 @@ _qv(qv)
 std::string PrimaryKeyPredicate::operator()(std::vector<boost::shared_ptr<sqlite::result> > &data_row_results)
 {
   std::string predicate;
-  sqlite::Variant v;
+  sqlite::variant_t v;
   BOOST_FOREACH (ColumnId col, *_pkey_columns)
   {
     size_t partition;
     ColumnId partition_column= Recordset::translate_data_swap_db_column(col, &partition);
     boost::shared_ptr<sqlite::result> &data_row_rs= data_row_results[partition];
-
-    data_row_rs->get_variant(partition_column, v);
+    
+    v = data_row_rs->get_variant(partition_column);
     predicate+= "`" + (*_column_names)[col] + "`=" + boost::apply_visitor(*_qv, (*_column_types)[col], v) + " and";
   }
   if (!predicate.empty())
@@ -107,6 +108,7 @@ void Recordset_sql_storage::init_variant_quoter(sqlide::QuoteVar &qv) const
   }
   else
   {
+    // used for sqlite storage and others
     qv.escape_string= boost::bind(sqlide::QuoteVar::escape_ansi_sql_string, _1);
     // swap db (sqlite) stores unknown values as quoted strings
     qv.store_unknown_as_string= true;
@@ -142,9 +144,9 @@ void Recordset_sql_storage::do_unserialize(Recordset *recordset, sqlite::connect
     real_column_types.reserve(column_names.size());
     std::fill_n(std::back_inserter(real_column_types), column_names.size(), std::string());
 
-    //XXX CHECK if above code is ok, what is this supposed to do? is it even called ?
     column_quoting.reserve(column_names.size());
-    
+    std::fill_n(std::back_inserter(column_quoting), column_names.size(), true);
+
     if (!column_names.empty())
     {
       // data
@@ -225,7 +227,7 @@ void Recordset_sql_storage::load_insert_statement(
     if ((col_index_map.end() != i) && !null_fields[i->second])
       var_list->push_back(fields_values[i->second]);
     else
-      var_list->push_back(sqlite::Null());
+      var_list->push_back(sqlite::null_t());
   }
 }
 
@@ -252,9 +254,9 @@ void Recordset_sql_storage::do_apply_changes(const Recordset *recordset, sqlite:
 }
 
 
-void Recordset_sql_storage::fetch_blob_value(Recordset *recordset, sqlite::connection *data_swap_db, RowId rowid, ColumnId column, sqlite::Variant &blob_value)
+void Recordset_sql_storage::fetch_blob_value(Recordset *recordset, sqlite::connection *data_swap_db, RowId rowid, ColumnId column, sqlite::variant_t &blob_value)
 {
-  blob_value= sqlite::Null();
+  blob_value= sqlite::null_t();
 
   // first check if requested blob is already in cache
   {
@@ -265,7 +267,7 @@ void Recordset_sql_storage::fetch_blob_value(Recordset *recordset, sqlite::conne
     if (blob_query.emit())
     {
       boost::shared_ptr<sqlite::result> rs= blob_query.get_result();
-      rs->get_variant(0, blob_value);
+      blob_value = rs->get_variant(0);
     }
   }
 
@@ -276,7 +278,7 @@ void Recordset_sql_storage::fetch_blob_value(Recordset *recordset, sqlite::conne
 }
 
 
-void Recordset_sql_storage::do_fetch_blob_value(Recordset *recordset, sqlite::connection *data_swap_db, RowId rowid, ColumnId column, sqlite::Variant &blob_value)
+void Recordset_sql_storage::do_fetch_blob_value(Recordset *recordset, sqlite::connection *data_swap_db, RowId rowid, ColumnId column, sqlite::variant_t &blob_value)
 {
 }
 
@@ -328,13 +330,15 @@ void Recordset_sql_storage::get_pkey_predicate_for_data_cache_rowid(Recordset *r
   Recordset::prepare_partition_queries(data_swap_db, "select * from `data%s` where id=?", data_row_queries);
 
   std::vector<boost::shared_ptr<sqlite::result> > data_row_results(data_row_queries.size());
-  std::list<sqlite::Variant> bind_vars;
+  std::list<sqlite::variant_t> bind_vars;
   bind_vars.push_back((int)rowid);
   if (!Recordset::emit_partition_queries(data_swap_db, data_row_queries, data_row_results, bind_vars))
     return;
 
   sqlide::QuoteVar qv;
   init_variant_quoter(qv);
+  // turn blob values into hex strings when building a primary key, since we can't bind those
+  //  qv.blob_to_string= boost::bind(sqlide::QuoteVar::blob_to_hex_string, _1, _2);
   PrimaryKeyPredicate pkey_pred(&column_types, &column_names, &_pkey_columns, &qv);
   pkey_predicate= pkey_pred(data_row_results);
 }
@@ -342,6 +346,8 @@ void Recordset_sql_storage::get_pkey_predicate_for_data_cache_rowid(Recordset *r
 
 void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqlite::connection *data_swap_db, Sql_script &sql_script, bool is_update_script)
 {
+  // this one is used mostly internally, for eg.: sqlite inserts storage
+
   if (_is_sql_script_substitute_enabled)
   {
     sql_script= _sql_script_substitute;
@@ -359,6 +365,12 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
   ColumnId editable_col_count= recordset->get_column_count();
   sqlide::QuoteVar qv;
   init_variant_quoter(qv);
+
+  sqlide::QuoteVar pk_qv;
+  init_variant_quoter(pk_qv);
+  // turn blob values into hex strings when building a primary key, since we can't bind those
+  pk_qv.blob_to_string= boost::bind(sqlide::QuoteVar::blob_to_hex_string, _1, _2);
+
   std::string full_table_name= this->full_table_name();
 
   if (0 == editable_col_count)
@@ -370,7 +382,7 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
 
   if (is_update_script)
   {
-    PrimaryKeyPredicate pkey_pred(&column_types, &column_names, &_pkey_columns, &qv);
+    PrimaryKeyPredicate pkey_pred(&real_column_types, &column_names, &_pkey_columns, &pk_qv);
 
     std::list<boost::shared_ptr<sqlite::query> > data_row_queries(partition_count);
     Recordset::prepare_partition_queries(data_swap_db, "select * from `data%s` where id=?", data_row_queries);
@@ -415,7 +427,7 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
         case -1:
           {
             std::vector<boost::shared_ptr<sqlite::result> > deleted_row_results(deleted_row_queries.size());
-            std::list<sqlite::Variant> bind_vars;
+            std::list<sqlite::variant_t> bind_vars;
             bind_vars.push_back((int)rowid);
             if (Recordset::emit_partition_queries(data_swap_db, deleted_row_queries, deleted_row_results, bind_vars))
             {
@@ -428,7 +440,7 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
           {
             std::vector<boost::shared_ptr<sqlite::result> > data_row_results(data_row_queries.size());
             {
-              std::list<sqlite::Variant> bind_vars;
+              std::list<sqlite::variant_t> bind_vars;
               bind_vars.push_back((int)rowid);
               if (!Recordset::emit_partition_queries(data_swap_db, data_row_queries, data_row_results, bind_vars))
                 break;
@@ -444,7 +456,7 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
 
               std::string col_names;
               std::string values;
-              sqlite::Variant v;
+              sqlite::variant_t v;
 
               do
               {
@@ -455,11 +467,10 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
                 ColumnId partition_column= Recordset::translate_data_swap_db_column(column, &partition);
                 boost::shared_ptr<sqlite::result> &data_row_rs= data_row_results[partition];
 
-                data_row_rs->get_variant(partition_column, v);
+                v = data_row_rs->get_variant(partition_column);
+
                 values+= strfmt("%s, ",
-                  column_quoting[column] || sqlide::is_var_null(v)?
-                  boost::apply_visitor(qv, column_types[column], v).c_str():
-                  boost::apply_visitor(var_to_str, v).c_str());
+                    boost::apply_visitor(qv, column_types[column], v).c_str());
                 if (blob_columns[column])
                   sql_bindings.push_back(v);
               }
@@ -468,7 +479,6 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
                 col_names.resize(col_names.size()-2);
               if (!values.empty())
                 values.resize(values.size()-2);
-
               sql= strfmt("INSERT INTO %s (%s) VALUES (%s)", _omit_schema_qualifier ? (std::string("`")+table_name()+std::string("`")).c_str() : full_table_name.c_str(), col_names.c_str(), values.c_str());
             }
           }
@@ -478,7 +488,7 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
           {
             std::vector<boost::shared_ptr<sqlite::result> > data_row_results(data_row_queries.size());
             {
-              std::list<sqlite::Variant> bind_vars;
+              std::list<sqlite::variant_t> bind_vars;
               bind_vars.push_back((int)rowid);
               if (!Recordset::emit_partition_queries(data_swap_db, data_row_queries, data_row_results, bind_vars))
                 break;
@@ -491,7 +501,7 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
               boost::shared_ptr<sqlite::result> changed_row_columns_rs= changed_row_columns_query.get_result();
 
               std::string values;
-              sqlite::Variant v;
+              sqlite::variant_t v;
               do
               {
                 ColumnId column= changed_row_columns_rs->get_int(0);
@@ -500,19 +510,16 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
                 ColumnId partition_column= Recordset::translate_data_swap_db_column(column, &partition);
                 boost::shared_ptr<sqlite::result> &data_row_rs= data_row_results[partition];
 
-                data_row_rs->get_variant(partition_column, v);
+                v = data_row_rs->get_variant(partition_column);
                 values+= strfmt("`%s`=%s, ",
                   column_names[column].c_str(),
-                  column_quoting[column] || sqlide::is_var_null(v)?
-                  boost::apply_visitor(qv, column_types[column], v).c_str():
-                  boost::apply_visitor(var_to_str, v).c_str());
+                  boost::apply_visitor(qv, column_types[column], v).c_str());
                 if (blob_columns[column] && _binding_blobs)
                   sql_bindings.push_back(v);
               }
               while (changed_row_columns_rs->next_row());
               if (!values.empty())
                 values.resize(values.size()-2);
-
               sql= strfmt("UPDATE %s SET %s WHERE %s", full_table_name.c_str(), values.c_str(), pkey_pred(data_row_results).c_str());
             }
           }
@@ -541,7 +548,7 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
       bool next_row_exists= true;
       do
       {
-        sqlite::Variant v;
+        sqlite::variant_t v;
         std::string values;
         for (size_t partition= 0; partition < partition_count; ++partition)
         {
@@ -550,7 +557,7 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
             col_end= std::min<ColumnId>(editable_col_count, (partition + 1) * Recordset::DATA_SWAP_DB_TABLE_MAX_COL_COUNT); col < col_end; ++col)
           {
             ColumnId partition_column= col - col_begin;
-            data_rs->get_variant(partition_column, v);
+            v = data_rs->get_variant(partition_column);
             values+= strfmt("%s, ",
                   column_quoting[partition_column] || sqlide::is_var_null(v)?
                   boost::apply_visitor(qv, column_types[partition_column], v).c_str():
@@ -574,6 +581,8 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
 
 void Recordset_sql_storage::generate_inserts(const Recordset *recordset, sqlite::connection *data_swap_db, Sql_script &sql_script)
 {
+  // This one is used for generating inserts for export
+
   if (_is_sql_script_substitute_enabled)
   {
     sql_script= _sql_script_substitute;
@@ -599,46 +608,55 @@ void Recordset_sql_storage::generate_inserts(const Recordset *recordset, sqlite:
   for (ColumnId col= 0; editable_col_count > col; ++col)
     blob_columns[col]= sqlide::is_var_blob(real_column_types[col]);
 
-    std::string col_names;
-    for (ColumnId col= 0; editable_col_count > col; ++col)
-      col_names += strfmt("`%s`, ", column_names[col].c_str());
-    if (!col_names.empty())
-      col_names.resize(col_names.size()-2);
+  std::string col_names;
+  for (ColumnId col= 0; editable_col_count > col; ++col)
+    col_names += strfmt("`%s`, ", column_names[col].c_str());
+  if (!col_names.empty())
+    col_names.resize(col_names.size()-2);
 
-    std::list<boost::shared_ptr<sqlite::query> > data_queries(partition_count);
-    Recordset::prepare_partition_queries(data_swap_db, "select * from `data%s`", data_queries);
-    std::vector<boost::shared_ptr<sqlite::result> > data_results(data_queries.size());
-    if (Recordset::emit_partition_queries(data_swap_db, data_queries, data_results))
+  std::list<boost::shared_ptr<sqlite::query> > data_queries(partition_count);
+  Recordset::prepare_partition_queries(data_swap_db, "select * from `data%s`", data_queries);
+  std::vector<boost::shared_ptr<sqlite::result> > data_results(data_queries.size());
+  if (Recordset::emit_partition_queries(data_swap_db, data_queries, data_results))
+  {
+    bool next_row_exists= true;
+    do
     {
-      bool next_row_exists= true;
-      do
+      sqlite::variant_t v;
+      std::string values;
+      for (size_t partition= 0; partition < partition_count; ++partition)
       {
-        sqlite::Variant v;
-        std::string values;
-        for (size_t partition= 0; partition < partition_count; ++partition)
+        boost::shared_ptr<sqlite::result> &data_rs= data_results[partition];
+        for (ColumnId col_begin= partition * Recordset::DATA_SWAP_DB_TABLE_MAX_COL_COUNT, col= col_begin,
+          col_end= std::min<ColumnId>(editable_col_count, (partition + 1) * Recordset::DATA_SWAP_DB_TABLE_MAX_COL_COUNT); col < col_end; ++col)
         {
-          boost::shared_ptr<sqlite::result> &data_rs= data_results[partition];
-          for (ColumnId col_begin= partition * Recordset::DATA_SWAP_DB_TABLE_MAX_COL_COUNT, col= col_begin,
-            col_end= std::min<ColumnId>(editable_col_count, (partition + 1) * Recordset::DATA_SWAP_DB_TABLE_MAX_COL_COUNT); col < col_end; ++col)
+          ColumnId partition_column= col - col_begin;
+          v = data_rs->get_variant(partition_column);
+          sqlide::VarToStr var_to_str;
+
+          std::string value;
+          if (column_quoted[col] || sqlide::is_var_null(v))
+            value = boost::apply_visitor(qv, column_types[col], v);
+          else
           {
-            ColumnId partition_column= col - col_begin;
-            data_rs->get_variant(partition_column, v);
-            sqlide::VarToStr var_to_str;
-
-            values+= strfmt("%s, ",//boost::apply_visitor(var_to_str, v).c_str());
-                            column_quoted[col] || sqlide::is_var_null(v)?
-                            boost::apply_visitor(qv, column_types[col], v).c_str():
-                            boost::apply_visitor(var_to_str, v).c_str());
+            value = boost::apply_visitor(var_to_str, v);
+            static std::string func_escape = "\\func ";
+            static std::string func_escape_ape = "\\\\func ";
+            if (value.substr(0, func_escape.size()) == func_escape
+                || value.substr(0, func_escape_ape.size()) == func_escape_ape)
+              value = boost::apply_visitor(qv, column_types[col], v);
           }
+          values+= strfmt("%s, ", value.c_str());//boost::apply_visitor(var_to_str, v).c_str());
         }
-        if (!values.empty())
-          values.resize(values.size()-2);
-        std::string sql= strfmt("INSERT INTO %s (%s) VALUES (%s)", _omit_schema_qualifier ? (std::string("`")+table_name()+std::string("`")).c_str() : full_table_name.c_str(), col_names.c_str(), values.c_str());
-        sql_script.statements.push_back(sql);
-
-        BOOST_FOREACH (boost::shared_ptr<sqlite::result> &data_rs, data_results)
-          next_row_exists= data_rs->next_row();
       }
-      while (next_row_exists);
+      if (!values.empty())
+        values.resize(values.size()-2);
+      std::string sql= strfmt("INSERT INTO %s (%s) VALUES (%s)", _omit_schema_qualifier ? (std::string("`")+table_name()+std::string("`")).c_str() : full_table_name.c_str(), col_names.c_str(), values.c_str());
+      sql_script.statements.push_back(sql);
+
+      BOOST_FOREACH (boost::shared_ptr<sqlite::result> &data_rs, data_results)
+        next_row_exists= data_rs->next_row();
     }
+    while (next_row_exists);
   }
+}
