@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -54,10 +54,10 @@ private:
   bool _treat_blobnull_as_blob;
 
 public:
-  template<typename T> result_type operator()(const T &t, const sqlite::Null &v) const { return _null_icon; }
-  result_type operator()(const sqlite::Null &v) const { return _null_icon; }
-  result_type operator()(const sqlite::BlobRef &t, const sqlite::Null &v) const { return _treat_blobnull_as_blob ? _blob_icon : _null_icon; }
-  template<typename V> result_type operator()(const sqlite::BlobRef &t, const V &v) const { return _blob_icon; }
+  template<typename T> result_type operator()(const T &t, const sqlite::null_t &v) const { return _null_icon; }
+  result_type operator()(const sqlite::null_t &v) const { return _null_icon; }
+  result_type operator()(const sqlite::blob_ref_t &t, const sqlite::null_t &v) const { return _treat_blobnull_as_blob ? _blob_icon : _null_icon; }
+  template<typename V> result_type operator()(const sqlite::blob_ref_t &t, const V &v) const { return _blob_icon; }
 
   template<typename T, typename V>
   result_type operator()(const T &t, const V &v) const { return 0; }
@@ -83,14 +83,14 @@ _edited_field_col(-1)
 
 VarGridModel::~VarGridModel()
 {
+  _refresh_connection.disconnect();
+
   g_static_rec_mutex_free(&_data_mutex);
 
+  _data_swap_db.reset();
   // clean temporary file to prevent crowding of files
   if (!_data_swap_db_path.empty())
-  {
-    _data_swap_db.reset();
     g_remove(_data_swap_db_path.c_str());
-  }
 }
 
 
@@ -166,7 +166,7 @@ int VarGridModel::refresh_ui()
     return refresh_ui_cb();
   else
   {
-    _grtm->run_once_when_idle(refresh_ui_cb);
+    _refresh_connection = _grtm->run_once_when_idle(refresh_ui_cb);
     return 0;
   }
 }
@@ -182,7 +182,7 @@ class VarType
 {
 public:
   typedef VarGridModel::ColumnType result_type;
-  result_type operator()(const sqlite::BlobRef &) const { return VarGridModel::BlobType; }
+  result_type operator()(const sqlite::blob_ref_t &) const { return VarGridModel::BlobType; }
   result_type operator()(int) const { return VarGridModel::NumericType; }
   result_type operator()(const boost::int64_t &) const { return VarGridModel::NumericType; }
   result_type operator()(const long double &) const { return VarGridModel::FloatType; }
@@ -262,7 +262,7 @@ bool VarGridModel::is_field_null(const NodeId &node, int column)
 
 bool VarGridModel::set_field_null(const bec::NodeId &node, int column)
 {
-  return is_field_null(node, column) ? true : set_field(node, column, sqlite::Null());
+  return is_field_null(node, column) ? true : set_field(node, column, sqlite::null_t());
 }
 
 
@@ -271,10 +271,10 @@ IconId VarGridModel::get_field_icon(const NodeId &node, int column, IconSize siz
   GStaticRecMutexLock data_mutex UNUSED (_data_mutex);
 
   Cell cell;
-  static const sqlite::Variant null_value= sqlite::Null();
+  static const sqlite::variant_t null_value= sqlite::null_t();
   if ((column < 0) || (column + 1 >= (int) _column_types.size()))
     return 0;
-  const sqlite::Variant &var= get_cell(cell, node, column, false) ? *cell : null_value;
+  const sqlite::variant_t &var= get_cell(cell, node, column, false) ? *cell : null_value;
   return boost::apply_visitor(*_icon_for_val, _column_types[column], var);
 }
 
@@ -302,6 +302,14 @@ bool VarGridModel::get_field_repr(const NodeId &node, int column, std::string &v
   return get_field_repr_(node, column, value);
 }
 
+bool VarGridModel::get_field_repr_no_truncate(const bec::NodeId &node, int column, std::string &value)
+{
+  Cell cell;
+  bool res= get_cell(cell, node, column, false);
+  if (res)
+    value= boost::apply_visitor(_var_to_str_repr, *cell);
+  return res;  
+}
 
 bool VarGridModel::get_field_repr_(const NodeId &node, int column, std::string &value)
 {
@@ -320,14 +328,14 @@ bool VarGridModel::get_field_repr_(const NodeId &node, int column, std::string &
 }
 
 
-bool VarGridModel::get_field(const NodeId &node, int column, sqlite::Variant &value)
+bool VarGridModel::get_field(const NodeId &node, int column, sqlite::variant_t &value)
 {
   GStaticRecMutexLock data_mutex UNUSED (_data_mutex);
   return get_field_(node, column, value);
 }
 
 
-bool VarGridModel::get_field_(const NodeId &node, int column, sqlite::Variant &value)
+bool VarGridModel::get_field_(const NodeId &node, int column, sqlite::variant_t &value)
 {
   Cell cell;
   bool res= get_cell(cell, node, column, false);
@@ -398,7 +406,7 @@ bool VarGridModel::get_field_grt(const NodeId &node, int column, grt::ValueRef &
 }
 
 
-bool VarGridModel::set_field(const NodeId &node, int column, const sqlite::Variant &value)
+bool VarGridModel::set_field(const NodeId &node, int column, const sqlite::variant_t &value)
 {
   bool res= false;
 
@@ -430,29 +438,29 @@ bool VarGridModel::set_field(const NodeId &node, int column, const sqlite::Varia
 
 bool VarGridModel::set_field(const NodeId &node, int column, const std::string &value)
 {
-  return set_field(node, column, sqlite::Variant(value));
+  return set_field(node, column, sqlite::variant_t(value));
 }
 
 
 bool VarGridModel::set_field(const NodeId &node, int column, double value)
 {
-  return set_field(node, column, sqlite::Variant((long double)value));
+  return set_field(node, column, sqlite::variant_t((long double)value));
 }
 
 
 bool VarGridModel::set_field(const NodeId &node, int column, int value)
 {
-  return set_field(node, column, sqlite::Variant(value));
+  return set_field(node, column, sqlite::variant_t(value));
 }
 
 
 bool VarGridModel::set_field(const NodeId &node, int column, long long value)
 {
-  return set_field(node, column, sqlite::Variant((boost::int64_t)value));
+  return set_field(node, column, sqlite::variant_t((boost::int64_t)value));
 }
 
 
-void VarGridModel::add_column(const std::string &name, const sqlite::Variant &type)
+void VarGridModel::add_column(const std::string &name, const sqlite::variant_t &type)
 {
   _column_names.push_back(name);
   _column_types.push_back(type);
@@ -511,7 +519,7 @@ void VarGridModel::cache_data_frame(RowId center_row, bool force_reload)
 
     std::list<boost::shared_ptr<sqlite::query> > data_queries(partition_count);
     prepare_partition_queries(data_swap_db.get(), "select d.* from `data%s` d inner join `data_index` di on (di.`id`=d.`id`) order by di.`rowid` limit ? offset ?", data_queries);
-    std::list<sqlite::Variant> bind_vars;
+    std::list<sqlite::variant_t> bind_vars;
     bind_vars.push_back((int)row_count);
     bind_vars.push_back((int)_data_frame_begin);
     std::vector<boost::shared_ptr<sqlite::result> > data_results(data_queries.size());
@@ -520,7 +528,7 @@ void VarGridModel::cache_data_frame(RowId center_row, bool force_reload)
       bool next_row_exists= true;
 
       std::vector<bool> blob_columns(_column_count);
-      for (ColumnId col= 0; _column_count > (int) col; ++col)
+      for (ColumnId col= 0; _column_count > col; ++col)
         blob_columns[col]= sqlide::is_var_blob(_real_column_types[col]);
 
       _data.reserve(row_count * _column_count);
@@ -532,15 +540,15 @@ void VarGridModel::cache_data_frame(RowId center_row, bool force_reload)
           for (ColumnId col_begin= partition * DATA_SWAP_DB_TABLE_MAX_COL_COUNT, col= col_begin,
             col_end= std::min<ColumnId>(_column_count, (partition + 1) * DATA_SWAP_DB_TABLE_MAX_COL_COUNT); col < col_end; ++col)
           {
-            sqlite::Variant v;
+            sqlite::variant_t v;
             if (_optimized_blob_fetching && blob_columns[col])
             {
-              v= sqlite::Null();
+              v= sqlite::null_t();
             }
             else
             {
               ColumnId partition_column= col - col_begin;
-              data_rs->get_variant(partition_column, v);
+              v = data_rs->get_variant(partition_column);
               v= boost::apply_visitor(_var_cast, _column_types[col], v);
             }
             _data.push_back(v);
@@ -602,7 +610,7 @@ void VarGridModel::prepare_partition_queries(sqlite::connection *data_swap_db, c
 }
 
 
-bool VarGridModel::emit_partition_queries(sqlite::connection *data_swap_db, std::list<boost::shared_ptr<sqlite::query> > &queries, std::vector<boost::shared_ptr<sqlite::result> > &results, const std::list<sqlite::Variant> &bind_vars)
+bool VarGridModel::emit_partition_queries(sqlite::connection *data_swap_db, std::list<boost::shared_ptr<sqlite::query> > &queries, std::vector<boost::shared_ptr<sqlite::result> > &results, const std::list<sqlite::variant_t> &bind_vars)
 {
   bool no_results_returned= false;
   size_t partition= 0;
@@ -610,7 +618,7 @@ bool VarGridModel::emit_partition_queries(sqlite::connection *data_swap_db, std:
   {
     query->clear();
     sqlide::BindSqlCommandVar bind_sql_command_var(query.get());
-    BOOST_FOREACH (const sqlite::Variant &var, bind_vars)
+    BOOST_FOREACH (const sqlite::variant_t &var, bind_vars)
       boost::apply_visitor(bind_sql_command_var, var);
     if (!query->emit())
     {
@@ -624,14 +632,14 @@ bool VarGridModel::emit_partition_queries(sqlite::connection *data_swap_db, std:
 }
 
 
-void VarGridModel::emit_partition_commands(sqlite::connection *data_swap_db, size_t partition_count, const std::string &command_text_template, const std::list<sqlite::Variant> &bind_vars)
+void VarGridModel::emit_partition_commands(sqlite::connection *data_swap_db, size_t partition_count, const std::string &command_text_template, const std::list<sqlite::variant_t> &bind_vars)
 {
   for (size_t partition= 0; partition < partition_count; ++partition)  
   {
     std::string partition_suffix= data_swap_db_partition_suffix(partition);
     sqlite::command command(*data_swap_db, strfmt(command_text_template.c_str(), partition_suffix.c_str()));
     sqlide::BindSqlCommandVar bind_sql_command_var(&command);
-    BOOST_FOREACH (const sqlite::Variant &var, bind_vars)
+    BOOST_FOREACH (const sqlite::variant_t &var, bind_vars)
       boost::apply_visitor(bind_sql_command_var, var);
     command.emit();
   }
