@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,7 +18,6 @@
  */
 #include "stdafx.h"
 
-#include "wb_config.h"
 #include "workbench_physical_model_impl.h"
 #include "workbench_physical_diagram_impl.h"
 #include "workbench_physical_connection_impl.h"
@@ -128,6 +127,7 @@ std::string workbench_physical_Model::ImplData::get_line_end_caption(bool mand,
     break;
 
   case PRFromColumnNotation:
+    //TODO: if else has same code is this intentional
     if (start)
     {
       if (many) return "\xe2\x88\x9e";
@@ -145,6 +145,7 @@ std::string workbench_physical_Model::ImplData::get_line_end_caption(bool mand,
 
   case PRClassicNotation:
   case PRUMLNotation:
+    //TODO: if else has same code is this intentional
     if (start)
     {
       if (!mand && many) return "0..*";
