@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -34,8 +34,6 @@
 #include "base/string_utilities.h"
 
 using namespace base;
-using namespace MySQL::Geometry;
-using namespace MySQL::Drawing;
 
 workbench_physical_TableFigure::ImplData::ImplData(workbench_physical_TableFigure *self)
   : super(self), _figure(0)
@@ -293,11 +291,11 @@ void workbench_physical_TableFigure::ImplData::sync_columns()
         // truncate enum and set types if its too long
         if (max_type_length > 0 && (int) type.length() > max_type_length)
         {
-          if (g_strncasecmp(type.c_str(), "set(", 4) == 0)
+          if (g_ascii_strncasecmp(type.c_str(), "set(", 4) == 0)
           {
             type= type.substr(0, 3).append("(...)");
           }
-          else if (g_strncasecmp(type.c_str(), "enum(", 5) == 0)
+          else if (g_ascii_strncasecmp(type.c_str(), "enum(", 5) == 0)
           {
             type= type.substr(0, 4).append("(...)");
           }
@@ -366,17 +364,17 @@ void workbench_physical_TableFigure::ImplData::sync_triggers()
       db_TriggerRef trigger(triggers.get(i));
       std::string text;
       
-      if (g_strcasecmp(trigger->timing().c_str(), "AFTER") == 0)
+      if (g_ascii_strcasecmp(trigger->timing().c_str(), "AFTER") == 0)
         text = "AFT";
       else
         text = "BEF";
       
       std::string event = trigger->event();
-      if (g_strcasecmp(event.c_str(), "INSERT") == 0)
+      if (g_ascii_strcasecmp(event.c_str(), "INSERT") == 0)
         text.append(" INSERT ");
-      else if (g_strcasecmp(event.c_str(), "UPDATE") == 0)
+      else if (g_ascii_strcasecmp(event.c_str(), "UPDATE") == 0)
         text.append(" UPDATE ");
-      else if (g_strcasecmp(event.c_str(), "DELETE") == 0)
+      else if (g_ascii_strcasecmp(event.c_str(), "DELETE") == 0)
         text.append(" DELETE ");
       
       text.append(trigger->name());
@@ -482,7 +480,7 @@ bool workbench_physical_TableFigure::ImplData::realize()
 
     std::string sfont= self()->owner()->owner()->get_data()->get_string_option(strfmt("%s:SectionFont", self()->class_name().c_str()), "");
     if (!sfont.empty())
-      _figure->set_section_font(parse_font_spec(sfont));
+      _figure->set_section_font(mdc::FontSpec::from_string(sfont));
 
     _figure->toggle(*self()->_expanded!=0);
     _figure->toggle_indexes(*self()->_indicesExpanded!=0);
