@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -240,7 +240,7 @@ PythonDebugger *PythonDebugger::from_cobject(PyObject *cobj)
 PythonDebugger::PythonDebugger(GRTShellWindow *shell, mforms::TabView *tabview)
 : _shell(shell), _lower_tabs(tabview), _stack_position_editor(0), _stack_position_line(0)
 {
-  _breakpoint_list= manage(new TreeView(TreeDefault));
+  _breakpoint_list= manage(new TreeNodeView(TreeDefault|TreeFlatList));
 //  _breakpoint_list->add_column(CheckColumnType, " ", 30, true);
   _breakpoint_list->add_column(StringColumnType, "Breakpoint", 200, false);
   _breakpoint_list->add_column(StringColumnType, "Location", 200, false);
@@ -253,7 +253,7 @@ PythonDebugger::PythonDebugger(GRTShellWindow *shell, mforms::TabView *tabview)
   
   _lower_tabs->add_page(spl, _("Debug Info"));
 
-  _stack_list= manage(new TreeView(TreeDefault));
+  _stack_list= manage(new TreeNodeView(TreeDefault|TreeFlatList));
   _stack_list->add_column(StringColumnType, "#", 30, false);
   _stack_list->add_column(StringColumnType, "Stack Location", 300, false);
   _stack_list->add_column(StringColumnType, "File", 300, false);
@@ -262,7 +262,7 @@ PythonDebugger::PythonDebugger(GRTShellWindow *shell, mforms::TabView *tabview)
   scoped_connect(_stack_list->signal_changed(),boost::bind(&PythonDebugger::stack_selected, this));
   //_stack_list->signal_row_activated().connect();
   
-  _variable_list= manage(new TreeView(TreeDefault));
+  _variable_list= manage(new TreeNodeView(TreeDefault|TreeFlatList));
   _variable_list->add_column(StringColumnType, "Variable", 200, false);
   _variable_list->add_column(StringColumnType, "Value", 400, false);
   _variable_list->end_columns();
@@ -377,9 +377,10 @@ void PythonDebugger::editor_text_changed(int line, int linesAdded, GRTCodeEditor
 }
 
 
-void PythonDebugger::edit_breakpoint(int row, int column, std::string value)
+void PythonDebugger::edit_breakpoint(mforms::TreeNodeRef node, int column, std::string value)
 {
-  if (column == 2) // edit bp condition
+  int row = _breakpoint_list->row_for_node(node);
+  if (column == 2 && row >= 0) // edit bp condition
   {
     grt::WillEnterPython lock;
     grt::AutoPyObject r(PyObject_CallMethod(_pdb, (char*)"wdb_set_bp_condition", (char*)"(is)", row, value.c_str()), 
@@ -395,7 +396,7 @@ void PythonDebugger::edit_breakpoint(int row, int column, std::string value)
     
     if (PyBool_Check(r) && r == Py_True)
     {
-      _breakpoint_list->set(row, column, value);
+      node->set_string(column, value);
     }
   }
 }
@@ -420,19 +421,27 @@ void PythonDebugger::run(GRTCodeEditor *editor, bool stepping)
 
   _pause_clicked = false;
   // run the script
-  grt::AutoPyObject r(PyObject_CallMethod(_pdb, (char*)"wdb_run", (char*)"(si)", editor->get_path().c_str(), stepping?1:0), 
-                      false);
-  if (!r)
+  try 
+  {
+    grt::AutoPyObject r(PyObject_CallMethod(_pdb, (char*)"wdb_run", (char*)"(si)", editor->get_path().c_str(), stepping?1:0), 
+                        false);
+    if (!r)
+    {
+      // exception while running, dump the exception to the debugger console
+      // exceptions in the script being debugged will be caught and reported by wdb_run
+      debug_print("There was an unhandled internal exception executing the script.\n");
+      PyErr_Print();
+      PyErr_Clear();
+    }
+  }
+  catch (std::exception &e)
   {
-    // exception while running, dump the exception to the debugger console
-    // exceptions in the script being debugged will be caught and reported by wdb_run
     debug_print("There was an unhandled internal exception executing the script.\n");
-    PyErr_Print();
-    PyErr_Clear();
+    debug_print(e.what());
   }
 
-  _variable_list->clear_rows();
-  _stack_list->clear_rows();
+  _variable_list->clear();
+  _stack_list->clear();
   
   debug_print("Execution finished\n");
 }
@@ -481,7 +490,7 @@ void PythonDebugger::debug_print(const std::string &s)
 
 void PythonDebugger::ui_clear_breakpoints()
 {
-  _breakpoint_list->clear_rows();
+  _breakpoint_list->clear();
 }
 
 
@@ -489,17 +498,15 @@ void PythonDebugger::ui_add_breakpoint(const char *file, int line, const char *c
 {
   if (!file)
     file = "";
-  else
-    file = g_path_get_basename(file);
     
   if (!condition)
     condition = "";
   
-  int row = _breakpoint_list->add_row();
+  mforms::TreeNodeRef node = _breakpoint_list->add_node();
   //_breakpoint_list->set(row, 0, true);
-  _breakpoint_list->set(row, 0, base::strfmt("%s:%i", file, line));
-  _breakpoint_list->set(row, 1, "");
-  _breakpoint_list->set(row, 2, condition);
+  node->set_string(0, base::strfmt("%s:%i", base::basename(file).c_str(), line));
+  node->set_string(1, "");
+  node->set_string(2, condition);
 }
 
 
@@ -577,7 +584,7 @@ const char *PythonDebugger::ui_program_stopped(const char *file, int line, int r
   
   if (reason != 5)
   {
-    _stack_list->set_selected(0);
+    _stack_list->select_node(_stack_list->node_at_row(0));
     stack_selected();
   }
   
@@ -671,7 +678,7 @@ bool PythonDebugger::heartbeat_timeout()
 
 void PythonDebugger::ui_clear_stack()
 {
-  _stack_list->clear_rows();
+  _stack_list->clear();
 }
 
 
@@ -680,29 +687,29 @@ void PythonDebugger::ui_add_stack(const char *location, const char *file, int li
   if (!file)
     file = "";
   
-  int row = _stack_list->add_row();
-  _stack_list->set_row_tag(row, file);  
-  _stack_list->set(row, 0, row);
-  _stack_list->set(row, 1, location);
-  _stack_list->set(row, 2, base::strfmt("%s:%i", g_path_get_basename(file), line));
+  mforms::TreeNodeRef node = _stack_list->add_node();
+  node->set_tag(file);
+  node->set_int(0, _stack_list->row_for_node(node));
+  node->set_string(1, location);
+  node->set_string(2, base::strfmt("%s:%i", base::basename(file).c_str(), line));
 }
 
 
 void PythonDebugger::ui_clear_variables()
 {
-  _variable_list->clear_rows();
+  _variable_list->clear();
 }
 
 void PythonDebugger::ui_add_variable(const char *varname, const char *value)
 {
-  int row = _variable_list->add_row();
-  _variable_list->set(row, 0, varname);
-  _variable_list->set(row, 1, value);
+  mforms::TreeNodeRef node = _variable_list->add_node();
+  node->set_string(0, varname);
+  node->set_string(1, value);
 }
 
 void PythonDebugger::stack_selected()
 {
-  int i = _stack_list->get_selected();
+  mforms::TreeNodeRef node(_stack_list->get_selected_node());
   int show_frame = 0;
 
   if (_stack_position_editor)
@@ -710,14 +717,14 @@ void PythonDebugger::stack_selected()
     _stack_position_editor->get_editor()->remove_markup(LineMarkupCurrent, _stack_position_line);
     _stack_position_editor = 0;
     _stack_position_line = 0;
-    _variable_list->clear_rows();
+    _variable_list->clear();
   }
   
-  if (i >= 0)
+  if (node)
   {
-    std::string location = _stack_list->get_string(i, 2);
+    std::string location = node->get_string(2);
     std::string::size_type pos = location.rfind(':');
-    std::string file = _stack_list->get_row_tag(i);
+    std::string file = node->get_tag();
     int line = atoi(location.substr(pos+1).c_str());
     
     if (!file.empty() && line > 0)
@@ -728,7 +735,7 @@ void PythonDebugger::stack_selected()
       _stack_position_editor = editor;
       _stack_position_line = line-1;
     }
-    show_frame = -1*(i+1);
+    show_frame = -1*(_stack_list->row_for_node(node)+1);
   }
   
   grt::WillEnterPython lock;
