@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,20 +18,31 @@
  */
 
 #include "stdafx.h"
-#include "workbench/wb_context_ui.h"
-#include "user_defined_type_editor.h"
-#include "grtui/string_list_editor.h"
+
 #include "base/util_functions.h"
 #include "base/string_utilities.h"
+#include "base/ui_form.h"
+
+#include "mforms/utilities.h"
 #include "mforms/checkbox.h"
 
+#include "grt/editor_base.h"
+
+#include "workbench/wb_context.h"
+#include "workbench/wb_context_ui.h"
+
+#include "user_defined_type_editor.h"
+#include "grtui/string_list_editor.h"
+#include "model/wb_context_model.h"
+
 using namespace base;
 
 UserDefinedTypeEditor::UserDefinedTypeEditor(wb::WBContextUI *wbui, const workbench_physical_ModelRef &model)
 : mforms::Form(0, mforms::FormResizable), _wbui(wbui), _model(model),
-_vbox(false), _type_list(mforms::TreeAllowReorderRows), _args_box(true), _flags_box(false), _button_box(true)
+_vbox(false), _type_list(mforms::TreeAllowReorderRows|mforms::TreeFlatList), _args_box(true), _flags_box(false), _button_box(true)
 {
   set_title(_("User Defined Types"));
+  set_name("user_type_editor");
   
   set_content(&_vbox);
   _vbox.set_spacing(8);
@@ -76,38 +87,38 @@ _vbox(false), _type_list(mforms::TreeAllowReorderRows), _args_box(true), _flags_
   _table.add(&_flags_box, 1, 2, 3, 4, mforms::HFillFlag|mforms::VFillFlag|mforms::HExpandFlag);
   _add_button.enable_internal_padding(true);
   _delete_button.enable_internal_padding(true);
-  _apply_button.enable_internal_padding(true);
-  _close_button.enable_internal_padding(true);
+  _ok_button.enable_internal_padding(true);
+  _cancel_button.enable_internal_padding(true);
 
   _button_box.set_padding(12);
-  _button_box.set_spacing(12);
+  _button_box.set_spacing(8);
   
   _add_button.set_text(_("Add"));
-  _add_button.set_size(75, -1);
+  _add_button.set_size(80, -1);
   _button_box.add(&_add_button, false, false);
   _delete_button.set_text(_("Delete"));
-  _delete_button.set_size(75, -1);
+  _delete_button.set_size(80, -1);
   _button_box.add(&_delete_button, false, false);
   
-  _apply_button.set_text(_("Apply"));
-  _apply_button.set_size(75, -1);
-  _button_box.add_end(&_apply_button, false, false);
-  _close_button.set_text(_("Close"));
-  _close_button.set_size(75, -1);
-  _button_box.add_end(&_close_button, false, false);
+  _ok_button.set_text(_("OK"));
+  _ok_button.set_size(80, -1);
+  _button_box.add_end(&_ok_button, false, false);
+  _cancel_button.set_text(_("Cancel"));
+  _cancel_button.set_size(80, -1);
+  _button_box.add_end(&_cancel_button, false, false);
   
   set_size(480, 480);
   center();
   
   refresh();
   if (_type_list.count() > 0)
-    _type_list.set_selected(0);
+    _type_list.select_node(_type_list.root_node()->get_child(0));
   
   scoped_connect(_add_button.signal_clicked(),boost::bind(&UserDefinedTypeEditor::add_clicked, this));
   scoped_connect(_delete_button.signal_clicked(),boost::bind(&UserDefinedTypeEditor::delete_clicked, this));
 
-  scoped_connect(_apply_button.signal_clicked(),boost::bind(&UserDefinedTypeEditor::apply_clicked, this));
-  scoped_connect(_close_button.signal_clicked(),boost::bind(&UserDefinedTypeEditor::close_clicked, this));
+  scoped_connect(_ok_button.signal_clicked(),boost::bind(&UserDefinedTypeEditor::ok_clicked, this));
+  scoped_connect(_cancel_button.signal_clicked(),boost::bind(&UserDefinedTypeEditor::cancel_clicked, this));
 
   scoped_connect(_name.signal_changed(),boost::bind(&UserDefinedTypeEditor::name_changed, this));
   scoped_connect(_args.signal_changed(),boost::bind(&UserDefinedTypeEditor::args_changed, this));
@@ -131,16 +142,16 @@ void UserDefinedTypeEditor::refresh()
   grt::ListRef<db_UserDatatype> udts(_model->catalog()->userDatatypes());
     
   _user_types.clear();
-  _type_list.clear_rows();
+  _type_list.clear();
   for (grt::ListRef<db_UserDatatype>::const_iterator t= udts.begin(); t != udts.end(); ++t)
   {
     // skip system entries
     if (!g_str_has_prefix((*t).id().c_str(), "com.mysql.rdbms.mysql.userdatatype"))
     {
-      int row= _type_list.add_row();
-      _type_list.set(row, 0, (*t)->name().c_str());
-      _type_list.set(row, 1, (*t)->sqlDefinition().c_str());
-      _type_list.set(row, 2, (*t)->flags().c_str());
+      mforms::TreeNodeRef node = _type_list.add_node();
+      node->set_string(0, (*t)->name().c_str());
+      node->set_string(1, (*t)->sqlDefinition().c_str());
+      node->set_string(2, (*t)->flags().c_str());
 
       _user_types.push_back(*t);
     }
@@ -150,8 +161,8 @@ void UserDefinedTypeEditor::refresh()
 
 void UserDefinedTypeEditor::selected_row()
 {
-  int row= _type_list.get_selected();
-  if (row < 0)
+  mforms::TreeNodeRef node= _type_list.get_selected_node();
+  if (!node)
   {
     for (std::vector<mforms::CheckBox*>::iterator iter= _flags.begin();
          iter != _flags.end(); ++iter)
@@ -172,13 +183,13 @@ void UserDefinedTypeEditor::selected_row()
   }
   else
   {
-    _name.set_value(_type_list.get_string(row, 0));
+    _name.set_value(node->get_string(0));
     _name.set_enabled(true);    
     _args.set_enabled(true);
     _type.set_enabled(true);
     
-    std::string typespec= _type_list.get_string(row, 1);
-    std::vector<std::string> flags= base::split(_type_list.get_string(row, 2), " ");
+    std::string typespec= node->get_string(1);
+    std::vector<std::string> flags= base::split(node->get_string(2), " ");
     
     std::string::size_type arg_start= typespec.find('(');
     std::string type_name;
@@ -217,7 +228,7 @@ void UserDefinedTypeEditor::selected_row()
     {
       for (std::vector<std::string>::const_iterator fl= flags.begin(); fl != flags.end(); ++fl)
       {
-        if (g_strcasecmp(fl->c_str(), (*iter).c_str()) == 0)
+        if (g_ascii_strcasecmp(fl->c_str(), (*iter).c_str()) == 0)
         {
           _flags[i]->set_active(true);
           break;
@@ -242,7 +253,9 @@ void UserDefinedTypeEditor::flag_toggled()
     }
   }
   
-  _type_list.set(_type_list.get_selected(), 2, flags);
+  mforms::TreeNodeRef node = _type_list.get_selected_node();
+  if (node)
+    node->set_string(2, flags);
 }
 
 
@@ -297,17 +310,22 @@ void UserDefinedTypeEditor::type_changed()
 
 void UserDefinedTypeEditor::add_clicked()
 {
-  int row= _type_list.add_row();
-  _type_list.set(row, 0, "usertype");
-  _type_list.set(row, 1, "INT(11)");
-  _type_list.set_selected(row);
+  mforms::TreeNodeRef node = _type_list.add_node();
+  node->set_string(0, "usertype");
+  node->set_string(1, "INT(11)");
+  _type_list.select_node(node);
   selected_row();
   
   _user_types.push_back(db_UserDatatypeRef());
 }
 
+//--------------------------------------------------------------------------------------------------
 
-bool UserDefinedTypeEditor::check_if_type_used(const db_UserDatatypeRef &type)
+/**
+ * Checks all currently defined tables in the model to see if the given column is used in any of them.
+ * Returns true if so, otherwise false.
+ */
+bool UserDefinedTypeEditor::is_type_used(const db_UserDatatypeRef &type)
 {
   grt::ListRef<db_Schema> schemata(_model->catalog()->schemata());
   for (grt::ListRef<db_Schema>::const_iterator sche= schemata.begin(); sche != schemata.end(); ++sche)
@@ -327,23 +345,25 @@ bool UserDefinedTypeEditor::check_if_type_used(const db_UserDatatypeRef &type)
   return false;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void UserDefinedTypeEditor::delete_clicked()
 {
-  int row= _type_list.get_selected();
-  if (row >= 0)
+  mforms::TreeNodeRef node = _type_list.get_selected_node();
+  if (node)
   {
-    if (_user_types[row].is_valid() && !check_if_type_used(_user_types[row]))
+    int row = _type_list.get_selected_row();
+    if (_user_types[row].is_valid() && is_type_used(_user_types[row]))
     {
-      mforms::Utilities::show_error(_("Delete User Type"), 
-                                    strfmt(_("Type '%s' is used in a column and cannot be deleted."),
-                                    _type_list.get_string(row, 0).c_str()), _("OK"));
+      mforms::Utilities::show_error(_("Delete User Type"),
+        strfmt(_("Type '%s' is used in a column and cannot be deleted."), node->get_string(0).c_str()),
+        _("OK"));
       
       return;
     }
-    _type_list.delete_row(row);
+    node->remove_from_parent();
     
-    _user_types.erase(_user_types.begin()+row);
+    _user_types.erase(_user_types.begin() + row);
   }
   selected_row();
 }
@@ -363,7 +383,7 @@ static bool is_missing(const grt::ObjectRef &item, const std::vector<db_UserData
 }
 
 
-void UserDefinedTypeEditor::apply_clicked()
+void UserDefinedTypeEditor::ok_clicked()
 {
   // check if everything is ok and then commit
   
@@ -371,7 +391,6 @@ void UserDefinedTypeEditor::apply_clicked()
   std::set<std::string> systemNames;
   
   grt::ListRef<db_UserDatatype> udts(_model->catalog()->userDatatypes());
-  int old_count = udts->count();
   for (grt::ListRef<db_UserDatatype>::const_iterator udt= udts.begin(); udt != udts.end(); ++udt)
   {
     if (g_str_has_prefix((*udt).id().c_str(), "com.mysql.rdbms.mysql.userdatatype"))
@@ -386,7 +405,7 @@ void UserDefinedTypeEditor::apply_clicked()
   // check for duplicate or invalid names
   for (int c= _type_list.count(), i= 0; i < c; i++)
   {
-    std::string name= _type_list.get_string(i, 0);
+    std::string name= _type_list.root_node()->get_child(i)->get_string(0);
     bool ok= true;
    
     for (std::string::size_type i= 0; i < name.size(); i++)
@@ -432,18 +451,21 @@ void UserDefinedTypeEditor::apply_clicked()
   for (size_t i= 0; i < _user_types.size(); i++)
   {
     db_UserDatatypeRef type;
+    mforms::TreeNodeRef node(_type_list.node_at_row(i));
+    if (!node)
+      continue;
     
     if (!_user_types[i].is_valid())
       type= db_UserDatatypeRef(udts.get_grt());
     else
       type= _user_types[i];
     
-    if (*type->name() != _type_list.get_string(i, 0))
-      type->name(_type_list.get_string(i, 0));
+    if (*type->name() != node->get_string(0))
+      type->name(node->get_string(0));
     
-    if (*type->sqlDefinition() != _type_list.get_string(i, 1))
+    if (*type->sqlDefinition() != node->get_string(1))
     {
-      std::string typespec= _type_list.get_string(i, 1);
+      std::string typespec= node->get_string(1);
       type->sqlDefinition(typespec);
       std::string::size_type arg_start= typespec.find('(');
       std::string type_name;
@@ -461,8 +483,8 @@ void UserDefinedTypeEditor::apply_clicked()
       if (type->actualType() != simpleType)
         type->actualType(simpleType);
     }
-    if (*type->flags() != _type_list.get_string(i, 2))
-      type->flags(_type_list.get_string(i, 2));
+    if (*type->flags() != node->get_string(2))
+      type->flags(node->get_string(2));
   
     // this is a new type
     if (!_user_types[i].is_valid())
@@ -471,11 +493,14 @@ void UserDefinedTypeEditor::apply_clicked()
   
   undo.end_or_cancel_if_empty(_("Edit User Defined Types"));
 
-  _wbui->get_usertypes_tree()->tree_changed(old_count);
+  if (_wbui->get_wb()->get_model_context())
+    _wbui->get_wb()->get_model_context()->_udt_list_changed();
+
+  close();
 }
 
 
-void UserDefinedTypeEditor::close_clicked()
+void UserDefinedTypeEditor::cancel_clicked()
 {
   close();
 }
@@ -483,7 +508,9 @@ void UserDefinedTypeEditor::close_clicked()
 
 void UserDefinedTypeEditor::name_changed()
 {
-  _type_list.set(_type_list.get_selected(), 0, _name.get_string_value());
+  mforms::TreeNodeRef node = _type_list.get_selected_node();
+  if (node)
+    node->set_string(0, _name.get_string_value());
 }
 
 
@@ -491,11 +518,15 @@ void UserDefinedTypeEditor::args_changed()
 {
   std::string typespec= _type.get_string_value();
   std::string args= _args.get_string_value();
+  mforms::TreeNodeRef node = _type_list.get_selected_node();
   
-  if (args.empty())
-    _type_list.set(_type_list.get_selected(), 1, typespec);
-  else
-    _type_list.set(_type_list.get_selected(), 1, typespec+"("+args+")");
+  if (node)
+  {
+    if (args.empty())
+      node->set_string(1, typespec);
+    else
+      node->set_string(1, typespec+"("+args+")");
+  }
 }
 
 
