@@ -1,3 +1,22 @@
+/*
+ * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
+
 #ifndef __ACTIVE_LABEL_H__
 #define __ACTIVE_LABEL_H__
 
@@ -22,7 +41,9 @@ class ActiveLabel : public Gtk::HBox
     void set_text(const std::string& lbl);
     std::string get_text() const {return _text_label.get_text();}
 
-    mforms::Menu* get_menu() {return &_menu;}
+    mforms::Menu* get_menu() {return _menu;}
+    void set_menu(mforms::Menu* m);
+    bool has_menu();
     void start_busy();
     void stop_busy();
 
@@ -36,7 +57,7 @@ class ActiveLabel : public Gtk::HBox
     Gtk::Label                         _label;
     Gtk::EventBox                      _text_label_eventbox;
     Gtk::Label                         _text_label;
-    mforms::Menu                       _menu;
+    mforms::Menu                       *_menu;
     #if GTK_VERSION_GE(2,20)
     Gtk::Spinner                       _spinner;
     #endif
