@@ -22,7 +22,7 @@ Glib::RefPtr<Gdk::Pixbuf> ImageCache::image_from_path(const std::string &path, b
 
   if (!path.empty())
   {
-    bec::GStaticMutexLock lock(_sync);
+    base::GStaticMutexLock lock(_sync);
     ImageMap::iterator it = _images.find(path);
     if ( _images.end() != it )
       im = it->second;
