@@ -2,7 +2,6 @@
 #include "gtk_helpers.h"
 #include <gtkmm/cellrenderercombo.h>
 #include <gtkmm/icontheme.h>
-#include "custom_renderers.h"
 #include "grt/common.h"
 
 //#define DEBUG
@@ -35,13 +34,11 @@ static void run_menu_and_forward_action(const bec::MenuItemList& items
                                         ,const sigc::slot<void, const std::string&, const std::vector<bec::NodeId>&> fe_menu_handler
                                         ,Gtk::Menu *menu)
 {
-  if (items.size() > 0)
-  {
+  if (!items.empty())
     run_popup_menu(items
                    ,time
                    ,sigc::bind(sigc::ptr_fun(process_menu_actions), model, nodes, fe_menu_handler)
                    ,menu);
-  }
 }
 
 
@@ -76,7 +73,7 @@ Index::Index(GtkTreeIter* it, const bec::NodeId& node)
   }
   else if (m == ListNode)
   {
-    it->user_data = (void*)(node[0]);
+    it->user_data = (void*)(intptr_t)(node[0]);
   }
 }
 
@@ -506,22 +503,29 @@ void ColumnsModel::add_model_column(Gtk::TreeModelColumnBase *col, int bec_tm_id
 
 //------------------------------------------------------------------------------
 //------------------------------------------------------------------------------
+
 ListModelWrapper::ListModelWrapper(bec::ListModel* tm, Gtk::TreeView *treeview, const std::string& name)
                     : Glib::ObjectBase(typeid(ListModelWrapper))
                     , Glib::Object()
                     , Gtk::TreeModel()
-                    , _tm(tm)
                     , _treeview(treeview)
                     , _iconview(0)
                     , _context_menu(0)
                     , _stamp(1)
                     , _columns(this, treeview)
                     , _icon_size(bec::Icon16)
-                    , _self_ref(new ListModelWrapper*(this))
                     , _name(name)
 {
+  _invalid = false;
+
   scoped_connect(tm->tree_changed_signal(), boost::bind(&::ListModelWrapper::model_changed, this, _1, _2));
-  tm->add_destroy_notify_callback(_self_ref, &ListModelWrapper::on_bec_model_destroyed);
+
+  // keep an indirect pointer because sometimes bec::ListModel is destroyed before the wrapper
+  // and in that case we don't want to keep using the dead pointer
+  _tm = new bec::ListModel*();
+  *_tm = tm;
+
+  tm->add_destroy_notify_callback(_tm, &ListModelWrapper::on_bec_model_destroyed);
   #ifndef NO_MENU_MANAGER
   if (_treeview)
     _treeview->signal_event().connect(sigc::mem_fun(this, &ListModelWrapper::handle_popup_event));
@@ -531,12 +535,8 @@ ListModelWrapper::ListModelWrapper(bec::ListModel* tm, Gtk::TreeView *treeview,
 //------------------------------------------------------------------------------
 void* ListModelWrapper::on_bec_model_destroyed(void* data)
 {
-  ListModelWrapper** self = (ListModelWrapper**)data;
-  if (self && *self)
-  {
-    (*self)->_tm = 0;
-  }
-  delete self;
+  bec::ListModel **ptr = (bec::ListModel**)data;
+  *ptr = 0;
   return 0;
 }
 
@@ -544,11 +544,9 @@ void* ListModelWrapper::on_bec_model_destroyed(void* data)
 ListModelWrapper::~ListModelWrapper()
 {
   delete _context_menu;
-  if (_tm)
-    _tm->remove_destroy_notify_callback(_self_ref);
-  *_self_ref= 0;
-
-  _tm= 0;
+  if (*_tm)
+    (*_tm)->remove_destroy_notify_callback(_tm);
+  *_tm= 0;
 }
 
 //------------------------------------------------------------------------------
@@ -583,11 +581,13 @@ ListModelWrapper::NodeIdArray ListModelWrapper::get_selection() const
 //------------------------------------------------------------------------------
 void ListModelWrapper::set_be_model(bec::ListModel* tm)
 {
-  if (_tm)
-    _tm->remove_destroy_notify_callback(_self_ref);
-  _tm = tm;
-  if (_tm)
-    _tm->add_destroy_notify_callback(_self_ref, &ListModelWrapper::on_bec_model_destroyed);
+  if (*_tm)
+    (*_tm)->remove_destroy_notify_callback(_tm);
+  *_tm = tm;
+  if (*_tm)
+  {
+    (*_tm)->add_destroy_notify_callback(_tm, &ListModelWrapper::on_bec_model_destroyed);
+  }
 }
 
 #ifndef NO_MENU_MANAGER
@@ -610,9 +610,7 @@ void ListModelWrapper::handle_popup(const int x, const int y, const int time, Gd
     there_is_path_at_pos = path.gobj() && !path.empty();
   }
 
-  if ( !there_is_path_at_pos )
-    list.clear();
-  else
+  if ( there_is_path_at_pos )
   {
     // Check that @path is on selection, otherwise add @path to selection
     bec::NodeId node = get_node_for_path(path);
@@ -647,12 +645,11 @@ void ListModelWrapper::handle_popup(const int x, const int y, const int time, Gd
 
       list = get_selection();
     }
-
-    if (!_context_menu)
-      _context_menu = new Gtk::Menu();
-
-    run_menu_and_forward_action(_tm->get_popup_items_for_nodes(list), x, y, time, _tm, list, _fe_menu_handler, _context_menu);
   }
+  if (!_context_menu)
+    _context_menu = new Gtk::Menu();
+
+  run_menu_and_forward_action((*_tm)->get_popup_items_for_nodes(list), x, y, time, *_tm, list, _fe_menu_handler, _context_menu);
 }
 
 //------------------------------------------------------------------------------
@@ -688,7 +685,7 @@ bec::NodeId ListModelWrapper::get_node_for_path(const Gtk::TreeModel::Path &path
 //------------------------------------------------------------------------------
 bool ListModelWrapper::init_gtktreeiter(GtkTreeIter *it, const bec::NodeId &node) const
 {
-  if ( _tm && it && node.is_valid() )
+  if ( *_tm && it && node.is_valid() )
   {
     Index id(it, node);
     id.stamp(_stamp);
@@ -716,7 +713,7 @@ bec::NodeId ListModelWrapper::node_for_iter(const iterator &iter) const
 //------------------------------------------------------------------------------
 void ListModelWrapper::refresh()
 {
-  if (_tm) _tm->refresh();
+  if (*_tm) (*_tm)->refresh();
   model_changed(bec::NodeId(), -1);
 }
 
@@ -724,10 +721,10 @@ void ListModelWrapper::refresh()
 //------------------------------------------------------------------------------
 void ListModelWrapper::note_row_added()
 {
-  if (_tm)
+  if (*_tm)
   {
-    _tm->refresh();
-    Gtk::TreePath path(_tm->count()-1);
+    (*_tm)->refresh();
+    Gtk::TreePath path((*_tm)->count()-1);
 
     row_inserted(path, get_iter(path));
   }
@@ -775,14 +772,14 @@ bool ListModelWrapper::iter_next_vfunc(const iterator& iter, iterator& iter_next
   // Invalidate iter_next
   reset_iter(iter_next);
 
-  if (_tm)
+  if (*_tm)
   {
-    if (current_node.is_valid() && _tm->has_next(current_node))
+    if (current_node.is_valid() && (*_tm)->has_next(current_node))
     {
       try
       {
         // Obtain parent of the current node to get number of children of the parent node
-        current_node = _tm->get_next(current_node);
+        current_node = (*_tm)->get_next(current_node);
       }
       catch (...)
       {
@@ -805,11 +802,11 @@ bool ListModelWrapper::get_iter_vfunc(const Path& path, iterator& iter) const
   // Invalidate iter_next
   reset_iter(iter);
 
-  if (_tm)
+  if (*_tm)
   {
     bec::NodeId node(path.to_string());
 
-    if ( node.is_valid() && node.back() < _tm->count() )
+    if ( node.is_valid() && node.back() < (*_tm)->count() )
       ret = init_gtktreeiter(it, node);
   }
   return ret;
@@ -841,9 +838,9 @@ bool ListModelWrapper::iter_nth_root_child_vfunc(int n, iterator& iter) const
   //index is 0.  If @a n is too big, or if there are no children, @a iter is set
   //to an invalid iterator and false is returned.
   //See also iter_nth_child_vfunc().
-  if ( _tm && n >= 0 && n < iter_n_root_children_vfunc() )
+  if ( *_tm && n >= 0 && n < iter_n_root_children_vfunc() )
   {
-    bec::NodeId node = _tm->get_node(n);
+    bec::NodeId node = (*_tm)->get_node(n);
     init_gtktreeiter(iter.gobj(), node);
     ret = true;
   }
@@ -866,7 +863,7 @@ int ListModelWrapper::iter_n_children_vfunc(const iterator& iter) const
 //------------------------------------------------------------------------------
 int ListModelWrapper::iter_n_root_children_vfunc() const
 {
-  const int ret = _tm ? _tm->count() : 0;
+  const int ret = *_tm ? (*_tm)->count() : 0;
   return ret;
 }
 
@@ -890,14 +887,14 @@ Gtk::TreeModel::Path ListModelWrapper::get_path_vfunc(const iterator& iter) cons
 //------------------------------------------------------------------------------
 void ListModelWrapper::get_icon_value(const iterator& iter, int column, const bec::NodeId &node, Glib::ValueBase& value) const
 {
-  if (!_tm)
+  if (!*_tm)
     return;
 
   static ImageCache       *pixbufs      =  ImageCache::get_instance();
   static Glib::RefPtr<Gtk::IconTheme> icon_theme= Gtk::IconTheme::get_default();
   GValue *gval = value.gobj();
 
-  bec::IconId               icon_id = _tm->get_field_icon(node, column, get_icon_size());
+  bec::IconId               icon_id = (*_tm)->get_field_icon(node, column, get_icon_size());
 
   g_value_init(gval, GDK_TYPE_PIXBUF);
   if (icon_id != 0)
@@ -914,7 +911,7 @@ void ListModelWrapper::get_icon_value(const iterator& iter, int column, const be
 //------------------------------------------------------------------------------
 void ListModelWrapper::get_value_vfunc(const iterator& iter, int column, Glib::ValueBase& value) const
 {
-  if (!_tm)
+  if (!*_tm)
     return;
 
   bec::NodeId node= node_for_iter(iter);
@@ -941,7 +938,7 @@ void ListModelWrapper::get_value_vfunc(const iterator& iter, int column, Glib::V
       case G_TYPE_BOOLEAN:
         {
           bool bv = false;
-          _tm->get_field(node, column, bv);
+          (*_tm)->get_field(node, column, bv);
           set_glib_bool(value, bv);
           lmwdprint("LMW::get_value_vfunc: %s: node %s, col %i: value: %i\n", _name.c_str(), node.repr().c_str(), column, bv);
           break;
@@ -950,7 +947,7 @@ void ListModelWrapper::get_value_vfunc(const iterator& iter, int column, Glib::V
       case G_TYPE_UINT:
         {
           int iv = 0;
-          _tm->get_field(node, column, iv);
+          (*_tm)->get_field(node, column, iv);
           set_glib_int(value, iv);
           lmwdprint("LMW::get_value_vfunc: %s: node %s, col %i: value: %i\n", _name.c_str(), node.repr().c_str(), column, iv);
           break;
@@ -967,7 +964,7 @@ void ListModelWrapper::get_value_vfunc(const iterator& iter, int column, Glib::V
       case G_TYPE_DOUBLE:
         {
           double dv = 0.0;
-          _tm->get_field(node, column, dv);
+          (*_tm)->get_field(node, column, dv);
           set_glib_double(value, dv);
           lmwdprint("LMW::get_value_vfunc: %s: node %s, col %i: value: %f\n", _name.c_str(), node.repr().c_str(), column, dv);
           break;
@@ -976,9 +973,9 @@ void ListModelWrapper::get_value_vfunc(const iterator& iter, int column, Glib::V
         {
           std::string sv;
           if (column < 0)
-            sv= _tm->get_field_description(node, -column);
+            sv= (*_tm)->get_field_description(node, -column);
           else
-            _tm->get_field_repr(node, column, sv);
+            (*_tm)->get_field_repr(node, column, sv);
           set_glib_string(value, sv, true);
           lmwdprint("LMW::get_value_vfunc: %s: node %s, col %i: value: '%s'\n", _name.c_str(), node.repr().c_str(), column, sv.c_str());
           break;
@@ -1002,7 +999,7 @@ bool ListModelWrapper::iter_is_valid(const iterator& iter) const
 //------------------------------------------------------------------------------
 void ListModelWrapper::set_value_impl(const iterator& row, int column, const Glib::ValueBase& value)
 {
-  if (!_tm)
+  if (!*_tm)
     return;
 
   bec::NodeId node(node_for_iter(row));
@@ -1026,7 +1023,7 @@ void ListModelWrapper::set_value_impl(const iterator& row, int column, const Gli
           Glib::Value<bool>   v;
           v.init(value.gobj());
           lmwdprint("LMW::set_value_impl:%s node %s, column %i, value %i\n", _name.c_str(), node.repr().c_str(), column, v.get());
-          _tm->set_field(node, column, v.get());
+          (*_tm)->set_field(node, column, v.get());
           break;
         }
       case G_TYPE_INT:
@@ -1035,7 +1032,7 @@ void ListModelWrapper::set_value_impl(const iterator& row, int column, const Gli
           Glib::Value<int>   v;
           v.init(value.gobj());
           lmwdprint("LMW::set_value_impl: node %s, column %i, value %i\n", node.repr().c_str(), column, v.get());
-          _tm->set_field(node, column, v.get());
+          (*_tm)->set_field(node, column, v.get());
           break;
         }
       case G_TYPE_FLOAT:
@@ -1043,14 +1040,14 @@ void ListModelWrapper::set_value_impl(const iterator& row, int column, const Gli
         {
           Glib::Value<double>  v;
           v.init(value.gobj());
-          _tm->set_field(node, column, v.get());
+          (*_tm)->set_field(node, column, v.get());
           break;
         }
       case G_TYPE_STRING:
         {
           Glib::Value<std::string>   v;
           v.init(value.gobj());
-          _tm->set_field(node, column, v.get());
+          (*_tm)->set_field(node, column, v.get());
           lmwdprint("LMW::set_value: %s '%s'\n", _name.c_str(), v.get().c_str());
           break;
         }
@@ -1108,10 +1105,21 @@ bool ListModelWrapper::drag_data_received_vfunc(const Gtk::TreeModel::Path& dest
   // Currently this works for linear lists
   try
   {
-    _tm->reorder(bec::NodeId((const char*)selection_data.get_data()), dest.front());
+    (*_tm)->reorder(bec::NodeId((const char*)selection_data.get_data()), dest.front());
     ret = true;
   }
   catch (const std::logic_error& e)
   {}
   return ret;
 }
+
+
+
+//------------------------------------------------------------------------------
+void ListModelWrapper::invalidate()
+{
+  _invalid= true;
+  *_tm= 0;
+  _stamp++;
+}
+
