@@ -254,6 +254,13 @@ class ListModelWrapper : public Glib::Object, public Gtk::TreeModel,
       return Glib::RefPtr<ListModelWrapper>(new ListModelWrapper(tm, treeview, name));
     }
 
+   /**
+    Marks the tree as invalid.
+    Methods like get_iter_vfunc will return false and the backend tree model
+    will be treated as no longer valid.
+    */
+    void invalidate();
+
     void set_iconview(Gtk::IconView* iv);
 
     template <class T>
@@ -304,7 +311,7 @@ class ListModelWrapper : public Glib::Object, public Gtk::TreeModel,
 
     virtual bec::NodeId get_node_for_path(const Gtk::TreeModel::Path &path) const;
   
-    bec::ListModel *get_be_model() const { return _tm; }
+    bec::ListModel *get_be_model() const { return *_tm; }
     void set_be_model(bec::ListModel* tm);
 
     void set_name(const std::string& nm) { _name = nm; }
@@ -525,14 +532,17 @@ class ListModelWrapper : public Glib::Object, public Gtk::TreeModel,
     virtual bool init_gtktreeiter(GtkTreeIter* it, const bec::NodeId &uid= bec::NodeId()) const;
     void reset_iter(iterator& iter) const throw();
 
-    bec::ListModel  *_tm;
+  private:
+    bec::ListModel  **_tm;
+  protected:
     Gtk::TreeView   *_treeview;
     Gtk::IconView   *_iconview;
     Gtk::Menu       *_context_menu;
     int              _stamp;              //!< stamp to check if an iterator is invalid
     ColumnsModel     _columns;            //!< Generic model of columns
     bec::IconSize    _icon_size;          //!< icon size to set by mode view changed buttons
-    ListModelWrapper** _self_ref; // hack to workaround a crash.. 
+    struct LMPtr    *_self_ref; // hack to workaround a crash.. 
+    bool             _invalid;
 
     std::string      _name;
 
