@@ -2,7 +2,6 @@
 
 #include "lf_wizard.h"
 #include "lf_utilities.h"
-#include "mforms_grttreeview.h"
 #include <mforms/mforms.h>
 #include <gtkmm.h>
 #include <stdio.h>
@@ -15,16 +14,6 @@
 
 using base::strfmt;
 
-#include "config.h"
-
-#ifdef EDITION_OSS
-# define SPLASH_IMAGE "wb_splashscreen_oss.png"
-#else
-# ifdef EDITION_SE
-#  define SPLASH_IMAGE "wb_splashscreen_se.png"
-# endif
-#endif
-
 //==============================================================================
 // We need recursive mutex to lock gtk main loop. For that we supply enter/leave
 // callbacks to glib via gdk_threads_set_lock_functions(). Out lock/unlock
@@ -48,48 +37,6 @@ inline void init_gdk_thread_callbacks()
 
 //==============================================================================
 
-
-static Gtk::Window *show_splash()
-{
-  {    
-    const char *datadir= getenv("MWB_DATA_DIR");
-    if (datadir)
-    {
-      Gtk::RC::add_default_file(std::string(datadir).append("/workbench.rc"));
-      
-      Gtk::Window *window= new Gtk::Window(Gtk::WINDOW_TOPLEVEL);
-      window->set_decorated(false);
-      window->set_default_size(560, 322);
-      window->modify_bg_pixmap(Gtk::STATE_NORMAL, std::string(datadir).append("/images/"SPLASH_IMAGE));
-      window->set_position(Gtk::WIN_POS_CENTER);
-      
-      Gtk::Fixed *fixed= Gtk::manage(new Gtk::Fixed());
-      window->add(*fixed);
-      
-      Gtk::Label *label= Gtk::manage(new Gtk::Label("Version " PACKAGE_VERSION));
-      fixed->put(*label, 460, 150);
-      label->set_alignment(1.0, 0.0);
-      label->set_justify(Gtk::JUSTIFY_RIGHT);
-      label->modify_fg(Gtk::STATE_NORMAL, label->get_style()->get_black());
-      fixed->show();
-      label->show();
-      
-      window->show_now();
-      
-      return window;
-    }
-  }
-  return 0;
-}
-
-static bool close_splash(Gtk::Window *w)
-{
-  w->hide();
-  delete w;
-  return false;
-}
-
-
 int main(int argc, char **argv)
 {
 
@@ -113,7 +60,7 @@ int main(int argc, char **argv)
   wb::WBOptions wboptions;
 
   wboptions.user_data_dir = std::string(g_get_home_dir()).append("/.mysql/workbench");
-  base::Logger log(wboptions.user_data_dir);
+  base::Logger log(wboptions.user_data_dir, getenv("MWB_LOG_TO_STDERR")!=NULL);
 
   wboptions.basedir = getenv("MWB_DATA_DIR");
   wboptions.plugin_search_path = getenv("MWB_PLUGIN_DIR");
@@ -138,7 +85,6 @@ int main(int argc, char **argv)
   }
 
   mforms::gtk::init(getenv("WB_FORCE_SYSTEM_COLORS") != NULL);
-  mforms::gtk::GRTTreeViewImpl::init();
   mforms::gtk::WizardImpl::set_icon_path(wboptions.basedir+"/images");
 
   Gtk::RC::add_default_file(wboptions.basedir+"/workbench.rc");
@@ -152,17 +98,6 @@ int main(int argc, char **argv)
     XSynchronize(gdk_display, 1);
   }
 
-  if (!wboptions.quit_when_done || wboptions.no_logo)
-  {
-    Gtk::Window *splash = show_splash();
-    splash->get_window()->freeze_updates(); // Added to stop idle window update requests storm in gnome-3
-
-    while (app.events_pending())
-      app.iteration(false);
-
-    Glib::signal_idle().connect(sigc::bind(sigc::ptr_fun(close_splash), splash));
-  }
-
   Program program(wboptions);
   
   mforms::gtk::check();
