@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -30,6 +30,10 @@
 #include "base/util_functions.h"
 #include "base/string_utilities.h"
 
+#include "base/log.h"
+
+DEFAULT_LOG_DOMAIN(DOMAIN_BASE);
+
 #ifdef WIN32
 #define vsnprintf _vsnprintf
 #endif
@@ -37,32 +41,25 @@
 using namespace std;
 using namespace base;
 
-// Maximum size for internal buffers for loading data.
-#define MAX_BUFFER_SIZE 512
-
-// What character starts a comment line.
-const std::string CommentIndicators = ";#";
-
 // What character separates key and value.
 const std::string EqualIndicators = "=:"; 
 
-// White spaces to skip.
-const std::string WhiteSpace = " \t\n\r";
-
 typedef struct
 {
   std::string key;
   std::string value;
-  std::string comment; // Note: comments must be before the value entry in the file.
+  std::string pre_comment; // A comment before the entry (separate line(s)).
+  std::string post_comment; // A comment after the entry (same line).
 } ConfigEntry;
 
-typedef std::vector<ConfigEntry> EntryList;
+typedef std::vector<ConfigEntry> EntryList; // Must be a vector instead of a map to preserve order and
+                                            // allow multiple values with the same key (includes).
 typedef EntryList::iterator EntryListIterator;
 
 typedef struct
 {
   std::string name;
-  std::string comment;
+  std::string comment; // A comment placed before a section start.
   EntryList keys;
 } ConfigSection;
 
@@ -91,62 +88,45 @@ std::string extract_next_word(std::string& line)
   return base::trim(word);
 }
 
-//--------------------------------------------------------------------------------------------------
-
-/**
- * Writes the formatted output to the file stream, returning the number of bytes written.
- */
-int write_to_stream(fstream& stream, const char* fmt, ...)
-{
-  char buf[MAX_BUFFER_SIZE];
-  int length;
-  
-  memset(buf, 0, MAX_BUFFER_SIZE);
-  va_list args;
-  
-  va_start(args, fmt);
-  length = vsnprintf(buf, MAX_BUFFER_SIZE, fmt, args);
-  va_end(args);
-  
-  if (buf[length] != '\n' && buf[length] != '\r')
-    buf[length++] = '\n';
-  
-  stream.write(buf, length);
-  
-  return length;
-}
-
-//----------------- ConfigurationFile::Private ------------------------------------------------------
+//----------------- ConfigurationFile::Private -----------------------------------------------------
 
 class ConfigurationFile::Private
 {
+public:
   int _flags;
   SectionList _sections;
   bool _dirty;
+  std::string _trailing_comments; // Any comments after the last key-value or section entry.
   
-public:
   Private(std::string file_name, ConfigFileFlags flags);
   
-  ConfigEntry* get_entry_in_section(std::string key, std::string section);
-  ConfigSection*	get_section(std::string section);
+  ConfigEntry* get_entry_in_section(std::string key, std::string section, bool auto_create);
+  ConfigSection*	get_section(std::string section, bool auto_create);
   
-  bool set_value(std::string key_name, std::string value, std::string comment, std::string section_name);
+  void clear_includes(const std::string &section_name);
+  void add_include(const std::string &section_name, const std::string &include);
+  void add_include_dir(const std::string &section_name, const std::string &include);
+  std::vector<std::string> get_includes(const std::string &section_name);
+
+  bool set_value(std::string key_name, std::string value, std::string section_name);
   void set_dirty();
   void clear();
   
-  bool create_key(std::string key, std::string value, std::string comment, std::string section);
+  bool create_key(std::string key, std::string value, std::string pre_comment, std::string post_comment,
+    std::string section);
   bool delete_key(std::string key, std::string section_name);
   bool create_section(std::string section_name, std::string comment);
   bool delete_section(std::string name);
   
   int section_count();
   int key_count();
+  int key_count_for_section(const std::string &section_name);
   bool is_dirty();
   
-  std::string comment_string(std::string comment);
+  std::string make_comment(const std::string &text);
   
-  bool load(const std::string& file_name);
-  bool save(const std::string& file_name);
+  bool load(const std::string &file_name);
+  bool save(const std::string &file_name);
   
 };
 
@@ -157,11 +137,11 @@ ConfigurationFile::Private::Private(std::string file_name, ConfigFileFlags flags
   _dirty = false;
   _flags = flags;
   
-  // If a filename was given load the file, otherwise create a default section.
-  if (file_name.size() > 0)
+  // Always create a default section.
+  _sections.push_back(ConfigSection());
+  
+  if (!file_name.empty())
     load(file_name);
-  else
-    _sections.push_back(ConfigSection());
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -169,9 +149,10 @@ ConfigurationFile::Private::Private(std::string file_name, ConfigFileFlags flags
 /**
  * Looks up the given key in the given section and returns its entry if found, otherwise NULL.
  */
-ConfigEntry* ConfigurationFile::Private::get_entry_in_section(std::string key, std::string section_name)
+ConfigEntry* ConfigurationFile::Private::get_entry_in_section(std::string key, std::string section_name,
+  bool auto_create)
 {
-  ConfigSection* section = get_section(section_name);
+  ConfigSection* section = get_section(section_name, auto_create && (_flags & AutoCreateSections) != 0);
   
   if (section == NULL)
     return NULL;
@@ -182,68 +163,131 @@ ConfigEntry* ConfigurationFile::Private::get_entry_in_section(std::string key, s
       return &(*iterator);
   }
   
+  if (auto_create)
+  {
+    _dirty = true;
+    ConfigEntry new_entry;
+    new_entry.key = base::trim(key);
+    section->keys.push_back(new_entry);
+    return &section->keys.back();
+  }
+
   return NULL;
 }
 
 //--------------------------------------------------------------------------------------------------
 
 /**
- * Looks up the given section in our list and returns a pointer to it if found. Otherwise NULL
- * is returned.
+ * Looks up the given section in our list and returns a pointer to it if found.
+ * If the section could not be found and auto_create is true then it is created and returned.
+ * Otherwise NULL is returned.
  */
-ConfigSection* ConfigurationFile::Private::get_section(std::string section_name)
+ConfigSection* ConfigurationFile::Private::get_section(std::string section_name, bool auto_create)
 {
+  section_name = base::trim(section_name);
   for (SectionListIterator iterator = _sections.begin(); iterator != _sections.end(); iterator++)
   {
     if (strcasecmp(iterator->name.c_str(), section_name.c_str()) == 0)
       return &(*iterator);
   }
   
+  if (auto_create)
+  {
+    create_section(section_name, "");
+    return &_sections.back();
+  }
+
   return NULL;
 }
 
 //--------------------------------------------------------------------------------------------------
 
-bool ConfigurationFile::Private::set_value(std::string key_name, std::string value,
-                                           std::string comment, std::string section_name)
+bool is_include(ConfigEntry &entry)
 {
-  ConfigEntry* entry = get_entry_in_section(key_name, section_name);
-  ConfigSection* section = get_section(section_name);
-  
+  std::string key = base::tolower(entry.key);
+  return key == "!include" || key == "!includedir";
+}
+
+void ConfigurationFile::Private::clear_includes(const std::string &section_name)
+{
+  ConfigSection *section = get_section(section_name, (_flags & AutoCreateSections) != 0);
+
   if (section == NULL)
+    return;
+
+  std::remove_if(section->keys.begin(), section->keys.end(), is_include);
+  _dirty = true;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void ConfigurationFile::Private::add_include(const std::string &section_name, const std::string &include)
+{
+  ConfigSection *section = get_section(section_name, (_flags & AutoCreateSections) != 0);
+
+  if (section == NULL)
+    return;
+
+  ConfigEntry* entry = get_entry_in_section("!include", section->name, true);
+  entry->value = include;
+
+  _dirty = true;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void ConfigurationFile::Private::add_include_dir(const std::string &section_name, const std::string &include)
+{
+  ConfigSection *section = get_section(section_name, (_flags & AutoCreateSections) != 0);
+
+  if (section == NULL)
+    return;
+
+  ConfigEntry* entry = get_entry_in_section("!includedir", section->name, true);
+  entry->value = include;
+
+  _dirty = true;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+std::vector<std::string> ConfigurationFile::Private::get_includes(const std::string &section_name)
+{
+  std::vector<std::string> result;
+  ConfigSection *section = get_section(section_name, (_flags & AutoCreateSections) != 0);
+
+  if (section != NULL)
   {
-    if (!(_flags & AutoCreateSections) || !create_section(section_name, ""))
-      return false;
-    
-    section = get_section(section_name);
+    for (EntryListIterator iterator = section->keys.begin(); iterator != section->keys.end(); iterator++)
+    {
+      if (is_include(*iterator))
+        result.push_back(iterator->value);
+    }
   }
+
+  return result;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Sets the value for the given key in the given section. If the section does not exist yet and
+ * the AutoCreateSections flag is set then it is created first. Similarly for the key (and the
+ * AutoCreateKeys flag).
+ * Returns true if the value could be set, false otherwise.
+ */
+bool ConfigurationFile::Private::set_value(std::string key_name, std::string value,
+                                           std::string section_name)
+{
+  ConfigEntry* entry = get_entry_in_section(key_name, section_name, (_flags & AutoCreateKeys) != 0);
   
-  // Sanity check...
-  if (section == NULL)
+  if (entry == NULL)
     return false;
   
-  // Create key if it does not yet exist.
-  // Note: we allow to insert empty values for a key too. This allows to write key-only entries to the file.
-  if (entry == NULL && (_flags & AutoCreateKeys))
-  {
-    ConfigEntry new_entry;
-    new_entry.key = key_name;
-    new_entry.value = value;
-    new_entry.comment = comment;
-    section->keys.push_back(new_entry);
-  }
-  
-  if (entry != NULL)
-  {
-    entry->value = value;
-    entry->comment = comment;
-    
-    _dirty = true;
-    
-    return true;
-  }
-  
-  return false;
+  entry->value = base::trim(value);
+  _dirty = true;
+
+  return true;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -258,13 +302,19 @@ void ConfigurationFile::Private::set_dirty()
 void ConfigurationFile::Private::clear()
 {
   _dirty = false;
+  _trailing_comments = "";
   _sections.clear();
+  _sections.push_back(ConfigSection()); // Make sure we always have a default section.
 }
 
 //--------------------------------------------------------------------------------------------------
 
 bool ConfigurationFile::Private::delete_section(std::string name)
 {
+  name = base::trim(name);
+  if (name.empty()) // Don't remove the default section.
+    return false;
+
   for (SectionListIterator iterator = _sections.begin(); iterator != _sections.end(); iterator++)
   {
     if (strcasecmp((*iterator).name.c_str(), name.c_str()) == 0)
@@ -284,11 +334,12 @@ bool ConfigurationFile::Private::delete_section(std::string name)
  */
 bool ConfigurationFile::Private::delete_key(std::string key, std::string section_name)
 {
-  ConfigSection* section = get_section(section_name);
+  ConfigSection* section = get_section(section_name, false);
   
   if (section == NULL)
     return false;
   
+  key = base::trim(key);
   for (EntryListIterator iterator = section->keys.begin(); iterator != section->keys.end(); iterator++)
   {
     if (strcasecmp(iterator->key.c_str(), key.c_str()) == 0)
@@ -309,18 +360,20 @@ bool ConfigurationFile::Private::delete_key(std::string key, std::string section
  * the new value. If it does not locate the key, it will create a new key with
  * the proper value and place it in the section requested.
  */
-bool ConfigurationFile::Private::create_key(std::string key, std::string value, std::string comment, std::string section)
+bool ConfigurationFile::Private::create_key(std::string key_name, std::string value, 
+  std::string pre_comment, std::string post_comment, std::string section_name)
 {
-  int old_flags = _flags;
-  bool result  = false;
+  ConfigEntry* entry = get_entry_in_section(key_name, section_name, true);
+
+  if (entry == NULL)
+    return false;
+
+  entry->value = base::trim(value);
+  entry->pre_comment = pre_comment;
+  entry->post_comment = post_comment;
+  _dirty = true;
   
-  _flags |= AutoCreateKeys;
-  
-  result = set_value(key, value, comment, section);
-  
-  _flags = old_flags;
-  
-  return result;
+  return true;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -331,12 +384,12 @@ bool ConfigurationFile::Private::create_key(std::string key, std::string value,
  */
 bool ConfigurationFile::Private::create_section(std::string section_name, std::string comment)
 {
-  
-  if (get_section(section_name) != NULL)
+  // TODO: this should be necessary.
+  if (get_section(section_name, false) != NULL)
     return false;
   
   ConfigSection section;
-  section.name = section_name;
+  section.name = base::trim(section_name);
   section.comment = comment;
   _sections.push_back(section);
   _dirty = true;
@@ -371,6 +424,20 @@ int ConfigurationFile::Private::key_count()
 
 //--------------------------------------------------------------------------------------------------
 
+/**
+ * Returns the number of keys in the given section. Returns 0 if the section could not be found.
+ */
+int ConfigurationFile::Private::key_count_for_section(const std::string &section_name)
+{
+  ConfigSection *section = get_section(section_name, false);
+  if (section != NULL)
+    return section->keys.size();
+  
+  return 0;
+}
+
+//--------------------------------------------------------------------------------------------------
+
 bool ConfigurationFile::Private::is_dirty()
 {
   return _dirty;
@@ -382,25 +449,15 @@ bool ConfigurationFile::Private::is_dirty()
  * Converts the given text to a comment string (prefixing it with the comment character) if it is 
  * not empty and does not yet have a comment character at the first position.
  */
-std::string ConfigurationFile::Private::comment_string(std::string comment)
+std::string ConfigurationFile::Private::make_comment(const std::string &text)
 {
-  std::string result;
+  if (text.size() == 0)
+    return text;
   
-  comment = base::trim(comment);
+  if (text[0] != '#' && text[0] != ';')
+    return "# " + text;
   
-  if (comment.size() == 0)
-    return comment;
-  
-  if (comment.find_first_of(CommentIndicators) != 0)
-  {
-    // Add a comment indicator at the first position in the result.
-    result = CommentIndicators[0];
-    result += " ";
-  }
-  
-  result += comment;
-  
-  return result;
+  return text;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -412,62 +469,105 @@ bool ConfigurationFile::Private::load(const std::string& file_name)
   if (file.is_open())
   {
     bool done = false;
-    int old_flags = _flags;
     
     std::string line;
     std::string comment;
-    char buffer[MAX_BUFFER_SIZE]; 
-    ConfigSection* section = get_section("");
-    
-    // These need to be set for loading. We'll restore the original values later.
-    _flags |= AutoCreateKeys;
-    _flags |= AutoCreateSections;
+    ConfigSection* section = get_section("", true); // Start with the default section which must always be there.
     
     while (!done)
     {
-      memset(buffer, 0, MAX_BUFFER_SIZE);
-      file.getline(buffer, MAX_BUFFER_SIZE);
+      std::getline(file, line);
       
-      line = base::trim(buffer);
+      line = base::trim(line);
       done = file.eof() || file.bad() || file.fail();
       
       // Skip empty lines.
       if (line.size() == 0)
         continue;
       
-      if (line.find_first_of(CommentIndicators) == 0)
+      switch (line[0])
       {
-        comment += "\n";
-        comment += line;
-      }
-      else
-        if (line.find_first_of('[') == 0)
-        {
-          // Start a new section.
-          line.erase(0, 1);
-          line.erase(line.find_last_of(']'), 1);
-          
-          create_section(line, comment);
-          section = get_section(line);
-          comment = "";
-        }
-        else
+        case '#':
+        case ';':
+          if (!comment.empty())
+            comment += "\n";
+          comment += line;
+          break;
+
+        case '[':
+          {
+            // Start a new section.
+            size_t closing_bracket_position = line.rfind(']');
+            if (closing_bracket_position != std::string::npos)
+              line.erase(closing_bracket_position, 1);
+            else
+              log_warning("Unterminated section specifier found in file %s: %s\n", file_name.c_str(), line.c_str());
+
+            line.erase(0, 1);
+            section = get_section(line, true);
+            section->comment = comment;
+            comment = "";
+          }
+          break;
+
+        case '!':
+          {
+            std::string post_comment;
+            std::string::size_type comment_position = line.find('#'); // For post comments only # is allowed.
+            if (comment_position != std::string::npos)
+            {
+              post_comment = line.substr(comment_position, line.size());
+              line = line.substr(0, comment_position - 1);
+            }
+
+            std::vector<std::string> parts = base::split(line, " ");
+            if (parts.size() == 2)
+            {
+              std::string key = base::tolower(parts[0]);
+              if (key == "!include" || key == "!includedir")
+              {
+                ConfigEntry* entry = get_entry_in_section(key, section->name, true);
+                entry->value = parts[1];
+                entry->pre_comment = comment;
+                entry->post_comment = post_comment;
+                comment = "";
+              }
+              else
+                log_warning("Invalid include sequence found in file %s: %s\n", file_name.c_str(), line.c_str());
+            }
+            else
+              log_warning("Invalid include sequence found in file %s: %s\n", file_name.c_str(), line.c_str());
+          }
+          break;
+
+        default:
           if (line.size() > 0)
           {
             // Normal key/value pair or a single key.
+            std::string post_comment;
+            std::string::size_type comment_position = line.find('#'); // For post comments only # is allowed.
+            if (comment_position != std::string::npos)
+            {
+              post_comment = line.substr(comment_position, line.size());
+              line = line.substr(0, comment_position - 1);
+            }
             std::string key = extract_next_word(line);
             std::string value = line;
-            
+
             if (key.size() > 0)
             {
-              set_value(key, value, comment, section->name);
+              ConfigEntry* entry = get_entry_in_section(key, section->name, true);
+              entry->value = base::trim(value);
+              entry->pre_comment = comment;
+              entry->post_comment = post_comment;
               comment = "";
             }
           }
+          break;
+      }
     }
-    
-    // Restore the original flag values.
-    _flags = old_flags;
+
+    _trailing_comments = comment;
   }
   else
     return false;
@@ -484,62 +584,80 @@ bool ConfigurationFile::Private::save(const std::string& file_name)
   if (file_name.size() == 0)
     return false;
   
-  fstream file(file_name.c_str(), ios::out | ios::trunc);
+  fstream output(file_name.c_str(), ios::out | ios::trunc);
   
-  if (file.is_open())
+  if (output.is_open())
   {
     SectionListIterator section_iterator;
     EntryListIterator entry_iterator;
     ConfigEntry entry;
     
+    bool empty = true; // No output written yet.
+
+    // We removed all empty lines on load, so add one for each pre-comment, section comment or section entry
+    // (except for the very first entry).
     for (section_iterator = _sections.begin(); section_iterator != _sections.end(); section_iterator++)
     {
-      bool wrote_comment = false;
-      
-      if (section_iterator->comment.size() > 0)
+      if (!empty)
+        output << '\n';
+
+      if (!section_iterator->comment.empty())
       {
-        wrote_comment = true;
-        write_to_stream(file, "\n%s", comment_string(section_iterator->comment).c_str());
+        output << make_comment(section_iterator->comment) << '\n';
+        empty = false;
       }
       
-      if (section_iterator->name.size() > 0)
-        write_to_stream(file, "%s[%s]", wrote_comment ? "" : "\n", section_iterator->name.c_str());
+      if (!section_iterator->name.empty())
+      {
+        output << "[" << section_iterator->name.c_str() << "]\n";
+        empty = false;
+      }
       
       for (entry_iterator = section_iterator->keys.begin(); entry_iterator != section_iterator->keys.end();
            entry_iterator++)
       {
-        if (entry_iterator->key.size() > 0)
+        if (!entry_iterator->pre_comment.empty())
         {
-          if (entry_iterator->value.size() > 0)
-          {
-            write_to_stream(file, "%s%s%s%s%c%s",
-                            entry_iterator->comment.size() > 0 ? "\n" : "",
-                            comment_string(entry_iterator->comment).c_str(),
-                            entry_iterator->comment.size() > 0 ? "\n" : "",
-                            entry_iterator->key.c_str(),
-                            EqualIndicators[0],
-                            entry_iterator->value.c_str());
-          }
+          if (!empty)
+            output << '\n';
+
+          output << make_comment(entry_iterator->pre_comment) << '\n';
+        }
+
+        empty = false;
+        if (!entry_iterator->key.empty())
+        {
+          if (entry_iterator->key.find("!include") == 0) // Include entry?
+            output << entry_iterator->key << ' ' << entry_iterator->value;
           else
           {
-            write_to_stream(file, "%s%s%s%s",
-                            entry_iterator->comment.size() > 0 ? "\n" : "",
-                            comment_string(entry_iterator->comment).c_str(),
-                            entry_iterator->comment.size() > 0 ? "\n" : "",
-                            entry_iterator->key.c_str());
+            if (!entry_iterator->value.empty())
+              output << entry_iterator->key << ' ' << EqualIndicators[0] << ' ' << entry_iterator->value;
+            else
+              output << entry_iterator->key;
           }
         }
+        if (!entry_iterator->post_comment.empty())
+          output << ' ' << make_comment(entry_iterator->post_comment);
+        output << '\n';
       }
     }
     
+    if (!_trailing_comments.empty())
+    {
+      if (!empty)
+        output << '\n';
+      output << make_comment(_trailing_comments) << '\n';
+    }
+
   }
   else
     return false;
   
   _dirty = false;
   
-  file.flush();
-  file.close();
+  output.flush();
+  output.close();
   
   return true;
 }
@@ -567,6 +685,34 @@ ConfigurationFile::~ConfigurationFile()
 
 //--------------------------------------------------------------------------------------------------
 
+void ConfigurationFile::clear_includes(const std::string &section_name)
+{
+  data->clear_includes(section_name);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void ConfigurationFile::add_include(const std::string &section_name, const std::string &include)
+{
+  data->add_include(section_name, include);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void ConfigurationFile::add_include_dir(const std::string &section_name, const std::string &include)
+{
+  data->add_include_dir(section_name, include);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+std::vector<std::string> ConfigurationFile::get_includes(const std::string &section_name)
+{
+  return data->get_includes(section_name);
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void ConfigurationFile::clear()
 {
   data->clear();
@@ -586,14 +732,14 @@ bool ConfigurationFile::is_dirty()
  */
 bool ConfigurationFile::has_key(const std::string& key, const std::string& section)
 {
-  return data->get_entry_in_section(key, section) != NULL;
+  return data->get_entry_in_section(key, section, false) != NULL;
 }
 
 //--------------------------------------------------------------------------------------------------
 
 bool ConfigurationFile::has_section(const std::string& section_name)
 {
-  return data->get_section(section_name) != NULL;
+  return data->get_section(section_name, false) != NULL;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -621,14 +767,15 @@ bool ConfigurationFile::save(const std::string& file_name)
 /**
  * Sets the comment for a given entry. Returns true if the entry was found, otherwise false.
  */
-bool ConfigurationFile::set_key_comment(std::string key, std::string comment, std::string section_name)
+bool ConfigurationFile::set_key_pre_comment(std::string key, std::string comment, std::string section_name)
 {
-  ConfigEntry* entry = data->get_entry_in_section(key, section_name);
+  ConfigEntry* entry = data->get_entry_in_section(key, section_name, (data->_flags & AutoCreateKeys) != 0);
   if (entry == NULL)
     return false;
   
-  data->set_dirty();
-  entry->comment = comment;
+  data->_dirty = true;
+  entry->pre_comment = comment;
+
   return true;
 }
 
@@ -639,7 +786,7 @@ bool ConfigurationFile::set_key_comment(std::string key, std::string comment, st
  */
 bool ConfigurationFile::set_section_comment(std::string section_name, std::string comment)
 {
-  ConfigSection* section = data->get_section(section_name);
+  ConfigSection* section = data->get_section(section_name, (data->_flags & AutoCreateSections) != 0);
   if (section == NULL)
     return false;
   
@@ -650,53 +797,37 @@ bool ConfigurationFile::set_section_comment(std::string section_name, std::strin
 
 //--------------------------------------------------------------------------------------------------
 
-/**
- * Sets the value for the given key in the given section. If the section does not exist yet and
- * the AutoCreateSections flag is set then it is created first. Similarly for the key (and the
- * AutoCreateKeys flag).
- * Returns true if the value could be set, false otherwise.
- */
-bool ConfigurationFile::set_value(std::string key_name, std::string value, std::string comment,
-                                  std::string section_name)
+bool ConfigurationFile::set_value(std::string key_name, std::string value, std::string section_name)
 {
-  return data->set_value(key_name, value, comment, section_name);
+  return data->set_value(key_name, value, section_name);
   
 }
 
 //--------------------------------------------------------------------------------------------------
 
-/**
- * Sets a float value and comment for a given key in a given section.
- */
-bool ConfigurationFile::set_float(std::string key, float value, std::string comment, std::string section)
+bool ConfigurationFile::set_float(std::string key, float value, std::string section)
 {
   char buffer[64];
   snprintf(buffer, 64, "%f", value);
   
-  return data->set_value(key, buffer, comment, section);
+  return data->set_value(key, buffer, section);
 }
 
 //--------------------------------------------------------------------------------------------------
 
-/**
- * Sets a integer value and comment for a given key in a given section.
- */
-bool ConfigurationFile::set_int(std::string key, int value, std::string comment, std::string section)
+bool ConfigurationFile::set_int(std::string key, int value, std::string section)
 {
   char buffer[64];
   snprintf(buffer, 64, "%d", value);
   
-  return data->set_value(key, buffer, comment, section);
+  return data->set_value(key, buffer, section);
 }
 
 //--------------------------------------------------------------------------------------------------
 
-/**
- * Sets a bool value and comment for a given key in a given section.
- */
-bool ConfigurationFile::set_bool(std::string key, bool value, std::string comment, std::string section)
+bool ConfigurationFile::set_bool(std::string key, bool value, std::string section)
 {
-  return data->set_value(key, value ?  "True" : "False", comment, section);
+  return data->set_value(key, value ?  "True" : "False", section);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -707,72 +838,81 @@ bool ConfigurationFile::set_bool(std::string key, bool value, std::string commen
  */
 std::string ConfigurationFile::get_value(std::string key, std::string section) 
 {
-  ConfigEntry* entry = data->get_entry_in_section(key, section);
+  ConfigEntry* entry = data->get_entry_in_section(key, section, false);
   
   return (entry == NULL) ? "" : entry->value;
 }
 
 //--------------------------------------------------------------------------------------------------
 
-/**
- * Same as get_value. Just for convenience.
- */
-std::string ConfigurationFile::get_string(std::string key, std::string section)
+double ConfigurationFile::get_float(std::string key, std::string section)
 {
-  return get_value(key, section);
-}
-
-//--------------------------------------------------------------------------------------------------
-
-/**
- * Read a value as float.
- */
-float ConfigurationFile::get_float(std::string key, std::string section)
-{
-  std::string value = get_value(key, section);
+  std::string value = base::unquote_identifier(get_value(key, section));
   
   if (value.size() == 0)
     return FLT_MIN;
   
-  return (float) atof(value.c_str());
+  // This conversion handles invalid entries with more than one letter at the end
+  // (those are simply ignored). The right size computation is done in both cases.
+  // There's no error handling in case of an overflow.
+  double factor = 1;
+  switch (::tolower(value[value.size() - 1]))
+  {
+    // All cases fall through.
+  case 'g':
+    factor *= 1024;
+  case 'm':
+    factor *= 1024;
+  case 'k':
+    factor *= 1024;
+    value[value.size() - 1] = 0;
+  }
+  return factor * atof(value.c_str());
 }
 
 //--------------------------------------------------------------------------------------------------
 
-/**
- * Reade a value as integer.
- */
 int	ConfigurationFile::get_int(std::string key, std::string section)
 {
-  std::string value = get_value(key, section);
+  std::string value = base::unquote_identifier(get_value(key, section));
   
   if (value.size() == 0)
     return INT_MIN;
   
-  return atoi(value.c_str());
+  // Same as for get_float().
+  // Note: for int overflow can happen much quicker with G suffix or large values.
+  int factor = 1;
+  switch (::tolower(value[value.size() - 1]))
+  {
+    // All cases fall through.
+  case 'g':
+    factor *= 1024;
+  case 'm':
+    factor *= 1024;
+  case 'k':
+    factor *= 1024;
+    value[value.size() - 1] = 0;
+  }
+  return factor * atoi(value.c_str());
 }
 
 //--------------------------------------------------------------------------------------------------
 
-/**
- * Read a value as boolean.
- */
 bool ConfigurationFile::get_bool(std::string key, std::string section)
 {
-  bool result = false;
-  std::string value = get_value(key, section);
+  std::string value = base::tolower(base::unquote_identifier(get_value(key, section)));
   
-  if (value.find("1") == 0 || strcasecmp(value.c_str(), "true") == 0 || strcasecmp(value.c_str(), "yes") == 0)
-    result = true;
+  if (value == "true" || value == "yes")
+    return true;
   
-  return result;
+  if (atoi(value.c_str()) != 0)
+    return true;
+
+  return false;
 }
 
 //--------------------------------------------------------------------------------------------------
 
-/**
- * Removes the given section. Returns true if successful, otherwise false.
- */
 bool ConfigurationFile::delete_section(std::string name)
 {
   return data->delete_section(name);
@@ -780,9 +920,6 @@ bool ConfigurationFile::delete_section(std::string name)
 
 //--------------------------------------------------------------------------------------------------
 
-/**
- * Removes the given key from the given selection. Returns true if successful, otherwise false.
- */
 bool ConfigurationFile::delete_key(std::string key, std::string section_name)
 {
   return data->delete_key(key, section_name);
@@ -790,15 +927,10 @@ bool ConfigurationFile::delete_key(std::string key, std::string section_name)
 
 //--------------------------------------------------------------------------------------------------
 
-/**
- * Given a key, a value and a section, this function will attempt to locate the
- * Key within the given section, and if it finds it, change the keys value to
- * the new value. If it does not locate the key, it will create a new key with
- * the proper value and place it in the section requested.
- */
-bool ConfigurationFile::create_key(std::string key, std::string value, std::string comment, std::string section)
+bool ConfigurationFile::create_key(std::string key, std::string value, std::string pre_comment,
+  std::string post_comment, std::string section)
 {
-  return data->create_key(key, value, comment, section);
+  return data->create_key(key, value, pre_comment, post_comment, section);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -814,41 +946,6 @@ bool ConfigurationFile::create_section(std::string section_name, std::string com
 
 //--------------------------------------------------------------------------------------------------
 
-/**
- * Same as the previous function but accepts an additional entry list which is copied to the new
- * section (but only if a new section really has been created).
- 
- bool ConfigurationFile::create_section(std::string section_name, std::string comment, EntryList entries)
- {
- if (!create_section(section_name, comment))
- return false;
- 
- ConfigSection* section = get_section(section_name);
- 
- if (!section)
- return false;
- 
- section->name = section_name;
- for (EntryListIterator iterator = entries.begin(); iterator != entries.end(); iterator++)
- {
- ConfigEntry entry;
- entry.comment = iterator->comment;
- entry.key = iterator->key;
- entry.value = iterator->value;
- 
- section->keys.push_back(entry);
- }
- 
- _dirty = true;
- 
- return true;
- }
- */
-//--------------------------------------------------------------------------------------------------
-
-/**
- * Returns the number of sections.
- */
 int ConfigurationFile::section_count() 
 { 
   return data->section_count();
@@ -856,12 +953,16 @@ int ConfigurationFile::section_count()
 
 //--------------------------------------------------------------------------------------------------
 
-/**
- * Returns the total number of keys in all sections.
- */
 int ConfigurationFile::key_count()
 {
   return data->key_count();
 }
 
 //--------------------------------------------------------------------------------------------------
+
+int ConfigurationFile::key_count_for_secton(const std::string &section_name)
+{
+  return data->key_count_for_section(section_name);
+}
+
+//--------------------------------------------------------------------------------------------------
