@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -306,14 +306,14 @@ void BackLayer::repaint(const Rect &aBounds)
       // sides
       if (extra_offset.x > 0)
       {
-        cr->rectangle(-extra_offset.x, -extra_offset.y, extra_offset.x, view_size.height);
-        cr->rectangle(view_size.width - 2*extra_offset.x, -extra_offset.y, extra_offset.x, view_size.height);
+        cr->rectangle(-extra_offset.x + vrect.left(), -extra_offset.y + vrect.top(), extra_offset.x, view_size.height);
+        cr->rectangle(vrect.left() + view_size.width - 2*extra_offset.x, -extra_offset.y + vrect.top(), extra_offset.x, view_size.height);
       }
       // top/bottom
       if (extra_offset.y > 0)
       {
-        cr->rectangle(-extra_offset.x, -extra_offset.y, view_size.width, extra_offset.y);
-        cr->rectangle(-extra_offset.x, view_size.height - 2*extra_offset.y, view_size.width, extra_offset.y);
+        cr->rectangle(-extra_offset.x + vrect.left(), -extra_offset.y + vrect.top(), view_size.width, extra_offset.y);
+        cr->rectangle(-extra_offset.x + vrect.left(), view_size.height - 2*extra_offset.y + vrect.top(), view_size.width, extra_offset.y);
       }
       cr->fill();
 
