@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -40,8 +40,7 @@
 #define MAGNET_STICK_DISTANCE 5
 
 using namespace mdc;
-using namespace MySQL::Geometry;
-using namespace MySQL::Drawing;
+using namespace base;
 
 int mdc_live_item_count = 0;
 
