@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -40,9 +40,10 @@
 
 #include "mdc_line.h"
 
+#include "base/file_utilities.h"
+
 using namespace mdc;
-using namespace MySQL::Geometry;
-using namespace MySQL::Drawing;
+using namespace base;
 
 // XXX: use the values defined by the platform!
 #define DOUBLE_CLICK_TIME 0.5
@@ -112,8 +113,6 @@ static CanvasView *destroyed= 0;
 
 CanvasView::~CanvasView()
 {
-  g_static_rec_mutex_free(&_lock);
-
   destroyed= this;
 
   delete _blayer;
@@ -136,6 +135,7 @@ CanvasView::~CanvasView()
 
   if (_crsurface)
     cairo_surface_destroy(_crsurface);
+  g_static_rec_mutex_free(&_lock);
 }
 
 
@@ -816,7 +816,6 @@ void CanvasView::repaint_area(const Rect &aBounds, int wx, int wy, int ww, int w
     bounds= aBounds;
 
   CanvasAutoLock lock(this);
-  Size vsize(get_total_view_size());
   Rect clip;
 
   begin_repaint(wx, wy, ww, wh);
@@ -831,7 +830,6 @@ void CanvasView::repaint_area(const Rect &aBounds, int wx, int wy, int ww, int w
     apply_transformations_gl();    
 
   Rect vrect= get_viewport();
-  Point extra_offset= _extra_offset;
 
   if (_blayer->visible())
     _blayer->repaint(bounds);
@@ -942,7 +940,7 @@ void CanvasView::export_png(const std::string &filename, bool crop)
 {
   CanvasAutoLock lock(this);
 
-  FileHandle fh(filename.c_str(), "wb");
+  base::FileHandle fh(filename.c_str(), "wb");
   Size vsize= get_total_view_size();
   
   Rect bounds= get_content_bounds();
