@@ -79,35 +79,35 @@ public:
 
   virtual void update_view_size(int width, int height)= 0;
 
-  void set_offset(const MySQL::Geometry::Point &offs);
-  virtual void scroll_to(const MySQL::Geometry::Point &offs);
+  void set_offset(const base::Point &offs);
+  virtual void scroll_to(const base::Point &offs);
 
   void set_zoom(float zoom);
   float get_zoom() const { return _zoom; };
 
-  void set_page_size(const MySQL::Geometry::Size &size);
-  MySQL::Geometry::Size get_page_size() const { return _page_size; };
+  void set_page_size(const base::Size &size);
+  base::Size get_page_size() const { return _page_size; };
   void set_page_layout(Count xpages, Count ypages);
   void get_page_layout(Count &xpages, Count &ypages) { xpages= _x_page_num; ypages= _y_page_num; }
 
   // logical view size
-  MySQL::Geometry::Size get_total_view_size() const;
+  base::Size get_total_view_size() const;
   
   // physical view size
   inline void get_view_size(int &w, int &h) const { w= _view_width; h= _view_height; }
 
-  MySQL::Geometry::Size get_viewable_size() const;
+  base::Size get_viewable_size() const;
   
-  MySQL::Geometry::Rect get_viewport() const;
+  base::Rect get_viewport() const;
 
-  MySQL::Geometry::Rect get_viewport_range() const;
+  base::Rect get_viewport_range() const;
   
-  MySQL::Geometry::Rect get_content_bounds() const;
+  base::Rect get_content_bounds() const;
 
-  virtual MySQL::Geometry::Point window_to_canvas(int x, int y) const;
-  virtual MySQL::Geometry::Rect window_to_canvas(int x, int y, int w, int h) const;
-  virtual void canvas_to_window(const MySQL::Geometry::Point &pt, int &x, int &y) const;
-  virtual void canvas_to_window(const MySQL::Geometry::Rect &rect, int &x, int &y, int &w, int &h) const;
+  virtual base::Point window_to_canvas(int x, int y) const;
+  virtual base::Rect window_to_canvas(int x, int y, int w, int h) const;
+  virtual void canvas_to_window(const base::Point &pt, int &x, int &y) const;
+  virtual void canvas_to_window(const base::Rect &rect, int &x, int &y, int &w, int &h) const;
 
   void show_grid();
   void hide_grid();
@@ -116,8 +116,8 @@ public:
   void set_grid_snapping(bool flag);
   bool get_grid_snapping();
   
-  MySQL::Geometry::Point snap_to_grid(const MySQL::Geometry::Point &pos);
-  MySQL::Geometry::Size snap_to_grid(const MySQL::Geometry::Size &size);
+  base::Point snap_to_grid(const base::Point &pos);
+  base::Size snap_to_grid(const base::Size &size);
 
   void set_draws_line_hops(bool flag);
 
@@ -138,13 +138,13 @@ public:
   virtual void lower_layer(Layer *layer);
 
   CanvasItem *get_item_at(int x, int y);
-  CanvasItem *get_item_at(const MySQL::Geometry::Point &point);
+  CanvasItem *get_item_at(const base::Point &point);
   
   CanvasItem *get_leaf_item_at(int x, int y);
-  CanvasItem *get_leaf_item_at(const MySQL::Geometry::Point &point);
+  CanvasItem *get_leaf_item_at(const base::Point &point);
   
   typedef boost::function<bool (CanvasItem*)> ItemCheckFunc;
-  std::list<CanvasItem*> get_items_bounded_by(const MySQL::Geometry::Rect &rect, 
+  std::list<CanvasItem*> get_items_bounded_by(const base::Rect &rect, 
                                               const ItemCheckFunc &pred= ItemCheckFunc());
 
   void repaint();
@@ -153,7 +153,7 @@ public:
   void set_needs_repaint_all_items();
 
   void queue_repaint();
-  void queue_repaint(const MySQL::Geometry::Rect &bounds);
+  void queue_repaint(const base::Rect &bounds);
 
   virtual void handle_mouse_move(int x, int y, EventState state);
   virtual void handle_mouse_button(MouseButton button, bool press, int x, int y, EventState state);
@@ -161,14 +161,14 @@ public:
 
   bool handle_key(const KeyInfo &key, bool press, EventState state);
 
-  void start_dragging_rectangle(const MySQL::Geometry::Point &pos);
-  MySQL::Geometry::Rect finish_dragging_rectangle();
+  void start_dragging_rectangle(const base::Point &pos);
+  base::Rect finish_dragging_rectangle();
 
 
   bool focus_item(CanvasItem *item);
   CanvasItem *get_focused_item();
   
-  void select_items_inside(const MySQL::Geometry::Rect &rect, SelectType type, Group *group= 0);
+  void select_items_inside(const base::Rect &rect, SelectType type, Group *group= 0);
   
   Selection *get_selection() const { return _selection; };
   Selection::ContentType get_selected_items();
@@ -178,22 +178,22 @@ public:
   virtual bool initialize();
 
   const FontSpec &get_default_font();
-  MySQL::Drawing::Color get_selection_color() const { return MySQL::Drawing::Color(0.6, 0.85, 0.95, 1.0); }
-  MySQL::Drawing::Color get_highlight_color() const { return MySQL::Drawing::Color(1, 0.6, 0.0, 0.8); }
-  MySQL::Drawing::Color get_hover_color() const { return MySQL::Drawing::Color(0.85, 0.5, 0.5, 0.8); }
+  base::Color get_selection_color() const { return base::Color(0.6, 0.85, 0.95, 1.0); }
+  base::Color get_highlight_color() const { return base::Color(1, 0.6, 0.0, 0.8); }
+  base::Color get_hover_color() const { return base::Color(0.85, 0.5, 0.5, 0.8); }
 
   inline CairoCtx *cairoctx() const { return _cairo; }
   virtual bool has_gl() const= 0;
 
-  virtual Surface *create_temp_surface(const MySQL::Geometry::Size &size) const;
+  virtual Surface *create_temp_surface(const base::Size &size) const;
 
   void export_png(const std::string &filename, bool crop=false);
-  void export_pdf(const std::string &filename, const MySQL::Geometry::Size &size_in_pt);
-  void export_ps(const std::string &filename, const MySQL::Geometry::Size &size_in_pt);
-  void export_svg(const std::string &filename, const MySQL::Geometry::Size &size_in_pt);
+  void export_pdf(const std::string &filename, const base::Size &size_in_pt);
+  void export_ps(const std::string &filename, const base::Size &size_in_pt);
+  void export_svg(const std::string &filename, const base::Size &size_in_pt);
 
-  void set_event_callbacks(const boost::function<bool (CanvasView*, MouseButton, bool, MySQL::Geometry::Point, EventState)> &button_handler,
-                           const boost::function<bool (CanvasView*, MySQL::Geometry::Point, EventState)> &motion_handler,
+  void set_event_callbacks(const boost::function<bool (CanvasView*, MouseButton, bool, base::Point, EventState)> &button_handler,
+                           const boost::function<bool (CanvasView*, base::Point, EventState)> &motion_handler,
                            const boost::function<bool (CanvasView*, KeyInfo, EventState, bool)> &key_handler);
 
 
@@ -233,13 +233,13 @@ protected:
 
   Selection *_selection;
 
-  MySQL::Geometry::Size _page_size;
+  base::Size _page_size;
   Count _x_page_num;
   Count _y_page_num;
 
   float _zoom;
-  MySQL::Geometry::Point _offset;
-  MySQL::Geometry::Point _extra_offset;
+  base::Point _offset;
+  base::Point _extra_offset;
   int _view_width;
   int _view_height;
 
@@ -260,8 +260,8 @@ protected:
   boost::signals2::signal<void ()> _viewport_changed_signal;
   boost::signals2::signal<void ()> _zoom_changed_signal;
   
-  boost::function<bool (CanvasView*, MouseButton, bool, MySQL::Geometry::Point, EventState)> _button_event_relay;
-  boost::function<bool (CanvasView*, MySQL::Geometry::Point, EventState)> _motion_event_relay;
+  boost::function<bool (CanvasView*, MouseButton, bool, base::Point, EventState)> _button_event_relay;
+  boost::function<bool (CanvasView*, base::Point, EventState)> _motion_event_relay;
   boost::function<bool (CanvasView*, KeyInfo, EventState, bool)> _key_event_relay;
 
   CanvasView(int width, int height);
@@ -269,7 +269,7 @@ protected:
   virtual void begin_repaint(int wx, int wy, int ww, int wh)= 0;
   virtual void end_repaint()= 0;
 
-  void repaint_area(const MySQL::Geometry::Rect &rect, int wx, int wy, int ww, int wh);
+  void repaint_area(const base::Rect &rect, int wx, int wy, int ww, int wh);
 
   void update_offsets();
   void apply_transformations();
@@ -277,20 +277,20 @@ protected:
   void reset_transformations_gl();
   void apply_transformations_for_conversion(cairo_matrix_t *matrix) const;
 
-  bool perform_auto_scroll(const MySQL::Geometry::Point &mouse_pos);
+  bool perform_auto_scroll(const base::Point &mouse_pos);
   
-  void render_for_export(const MySQL::Geometry::Rect &bounds, CairoCtx *cr);
+  void render_for_export(const base::Rect &bounds, CairoCtx *cr);
   
 private:
   struct ClickInfo {
-    MySQL::Geometry::Point pos;
+    base::Point pos;
   };
 
   EventState _event_state;
   CanvasItem *_last_click_item;
   CanvasItem *_last_over_item;
   std::vector<ClickInfo> _last_click_info;
-  MySQL::Geometry::Point _last_mouse_pos;
+  base::Point _last_mouse_pos;
   
   GStaticRecMutex _lock;
 
