@@ -1,3 +1,22 @@
+/* 
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
+
 #ifndef _MDC_CANVAS_MANAGER_X11_H_
 #define _MDC_CANVAS_MANAGER_X11_H_
 
@@ -46,7 +65,7 @@ protected:
   int _depth;
   int _clip_x, _clip_y, _clip_w, _clip_h;
 
-  virtual void scroll_to(const MySQL::Geometry::Point &offs);
+  virtual void scroll_to(const base::Point &offs);
   virtual void update_view_size(int width, int height);
   virtual void make_current();
   virtual void begin_repaint(int x, int y, int w, int h);
