@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,8 +33,7 @@
 
 
 using namespace mdc;
-using namespace MySQL::Geometry;
-using namespace MySQL::Drawing;
+using namespace base;
 
 LineLayouter::LineLayouter()
 {
@@ -364,6 +363,7 @@ static void draw_line_end(CairoCtx *cr, LineEndType type, const Color &lcolor, c
 {
   switch (type)
   {
+  case DashedNormalEnd:
   case NormalEnd:
     break;
 
@@ -388,7 +388,8 @@ static void draw_line_end(CairoCtx *cr, LineEndType type, const Color &lcolor, c
     cr->set_color(lcolor);
     cr->stroke_preserve();
     break;
-    
+
+  case DashedChickenFootEnd:
   case ChickenFootEnd:
     cr->set_color(lcolor);
     cr->move_to(-5, 0);
@@ -497,6 +498,7 @@ static void draw_line_end_gl(LineEndType type, const Color &lcolor, const Color
 {
   switch (type)
   {
+    case DashedNormalEnd:
     case NormalEnd:
       break;
 
@@ -525,6 +527,7 @@ static void draw_line_end_gl(LineEndType type, const Color &lcolor, const Color
         break;
       }
 
+    case DashedChickenFootEnd:
     case ChickenFootEnd:
       gl_setcolor(lcolor);
       glBegin(GL_LINE_STRIP);
@@ -543,9 +546,9 @@ static void draw_line_end_gl(LineEndType type, const Color &lcolor, const Color
       glEnd();
 
       gl_setcolor(bcolor);
-      gl_arc(0, 12, 3, 0, 2 * M_PI, true);
+      gl_arc(0, 12, 4, 0, 2 * M_PI, true);
       gl_setcolor(lcolor);
-      gl_arc(0, 12, 3, 0, 2 * M_PI, false);
+      gl_arc(0, 12, 4, 0, 2 * M_PI, false);
       break;
 
     case ChickenFoot1End:
