@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -26,11 +26,11 @@ BEGIN_MDC_DECLS
 
 class VertexHandle : public ItemHandle {
 public:
-  VertexHandle(InteractionLayer *ilayer, CanvasItem *item, const MySQL::Geometry::Point &pos,
+  VertexHandle(InteractionLayer *ilayer, CanvasItem *item, const base::Point &pos,
                bool connectable);
   virtual ~VertexHandle();
   
-  virtual MySQL::Geometry::Rect get_bounds() const;
+  virtual base::Rect get_bounds() const;
   virtual void repaint(CairoCtx *cr);
 
   void set_connected(bool flag);
