@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,7 +23,7 @@
 #include <mdc_canvas_view_image.h>
 
 using namespace mdc;
-using namespace MySQL::Geometry;
+using namespace base;
 
 BEGIN_TEST_DATA_CLASS(canvas_view)
 
