@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,25 +19,12 @@
 
 #include "stdafx.h"
 
-#ifndef _WIN32
-#include <list>
-#include <string>
-#endif
-
-#ifdef _WIN32
-#include <windows.h>
-#endif
-
 #include "driver_manager.h"
-#include <gmodule.h>
-#include <cppconn/connection.h>
-#include <cppconn/statement.h>
-#include <cppconn/exception.h>
-#include <boost/foreach.hpp>
-#include <boost/scoped_array.hpp>
-#include <sstream>
 
-#include "boost/foreach.hpp"
+#include "mysql_driver.h"
+#include "cppconn/driver.h"
+#include "cppconn/statement.h"
+#include "cppconn/exception.h"
 
 namespace sql {
 
@@ -176,7 +163,7 @@ void DriverManager::set_driver_dir(const std::string &path)
   _driver_path= path;
 }
 
-TunnelConnection *DriverManager::getTunnel(const db_mgmt_ConnectionRef &connectionProperties)
+boost::shared_ptr<TunnelConnection> DriverManager::getTunnel(const db_mgmt_ConnectionRef &connectionProperties)
 {
   db_mgmt_DriverRef drv = connectionProperties->driver();
   if (!drv.is_valid())
@@ -184,9 +171,11 @@ TunnelConnection *DriverManager::getTunnel(const db_mgmt_ConnectionRef &connecti
 
   if (_createTunnel)
     return _createTunnel(connectionProperties);
-  return 0;
+  return boost::shared_ptr<TunnelConnection>();
 }
-  
+
+//--------------------------------------------------------------------------------------------------
+
 #define MYSQL_PASSWORD_CACHE_TIMEOUT 60
 
 ConnectionWrapper DriverManager::getConnection(const db_mgmt_ConnectionRef &connectionProperties, ConnectionInitSlot connection_init_slot)
@@ -195,7 +184,7 @@ ConnectionWrapper DriverManager::getConnection(const db_mgmt_ConnectionRef &conn
   if (!drv.is_valid())
     throw SQLException("Invalid connection settings: undefined connection driver");
 
-  TunnelConnection *tunnel = NULL;
+  boost::shared_ptr<TunnelConnection> tunnel;
   if (_createTunnel)
   {
     tunnel = _createTunnel(connectionProperties);
@@ -209,11 +198,16 @@ ConnectionWrapper DriverManager::getConnection(const db_mgmt_ConnectionRef &conn
   return getConnection(connectionProperties, tunnel, Authentication::Ref(), connection_init_slot);
 }
 
+//--------------------------------------------------------------------------------------------------
 
-ConnectionWrapper DriverManager::getConnection(const db_mgmt_ConnectionRef &connectionProperties, TunnelConnection *tunnel, Authentication::Ref password, ConnectionInitSlot connection_init_slot)
+ConnectionWrapper DriverManager::getConnection(const db_mgmt_ConnectionRef &connectionProperties,
+  boost::shared_ptr<TunnelConnection> tunnel, Authentication::Ref password,
+  ConnectionInitSlot connection_init_slot)
 {
+  // Load the driver for the connection dynamically. However for the C++ connector we have a static
+  // link anyway, so use this instead.
+
   // 0. determine correct driver filename
-  
   db_mgmt_DriverRef drv = connectionProperties->driver();
   
   std::string library= "";
@@ -221,41 +215,63 @@ ConnectionWrapper DriverManager::getConnection(const db_mgmt_ConnectionRef &conn
     library = drv->driverLibraryName();
   else
     throw SQLException("Invalid connection settings: undefined connection driver");
-#ifdef _WIN32
-  library.append(".dll");
-#elif defined(__APPLE__)
-  library.append(".dylib");
-#else
-  library.append(".so");
-#endif
 
-  // 1. find driver
-
-  GModule *gmodule= g_module_open((_driver_path + "/" + library).c_str(), G_MODULE_BIND_LOCAL);
-  if (NULL == gmodule)
+  Driver *driver;
+  if (library == "mysqlcppconn")
+    driver = reinterpret_cast<Driver *>(sql_mysql_get_driver_instance());
+  else
   {
-    fprintf(stderr, "Error: %s", g_module_error());
-    throw SQLException(std::string("Database driver: Failed to open library '").append(_driver_path + "/" + library).append("'. Check settings.").c_str());
-  }
+  #ifdef _WIN32
+    library.append(".dll");
+  #elif defined(__APPLE__)
+    library.append(".dylib");
+  #else
+    library.append(".so");
+  #endif
+
+    // 1. find driver
+    GModule *gmodule= g_module_open((_driver_path + "/" + library).c_str(), G_MODULE_BIND_LOCAL);
+    if (NULL == gmodule)
+    {
+      fprintf(stderr, "Error: %s", g_module_error());
+      throw SQLException(std::string("Database driver: Failed to open library '").append(_driver_path + "/" + library).append("'. Check settings.").c_str());
+    }
   
-  Driver *(* get_driver_instance)()= NULL;
-  g_module_symbol(gmodule, "sql_mysql_get_driver_instance", (gpointer*)&get_driver_instance);
-  if (NULL == get_driver_instance)
-    throw SQLException("Database driver: Failed to get library instance. Check settings.");
+    Driver *(* get_driver_instance)()= NULL;
+    g_module_symbol(gmodule, "sql_mysql_get_driver_instance", (gpointer*)&get_driver_instance);
+    if (NULL == get_driver_instance)
+      throw SQLException("Database driver: Failed to get library instance. Check settings.");
+
+    driver = get_driver_instance();
+  }
+  if (driver == NULL)
+    throw SQLException("Database driver: Failed to get driver instance. Check  settings.");
 
   // 2. call driver->connect()
   Param_types param_types;
   {
     grt::ListRef<db_mgmt_DriverParameter> params= connectionProperties->driver()->parameters();
-    for (size_t n= 0, count= params.count(); n < count; ++n)
+    for (size_t n = 0, count = params.count(); n < count; ++n)
     {
       db_mgmt_DriverParameterRef param= params.get(n);
       param_types[param->name()]= param->paramType();
     }
+    // set the values that take boolean options, so that they can be converted correctly in case they're specified manually by the user
+    param_types["OPT_REPORT_DATA_TRUNCATION"] = "boolean";
+    param_types["OPT_ENABLE_CLEARTEXT_PLUGIN"] = "boolean";
+    param_types["CLIENT_INTERACTIVE"] = "boolean";
+    param_types["CLIENT_COMPRESS"] = "boolean";
+    param_types["CLIENT_FOUND_ROWS"] = "boolean";
+    param_types["CLIENT_IGNORE_SIGPIPE"] = "boolean";
+    param_types["CLIENT_IGNORE_SPACE"] = "boolean";
+    param_types["CLIENT_LOCAL_FILES"] = "boolean";
+    param_types["CLIENT_MULTI_STATEMENTS"] = "boolean";
+    param_types["CLIENT_NO_SCHEMA"] = "boolean";
+
+    param_types["useLegacyAuth"] = "boolean";
   }
-  //std::map<std::string, ConnectPropertyVal> properties;
   ConnectOptionsMap properties;
-  grt::DictRef parameter_values= connectionProperties->parameterValues();
+  grt::DictRef parameter_values = connectionProperties->parameterValues();
   parameter_values.foreach(boost::bind(&conv_to_dbc_value, _1, _2, boost::ref(properties), param_types));
 
   {
@@ -263,34 +279,50 @@ ConnectionWrapper DriverManager::getConnection(const db_mgmt_ConnectionRef &conn
     const int conn_timeout = 60;
     const int read_timeout = 600;
     if (properties.find("OPT_CONNECT_TIMEOUT") == properties.end())
-      properties["OPT_CONNECT_TIMEOUT"]= conn_timeout;
+      properties["OPT_CONNECT_TIMEOUT"] = conn_timeout;
     if (properties.find("OPT_READ_TIMEOUT") == properties.end())
-      properties["OPT_READ_TIMEOUT"]= read_timeout;
+      properties["OPT_READ_TIMEOUT"] = read_timeout;
+  }
+  properties["OPT_CAN_HANDLE_EXPIRED_PASSWORDS"] = true;
+  properties["CLIENT_MULTI_STATEMENTS"] = true;
+  properties["metadataUseInfoSchema"] = false; // I_S is way too slow for many things as of MySQL 5.6.10, so disable it for now
+    
+  // If SSL is enabled but there's no certificate or anything, create the sslKey option to force enabling SSL without a key
+  // (equivalent to starting cmdline client with mysql --ssl-key=)
+  if (parameter_values.get_string("sslKey", "") == "" && parameter_values.get_int("useSSL", 0) != 0)
+    properties["sslKey"] = std::string();
+
+  // If SSL is not enabled, clear all ssl related props to not confuse the connector
+  if (parameter_values.get_int("useSSL", 0) == 0)
+  {
+    properties.erase("sslKey");
+    properties.erase("sslCert");
+    properties.erase("sslCA");
+    properties.erase("sslCAPath");
+    properties.erase("sslCipher");
   }
-  properties["CLIENT_MULTI_STATEMENTS"]= true;
 
   // If we are on a pipe connection then set the host name explicitly.
   // However, pipe connections can only be established on the local box (Win only).
   if (drv->name() == "MysqlNativeSocket")
   {
-    ConnectPropertyVal host = std::string(".");
 #ifdef _WIN32
-    properties["hostName"]= host;
+    properties["hostName"] = std::string(".");
 #else
-    properties["hostName"]= std::string();
+    properties["hostName"] = std::string();
 #endif
   }
 
   if (tunnel)
   {
-    // this can throw an exception if the tunnel can't be created
-    //!tunnel->connect();
-    
-    // make the driver connect to the local tunnel port
-    properties["port"]= tunnel->get_port();
-    properties["hostName"]= sql::SQLString("127.0.0.1");
+    // Make the driver connect to the local tunnel port.
+    properties["port"] = tunnel->get_port();
+    properties["hostName"] = sql::SQLString("127.0.0.1");
   }
-  
+  else
+    properties["hostName"] = "["+parameter_values.get_string("hostName")+"]"; // [quote] hostname so that ipv6 addresses aren't parsed as URIs
+
+
   Authentication::Ref authref;
 
   // Check if there is a stored or cached password, if there isn't try without one (blank)
@@ -351,21 +383,10 @@ retry:
       }
     }
   }
-  
-  Driver *driver= get_driver_instance();
-  if (NULL == driver)
-    throw SQLException("Database driver: Failed to get driver instance. Check  settings.");
 
   try
   {
     std::auto_ptr<Connection> conn(driver->connect(properties));
-
-    if (connection_init_slot)
-        connection_init_slot(conn.get(), connectionProperties);
-    
-    std::string def_schema= parameter_values.get_string("schema", "");
-    if (!def_schema.empty())
-      conn->setSchema(def_schema);
     
     // make sure the user we got logged in is the user we wanted
     {
@@ -395,6 +416,20 @@ retry:
       }
     }
     
+    std::string sql_mode = parameter_values.get_string("SQL_MODE", "");
+    if (!sql_mode.empty())
+    {
+      boost::scoped_ptr<sql::Statement> statement(conn.get()->createStatement());
+      statement->execute("SET SESSION SQL_MODE='"+sql_mode+"'");
+    }
+
+    if (connection_init_slot)
+      connection_init_slot(conn.get(), connectionProperties);
+    
+    std::string def_schema= parameter_values.get_string("schema", "");
+    if (!def_schema.empty())
+      conn->setSchema(def_schema);
+
     return ConnectionWrapper(conn, tunnel);
   }
   catch (sql::SQLException &exc)
