@@ -1,15 +1,36 @@
+/* 
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
+
 #ifndef _DRIVER_MANAGER_H_
 #define _DRIVER_MANAGER_H_
 
-
 #include "cppdbc_public_interface.h"
-#include "grts/structs.db.mgmt.h"
+
 #include <cppconn/driver.h>
 #include <boost/cstdint.hpp>
 #include <boost/shared_ptr.hpp>
 #include <memory>
 #include <set>
 
+#include "grts/structs.db.mgmt.h"
+#include "cppconn/connection.h"
+
 #ifndef THROW
 #ifdef _MSC_VER
 #define THROW(...)
@@ -18,7 +39,6 @@
 #endif
 #endif
 
-
 namespace sql
 {
 typedef boost::shared_ptr<Connection> ConnectionPtr;
@@ -30,6 +50,7 @@ public:
   virtual void connect(db_mgmt_ConnectionRef connectionProperties)= 0;
   virtual void disconnect()= 0;
   virtual int get_port()= 0;
+  virtual bool get_message(std::string &type, std::string &message) = 0;
 };
 
 
@@ -39,7 +60,7 @@ class ConnectionWrapper
   boost::shared_ptr<TunnelConnection> _tunnel_conn;
 public:
   ConnectionWrapper() {}
-  ConnectionWrapper(std::auto_ptr<Connection> conn,TunnelConnection* tunn_conn):_conn(conn),_tunnel_conn(tunn_conn) {}
+  ConnectionWrapper(std::auto_ptr<Connection> conn, boost::shared_ptr<TunnelConnection> tunn_conn):_conn(conn),_tunnel_conn(tunn_conn) {}
   void reset() {_conn.reset();}
   Connection* operator->() {return _conn.get();}
   Connection* get() {return _conn.get();}
@@ -73,13 +94,13 @@ public:
 
 typedef std::set<Authentication::Ref> AuthenticationSet;
 
-class CPPDBC_PUBLIC_FUNC AuthenticationError : public std::runtime_error
+class CPPDBC_PUBLIC_FUNC AuthenticationError : public grt::db_login_error
 {
   Authentication::Ref _authobj;
 public:
-  AuthenticationError(const AuthenticationError &other) : std::runtime_error(other), _authobj(other._authobj) {}
+  AuthenticationError(const AuthenticationError &other) : grt::db_login_error(other.what()), _authobj(other._authobj) {}
   AuthenticationError(const std::string &what, Authentication::Ref auth)
-  : std::runtime_error(what), _authobj(auth) {}
+  : grt::db_login_error(what), _authobj(auth) {}
   virtual ~AuthenticationError() THROW () {}
   
   Authentication::Ref authentication() { return _authobj; }
@@ -106,15 +127,15 @@ public:
   // Returns a Connection object for the give connection params
   ConnectionWrapper getConnection(const db_mgmt_ConnectionRef &connectionProperties, ConnectionInitSlot connection_init_slot= ConnectionInitSlot());
 
-  ConnectionWrapper getConnection(const db_mgmt_ConnectionRef &connectionProperties, TunnelConnection *tunnel, Authentication::Ref password, ConnectionInitSlot connection_init_slot= ConnectionInitSlot());
+  ConnectionWrapper getConnection(const db_mgmt_ConnectionRef &connectionProperties, boost::shared_ptr<TunnelConnection> tunnel, Authentication::Ref password, ConnectionInitSlot connection_init_slot= ConnectionInitSlot());
   
-  TunnelConnection *getTunnel(const db_mgmt_ConnectionRef &connectionProperties);
+  boost::shared_ptr<TunnelConnection> getTunnel(const db_mgmt_ConnectionRef &connectionProperties);
   
   // Returns the list of available drivers
   std::list<Driver *> getDrivers();
   
 public:
-  typedef boost::function<TunnelConnection* (const db_mgmt_ConnectionRef &)> TunnelFactoryFunction;
+  typedef boost::function<boost::shared_ptr<TunnelConnection> (const db_mgmt_ConnectionRef &)> TunnelFactoryFunction;
   typedef boost::function<bool (const db_mgmt_ConnectionRef &, std::string &)> PasswordFindFunction;
   typedef boost::function<std::string (const db_mgmt_ConnectionRef &, bool)> PasswordRequestFunction;
 
