@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,6 +17,7 @@
  * 02110-1301  USA
  */
 
+
 #include "tut_stdafx.h"
 
 #include "connection_helpers.h"
@@ -60,32 +61,68 @@ public:
   GRTManagerTest grtm;
   SqlFacade::Ref sql_splitter;
   GRT *grt;
+  
+  TEST_DATA_CONSTRUCTOR(module_dbc_statement_test)
+  {
+    sql_splitter = NULL;
+#ifdef _WIN32
+    std::string path = "../../Bin/Debug";
+    grtm.set_search_paths(path, path, path);
+#else
+    std::string path = ".";
+    std::string libraries = getenv("MWB_LIBRARY_DIR");
+    std::string modules = getenv("TEST_MODULES_DIR");
+    
+    grtm.set_search_paths(modules, path, libraries);
+#endif  
+
+    grtm.initialize(true, path);
+
+    grt= grtm.get_grt();
+
+    // load structs
+    grt->scan_metaclasses_in("../../res/grt/");
+    grt->end_loading_metaclasses();
+
+    ensure_equals("load structs", grt->get_metaclasses().size(), INT_METACLASS_COUNT);
+
+    sql_splitter= SqlFacade::instance_for_rdbms_name(grt, "Mysql");
+    ensure("failed to get sqlparser module", (NULL != sql_splitter));
+    
+    db_mgmt_ConnectionRef connectionProperties(grt);
+
+    setup_env(grt, connectionProperties);   
+    
+    sql::DriverManager *dm= sql::DriverManager::getDriverManager();
+    ensure("dm is NULL", dm != NULL);
+
+    sql::ConnectionWrapper wrapper= dm->getConnection(connectionProperties);
+    ensure("conn is NULL", wrapper.get() != NULL);
+
+    sql::Connection* connection= wrapper.get();
+    std::auto_ptr<sql::Statement> stmt(connection->createStatement());    
+    stmt->execute("DROP SCHEMA IF EXISTS test; CREATE SCHEMA test");
+  }
+  
+  TEST_DATA_DESTRUCTOR(module_dbc_statement_test)
+  {
+    db_mgmt_ConnectionRef connectionProperties(grt);
+
+    setup_env(grt, connectionProperties);   
+    sql::DriverManager *dm= sql::DriverManager::getDriverManager();
+    ensure("dm is NULL", dm != NULL);
+
+    sql::ConnectionWrapper wrapper= dm->getConnection(connectionProperties);
+    ensure("conn is NULL", wrapper.get() != NULL);
+
+    sql::Connection* connection= wrapper.get();
+    std::auto_ptr<sql::Statement> stmt(connection->createStatement());    
+    stmt->execute("DROP SCHEMA IF EXISTS test;");
+  }
 END_TEST_DATA_CLASS
 
 TEST_MODULE(module_dbc_statement_test, "DBC: statement tests");
 
-TEST_FUNCTION(1)
-{
-  sql_splitter = NULL;
-  std::string path = "../../Bin/Debug";
-  grtm.set_search_paths(path, path, path);
-  grtm.initialize(path);
-
-  grt= grtm.get_grt();
-
-  // load structs
-  grt->scan_metaclasses_in("../../res/grt/");
-  grt->end_loading_metaclasses();
-
-  ensure_equals("load structs", grt->get_metaclasses().size(), INT_METACLASS_COUNT);
-
-  sql_splitter= SqlFacade::instance_for_rdbms_name(grt, "Mysql");
-  ensure("failed to get sqlparser module", (NULL != sql_splitter));
-}
-
-// We don't test Statement::getWarnings() as such doesn't exist.
-// We don't test Statement::clearWarnings() as such doesn't exist.
-
 // Test construction of a statement object.
 TEST_FUNCTION(2)
 {
@@ -663,10 +700,16 @@ TEST_FUNCTION(16)
 			"CREATE TABLE dbc_statement_test_15.table3 (id int);"
 			"SELECT 1;";
 		std::list<std::string> statements;
-        ensure("failed to get sqlparser module", (NULL != sql_splitter));
-		sql_splitter->splitSqlScript(sql_script, statements);
-		sql::SqlBatchExec()(stmt.get(), statements);
+    
+    ensure("failed to get sqlparser module", (NULL != sql_splitter));
+    
+    sql_splitter->splitSqlScript(sql_script, statements);
+    sql::SqlBatchExec()(stmt.get(), statements);
 
+    //  Cleanup
+    sql_script= "DROP DATABASE IF EXISTS dbc_statement_test_15;";
+    sql_splitter->splitSqlScript(sql_script, statements);
+    sql::SqlBatchExec()(stmt.get(), statements);
   } catch (sql::SQLException &) {
     printf("ERR: Caught sql::SQLException\n");
     throw;
@@ -674,3 +717,4 @@ TEST_FUNCTION(16)
 }
 
 END_TESTS
+
