@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,15 +16,18 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
+
 #include "stdafx.h"
 
+#include "mforms/mforms.h"
+
 using namespace mforms;
 
-CheckBox::CheckBox()
+CheckBox::CheckBox(bool square)
 {
   _checkbox_impl= &ControlFactory::get_instance()->_checkbox_impl;
 
-  _checkbox_impl->create(this);
+  _checkbox_impl->create(this, square);
 }
 
 
