@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,8 +19,9 @@
 
 #include "stdafx.h"
 
-using namespace mforms;
+#include "mforms/mforms.h"
 
+using namespace mforms;
 
 Form::Form(Form *owner, FormFlag flag)
 {
@@ -34,7 +35,7 @@ Form::Form(Form *owner, FormFlag flag)
 
 Form::Form()
 {
-  _form_impl= 0;
+  _form_impl= &ControlFactory::get_instance()->_form_impl;
   _content= 0;
 }
 
@@ -48,7 +49,7 @@ Form *Form::main_form()
 
 Form::~Form()
 {
-  if (_content)
+  if (_content && !_content->release_on_add())
     _content->release();
 }
 
@@ -101,11 +102,16 @@ void Form::set_content(View *view)
 {
   if (_content != view && _form_impl)
   {
-    if (_content) _content->release();
-    _content= view;
-    _content->retain();
+    if (_content)
+      _content->release();
+    _content = view;
+    if (!_content->release_on_add())
+      _content->retain();
     _form_impl->set_content(this, view);
     _content->set_parent(this);
+
+    // Note: we are not adding the content view to the underlying view's child list and hence
+    //       have to take care for ref counting here.
   }
 }
 
