@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -65,6 +65,12 @@ void mforms::gtk::LabelImpl::realized()
         _label->modify_font(font);
       }
       break;
+    case ::mforms::VeryBigStyle:
+      {
+        font.set_size(font.get_size()*5/3);
+        _label->modify_font(font);
+      }
+      break;
     case ::mforms::BigBoldStyle:
       {
         font.set_size(font.get_size()*4/3);
