@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -30,6 +30,7 @@ namespace mforms {
 namespace gtk {
 
 //------------------------------------------------------------------------------
+
 PopupImpl::PopupImpl(::mforms::Popup *self, mforms::PopupStyle style)
   : ObjectImpl(self)
   , _width(-1)
@@ -37,6 +38,7 @@ PopupImpl::PopupImpl(::mforms::Popup *self, mforms::PopupStyle style)
   , _have_rgba(false)
   , _inside(false)
   , _result(-1)
+  , _style(style)
 {
   d("\n");
   mforms::Popup* selfc = dynamic_cast<mforms::Popup*>(owner);
@@ -59,13 +61,14 @@ PopupImpl::PopupImpl(::mforms::Popup *self, mforms::PopupStyle style)
   _wnd.property_skip_taskbar_hint() = true;
   _wnd.property_skip_pager_hint() = true;
   _wnd.property_decorated() = false;
+  _wnd.modify_bg(Gtk::STATE_NORMAL, Gdk::Color("black"));
 
   on_screen_changed(_wnd.get_screen());
   set_size(self, 825, 351);
   _wnd.set_transient_for(*get_mainwindow());
   _wnd.set_modal(true);
 #if GTK_VERSION_GE(2,12)
-  _wnd.set_opacity(0.92);
+  _wnd.set_opacity(0.7);
 #endif
 }
 
@@ -100,7 +103,7 @@ bool PopupImpl::handle_expose_event(GdkEventExpose *event)
     if (cr)
     {
       // Draw round corners
-      if (_width > 0 && _height > 0)
+      if (_width > 0 && _height > 0 && _style == mforms::PopupBezel)
       {
         //cairo_save(cr);
 
@@ -193,15 +196,16 @@ bool PopupImpl::mouse_button_event(GdkEventButton *event)
     }
     else if (event->type == GDK_BUTTON_RELEASE)
     {
+      self->retain();
       self->mouse_up(event->button-1, (int)event->x, (int)event->y);
       self->mouse_click(event->button-1, (int)event->x, (int)event->y); // Click must be called after mouse was up!
+      self->release();
     }
     else if (event->type == GDK_2BUTTON_PRESS)
       self->mouse_double_click(event->button-1, (int)event->x, (int)event->y);
   }
   else
     set_modal_result(self, 0);
-
   return false;
 }
 
@@ -222,6 +226,16 @@ bool PopupImpl::create(::mforms::Popup *self, ::mforms::PopupStyle style)
   return new PopupImpl(self, style) != 0;
 }
 
+//------------------------------------------------------------------------------
+
+void PopupImpl::destroy(::mforms::Popup *self)
+{
+  d("\n");
+  PopupImpl *impl = self->get_data<PopupImpl>();
+  self->set_data(NULL, NULL);
+  delete impl;
+}
+
 //------------------------------------------------------------------------------
 void PopupImpl::set_needs_repaint(::mforms::Popup *self)
 {
@@ -237,33 +251,34 @@ void PopupImpl::set_size(::mforms::Popup *self, int w, int h)
   PopupImpl *impl = self->get_data<PopupImpl>();
   d("\n");
 
-  impl->_width = w + 2*R;
-  impl->_height = h + 2*R;
-  impl->_wnd.set_size_request(impl->_width, impl->_height);}
+  impl->_width = w;// + 2*R;
+  impl->_height = h;// + 2*R;
+  impl->_wnd.set_size_request(impl->_width, impl->_height);
+}
 
 //------------------------------------------------------------------------------
 int PopupImpl::show(::mforms::Popup *self, int x, int y)
 {
   PopupImpl *impl = self->get_data<PopupImpl>();
-
   d("x=%i, y=%i\n", x, y);
   if (impl->_wnd.is_visible())
     impl->_wnd.hide();
-  else
+
   {
     Gtk::Window* main_window = get_mainwindow();
 
     const Gtk::Requisition req = impl->_wnd.size_request();
-    const int x1 = x - req.width;
-
     impl->_wnd.show();
-    impl->_wnd.move(x1,y-16);
+    impl->_wnd.move(x, y);
     
-    impl->_wnd.get_window()->pointer_grab(true, Gdk::BUTTON_PRESS_MASK|Gdk::BUTTON_RELEASE_MASK|Gdk::POINTER_MOTION_MASK, 0);
+    if (impl->_style == mforms::PopupBezel)
+    {
+      impl->_wnd.get_window()->pointer_grab(true, Gdk::BUTTON_PRESS_MASK|Gdk::BUTTON_RELEASE_MASK|Gdk::POINTER_MOTION_MASK, 0);
 
-    Gtk::Main::run();
-    impl->_wnd.set_modal(false);
-    impl->_wnd.hide();
+      Gtk::Main::run();
+      impl->_wnd.set_modal(false);
+      impl->_wnd.hide();
+    }
   }
 
   return impl->_result;
@@ -274,7 +289,10 @@ base::Rect PopupImpl::get_content_rect(::mforms::Popup *self)
 {
   d("\n");
   PopupImpl *impl = self->get_data<PopupImpl>();
-  return base::Rect(R, R, impl->_width-R, impl->_height-R);
+  if (impl->_style == mforms::PopupBezel)
+    return base::Rect(R, R, impl->_width-R, impl->_height-R);
+  else
+    return base::Rect(0, 0, impl->_width, impl->_height);
 }
 
 //------------------------------------------------------------------------------
@@ -283,8 +301,14 @@ void PopupImpl::set_modal_result(Popup *self, int result)
   d("\n");
   PopupImpl *impl = self->get_data<PopupImpl>();
   impl->_result = result;
-  if (result > -1)
+  impl->_wnd.hide();
+
+  if (result > -1 && impl->_style == mforms::PopupBezel)
     Gtk::Main::quit();
+
+  // must call closed() cb when idle, because it can delete the popup
+  // in the middle of an event handler
+  Glib::signal_idle().connect(sigc::bind_return(sigc::mem_fun(self, &Popup::closed), false));
 }
 
 //------------------------------------------------------------------------------
@@ -293,6 +317,7 @@ void PopupImpl::init()
   ::mforms::ControlFactory *f = ::mforms::ControlFactory::get_instance();
 
   f->_popup_impl.create            = &PopupImpl::create;
+  f->_popup_impl.destroy           = &PopupImpl::destroy;
   f->_popup_impl.set_needs_repaint = &PopupImpl::set_needs_repaint;
   f->_popup_impl.set_size          = &PopupImpl::set_size;
   f->_popup_impl.show              = &PopupImpl::show;
