@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -245,8 +245,16 @@ bool ViewImpl::is_enabled(::mforms::View *self)
 }
 
 
-void ViewImpl::set_name(::mforms::View *view, const std::string &name)
+void ViewImpl::set_name(::mforms::View *self, const std::string &name)
 {
+  ViewImpl *view = self->get_data<ViewImpl>();
+  if ( view )
+    view->set_name(name);
+}
+
+void ViewImpl::set_name(const std::string &name)
+{
+
 }
 
 void ViewImpl::relayout(::mforms::View *view)
@@ -288,9 +296,20 @@ void ViewImpl::size_changed()
   }
 }
 
+void ViewImpl::on_focus_grab()
+{
+  if (get_outer() && get_outer()->is_realized())
+  {
+    ::mforms::View* owner_view = dynamic_cast< ::mforms::View*>(owner);
+    if (owner_view)
+      owner_view->focus_changed();
+  }
+}
+
 void ViewImpl::setup()
 {
   get_outer()->signal_size_allocate().connect(sigc::hide(sigc::mem_fun(this, &ViewImpl::size_changed)));
+  get_outer()->signal_grab_focus().connect(sigc::mem_fun(this, &ViewImpl::on_focus_grab));
   get_outer()->signal_realize().connect(sigc::mem_fun(this, &ViewImpl::size_changed));
   get_outer()->show();
 }
@@ -316,12 +335,31 @@ void ViewImpl::set_front_color(::mforms::View *self, const std::string &color)
       w->modify_fg(Gtk::STATE_NORMAL, c);
     }
   }
+  view->set_front_color(color);
 }
 
 void ViewImpl::set_back_color(::mforms::View *self, const std::string &color)
 {
   ViewImpl *view= self->get_data<ViewImpl>();
-  view->set_back_color(color); //XXX should use modify_bg()
+  if (view)
+  {
+    Gtk::Widget *w = view->get_inner();
+    if (w)
+    {
+      if (color.empty())
+      {
+        w->unset_bg(Gtk::STATE_NORMAL);
+        w->unset_base(Gtk::STATE_NORMAL);
+      }
+      else
+      {
+        Gdk::Color gcolor(color);
+        w->get_colormap()->alloc_color(gcolor);
+        w->modify_bg(Gtk::STATE_NORMAL, gcolor);
+        w->modify_base(Gtk::STATE_NORMAL, gcolor);
+      }
+    }
+  }
 }
 
 std::string ViewImpl::get_front_color(::mforms::View *self)
@@ -334,17 +372,106 @@ std::string ViewImpl::get_front_color(::mforms::View *self)
 std::string ViewImpl::get_back_color(::mforms::View *self)
 {
   ViewImpl *view= self->get_data<ViewImpl>();
-  //TODO
+  double *color = (double*)g_object_get_data(G_OBJECT(view->get_inner()->gobj()), "bg");
+  if (color)
+  {
+    char buffer[32];
+    sprintf(buffer, "#%02x%02x%02x", (int)(color[0]*255), (int)(color[1]*255), (int)(color[2]*255));
+    return buffer;
+  }
   return "";
 }
 
 
-
-void ViewImpl::set_back_image(::mforms::View *self, const std::string &path, mforms::ImageLayout layout)
+bool ViewImpl::on_expose_event(GdkEventExpose *event, Gtk::Widget *target)
 {
-  // TODO: implement background image.
+  int x, y;
+  int iwidth, fwidth;
+  int iheight, fheight;
+
+  if (_back_image)
+  {
+    x = 0;
+    y = 0;
+    iwidth = _back_image->get_width();
+    iheight = _back_image->get_height();
+    fwidth = target->get_width();
+    fheight = target->get_height();
+
+    switch (_back_image_alignment)
+    {
+      case mforms::TopLeft:
+        x = 0;
+        y = 0;
+        break;
+      case mforms::TopCenter:
+        x = (iwidth+fwidth) / 2;
+        y = 0;
+        break;
+      case mforms::TopRight:
+        x = fwidth - iwidth;
+        y = 0;
+        break;
+      case mforms::MiddleLeft:
+        x = 0;
+        y = (iheight+fheight) / 2;
+        break;
+      case mforms::MiddleCenter:
+        x = (iwidth+fwidth) / 2;
+        y = (iheight+fheight) / 2;
+        break;
+      case mforms::MiddleRight:
+        x = fwidth - iwidth;
+        y = (iheight+fheight) / 2;
+        break;
+      case mforms::BottomLeft:
+        x = 0;
+        y = fheight - iheight;
+        break;
+      case mforms::BottomCenter:
+        x = (iwidth+fwidth) / 2;
+        y = fheight - iheight;
+        break;
+      case mforms::BottomRight:
+        x = fwidth - iwidth;
+        y = fheight - iheight;
+        break;
+      default:
+        break;
+    }
+
+    _back_image->render_to_drawable(target->get_window(), target->get_style()->get_fg_gc(Gtk::STATE_NORMAL), 0, 0, x, y, iwidth, iheight, Gdk::RGB_DITHER_NONE, 0, 0);
+
+    return true;
+  }
+  return false;
 }
 
+void ViewImpl::set_back_image(::mforms::View *self, const std::string &path, mforms::Alignment ali)
+{
+  ViewImpl *view= self->get_data<ViewImpl>();
+  view->set_back_image(path, ali);
+}
+
+void ViewImpl::set_back_image(const std::string &path, mforms::Alignment alig)
+{
+  if (path.empty())
+  {
+    _back_image.reset();
+    return;
+  }
+
+  std::string file = mforms::App::get()->get_resource_path(path);
+  try
+  {
+    _back_image = Gdk::Pixbuf::create_from_file(file);
+    _back_image_alignment = alig;
+  }
+  catch (Glib::Error &exc)
+  {
+    log_error("Could not load image for background: %s: %s\n", file.c_str(), exc.what().c_str());
+  }
+}
 
 void ViewImpl::flush_events(::mforms::View *self)
 {
@@ -407,6 +534,15 @@ void set_bgcolor(Gtk::Widget* w, const std::string& strcolor)
   }
 }
 
+
+void ViewImpl::focus(::mforms::View *self)
+{
+  ViewImpl *view = self->get_data<ViewImpl>();
+  if (view && view->get_inner())
+    view->get_inner()->grab_focus();
+}
+
+
 void ViewImpl::init()
 {
   ::mforms::ControlFactory *f= ::mforms::ControlFactory::get_instance();
@@ -443,14 +579,32 @@ void ViewImpl::init()
   f->_view_impl.set_back_image       = &ViewImpl::set_back_image;
   f->_view_impl.flush_events         = &ViewImpl::flush_events;
   f->_view_impl.set_padding          = &ViewImpl::set_padding;
+  f->_view_impl.focus                = &ViewImpl::focus;
 };
 
 
 Gtk::Widget* ViewImpl::get_widget_for_view(mforms::View *view)
 {
   ViewImpl *vi = view->get_data<ViewImpl>();
-  return vi ? vi->get_outer() : 0;
+
+  if (vi)
+  {
+    Gtk::Widget *w = vi->get_outer();
+    w->set_data("mforms::View", view);
+    return w;
+  }
+  return NULL;
 }
 
+
+mforms::View *ViewImpl::get_view_for_widget(Gtk::Widget *w)
+{
+  mforms::View *view = (mforms::View*)w->get_data("mforms::View");
+  if (view)
+    return view;
+  return NULL;
+}
+
+
 };
 };
