@@ -50,7 +50,7 @@ namespace mforms {
     
     std::string (__stdcall *get_resource_path)(App *app, const std::string &file);
     std::string (__stdcall *get_executable_path)(App *app, const std::string &file);
-    std::string (__stdcall *get_bounds)(App *app);
+    base::Rect (__stdcall *get_application_bounds)(App *app);
     
     int (__stdcall *enter_event_loop)(App *app, float max_wait_time);
     void (__stdcall *exit_event_loop)(App *app, int result);
@@ -60,7 +60,7 @@ namespace mforms {
 
     base::Color (__stdcall *get_system_color)(SystemColor type);
 
-    void (__stdcall *close_splash)(App *app); // TODO: Linux
+    float (__stdcall *backing_scale_factor)(App *app);
   };
 #endif
 #endif
@@ -102,7 +102,7 @@ namespace mforms {
     std::string get_user_data_folder() { return _user_data_folder; }
     
     /** Gets the bounds of the main window. */
-    void get_bounds(int* x, int* y, int* w, int* h);
+    base::Rect get_application_bounds();
     
     /** Enters the event loop and exit only once exit_event_loop() is called.
      @param timeout Given in seconds. If > 0.0, the function will return -1 after that time.
@@ -119,8 +119,7 @@ namespace mforms {
     // Retrieve system information like predefined colors, screen size, monitor count etc.
     base::Color get_system_color(SystemColor type);
 
-    void close_splash();
-
+    float backing_scale_factor();
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 #ifndef SWIG
     void set_user_data_folder_path(const std::string &path) { _user_data_folder = path; }
@@ -129,6 +128,7 @@ namespace mforms {
   protected:
     AppImplPtrs *_app_impl;
     
+  private:
     std::string _user_data_folder;
   };
 };
