@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -56,6 +56,8 @@ namespace mforms {
     std::string (__stdcall *get_text)(TextEntry *self);
     void (__stdcall *set_read_only)(TextEntry *self, bool flag);
     void (__stdcall *set_placeholder_text)(TextEntry *self, const std::string &text);
+    void (__stdcall *set_placeholder_color)(TextEntry *self, const std::string &color);
+    void (__stdcall *set_bordered)(TextEntry *self, bool flag);
   };
 #endif
 #endif
@@ -70,16 +72,25 @@ namespace mforms {
 
     /** Sets the max allowed length that the user can type */
     void set_max_length(int len);
+
     /** Sets the text in the entry */
     void set_value(const std::string &text);
+    
     /** Gets the text in the entry */
     virtual std::string get_string_value();
+    
     /** Sets whether contents of entry can be edited by the user */
     void set_read_only(bool flag);
 
     /** Sets the text to be shown as a placeholder, when there's no text in the entry */
     void set_placeholder_text(const std::string &text);
 
+    /** Foreground color for the placeholder text. */
+    void set_placeholder_color(const std::string &color);
+
+    /** Sets whether to draw a border around the text box. Default is bordered. */
+    void set_bordered(bool flag);
+
 #ifndef SWIG
     /** Signal emitted when the entry is edited 
      
