@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,7 +20,6 @@
 #ifndef _MFORMS_TOOLBAR_H_
 #define _MFORMS_TOOLBAR_H_
 
-#include "base/trackable.h"
 #include <mforms/box.h>
 #include <mforms/app.h>
 #include <vector>
@@ -53,7 +52,7 @@ namespace mforms {
     ExpanderItem,
     ImageBoxItem
   };
-  
+#ifndef SWIG
   struct MFORMS_EXPORT ToolBarImplPtrs
   {
     bool (__stdcall *create_tool_bar)(ToolBar *item, ToolBarType type);
@@ -74,10 +73,10 @@ namespace mforms {
     // For selector items only.
     void (__stdcall *set_selector_items)(ToolBarItem* item, const std::vector<std::string>& values);
   }; 
-
+#endif
 
   // base abstract class for menuitem and menubase
-  class MFORMS_EXPORT ToolBar : public Container, public base::trackable
+  class MFORMS_EXPORT ToolBar : public Container
   {
   protected:    
     ToolBarImplPtrs *_impl;
@@ -122,7 +121,9 @@ namespace mforms {
     void set_tooltip(const std::string &text);
 
     void set_icon(const std::string &path);
+    std::string get_icon() { return _icon; };
     void set_alt_icon(const std::string &path);
+    std::string get_alt_icon() { return _alt_icon; };
 
     void set_enabled(bool flag);
     bool get_enabled();
@@ -147,6 +148,8 @@ namespace mforms {
   private:
     ToolBarImplPtrs *_impl;
     std::string _name;
+    std::string _icon;
+    std::string _alt_icon;
     ToolBarItemType _type;
     const bool _expandable;
     boost::signals2::signal<void (ToolBarItem*)> _clicked_signal;
