@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,7 +24,7 @@
 namespace mforms {
 namespace stub {
 
-DrawBoxImpl::DrawBoxImpl(::mforms::DrawBox *self)
+DrawBoxImpl::DrawBoxImpl(mforms::DrawBox *self)
   : ViewImpl(self)
 {
 }
@@ -35,21 +35,41 @@ void DrawBoxImpl::set_size(int width, int height)
 }
 
 
-bool __stdcall DrawBoxImpl::create(::mforms::DrawBox *self)
+bool DrawBoxImpl::create(mforms::DrawBox *self)
 {
   return true;
 }
 
-void __stdcall DrawBoxImpl::set_needs_repaint(::mforms::DrawBox *self)
+void DrawBoxImpl::set_needs_repaint(mforms::DrawBox *self)
 {
 }
 
+void DrawBoxImpl::add(mforms::DrawBox *, mforms::View *, mforms::Alignment alignment)
+{
+
+}
+
+void DrawBoxImpl::remove(mforms::DrawBox *, mforms::View *)
+{
+
+}
+
+void DrawBoxImpl::move(mforms::DrawBox *, mforms::View *, int x, int y)
+{
+
+}
+
+
 void DrawBoxImpl::init()
 {
   ::mforms::ControlFactory *f = ::mforms::ControlFactory::get_instance();
 
   f->_drawbox_impl.create         = &DrawBoxImpl::create;
   f->_drawbox_impl.set_needs_repaint = &DrawBoxImpl::set_needs_repaint;
+  f->_drawbox_impl.add = &DrawBoxImpl::add;
+  f->_drawbox_impl.remove = &DrawBoxImpl::remove;
+  f->_drawbox_impl.move = &DrawBoxImpl::move;
+
 }
 
 };
