@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -26,12 +26,11 @@
 
 #include "base/string_utilities.h"
 #include "base/file_functions.h"
+#include "base/log.h"
 
-#include <math.h>
+#include "mforms/mforms.h"
 
-#include <boost/shared_ptr.hpp>
-
-#include <glib/gstdio.h>
+DEFAULT_LOG_DOMAIN(DOMAIN_MFORMS_BE);
 
 using namespace mforms;
 using namespace base;
@@ -139,6 +138,7 @@ void Utilities::forget_message_answers()
 
 void Utilities::show_wait_message(const std::string &title, const std::string &text)
 {
+  // The wait message is a special window, so there's no need to hide the splash screen.
   ControlFactory::get_instance()->_utilities_impl.show_wait_message(title, text);
 }
 
@@ -163,7 +163,15 @@ static void* cancellable_task_thread(CancellableTaskData *data)
 {
   std::auto_ptr<CancellableTaskData> autodel(data);
 
-  void *ptr = data->task();
+  void *ptr = NULL;
+  try
+  {
+    ptr = data->task();
+  }
+  catch (std::exception &exc)
+  {
+    log_error("Cancellable task threw uncaught exception: %s", exc.what());
+  }
   g_mutex_lock(data->mutex.get());
   *data->result_ptr = ptr;
   *data->finished_flag = true;
@@ -239,6 +247,7 @@ retry:
   }
   else
   {
+    log_debug2("run_cancelable_wait_message returned false\n");
     // task canceled by user
     return false;
   }
@@ -294,6 +303,12 @@ bool Utilities::move_to_trash(const std::string &path)
   }
 }
 
+//--------------------------------------------------------------------------------------------------
+void Utilities::reveal_file(const std::string &path)
+{
+  ControlFactory::get_instance()->_utilities_impl.reveal_file(path);
+}
+
 //--------------------------------------------------------------------------------------------------
 
 void Utilities::add_timeout(float interval, const boost::function<bool ()> &callback)
@@ -320,8 +335,8 @@ bool Utilities::request_input(const std::string &title, const std::string &descr
                               const std::string &default_value, std::string &ret_value)
 {
   // In order to avoid trouble with window z-ordering we explicitly ask to hide any wait window
-  // that should get in the way.
-  hide_wait_message(); 
+  // that could get in the way. Same for the splash screen.
+  hide_wait_message();
 
   mforms::Form input_form(NULL, (FormFlag) (FormDialogFrame | FormStayOnTop));
   mforms::Table content;
@@ -372,8 +387,7 @@ bool Utilities::request_input(const std::string &title, const std::string &descr
 
 void Utilities::store_password(const std::string &service, const std::string &account, const std::string &password)
 {
-  Logger::log(Logger::LogDebug, DOMAIN_MFORMS_BE, "Storing password for '%s'@'%s'\n",
-    account.c_str(), service.c_str());
+  log_debug("Storing password for '%s'@'%s'\n", account.c_str(), service.c_str());
   ControlFactory::get_instance()->_utilities_impl.store_password(service, account, password);
 }
 
@@ -382,8 +396,7 @@ void Utilities::store_password(const std::string &service, const std::string &ac
 bool Utilities::find_password(const std::string &service, const std::string &account, std::string &password)
 {
   const bool ret = ControlFactory::get_instance()->_utilities_impl.find_password(service, account, password);
-  Logger::log(Logger::LogDebug, DOMAIN_MFORMS_BE, "Looking up password for '%s'@'%s' has %s\n",
-    account.c_str(), service.c_str(), ret ? "succeeded" : "failed");
+  log_debug("Looking up password for '%s'@'%s' has %s\n", account.c_str(), service.c_str(), ret ? "succeeded" : "failed");
   return ret;
 }
 
@@ -391,8 +404,7 @@ bool Utilities::find_password(const std::string &service, const std::string &acc
 
 void Utilities::forget_password(const std::string &service, const std::string &account)
 {
-  Logger::log(Logger::LogDebug, DOMAIN_MFORMS_BE, "Forgetting password for '%s'@'%s\n'",
-    account.c_str(), service.c_str());
+  log_debug("Forgetting password for '%s'@'%s'\n", account.c_str(), service.c_str());
   ControlFactory::get_instance()->_utilities_impl.forget_password(service, account);
 }
 
@@ -408,7 +420,7 @@ void *Utilities::perform_from_main_thread(const boost::function<void* ()> &slot,
 static void *_ask_for_password_main(const std::string &title, const std::string &service, std::string *username /*in/out*/,
                              bool prompt_storage, std::string *ret_password /*out*/, bool *ret_store /*out*/)
 {
-  Logger::log(Logger::LogDebug, DOMAIN_MFORMS_BE, "Creating and showing password dialog\n");
+  log_debug("Creating and showing password dialog\n");
 
   Utilities::hide_wait_message(); 
 
@@ -535,10 +547,11 @@ static bool _ask_for_password(const std::string &title, const std::string &servi
 //--------------------------------------------------------------------------------------------------
 
 bool Utilities::ask_for_password(const std::string &title, const std::string &service,
-                                 std::string &username /*in/out*/, std::string &ret_password /*out*/)
+                                 const std::string &username, std::string &ret_password /*out*/)
 {
+  std::string ret_username = username;
   bool dummy= false;
-  return _ask_for_password(title, service, username, false, ret_password, dummy);
+  return _ask_for_password(title, service, ret_username, false, ret_password, dummy);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -598,7 +611,7 @@ bool Utilities::credentials_for_service(const std::string &title, const std::str
       {
         show_warning(title.empty() ? _("Error Storing Password") : title, 
                      std::string("There was an error storing the password:\n")+exc.what(), "OK");
-        Logger::log(Logger::LogWarning, DOMAIN_MFORMS_BE, "Could not store password vault: \n", exc.what());
+        log_warning("Could not store password vault: %s\n", exc.what());
       }
     }
     return true;
