@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,15 +19,12 @@
 
 #include "stdafx.h"
 
-
-#define _USE_MATH_DEFINES
-#include <math.h>
-#include <vector>
+#include "mforms/mforms.h"
 
 using namespace std;
 
 using namespace mforms;
-using namespace MySQL::Geometry;
+using namespace base;
 
 // Animation timer. One timer for all widget animations.
 static int animation_timer_refcount= 0;
@@ -64,8 +61,8 @@ static vector<BaseWidget*> animated_widgets;
 #endif
 
 #define WIDGET_NORMAL_FONT_SIZE 11
-#define WIDGET_DESCRIPTION_FONT_SIZE 9
-#define WIDGET_WARNING_FONT_SIZE 8
+#define WIDGET_DESCRIPTION_FONT_SIZE 11
+#define WIDGET_WARNING_FONT_SIZE 9
 
 //--------------------------------------------------------------------------------------------------
 
@@ -130,16 +127,24 @@ static void stop_animation_timer_for(BaseWidget* widget)
 //--------------------------------------------------------------------------------------------------
 
 BaseWidget::BaseWidget()
-: _background(NULL), _description(""), _description_height(0)
 {
   g_static_mutex_init(&_lock);
+
+  _background = NULL;
   _lower_limit= 0;
   _upper_limit= 1;
+  _right_align = false; // Right alignment of the description.
   _auto_scale= false;
   _layout_width= 100;
-  _layout_height= 50;
+  _layout_height= 64;
   _layout_surface= NULL;
   _layout_context= NULL;
+
+  _description = "";
+  _description_offset = 0;
+
+  _last_width = 0;
+  _last_height = 0;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -187,23 +192,6 @@ void BaseWidget::destroy_background()
 
 //--------------------------------------------------------------------------------------------------
 
-/**
- * Returns the usable area for the core widget area, i.e. where descendants can draw into.
- * It excludes the area needed to draw the description.
- */
-Rect BaseWidget::get_diagram_area()
-{
-  create_context_for_layout();
-  layout(_layout_context);
-  
-  Rect result(0, 0, get_width(), get_height());
-  result.size.height -= _description_height;
-  
-  return result;
-}
-
-//--------------------------------------------------------------------------------------------------
-
 /**
  * Normalizes the input value to the range [0..1] depending on the set input value range.
  */
@@ -287,10 +275,24 @@ void BaseWidget::get_minmax_values(double* min, double* max)
 
 //--------------------------------------------------------------------------------------------------
 
+void BaseWidget::set_right_align(bool flag)
+{
+  lock();
+  if (_right_align != flag)
+  {
+    _right_align = flag;
+    set_layout_dirty(true);
+    set_needs_repaint();
+  }
+  unlock();
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void BaseWidget::enable_auto_scale(bool enable)
 {
   lock();
-  _auto_scale= enable; // Change will be visible when next value comes in.
+  _auto_scale = enable; // Change will be visible when next value comes in.
   unlock();
 }
 
@@ -348,13 +350,10 @@ void BaseWidget::set_description(const std::string& description)
 {
   if (_description != description)
   {
-    _description= description;
+    _description = description;
     create_context_for_layout();
     if (layout(_layout_context))
-    {
       set_layout_dirty(true);
-      relayout();
-    }
     set_needs_repaint();
   }
 }
@@ -363,7 +362,7 @@ void BaseWidget::set_description(const std::string& description)
 
 void BaseWidget::repaint(cairo_t *cr, int areax, int areay, int areaw, int areah)
 {
-  if (is_layout_dirty())
+  if (is_layout_dirty() || _last_height != get_height() || _last_width != get_width())
     layout(cr);
   
   prepare_background();
@@ -377,18 +376,13 @@ void BaseWidget::repaint(cairo_t *cr, int areax, int areay, int areaw, int areah
     cairo_paint(cr);
   }
 
-  // Erase background behind the description.
-//  cairo_set_source_rgb(cr, BK_RED, BK_GREEN, BK_BLUE); // Same as the widget box.
-//  cairo_rectangle(cr, 0, get_height() - _description_height, get_width(), _description_height);
-//  cairo_fill(cr);
-
   if (_description != "")
   {
-    cairo_select_font_face(cr, WIDGET_SMALL_FONT, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_BOLD);
+    cairo_select_font_face(cr, WIDGET_SMALL_FONT, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
     cairo_set_font_size(cr, WIDGET_DESCRIPTION_FONT_SIZE);
 
-    cairo_set_source_rgb(cr, 66 / 255.0, 73 / 255.0, 79 / 255.0);
-    cairo_move_to(cr, 0, get_height() - 6); // Small extra offset for bottom spacing.
+    cairo_set_source_rgb(cr, 0x5f / 255.0, 0x5f / 255.0, 0x5f / 255.0);
+    cairo_move_to(cr, _description_offset, get_height() - 4);
     cairo_show_text(cr, _description.c_str());
     cairo_stroke(cr);
   }
@@ -398,7 +392,7 @@ void BaseWidget::repaint(cairo_t *cr, int areax, int areay, int areaw, int areah
 //--------------------------------------------------------------------------------------------------
 
 /**
- * Returns the computed size for this widget.
+ * Returns the computed size for this widget (min size).
  */
 void BaseWidget::get_layout_size(int* w, int* h)
 {
@@ -408,8 +402,8 @@ void BaseWidget::get_layout_size(int* w, int* h)
     layout(_layout_context);
   }
   
-  *w= _layout_width;
-  *h= _layout_height;
+  *w = _layout_width;
+  *h = _layout_height;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -422,29 +416,35 @@ void BaseWidget::get_layout_size(int* w, int* h)
 bool BaseWidget::layout(cairo_t *cr)
 {
   lock();
-  
-  bool result= false;
+
+  set_layout_dirty(false);
+  _last_height = get_height();
+  _last_width = get_width();
+
+  bool result = false;
   if (_description != "")
   {
-    cairo_select_font_face(cr, WIDGET_SMALL_FONT, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_BOLD);
+    cairo_select_font_face(cr, WIDGET_SMALL_FONT, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
     cairo_set_font_size(cr, WIDGET_DESCRIPTION_FONT_SIZE);
     
-    cairo_text_extents_t extents;
-    cairo_text_extents(cr, _description.c_str(), &extents);
-    int new_height= (int) ceil(extents.height + 8);
-    if (new_height != _description_height)
-    {
-      _description_height= new_height;
-      result= true;
-    }
+    // Make description area height text independent or it will vary with the text.
+    cairo_font_extents_t extents;
+    cairo_font_extents(cr, &extents);
+    _diagram_area = base::Rect(0, 0, get_width(), get_height() - (int) ceil(extents.height) - 4);
     
-    int new_width= (int) (extents.x_advance + extents.x_bearing);
+    cairo_text_extents_t text_extents;
+    cairo_text_extents(cr, _description.c_str(), &text_extents);
+
+    int new_width = (int)ceil(text_extents.width);
     if (new_width > _layout_width)
     {
-      _layout_width= new_width; // Grow only, otherwise the layout might constantly change.
-      result= true;
+      _layout_width = new_width; // Grow only, otherwise the layout might constantly change.
+      result = true;
     }
+
+    _description_offset = _right_align ? get_width() - (int)ceil(text_extents.x_advance) : 0;
   }
+
   unlock();
   
   return result;
@@ -486,46 +486,6 @@ void WidgetSeparator::repaint(cairo_t *cr, int areax, int areay, int areaw, int
   cairo_stroke(cr);
 }
 
-//----------------- WidgetContainer ----------------------------------------------------------------
-
-WidgetContainer::WidgetContainer(const std::string& title)
-: Box(false), _title(title), _content(true)
-{
-//  set_back_color(BACKGROUND_COLOR);
-
-  set_padding(4);
-  //set_spacing(4);
-
-  _title.set_style(BoldInfoCaptionStyle);
-  _title.set_wrap_text(false);
-  _title.set_color("#42494F");
-  add(&_title, false, true);
-
-  _content.set_padding(4);
-  _content.set_spacing(8);
-  add(&_content, true, true);
-}
-
-//--------------------------------------------------------------------------------------------------
-
-WidgetContainer::~WidgetContainer()
-{
-}
-
-//--------------------------------------------------------------------------------------------------
-
-void WidgetContainer::add_widget(BaseWidget* widget, bool expand)
-{
-  _content.add(widget, expand, true);
-}
-
-//--------------------------------------------------------------------------------------------------
-
-void WidgetContainer::set_title(const std::string& title)
-{
-  _title.set_text(title);
-}
-
 //----------------- HeartbeatWidget ----------------------------------------------------------------
 
 HeartbeatWidget::HeartbeatWidget()
@@ -613,14 +573,12 @@ void HeartbeatWidget::step()
  */
 void HeartbeatWidget::prepare_background()
 {
-  Rect bounds= get_diagram_area();
-
   if (_background == NULL 
-    || cairo_image_surface_get_width(_background) != bounds.width()
-    || cairo_image_surface_get_height(_background) != bounds.height())
+    || cairo_image_surface_get_width(_background) != _diagram_area.width()
+    || cairo_image_surface_get_height(_background) != _diagram_area.height())
   {
     destroy_background();
-    _background= cairo_image_surface_create(CAIRO_FORMAT_ARGB32, (int) bounds.width(), (int) bounds.height());
+    _background = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, (int)_diagram_area.width(), (int)_diagram_area.height());
     cairo_t* cr= cairo_create(_background);
 
     // Erase background to constant color.
@@ -635,7 +593,7 @@ void HeartbeatWidget::prepare_background()
     cairo_set_source(cr, gradient);
     cairo_set_line_width(cr, 5);
     cairo_move_to(cr, 2.5, 3);
-    cairo_line_to(cr, bounds.width() - 2.5, 3);
+    cairo_line_to(cr, _diagram_area.width() - 2.5, 3);
     cairo_stroke(cr);
     
     cairo_pattern_destroy(gradient);
@@ -648,10 +606,10 @@ void HeartbeatWidget::prepare_background()
     cairo_set_line_width(cr, 1);
     
     double y= 4.5;
-    while (y < bounds.height())
+    while (y < _diagram_area.height())
     {
       cairo_move_to(cr, 4.5, y);
-      cairo_line_to(cr, bounds.width() - 4.5, y);
+      cairo_line_to(cr, _diagram_area.width() - 4.5, y);
       y += 7;
     }
     cairo_stroke(cr);
@@ -700,7 +658,7 @@ void HeartbeatWidget::repaint(cairo_t *cr, int areax, int areay, int areaw, int
 {
   BaseWidget::repaint(cr, areax, areay, areaw, areah);
 
-  Rect bounds= get_diagram_area();
+  Rect bounds = _diagram_area;
 
   // Draw the heartbeat line.
   cairo_set_line_width(cr, 2);
@@ -733,14 +691,11 @@ void HeartbeatWidget::repaint(cairo_t *cr, int areax, int areay, int areaw, int
   unlock();
 }
 
-//----------------- ServerInfoWidget ---------------------------------------------------------------
+//----------------- ServerStatusWidget ---------------------------------------------------------------
 
-ServerInfoWidget::ServerInfoWidget()
+ServerStatusWidget::ServerStatusWidget()
 {
   _status= -1;
-  _instance= "Localhost 5.1";
-  _ip_address= "127.0.0.1";
-  _server= "MySQL 5.1.32 via socket";
   _image_unknown= Utilities::load_icon("admin_info_unknown.png");
   _image_running= Utilities::load_icon("admin_info_running.png");
   _image_stopped= Utilities::load_icon("admin_info_stopped.png");
@@ -748,7 +703,7 @@ ServerInfoWidget::ServerInfoWidget()
 
 //--------------------------------------------------------------------------------------------------
 
-ServerInfoWidget::~ServerInfoWidget()
+ServerStatusWidget::~ServerStatusWidget()
 {
  cairo_surface_destroy(_image_unknown);
  cairo_surface_destroy(_image_running);
@@ -757,26 +712,7 @@ ServerInfoWidget::~ServerInfoWidget()
 
 //--------------------------------------------------------------------------------------------------
 
-void ServerInfoWidget::set_server_info(const std::string& instance, 
-                                       const std::string& ip_address,
-                                       const std::string& server)
-{
-  if (_instance != instance || _ip_address != ip_address || _server != server)
-  {
-    lock();
-    _instance= instance;
-    _ip_address= ip_address;
-    _server= server;
-
-    set_layout_dirty(true);
-    set_needs_repaint();
-    unlock();
-  }
-}
-
-//--------------------------------------------------------------------------------------------------
-
-void ServerInfoWidget::set_server_status(int status)
+void ServerStatusWidget::set_server_status(int status)
 {
   // Sanity check.
   if (status < -1 || status > 1)
@@ -794,12 +730,9 @@ void ServerInfoWidget::set_server_status(int status)
 
 //--------------------------------------------------------------------------------------------------
 
-#define ICON_OFFSET 15 // Distance between left border and icon.
-#define SPACING 16     // Horizontal distance between icon and text.
-#define TEXT_SPACING 8 // Distance between variable and fixed text.
-#define LINE_SPACING 2 // Extra spacing between lines.
+#define LINE_SPACING 4 // Extra spacing between lines.
 
-void ServerInfoWidget::repaint(cairo_t *cr, int areax, int areay, int areaw, int areah)
+void ServerStatusWidget::repaint(cairo_t *cr, int areax, int areay, int areaw, int areah)
 {
   BaseWidget::repaint(cr, areax, areay, areaw, areah);
 
@@ -807,75 +740,26 @@ void ServerInfoWidget::repaint(cairo_t *cr, int areax, int areay, int areaw, int
 
   cairo_save(cr);
 
-  // Erase background.
-//  cairo_set_source_rgb(cr, BK_RED, BK_GREEN, BK_BLUE);
-//  cairo_paint(cr);
-
   cairo_surface_t* icon;
-  std::string status_text;
   switch (_status)
   {
     case 1:
       icon= _image_running;
-      status_text= "Running";
       break;
     case 0:
       icon= _image_stopped;
-      status_text= "Stopped";
       break;
     default:
       icon= _image_unknown;
-      status_text= "Unknown";
   }
 
-  // Variable text.
-  cairo_select_font_face(cr, WIDGET_FONT, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_BOLD);
-  cairo_set_font_size(cr, WIDGET_NORMAL_FONT_SIZE);
-  double y= _content_top;
-  cairo_set_source_rgb(cr, 25 / 255.0, 25 / 255.0, 25 / 255.0);
-  cairo_move_to(cr, _var_text_left, y);
-  cairo_show_text(cr, _instance.c_str());
-  cairo_stroke(cr);
-  y += _text_y_advance;
-  cairo_move_to(cr, _var_text_left, y);
-  cairo_show_text(cr, _ip_address.c_str());
-  cairo_stroke(cr);
-  y += _text_y_advance;
-  cairo_move_to(cr, _var_text_left, y);
-  cairo_show_text(cr, _server.c_str());
-  cairo_stroke(cr);
-  y += _text_y_advance;
-  cairo_move_to(cr, _var_text_left, y);
-  cairo_show_text(cr, status_text.c_str());
-  cairo_stroke(cr);
-
   // Icon.
   if (icon != NULL)
   {
-    cairo_set_source_surface(cr, icon, ICON_OFFSET, 0);
+    cairo_set_source_surface(cr, icon, 0, 0);
     cairo_paint(cr);
   }
 
-  // Fixed text.
-  cairo_select_font_face(cr, WIDGET_FONT, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
-  y= _content_top;
-  cairo_set_source_rgb(cr, 98 / 255.0, 97 / 255.0, 97 / 255.0);
-  cairo_move_to(cr, _instance_left, y);
-  cairo_show_text(cr, "Name:");
-  cairo_stroke(cr);
-  y += _text_y_advance;
-  cairo_move_to(cr, _ip_left, y);
-  cairo_show_text(cr, "Host:");
-  cairo_stroke(cr);
-  y += _text_y_advance;
-  cairo_move_to(cr, _server_left, y);
-  cairo_show_text(cr, "Server:");
-  cairo_stroke(cr);
-  y += _text_y_advance;
-  cairo_move_to(cr, _status_left, y);
-  cairo_show_text(cr, "Status:");
-  cairo_stroke(cr);
-
   cairo_restore(cr);
   unlock();
 }
@@ -885,115 +769,42 @@ void ServerInfoWidget::repaint(cairo_t *cr, int areax, int areay, int areaw, int
 /**
  * This function computes the overall layout of the widget and sets its size.
  */
-bool ServerInfoWidget::layout(cairo_t* cr)
+bool ServerStatusWidget::layout(cairo_t* cr)
 {
+  _layout_width = 0;
+  _layout_height = 0;
   BaseWidget::layout(cr);
 
   cairo_save(cr);
     
-  cairo_select_font_face(cr, WIDGET_FONT, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_BOLD);
+  cairo_select_font_face(cr, WIDGET_FONT, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
   cairo_set_font_size(cr, WIDGET_NORMAL_FONT_SIZE);
 
-  cairo_text_extents_t extents;
-  double total_height= 4;
-  double total_width= ICON_OFFSET;
-
   lock();
   
   // Icon and main text position.
   // Icon size for left text offset and total size.
   cairo_surface_t* icon;
-  std::string status_text;
   switch (_status)
   {
     case 1:
       icon= _image_running;
-      status_text= "Running";
       break;
     case 0:
       icon= _image_stopped;
-      status_text= "Stopped";
       break;
     default:
       icon= _image_unknown;
-      status_text= "Unknown";
   }
 
-  double icon_width= 0;
-  double icon_height= 0;
   if (icon != NULL)
   {
-    icon_width= cairo_image_surface_get_width(icon);
-    icon_height= cairo_image_surface_get_height(icon);
+    _layout_width = MAX(_layout_width, cairo_image_surface_get_width(icon));
+    _layout_height += cairo_image_surface_get_height(icon) + LINE_SPACING;
   }
 
-  // Fixed text positions.
-  cairo_select_font_face(cr, WIDGET_FONT, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
-  double fixed_max_width= 0;
-  double fixed_left= ICON_OFFSET + icon_width + SPACING;
-
-  cairo_text_extents(cr, "Name:", &extents);
-  double instance_width= extents.x_advance;
-  if (instance_width > fixed_max_width)
-    fixed_max_width= instance_width;
-
-  _content_top= extents.height;
-
-  cairo_text_extents(cr, "Host:", &extents);
-  double ip_width= extents.x_advance;
-  if (ip_width > fixed_max_width)
-    fixed_max_width= ip_width;
-  cairo_text_extents(cr, "Server:", &extents);
-  double server_width= extents.x_advance;
-  if (server_width > fixed_max_width)
-    fixed_max_width= server_width;
-  cairo_text_extents(cr, "Status:", &extents);
-  double status_width= extents.x_advance;
-  if (status_width > fixed_max_width)
-    fixed_max_width= status_width;
-
-  // Now that we have all widths and the maximum width compute the final positions.
-  // The vertical offset between lines is computed for the bold, variable text (as this likely needs a bit more room).
-  _instance_left= fixed_left + fixed_max_width - instance_width;
-  _ip_left= fixed_left + fixed_max_width - ip_width;
-  _server_left= fixed_left + fixed_max_width - server_width;
-  _status_left= fixed_left + fixed_max_width - status_width;
-
-  cairo_select_font_face(cr, WIDGET_FONT, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_BOLD);
-  _var_text_left= fixed_left + fixed_max_width + TEXT_SPACING;
-  double var_max_width= 0;
-
-  cairo_text_extents(cr, _instance.c_str(), &extents);
-  if (extents.x_advance > var_max_width)
-    var_max_width= extents.x_advance;
-
-  if (extents.height > _content_top)
-    _content_top= extents.height;
-
-  cairo_text_extents(cr, _ip_address.c_str(), &extents);
-  if (extents.x_advance > var_max_width)
-    var_max_width= extents.x_advance;
-  cairo_text_extents(cr, _server.c_str(), &extents);
-  if (extents.x_advance > var_max_width)
-    var_max_width= extents.x_advance;
-  cairo_text_extents(cr, status_text.c_str(), &extents);
-  if (extents.x_advance > var_max_width)
-    var_max_width= extents.x_advance;
-
-  total_width= _var_text_left + var_max_width;
-
-  cairo_text_extents(cr, "Äg", &extents);
-  _text_y_advance= extents.height + LINE_SPACING;
-
-  if (icon_height > 4 * _text_y_advance)
-    total_height += icon_height;
-  else
-    total_height += 4 * _text_y_advance; // 4 lines of text.
-  
   cairo_restore(cr);
-  _layout_width= MIN(270, (int) total_width); // For now moor the size at 270px max.
-  _layout_height= (int) total_height;
-  
+
   unlock();
   
   return true;
@@ -1019,7 +830,7 @@ BarGraphWidget::~BarGraphWidget()
 
 void BarGraphWidget::prepare_background()
 {
-  Rect bounds= get_diagram_area();
+  Rect bounds = _diagram_area;
 
   if (_background == NULL 
     || cairo_image_surface_get_height(_background) != bounds.height())
@@ -1117,7 +928,7 @@ void BarGraphWidget::create_value_gradient()
   if (_value_gradient != NULL)
     cairo_pattern_destroy(_value_gradient);
   
-  Rect bounds= get_diagram_area();
+  Rect bounds= _diagram_area;
   double split_point= (bounds.height() - 2) * _value;
   split_point = 3 * (split_point / 3); // Integer div.
 
@@ -1147,7 +958,7 @@ void BarGraphWidget::repaint(cairo_t *cr, int areax, int areay, int areaw, int a
   BaseWidget::repaint(cr, areax, areay, areaw, areah);
 
   // Fill body with thick lines in either dark gray or a blue gradient depending on the current bar value.
-  Rect bounds= get_diagram_area();
+  Rect bounds= _diagram_area;
 
   lock();
 
@@ -1205,7 +1016,7 @@ void LineDiagramWidget::repaint(cairo_t *cr, int areax, int areay, int areaw, in
 {
   BaseWidget::repaint(cr, areax, areay, areaw, areah);
   
-  Rect bounds= get_diagram_area();
+  Rect bounds = _diagram_area;
   
   lock();
 
@@ -1289,10 +1100,10 @@ void LineDiagramWidget::set_value(double value)
   double timestamp= g_timer_elapsed(_clock, NULL);
   
   // Move values one step to the left.
-  memmove(&_deflection[0], &_deflection[1], sizeof(_deflection));
+  memmove(&_deflection[0], &_deflection[1], sizeof(_deflection) - sizeof(_deflection[0]));
   _deflection[LINE_SERIES_DATA_SIZE - 1]= value;
 
-  memmove(&_timestamp[0], &_timestamp[1], sizeof(_timestamp));
+  memmove(&_timestamp[0], &_timestamp[1], sizeof(_timestamp) - sizeof(_timestamp[0]));
   _timestamp[LINE_SERIES_DATA_SIZE - 1]= timestamp;
   
   unlock();
@@ -1303,7 +1114,7 @@ void LineDiagramWidget::set_value(double value)
 
 void LineDiagramWidget::prepare_background()
 {
-  Rect bounds= get_diagram_area();
+  Rect bounds = _diagram_area;
   
   if (_background == NULL 
      || cairo_image_surface_get_height(_background) != bounds.height()
@@ -1471,7 +1282,7 @@ void LineDiagramWidget::show_feedback(cairo_t* cr, const Rect& bounds)
 {
   if (_sleep_mode != Awake)
   {
-    cairo_select_font_face(cr, WIDGET_SMALL_FONT, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_BOLD);
+    cairo_select_font_face(cr, WIDGET_SMALL_FONT, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
     cairo_set_font_size(cr, WIDGET_WARNING_FONT_SIZE);
 
     if (_warning_extents.width == 0)
@@ -1571,6 +1382,9 @@ bool LineDiagramWidget::feedback_step()
 
         result= true;
         break;
+      case Awake:
+      case Sleeping:
+        break;
     }
   }
   return result;
