@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,8 +18,13 @@
  */
 
 #include "stdafx.h"
+
+#include "mforms/mforms.h"
+#include "wf_view.h"
 #include "wf_box.h"
 
+#include "ConvUtils.h"
+
 using namespace MySQL;
 using namespace MySQL::Forms;
 using namespace MySQL::Utilities;
@@ -50,8 +55,8 @@ void maximize_children(ControlList% list, bool horizontal, Size containerSize, i
   {
     for each (ControlEntry^ entry in list)
     {
-      ViewImpl::remove_auto_resize(entry->control, mforms::ResizeVertical);
-      entry->bounds.Y= padding;
+      ViewImpl::remove_auto_resize(entry->control, AutoResizeMode::ResizeVertical);
+      entry->bounds.Y = padding;
       entry->bounds.Height= containerSize.Height;
     };
   }
@@ -59,11 +64,11 @@ void maximize_children(ControlList% list, bool horizontal, Size containerSize, i
   {
     for each (ControlEntry^ entry in list)
     {
-      ViewImpl::remove_auto_resize(entry->control, mforms::ResizeHorizontal);
-      entry->bounds.X= padding;
+      ViewImpl::remove_auto_resize(entry->control, AutoResizeMode::ResizeHorizontal);
+      entry->bounds.X = padding;
 
       // Determine the preferred size based on the new width (for labels, as they can be in auto wrap mode).
-      if (entry->control->GetType()->IsSubclassOf(Label::typeid))
+      if (entry->control->GetType()->IsSubclassOf(Windows::Forms::Label::typeid))
         entry->bounds.Size= entry->control->GetPreferredSize(Size(containerSize.Width, 0));
       else
         entry->bounds.Width= containerSize.Width;
@@ -84,42 +89,63 @@ void apply_bounds(ControlList% list, bool horizontal)
   int vertical_offset = 0;
   int horizontal_offset = 0;
 
+  HDWP hdwp = BeginDeferWindowPos(list.Count);
   for each (ControlEntry^ entry in list)
   {
     // The parameter horizontal tells us if the layout direction is horizontal or vertical.
-    // Always resize orthogonal to the layout direction but resize in layout direction only if fill is set.
-    ViewImpl::remove_auto_resize(entry->control, mforms::ResizeBoth);
+    // Always resize orthogonal to the layout direction but resize in layout direction only if fill is set
+    // or the computed size is smaller than the current size.
+    ViewImpl::remove_auto_resize(entry->control, AutoResizeMode::ResizeBoth);
+    Drawing::Rectangle newBounds = entry->control->Bounds;
+
     if (horizontal)
     {
       // Workaround: usually, scaling a button vertically does not produce good results.
       // Buttons are better left at their normal height, so we switch that auto-scaling off.
       // In vertical layout the scaling is controllable by the fill flag.
       if (entry->control->GetType() != Button::typeid)
-        entry->control->Height= entry->bounds.Height;
-      if (entry->fills)
-        entry->control->Width= entry->bounds.Width;
+        newBounds.Height = entry->bounds.Height;
+      if (entry->fills || entry->bounds.Width < newBounds.Width)
+        newBounds.Width = entry->bounds.Width;
     }
     else
     {
-      entry->control->Width= entry->bounds.Width;
-      if (entry->fills)
-        entry->control->Height= entry->bounds.Height;
+      newBounds.Width = entry->bounds.Width;
+      if (entry->fills || entry->bounds.Height < newBounds.Height)
+        newBounds.Height = entry->bounds.Height;
     }
 
     // Get the resulting control size and adjust the offset for following controls and the
     // middle alignment (for controls that don't resize).
-    int x = 0;
-    int y = 0;
-    x = entry->bounds.Left + horizontal_offset + (entry->bounds.Width - entry->control->Width) / 2;
-    y = entry->bounds.Top + vertical_offset + (entry->bounds.Height - entry->control->Height) / 2;
+    newBounds.X = entry->bounds.Left + horizontal_offset + (entry->bounds.Width - newBounds.Width) / 2;
+    newBounds.Y = entry->bounds.Top + vertical_offset + (entry->bounds.Height - newBounds.Height) / 2;
 
     if (horizontal)
-      horizontal_offset += entry->control->Width - entry->bounds.Width;
+      horizontal_offset += newBounds.Width - entry->bounds.Width;
     else
-      vertical_offset += entry->control->Height - entry->bounds.Height;
+      vertical_offset += newBounds.Height - entry->bounds.Height;
 
-    entry->control->Location = Point(x, y);
+    // The WebBrowser class doesn't like being resized via DeferWindowPos, so we do it the straight way.
+    // Also, if DeferWindowPos (or BeginDeferWindowPos for that matter) returned an empty handle
+    // then we need to do the resizing that way for all controls.
+    if (hdwp == 0  || is<Windows::Forms::WebBrowser>(entry->control))
+      entry->control->Bounds = newBounds;
+    else
+    {
+      HDWP new_hdwp = DeferWindowPos(hdwp, (HWND)entry->control->Handle.ToInt32(), 0, newBounds.X, newBounds.Y,
+        newBounds.Width, newBounds.Height, SWP_NOZORDER);
+      if (new_hdwp == 0)
+      {
+        // Something went wrong, so finish what we started so far and continue with straight resize.
+        EndDeferWindowPos(hdwp);
+      }
+
+      hdwp = new_hdwp;
+    }
   }
+
+  if (hdwp != 0)
+    EndDeferWindowPos(hdwp);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -178,7 +204,7 @@ int get_largest_height(ControlList% list1, ControlList% list2)
  */
 void expand_horizontally(ControlList% list, Box^ box, int fraction, bool mirrored)
 {
-  for (int i= 0; i < list.Count; i++)
+  for (int i = 0; i < list.Count; i++)
   {
     if (box->GetControlExpands(list[i]->control))
     {
@@ -265,14 +291,15 @@ System::Drawing::Size HorizontalGtkBoxLayout::ComputeLayout(Box^ box, System::Dr
   // First part: setup and vertical position and size computation.
   for each (Control^ control in box->Controls)
   {
+    // Also here we should use reflection to get the true visibility state, but that's expensive.
     if (!control->Visible)
       continue;
 
     // Make a copy of the current bounds of the control.
     // This is used in the computation before we finally manipulate the control itself.
-    ControlEntry^ entry= gcnew ControlEntry();
-    entry->control= control;
-    entry->fills= box->GetControlFills(control);
+    ControlEntry^ entry = gcnew ControlEntry();
+    entry->control = control;
+    entry->fills = box->GetControlFills(control);
 
     bool use_min_width = ViewImpl::use_min_width_for_layout(control);
     bool use_min_height = ViewImpl::use_min_height_for_layout(control);
@@ -282,9 +309,24 @@ System::Drawing::Size HorizontalGtkBoxLayout::ComputeLayout(Box^ box, System::Dr
     {
       entry->bounds= System::Drawing::Rectangle(Point(0, 0), control->GetPreferredSize(Size(0, proposedSize.Height)));
       if (use_min_width)
-        entry->bounds.Width= control->MinimumSize.Width;
+        entry->bounds.Width = control->MinimumSize.Width;
       if (use_min_height)
-        entry->bounds.Height= control->MinimumSize.Height;
+        entry->bounds.Height = control->MinimumSize.Height;
+    }
+
+    // Weird behavior of combo boxes. They report a one pixel too small height (you cannot set them
+    // to this smaller height, though). So this wrong value is messing up our computed overall height.
+    if (entry->control->GetType() == ComboBox::typeid)
+      entry->bounds.Height++;
+    
+    // Buttons have a very own understanding of their preferred size. Flat buttons always add 3 px padding
+    // in addition to their normal Padding. System buttons ignore the image in size calculations.
+    // We fix flat buttons for now as they can come with images only. Normal buttons usually have text.
+    if (entry->control->GetType() == Button::typeid)
+    {
+      Button ^button = (Button ^)entry->control;
+      if (button->FlatStyle == FlatStyle::Flat)
+        entry->bounds = Drawing::Rectangle::Inflate(entry->bounds, -3, -3);
     }
 
     // Sort control into the proper alignment list.
@@ -306,9 +348,9 @@ System::Drawing::Size HorizontalGtkBoxLayout::ComputeLayout(Box^ box, System::Dr
   // Adjust height of the container if it is too small or auto resizing is enabled.
   if (proposedSize.Height < max_height || ViewImpl::can_auto_resize_vertically(box))
   {
-    proposedSize.Height= max_height;
-    if (proposedSize.Height < box->MinimumSize.Height)
-      proposedSize.Height= box->MinimumSize.Height;
+    proposedSize.Height = max_height;
+    if (proposedSize.Height < box->MinimumSize.Height - box->Padding.Vertical)
+      proposedSize.Height = box->MinimumSize.Height - box->Padding.Vertical;
   }
 
   // Go again through the child list and adjust the height of each child control as well as
@@ -319,27 +361,27 @@ System::Drawing::Size HorizontalGtkBoxLayout::ComputeLayout(Box^ box, System::Dr
   // Second part: horizontal position and size computation.
   // We can have two special cases here: distributed and "use all" mode, but only if the container
   // is not set to auto resizing in horizontal direction (otherwise it adjusts itself to the content).
-  int common_width= 0;
+  int common_width = 0;
 
-  int control_count= left_aligned.Count + right_aligned.Count;
+  int control_count = left_aligned.Count + right_aligned.Count;
   if (control_count > 0 && box->Homogeneous)
   {
     // In this mode we resize all controls so that they entirely fill the width of the container.
     // However, if any of the child controls has a width larger than the computed common width
     // instead use this largest width and increase the container width accordingly.
-    common_width= (proposedSize.Width - (control_count - 1) * spacing) / control_count;
-    int max= get_largest_width(left_aligned,  right_aligned);
+    common_width = (proposedSize.Width - (control_count - 1) * spacing) / control_count;
+    int max = get_largest_width(left_aligned,  right_aligned);
     if (max > common_width)
-      common_width= max;
+      common_width = max;
   }
 
-  int offset= box->Padding.Left;
+  int offset = box->Padding.Left;
 
-  int resulting_width= 0;
+  int resulting_width = 0;
   for each (ControlEntry^ entry in left_aligned)
   {
     // Consider either a common width or the individual widths of the controls here.
-    entry->bounds.X= offset;
+    entry->bounds.X = offset;
     if (common_width > 0)
       entry->bounds.Width= common_width;
     offset += entry->bounds.Width + spacing;
@@ -350,18 +392,18 @@ System::Drawing::Size HorizontalGtkBoxLayout::ComputeLayout(Box^ box, System::Dr
     // Remove the left padding we used for positioning. It's applied later.
     resulting_width = offset - box->Padding.Left;
 
-    // Remove also the last spacing if there are no right aligned children.
+    // Remove also the last spacing if there are no (visible) right aligned children.
     if (right_aligned.Count == 0)
       resulting_width -= spacing;
   }
 
   // For right aligned controls we first compute relative coordinates.
-  offset= -box->Padding.Right;
+  offset = -box->Padding.Right;
   for each (ControlEntry^ entry in right_aligned)
   {
     // Consider either a common width or the individual widths of the controls here.
     if (common_width > 0)
-      entry->bounds.Width= common_width;
+      entry->bounds.Width = common_width;
     entry->bounds.X= offset - entry->bounds.Width;
     offset -= entry->bounds.Width + spacing;
   }
@@ -374,9 +416,9 @@ System::Drawing::Size HorizontalGtkBoxLayout::ComputeLayout(Box^ box, System::Dr
   // Adjust width of the container if it is too small or auto resizing is enabled.
   if (proposedSize.Width < resulting_width || ViewImpl::can_auto_resize_horizontally(box))
   {
-    proposedSize.Width= resulting_width;
-    if (proposedSize.Width < box->MinimumSize.Width)
-      proposedSize.Width= box->MinimumSize.Width;
+    proposedSize.Width = resulting_width;
+    if (proposedSize.Width < box->MinimumSize.Width - box->Padding.Horizontal)
+      proposedSize.Width= box->MinimumSize.Width - box->Padding.Horizontal;
   }
 
   if (resizeChildren)
@@ -415,9 +457,9 @@ System::Drawing::Size HorizontalGtkBoxLayout::ComputeLayout(Box^ box, System::Dr
 
 bool HorizontalGtkBoxLayout::Layout(Object^ container, LayoutEventArgs^ arguments)
 {
-  Box^ box= (Box^) container;
-
-  ControlUtilities::SuspendDrawing(box);
+  Box^ box = (Box^) container;
+  if (!ViewImpl::can_layout(box, arguments->AffectedProperty))
+    return false;
 
   ViewImpl::adjust_auto_resize_from_docking(box);
   System::Drawing::Size boxSize = ComputeLayout(box, box->Size, true);
@@ -427,9 +469,6 @@ bool HorizontalGtkBoxLayout::Layout(Object^ container, LayoutEventArgs^ argument
   if (parentLayoutNeeded)
     ViewImpl::resize_with_docking(box, boxSize);
 
-  ControlUtilities::ResumeDrawing(box);
-  box->Refresh();
-
   return parentLayoutNeeded;
 }
 
@@ -502,6 +541,11 @@ System::Drawing::Size VerticalGtkBoxLayout::ComputeLayout(Box^ box, System::Draw
         entry->bounds.Height= control->MinimumSize.Height;
     }
 
+    // Weird behavior of combo boxes. They report a one pixel too small height (you cannot set them
+    // to this smaller height, though). So this wrong value is messing up our computed overall height.
+    if (entry->control->GetType() == ComboBox::typeid)
+      entry->bounds.Height++;
+
     if (max_width < entry->bounds.Width)
       max_width= entry->bounds.Width;
 
@@ -514,9 +558,9 @@ System::Drawing::Size VerticalGtkBoxLayout::ComputeLayout(Box^ box, System::Draw
   // Adjust height of the container if it is too small or auto resizing is enabled.
   if (proposedSize.Width < max_width || ViewImpl::can_auto_resize_horizontally(box))
   {
-    proposedSize.Width= max_width;
-    if (proposedSize.Width < box->MinimumSize.Width)
-      proposedSize.Width= box->MinimumSize.Width;
+    proposedSize.Width = max_width;
+    if (proposedSize.Width < box->MinimumSize.Width - box->Padding.Horizontal)
+      proposedSize.Width = box->MinimumSize.Width - box->Padding.Horizontal;
   }
 
   // Go again through the child list and adjust the width of each child control as well as
@@ -528,28 +572,28 @@ System::Drawing::Size VerticalGtkBoxLayout::ComputeLayout(Box^ box, System::Draw
   // Second part: horizontal position and size computation.
   // We can have two special cases here: distributed and "use all" mode, but only if the container
   // is not set to auto resizing in horizontal direction (otherwise it adjusts itself to the content).
-  int common_height= 0;
-  int control_count= top_aligned.Count + bottom_aligned.Count;
+  int common_height = 0;
+  int control_count = top_aligned.Count + bottom_aligned.Count;
   if (control_count > 0 && box->Homogeneous)
   {
     // In this mode we resize all controls so that they entirely fill the width of the container.
     // However, if any of the child controls has a width larger than the computed common width
     // instead use this largest width and increase the container width accordingly.
-    common_height= (proposedSize.Height  - (control_count - 1) * spacing) / (control_count);
-    int max= get_largest_height(top_aligned, bottom_aligned);
+    common_height = (proposedSize.Height  - (control_count - 1) * spacing) / (control_count);
+    int max = get_largest_height(top_aligned, bottom_aligned);
     if (max > common_height)
-      common_height= max;
+      common_height = max;
   }
 
-  int offset= box->Padding.Top;
+  int offset = box->Padding.Top;
 
   int resulting_height= 0;
   for each (ControlEntry^ entry in top_aligned)
   {
     // Consider either a common height or the individual widths of the controls here.
-    entry->bounds.Y= offset;
+    entry->bounds.Y = offset;
     if (common_height > 0)
-      entry->bounds.Height= common_height;
+      entry->bounds.Height = common_height;
     offset += entry->bounds.Height + spacing;
   }
 
@@ -564,13 +608,13 @@ System::Drawing::Size VerticalGtkBoxLayout::ComputeLayout(Box^ box, System::Draw
   }
 
   // For bottom aligned controls we first compute relative coordinates.
-  offset= -box->Padding.Bottom;
+  offset = -box->Padding.Bottom;
   for each (ControlEntry^ entry in bottom_aligned)
   {
     // Consider either a common height or the individual widths of the controls here.
     if (common_height > 0)
-      entry->bounds.Height= common_height;
-    entry->bounds.Y= offset - entry->bounds.Height;
+      entry->bounds.Height = common_height;
+    entry->bounds.Y = offset - entry->bounds.Height;
     offset -= entry->bounds.Height + spacing;
   }
 
@@ -583,8 +627,8 @@ System::Drawing::Size VerticalGtkBoxLayout::ComputeLayout(Box^ box, System::Draw
   if (proposedSize.Height < resulting_height || ViewImpl::can_auto_resize_vertically(box))
   {
     proposedSize.Height= resulting_height;
-    if (proposedSize.Height < box->MinimumSize.Height)
-      proposedSize.Height= box->MinimumSize.Height;
+    if (proposedSize.Height < box->MinimumSize.Height - box->Padding.Vertical)
+      proposedSize.Height = box->MinimumSize.Height - box->Padding.Vertical;
   }
 
   if (resizeChildren)
@@ -623,12 +667,10 @@ System::Drawing::Size VerticalGtkBoxLayout::ComputeLayout(Box^ box, System::Draw
 
 bool VerticalGtkBoxLayout::Layout(Object^ container, LayoutEventArgs^ arguments)
 {
-  Box^ box= (Box^) container;
-  if (!box->IsHandleCreated)
+  Box^ box = (Box^) container;
+  if (!ViewImpl::can_layout(box, arguments->AffectedProperty))
     return false;
 
-  ControlUtilities::SuspendDrawing(box);
-
   ViewImpl::adjust_auto_resize_from_docking(box);
   System::Drawing::Size boxSize = ComputeLayout(box, box->Size, true);
 
@@ -637,9 +679,6 @@ bool VerticalGtkBoxLayout::Layout(Object^ container, LayoutEventArgs^ arguments)
   if (parentLayoutNeeded)
     ViewImpl::resize_with_docking(box, boxSize);
 
-  ControlUtilities::ResumeDrawing(box);
-  box->Refresh();
-
   return parentLayoutNeeded;
 }
 
@@ -653,26 +692,47 @@ System::Drawing::Size VerticalGtkBoxLayout::GetPreferredSize(Box^ container,
 
 //----------------- Box ---------------------------------------------------------------------------
 
+Box::Box()
+{
+  SetStyle(ControlStyles::UserPaint, true);
+  SetStyle(ControlStyles::AllPaintingInWmPaint, true);
+  SetStyle(ControlStyles::DoubleBuffer, true);
+  SetStyle(ControlStyles::SupportsTransparentBackColor, true);
+  SetStyle(ControlStyles::OptimizedDoubleBuffer, true);
+  UpdateStyles();
+
+  horizontal = false;
+  homogeneous = false;
+  spacing = 0;
+}
+
+//--------------------------------------------------------------------------------------------------
+
 System::Drawing::Size Box::GetPreferredSize(System::Drawing::Size proposedSize)
 {
-  System::Drawing::Size size= layoutEngine->GetPreferredSize(this, proposedSize);
-  if (size.Width < MinimumSize.Width)
-    size.Width= MinimumSize.Width;
-  if (size.Height < MinimumSize.Height)
-    size.Height= MinimumSize.Height;
-  return size;
+  return layoutEngine->GetPreferredSize(this, proposedSize);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void Box::OnPaintBackground(PaintEventArgs^ args)
+{
+  Panel::OnPaintBackground(args);
+
+  ViewImpl ^view = ObjectImpl::get_wrapper<ViewImpl>(this);
+  view->draw_background(args);
 }
 
 //----------------- BoxImpl -----------------------------------------------------------------------
 
-bool BoxImpl::create(::mforms::Box *self, bool horizontal)
+bool BoxImpl::create(mforms::Box *self, bool horizontal)
 {
-  BoxImpl ^box= gcnew BoxImpl(self);
+  BoxImpl ^box = gcnew BoxImpl(self);
 
   if (box != nullptr)
   {
-    Box^ native_box= ViewImpl::create<Box>(self, box);
-    native_box->Horizontal= horizontal;
+    Box^ native_box = ViewImpl::create<Box>(self, box);
+    native_box->Horizontal = horizontal;
     return true;
   }
   return false;
