@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,12 +17,11 @@
  * 02110-1301  USA
  */
 
-/**
- * Contains the implementation of the wrapper for a .NET form used by the mforms backend.
- */
-
 #include "stdafx.h"
 
+#include "mforms/mforms.h"
+#include "wf_view.h"
+#include "wf_button.h"
 #include "wf_form.h"
 #include "wf_utilities.h"
 
@@ -38,7 +37,18 @@ using namespace MySQL::Forms;
 
 bool FormFillLayout::Layout(Object^ container, LayoutEventArgs^ arguments)
 {
+  // Not using the can_layout check here, as a form is a bit special.
+  String ^reason = arguments->AffectedProperty;
+  if (reason != "Bounds" && reason != "Padding" && reason != "Visible")
+    return false;
+
   FillForm^ form = (FillForm^) container;
+  if (!form->IsHandleCreated)
+    return false;
+
+  mforms::View *backend = ViewImpl::get_backend_control<mforms::View>(form);
+  if (backend->is_destroying())
+    return false;
 
   ViewImpl::adjust_auto_resize_from_docking(form);
   System::Drawing::Size newSize = form->ComputeLayout(form->Size, true);
@@ -66,6 +76,12 @@ System::Drawing::Size FormFillLayout::GetPreferredSize(Control^ container, Syste
 
 //----------------- FillForm -----------------------------------------------------------------------
 
+FillForm::FillForm()
+{
+}
+
+//--------------------------------------------------------------------------------------------------
+
 /**
  * Computes the entire layout of the form.
  * 
@@ -95,12 +111,12 @@ System::Drawing::Size FillForm::ComputeLayout(System::Drawing::Size proposedSize
     proposedSize.Height -= vertical_padding;
 
     ViewImpl::set_full_auto_resize(content);
-    System::Drawing::Size contentSize= content->GetPreferredSize(proposedSize);
+    System::Drawing::Size contentSize = content->GetPreferredSize(proposedSize);
 
     if (ViewImpl::use_min_width_for_layout(content))
-      contentSize.Width= content->MinimumSize.Width;
+      contentSize.Width = content->MinimumSize.Width;
     if (ViewImpl::use_min_height_for_layout(content))
-      contentSize.Height= content->MinimumSize.Height;
+      contentSize.Height = content->MinimumSize.Height;
 
     // Adjust width of the container if it is too small or auto resizing is enabled.
     // Don't auto size if this is a standalone window (has no parent).
@@ -116,7 +132,7 @@ System::Drawing::Size FillForm::ComputeLayout(System::Drawing::Size proposedSize
     if (resizeChildren)
     {
       // Now stretch the client control to fill the entire display area.
-      ViewImpl::remove_auto_resize(content, mforms::ResizeBoth);
+      ViewImpl::remove_auto_resize(content, AutoResizeMode::ResizeBoth);
       content->Bounds= System::Drawing::Rectangle(inner.Location, proposedSize);
     }
 
@@ -131,23 +147,18 @@ System::Drawing::Size FillForm::ComputeLayout(System::Drawing::Size proposedSize
 
 System::Drawing::Size FillForm::GetPreferredSize(System::Drawing::Size proposedSize)
 {
-  System::Drawing::Size size= layoutEngine->GetPreferredSize(this, proposedSize);
-  if (size.Width < MinimumSize.Width)
-    size.Width= MinimumSize.Width;
-  if (size.Height < MinimumSize.Height)
-    size.Height= MinimumSize.Height;
-  return size;
+  return layoutEngine->GetPreferredSize(this, proposedSize);
 }
 
 //-------------------------------------------------------------------------------------------------
 
-bool FormImpl::create(::mforms::Form *self, ::mforms::Form *owner, ::mforms::FormFlag flag)
+bool FormImpl::create(mforms::Form *self, mforms::Form *owner, mforms::FormFlag flag)
 {
   FormImpl ^form= gcnew FormImpl(self, owner, flag);
 
   if (form != nullptr)
   {
-    Windows::Forms::Form ^f= ViewImpl::create<FillForm>(self, form);
+    Windows::Forms::Form ^f = ViewImpl::create<FillForm>(self, form);
     if (owner != NULL)
       f->StartPosition = FormStartPosition::CenterParent;
     else
@@ -184,7 +195,7 @@ bool FormImpl::create(::mforms::Form *self, ::mforms::Form *owner, ::mforms::For
       f->Owner = UtilitiesImpl::get_mainform();
 
     form->_hide_on_close= (flag & mforms::FormHideOnClose) != 0;
-    ViewImpl::remove_auto_resize(f, mforms::ResizeBoth);
+    ViewImpl::remove_auto_resize(f, AutoResizeMode::ResizeBoth);
 
     return true;
   }
@@ -193,7 +204,7 @@ bool FormImpl::create(::mforms::Form *self, ::mforms::Form *owner, ::mforms::For
 
 //--------------------------------------------------------------------------------------------------
 
-void FormImpl::set_title(::mforms::Form *self, const std::string &title)
+void FormImpl::set_title(mforms::Form *self, const std::string &title)
 {
   FormImpl^ form= (FormImpl^)ObjectImpl::FromUnmanaged(self);
 
@@ -203,7 +214,7 @@ void FormImpl::set_title(::mforms::Form *self, const std::string &title)
 
 //--------------------------------------------------------------------------------------------------
 
-void FormImpl::show_modal(::mforms::Form *self, ::mforms::Button *accept,::mforms::Button *cancel)
+void FormImpl::show_modal(mforms::Form *self, mforms::Button *accept,mforms::Button *cancel)
 {
   FormImpl^ form= (FormImpl^)ObjectImpl::FromUnmanaged(self);
 
@@ -233,7 +244,7 @@ void FormImpl::show_modal(::mforms::Form *self, ::mforms::Button *accept,::mform
 
 //--------------------------------------------------------------------------------------------------
 
-bool FormImpl::run_modal(::mforms::Form *self, ::mforms::Button *accept,::mforms::Button *cancel)
+bool FormImpl::run_modal(mforms::Form *self, mforms::Button *accept,mforms::Button *cancel)
 {
   FormImpl^ form= (FormImpl^)ObjectImpl::FromUnmanaged(self);
 
@@ -257,7 +268,7 @@ bool FormImpl::run_modal(::mforms::Form *self, ::mforms::Button *accept,::mforms
     if (f->InvokeRequired)
     {
       Object^ invocation_result= f->Invoke(gcnew FillForm::ShowModalDelegate(f, &Windows::Forms::Form::ShowDialog),
-        gcnew array<Object^> {form->parent});
+        gcnew array<Object^> {form->_owner});
       dialog_result= *(Windows::Forms::DialogResult^) (invocation_result);
     }
     else
@@ -270,7 +281,7 @@ bool FormImpl::run_modal(::mforms::Form *self, ::mforms::Button *accept,::mforms
         f->ActiveControl = acceptButton;
 
       mforms::Utilities::enter_modal_loop();
-      dialog_result= f->ShowDialog(form->parent);
+      dialog_result= f->ShowDialog(form->_owner);
       mforms::Utilities::leave_modal_loop();
     }
 
@@ -285,7 +296,7 @@ bool FormImpl::run_modal(::mforms::Form *self, ::mforms::Button *accept,::mforms
 
 //--------------------------------------------------------------------------------------------------
 
-void FormImpl::end_modal(::mforms::Form *self, bool result)
+void FormImpl::end_modal(mforms::Form *self, bool result)
 {
   FormImpl^ form= (FormImpl^)ObjectImpl::FromUnmanaged(self);
   if (form != nullptr)
@@ -300,7 +311,7 @@ void FormImpl::end_modal(::mforms::Form *self, bool result)
 /**
  * Called by the backend when a form is closed from the application side.
  */
-void FormImpl::close(::mforms::Form *self)
+void FormImpl::close(mforms::Form *self)
 {
   FormImpl^ form= (FormImpl^)ObjectImpl::FromUnmanaged(self);
 
@@ -310,7 +321,7 @@ void FormImpl::close(::mforms::Form *self)
 
 //--------------------------------------------------------------------------------------------------
 
-void FormImpl::set_content(::mforms::Form *self, ::mforms::View *view)
+void FormImpl::set_content(mforms::Form *self, mforms::View *view)
 {
   FormImpl^ form= (FormImpl^)ObjectImpl::FromUnmanaged(self);
 
@@ -325,7 +336,7 @@ void FormImpl::set_content(::mforms::Form *self, ::mforms::View *view)
 //--------------------------------------------------------------------------------------------------
 
 
-void FormImpl::center(::mforms::Form *self)
+void FormImpl::center(mforms::Form *self)
 {
   FormImpl^ form= (FormImpl^)ObjectImpl::FromUnmanaged(self);
   if (form != nullptr)
@@ -334,7 +345,7 @@ void FormImpl::center(::mforms::Form *self)
 
 //--------------------------------------------------------------------------------------------------
 
-void FormImpl::flush_events(::mforms::Form *self)
+void FormImpl::flush_events(mforms::Form *self)
 {
   FormImpl^ form= (FormImpl^)ObjectImpl::FromUnmanaged(self);
   if (form != nullptr)
@@ -343,22 +354,22 @@ void FormImpl::flush_events(::mforms::Form *self)
 
 //--------------------------------------------------------------------------------------------------
 
-FormImpl::FormImpl(::mforms::Form *form, ::mforms::Form *owner, ::mforms::FormFlag flag)
+FormImpl::FormImpl(mforms::Form *form, mforms::Form *owner, mforms::FormFlag flag)
   : ViewImpl(form)
 {
   if (owner != NULL)
   {
     // Meant is the window parent here, not the real owner.
     if (owner == mforms::Form::main_form())
-      parent = Application::OpenForms[0];
+      _owner = Application::OpenForms[0];
     else
     {
       FormImpl^ parent_form= (FormImpl^)ObjectImpl::FromUnmanaged(owner);
-      parent = parent_form->get_control<Windows::Forms::Form>();
+      _owner = parent_form->get_control<Windows::Forms::Form>();
     }
   }
   else
-    parent = nullptr;
+    _owner = nullptr;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -388,7 +399,7 @@ void FormImpl::disposed(System::Object ^sender, System::EventArgs ^e)
 
   if (native_form->Tag != nullptr)
   {
-    ::mforms::Form* form=ViewImpl::get_backend_control<mforms::Form>(native_form);
+    mforms::Form* form=ViewImpl::get_backend_control<mforms::Form>(native_form);
     form->release();
   }
 }
