@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,9 +18,14 @@
  */
 
 #include "stdafx.h"
+
+#include "mforms/mforms.h"
+#include "wf_view.h"
 #include "wf_table.h"
 #include "wf_label.h"
 
+#include "ConvUtils.h"
+
 using namespace MySQL;
 using namespace MySQL::Forms;
 using namespace MySQL::Utilities;
@@ -37,22 +42,44 @@ using namespace System::Drawing;
  */
 void apply_bounds(CellList% list)
 {
+  HDWP hdwp = BeginDeferWindowPos(list.Count);
   for each (CellEntry^ entry in list)
   {
-    ViewImpl::remove_auto_resize(entry->control, mforms::ResizeBoth);
-    if (entry->horizontalFill)
-      entry->control->Width= entry->bounds.Width;
-    if (entry->verticalFill)
-      entry->control->Height= entry->bounds.Height;
+    if (!entry->isVisible)
+      continue;
 
-    // Get the resulting control size and compute the final position.
-    int actual_width= entry->control->Width;
-    int actual_height= entry->control->Height;
-    
-    int left= entry->bounds.Left + (entry->bounds.Width - actual_width) / 2;
-    int top= entry->bounds.Top + (entry->bounds.Height - actual_height) / 2;
-    entry->control->Location= Point(left, top);
+    ViewImpl::remove_auto_resize(entry->control, AutoResizeMode::ResizeBoth);
+    Drawing::Rectangle newBounds = entry->control->Bounds;
+
+    if (entry->horizontalFill)
+      newBounds.Width = entry->bounds.Width;
+    if (entry->verticalFill)
+      newBounds.Height = entry->bounds.Height;
+
+    newBounds.X = entry->bounds.Left + (entry->bounds.Width - newBounds.Width) / 2;
+    newBounds.Y = entry->bounds.Top + (entry->bounds.Height - newBounds.Height) / 2;
+
+    // The WebBrowser class doesn't like being resized via DeferWindowPos, so we do it the straight way.
+    // Also, if DeferWindowPos (or BeginDeferWindowPos for that matter) returned an empty handle
+    // then we need to do the resizing that way for all controls.
+    if (hdwp == 0  || is<Windows::Forms::WebBrowser>(entry->control))
+      entry->control->Bounds = newBounds;
+    else
+    {
+      HDWP new_hdwp = DeferWindowPos(hdwp, (HWND)entry->control->Handle.ToInt32(), 0, newBounds.X, newBounds.Y,
+        newBounds.Width, newBounds.Height, SWP_NOZORDER);
+      if (new_hdwp == 0)
+      {
+        // Something went wrong, so finish what we started so far and continue with straight resize.
+        EndDeferWindowPos(hdwp);
+      }
+
+      hdwp = new_hdwp;
+    }
   }
+
+  if (hdwp != 0)
+    EndDeferWindowPos(hdwp);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -123,14 +150,12 @@ int CompareRowSpan(CellEntry^ entry1, CellEntry^ entry2)
  */
 bool GtkTableLayout::Layout(Object^ container, LayoutEventArgs^ arguments)
 {
-  Table^ table= (Table^) container;
-  if (!table->IsHandleCreated)
+  Table^ table = (Table^) container;
+  if (!ViewImpl::can_layout(table, arguments->AffectedProperty))
     return false;
 
   if (table->RowCount > 0 && table->ColumnCount > 0)
   {
-    ControlUtilities::SuspendDrawing(table);
-
     ViewImpl::adjust_auto_resize_from_docking(table);
     System::Drawing::Size tableSize = table->ComputeLayout(table->Size, true);
 
@@ -138,9 +163,6 @@ bool GtkTableLayout::Layout(Object^ container, LayoutEventArgs^ arguments)
     if (parentLayoutNeeded)
       ViewImpl::resize_with_docking(table, tableSize);
 
-    ControlUtilities::ResumeDrawing(table);
-    table->Refresh();
-
     return parentLayoutNeeded;
   }
   return false;
@@ -148,6 +170,17 @@ bool GtkTableLayout::Layout(Object^ container, LayoutEventArgs^ arguments)
 
 //----------------- Table ---------------------------------------------------------------------------
 
+Table::Table()
+{
+  homogeneous = false;
+  columnSpacing = 0;
+  rowSpacing = 0;
+  columnCount = 0;
+  rowCount = 0;
+}
+
+//--------------------------------------------------------------------------------------------------
+
 /**
  * Computes the entire layout of the table. This includes size and position of client controls.
  * 
@@ -200,8 +233,12 @@ System::Drawing::Size Table::ComputeLayout(System::Drawing::Size proposedSize, b
   for (int i= 0; i < ColumnCount; i++)
     horizontalExpandState[i]= false;
 
-  proposedSize.Width -= Padding.Horizontal;
-  proposedSize.Height -= Padding.Vertical;
+  if (!HorizontalCenter)
+    proposedSize.Width -= Padding.Horizontal;
+  if (!VerticalCenter)
+    proposedSize.Height -= Padding.Vertical;
+  bool useHorizontalCentering = resizeChildren && HorizontalCenter;
+  bool useVerticalCentering = resizeChildren && VerticalCenter;
 
   System::Drawing::Size newSize= System::Drawing::Size::Empty;
 
@@ -213,19 +250,34 @@ System::Drawing::Size Table::ComputeLayout(System::Drawing::Size proposedSize, b
   // Keep expand states so we can apply them later.
   for each (CellEntry^ entry in content)
   {
-    entry->isVisible= entry->control->Visible && (entry->rightAttachment > entry->leftAttachment)
-      && (entry->bottomAttachment > entry->topAttachment);
+    entry->isVisible = entry->control->Visible && (entry->rightAttachment > entry->leftAttachment)
+      && (entry->bottomAttachment > entry->topAttachment) && !entry->control->IsDisposed && !entry->control->Disposing;
     if (!entry->isVisible)
       continue;
 
     // Check if the width of the entry is larger than what we have already.
     // While we are at it, keep the expand state in the associated cells.
-    int currentWidth= 0;
-    for (int i= entry->leftAttachment; i < entry->rightAttachment; i++)
+    // However, if the current entry is expanding and covers a column which is already set to expand
+    // then don't apply expansion, as we only want to expand those columns.
+    int currentWidth = 0;
+    bool doExpand = entry->horizontalExpand;
+    if (doExpand)
+    {
+      useHorizontalCentering = false; // Expansion disables auto centering.
+      for (int i = entry->leftAttachment; i < entry->rightAttachment; i++)
+      {
+        if (horizontalExpandState[i])
+        {
+          doExpand = false;
+          break;
+        }
+      }
+    }
+    for (int i = entry->leftAttachment; i < entry->rightAttachment; i++)
     {
       currentWidth += widths[i];
-      if (entry->horizontalExpand)
-        horizontalExpandState[i]= true;
+      if (doExpand)
+        horizontalExpandState[i] = true;
     }
 
     bool use_min_width = ViewImpl::use_min_width_for_layout(entry->control);
@@ -242,6 +294,11 @@ System::Drawing::Size Table::ComputeLayout(System::Drawing::Size proposedSize, b
         entry->bounds.Height= entry->control->MinimumSize.Height;
     }
 
+    // Weird behavior of combo boxes. They report a one pixel too small height (you cannot set them
+    // to this smaller height, though). So this wrong value is messing up our computed overall height.
+    if (entry->control->GetType() == ComboBox::typeid)
+      entry->bounds.Height++;
+
     // Set all cells to the computed partial size if it is larger than what was found so far.
     // On the way apply the expand flag to all cells that are covered by that entry.
 
@@ -273,12 +330,10 @@ System::Drawing::Size Table::ComputeLayout(System::Drawing::Size proposedSize, b
     int max= 0;
     for (int i= 0; i < ColumnCount; i++)
       if (widths[i] > max)
-        max= widths[i];
+        max = widths[i];
 
-    if (proposedSize.Width > max * ColumnCount)
-      max = proposedSize.Width / ColumnCount;
     for (int i= 0; i < ColumnCount; i++)
-      widths[i]= max;
+      widths[i] = max;
   }
 
   // Compute overall width and handle expanded entries.
@@ -287,28 +342,29 @@ System::Drawing::Size Table::ComputeLayout(System::Drawing::Size proposedSize, b
   newSize.Width += (ColumnCount - 1) * ColumnSpacing;
 
   // Do auto sizing the table if enabled. Apply minimal bounds in any case.
-  if (newSize.Width > proposedSize.Width || ViewImpl::can_auto_resize_horizontally(this))
+  if (newSize.Width > proposedSize.Width || (ViewImpl::can_auto_resize_horizontally(this) && !useHorizontalCentering))
   {
-    proposedSize.Width= newSize.Width;
-    if (proposedSize.Width < MinimumSize.Width)
-      proposedSize.Width= MinimumSize.Width;
+    proposedSize.Width = newSize.Width;
+    if (proposedSize.Width < MinimumSize.Width - Padding.Horizontal)
+      proposedSize.Width = MinimumSize.Width - Padding.Horizontal;
   }
 
-  // Handle expansion of cells, which only applies to the table if it is not set to homogeneous mode.
-  // The following test will also fail if homogeneous mode is set because the cell sizes 
-  // have been adjusted already to fill the entire table.
-  if (newSize.Width < proposedSize.Width)
+  // Handle expansion of cells.
+  if (resizeChildren)
   {
-    int expandCount= 0;
-    for (int i= 0; i < ColumnCount; i++)
-      if (horizontalExpandState[i])
-        expandCount++;
-    if (expandCount > 0)
+    if (!useHorizontalCentering && (newSize.Width < proposedSize.Width))
     {
-      int fraction= (proposedSize.Width - newSize.Width) / expandCount;
+      int expandCount= 0;
       for (int i= 0; i < ColumnCount; i++)
         if (horizontalExpandState[i])
-          widths[i] += fraction;
+          expandCount++;
+      if (expandCount > 0)
+      {
+        int fraction= (proposedSize.Width - newSize.Width) / expandCount;
+        for (int i= 0; i < ColumnCount; i++)
+          if (horizontalExpandState[i])
+            widths[i] += fraction;
+      }
     }
   }
 
@@ -328,11 +384,25 @@ System::Drawing::Size Table::ComputeLayout(System::Drawing::Size proposedSize, b
     // On the way apply the expand flag to all cells that are covered by that entry.
 
     // Check if the height of the entry is larger than what we have already.
+    // Same expansion handling here as for horizontal expansion.
     int currentHeight= 0;
+    bool doExpand = entry->verticalExpand;
+    if (doExpand)
+    {
+      useVerticalCentering = false; // Expansion disables auto centering.
+      for (int i = entry->topAttachment; i < entry->bottomAttachment; i++)
+      {
+        if (verticalExpandState[i])
+        {
+          doExpand = false;
+          break;
+        }
+      }
+    }
     for (int i= entry->topAttachment; i < entry->bottomAttachment; i++)
     {
       currentHeight += heights[i];
-      if (entry->verticalExpand)
+      if (doExpand)
         verticalExpandState[i]= true;
     }
 
@@ -340,7 +410,7 @@ System::Drawing::Size Table::ComputeLayout(System::Drawing::Size proposedSize, b
     // Currently this applies only for wrapping labels.
     if (entry->control->GetType() == WrapControlLabel::typeid)
     {
-      int currentWidth= 0;
+      int currentWidth = 0;
       for (int i= entry->leftAttachment; i < entry->rightAttachment; i++)
         currentWidth += widths[i];
       entry->bounds= System::Drawing::Rectangle(Point(0, 0),
@@ -368,30 +438,26 @@ System::Drawing::Size Table::ComputeLayout(System::Drawing::Size proposedSize, b
   // Handle homogeneous mode.
   if (Homogeneous)
   {
-    int max= 0;
-    for (int i= 0; i < RowCount; i++)
+    int max = 0;
+    for (int i = 0; i < RowCount; i++)
       if (heights[i] > max)
-        max= heights[i];
+        max = heights[i];
 
-    // If the sum of all resized rows still does not fill the full height
-    // then distribute the remaining space too to make them fill.
-    if (proposedSize.Height > max * RowCount)
-      max = proposedSize.Height / RowCount;
-    for (int i= 0; i < RowCount; i++)
-      heights[i]= max;
+    for (int i = 0; i < RowCount; i++)
+      heights[i] = max;
   }
 
   // Compute overall size and handle expanded entries.
-  for (int i= 0; i < RowCount; i++)
+  for (int i = 0; i < RowCount; i++)
     newSize.Height += heights[i];
   newSize.Height += (RowCount - 1) * RowSpacing;
 
   // Do auto sizing the table if enabled. Apply minimal bounds in any case.
-  if (newSize.Height > proposedSize.Height || ViewImpl::can_auto_resize_vertically(this))
+  if (newSize.Height > proposedSize.Height || (ViewImpl::can_auto_resize_vertically(this) && !useVerticalCentering))
   {
-    proposedSize.Height= newSize.Height;
-    if (proposedSize.Height < MinimumSize.Height)
-      proposedSize.Height= MinimumSize.Height;
+    proposedSize.Height = newSize.Height;
+    if (proposedSize.Height < MinimumSize.Height - Padding.Vertical)
+      proposedSize.Height = MinimumSize.Height - Padding.Vertical;
   }
 
   // Handle expansion of cells (vertical case). Since this can happen only if the new size is
@@ -399,7 +465,7 @@ System::Drawing::Size Table::ComputeLayout(System::Drawing::Size proposedSize, b
   // have already our target size). Hence do it only if we are actually layouting the table.
   if (resizeChildren)
   {
-    if (newSize.Height < proposedSize.Height)
+    if (!useVerticalCentering && (newSize.Height < proposedSize.Height))
     {
       int expandCount= 0;
       for (int i= 0; i < RowCount; i++)
@@ -415,34 +481,41 @@ System::Drawing::Size Table::ComputeLayout(System::Drawing::Size proposedSize, b
     }
 
     // Compute target bounds from cell sizes. Compute one more column/row used as right/bottom border.
-    array<int>^ rowStarts= gcnew array<int>(RowCount + 1);
-    rowStarts[0]= Padding.Top;
+    array<int>^ rowStarts = gcnew array<int>(RowCount + 1);
+    rowStarts[0] = Padding.Top;
+    if (useVerticalCentering && (newSize.Height < proposedSize.Height))
+      rowStarts[0] = (proposedSize.Height - newSize.Height) / 2;
+
     for (int i= 1; i <= RowCount; i++)
-      rowStarts[i]= rowStarts[i - 1] + heights[i - 1] + RowSpacing;
+      rowStarts[i] = rowStarts[i - 1] + heights[i - 1] + RowSpacing;
 
     array<int>^ columnStarts= gcnew array<int>(ColumnCount + 1);
-    columnStarts[0]= Padding.Left;
+    columnStarts[0] = Padding.Left;
+    if (useHorizontalCentering && (newSize.Width < proposedSize.Width))
+      columnStarts[0] = (proposedSize.Width - newSize.Width) / 2;
+
     for (int i= 1; i <= ColumnCount; i++)
-      columnStarts[i]= columnStarts[i - 1] + widths[i - 1] + ColumnSpacing;
+      columnStarts[i] = columnStarts[i - 1] + widths[i - 1] + ColumnSpacing;
 
     for each (CellEntry^ entry in content)
     {
       if (!entry->isVisible)
         continue;
 
-      entry->bounds.X= columnStarts[entry->leftAttachment];
-      entry->bounds.Y= rowStarts[entry->topAttachment];
-      entry->bounds.Width= columnStarts[entry->rightAttachment] - columnStarts[entry->leftAttachment] - ColumnSpacing;
-      entry->bounds.Height= rowStarts[entry->bottomAttachment] - rowStarts[entry->topAttachment] - RowSpacing;
+      entry->bounds.X = columnStarts[entry->leftAttachment];
+      entry->bounds.Y = rowStarts[entry->topAttachment];
+      entry->bounds.Width = columnStarts[entry->rightAttachment] - columnStarts[entry->leftAttachment] - ColumnSpacing;
+      entry->bounds.Height = rowStarts[entry->bottomAttachment] - rowStarts[entry->topAttachment] - RowSpacing;
     }
 
     // Apply target bounds to cell content.
     apply_bounds(content);
-
   }
 
-  proposedSize.Width += Padding.Horizontal;
-  proposedSize.Height += Padding.Vertical;
+  if (!HorizontalCenter)
+    proposedSize.Width += Padding.Horizontal;
+  if (!VerticalCenter)
+    proposedSize.Height += Padding.Vertical;
 
   return proposedSize;
 }
@@ -466,6 +539,11 @@ System::Drawing::Size Table::GetPreferredSize(System::Drawing::Size proposedSize
 
 void Table::Add(Control^ control, int left, int right, int top, int bottom, int flags)
 {
+  if (left >= ColumnCount || right > ColumnCount)
+    throw gcnew IndexOutOfRangeException("Adding control to table at an invalid column position");
+  if (top >= RowCount || top > RowCount)
+    throw gcnew IndexOutOfRangeException("Adding control to table at an invalid row position");
+
   ViewImpl::set_layout_dirty(this, true);
 
   CellEntry^ entry= gcnew CellEntry();
@@ -488,20 +566,20 @@ void Table::Add(Control^ control, int left, int right, int top, int bottom, int
     top= 0;
   if (top > bottom)
   {
-    int temp= top;
-    top= bottom;
-    bottom= temp;
+    int temp = top;
+    top = bottom;
+    bottom = temp;
   }
   if (top == bottom)
     bottom++;
-  entry->leftAttachment= left;
-  entry->rightAttachment= right;
-  entry->topAttachment= top;
-  entry->bottomAttachment= bottom;
-  entry->horizontalExpand= (flags & ::mforms::HExpandFlag) != 0;
-  entry->verticalExpand= (flags & ::mforms::VExpandFlag) != 0;
-  entry->horizontalFill= (flags & ::mforms::HFillFlag) != 0;
-  entry->verticalFill= (flags & ::mforms::VFillFlag) != 0;
+  entry->leftAttachment = left;
+  entry->rightAttachment = right;
+  entry->topAttachment = top;
+  entry->bottomAttachment = bottom;
+  entry->horizontalExpand = (flags & mforms::HExpandFlag) != 0;
+  entry->verticalExpand = (flags & mforms::VExpandFlag) != 0;
+  entry->horizontalFill = (flags & mforms::HFillFlag) != 0;
+  entry->verticalFill = (flags & mforms::VFillFlag) != 0;
   content.Add(entry); // for us
   Controls->Add(control); // for Windows
 }
@@ -511,7 +589,6 @@ void Table::Add(Control^ control, int left, int right, int top, int bottom, int
 void Table::Remove(Control^ control)
 {
   ViewImpl::set_layout_dirty(this, true);
-  Controls->Remove(control);
 
   for each (CellEntry^ entry in content)
   {
@@ -521,11 +598,137 @@ void Table::Remove(Control^ control)
       break;
     }
   }
+
+  Controls->Remove(control);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+Layout::LayoutEngine^ Table::LayoutEngine::get()
+{
+  if (layoutEngine == nullptr)
+    layoutEngine = gcnew GtkTableLayout();
+
+  return layoutEngine;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+bool Table::Homogeneous::get()
+{
+  return homogeneous;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void Table::Homogeneous::set(bool value)
+{
+  if (homogeneous != value)
+  {
+    ViewImpl::set_layout_dirty(this, true);
+    homogeneous = value;
+    Refresh();
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
+int Table::RowSpacing::get()
+{
+  return rowSpacing;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void Table::RowSpacing::set(int value)
+{
+  if (rowSpacing != value)
+  {
+    ViewImpl::set_layout_dirty(this, true);
+    rowSpacing= value;
+    Refresh();
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
+int Table::ColumnSpacing::get()
+{
+  return columnSpacing;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void Table::ColumnSpacing::set(int value)
+{
+  if (columnSpacing != value)
+  {
+    ViewImpl::set_layout_dirty(this, true);
+    columnSpacing= value;
+    Refresh();
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
+int Table::RowCount::get() 
+{ 
+  return rowCount;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void Table::RowCount::set(int value)
+{
+  if (rowCount != value)
+  {
+    ViewImpl::set_layout_dirty(this, true);
+    rowCount = value;
+    Refresh();
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
+int Table::ColumnCount::get() 
+{ 
+  return columnCount;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void Table::ColumnCount::set(int value)
+{
+  if (columnCount != value)
+  {
+    ViewImpl::set_layout_dirty(this, true);
+    columnCount = value;
+    Refresh();
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
+CellList% Table::Content::get()
+{
+  return content;
 }
 
 //----------------- TableImpl -----------------------------------------------------------------------
 
-bool TableImpl::create(::mforms::Table *self)
+void TableImpl::set_padding(int left, int top, int right, int bottom)
+{
+  // Depending on what is specified as padding we apply a dynamic padding (centering so the content).
+  Table ^table = get_control<Table>();
+  table->HorizontalCenter = (left < 0 || right < 0);
+  table->VerticalCenter = (top < 0 || bottom < 0);
+  
+  ViewImpl::set_padding(left, top, right, bottom);
+}
+
+//-------------------------------------------------------------------------------------------------
+
+bool TableImpl::create(mforms::Table *self)
 {
   TableImpl ^table= gcnew TableImpl(self);
 
