@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,13 +19,19 @@
 
 #include "stdafx.h"
 
-#include "wf_toolbar.h"
+#include "mforms/mforms.h"
 #include "wf_view.h"
+#include "wf_toolbar.h"
+
+#include "base/log.h"
+#include "base/string_utilities.h"
+#include "ConvUtils.h"
 
 using namespace System::Windows::Forms;
 using namespace System::IO;
 using namespace System::Drawing;
 
+using namespace MySQL;
 using namespace MySQL::Forms;
 using namespace MySQL::Controls;
 using namespace MySQL::Utilities;
@@ -47,25 +53,17 @@ DEFAULT_LOG_DOMAIN(DOMAIN_MFORMS_WRAPPER)
 System::Drawing::Size MformsToolStripLayout::ComputeLayout(MformsToolStrip^ toolstrip,
   System::Drawing::Size proposedSize, bool preferredSizeOnly)
 {
-  int itemsTotalWidth= 0;
+  int itemsTotalWidth = 0;
   int maxHeight = 0;
-  int expandCount= 0;
+  int expandCount = 0;
 
-  // Consider extra pixels for the height if we have a main tool strip (it renders an inner border).
-  int borderOffset = 0;
-  if (toolstrip->Renderer->GetType() == FlatMainToolStripRenderer::typeid)
-    borderOffset = 2;
+  proposedSize.Width -= toolstrip->Padding.Horizontal + toolstrip->GripMargin.Horizontal;
+  if (proposedSize.Width < 0)
+    proposedSize.Width = 0;
 
-  if (!preferredSizeOnly)
-  {
-    proposedSize.Width -= toolstrip->Padding.Horizontal + toolstrip->GripMargin.Horizontal;
-    proposedSize.Height = toolstrip->Size.Height - toolstrip->Padding.Vertical - borderOffset;
-  }
-
-  // The Margin property of a ToolStripItem has a special meaning. The horizontal values determine
-  // the distances to neighbor items. The vertical values reduce the available vertical space into
-  // which an item can be placed. An item is always vertically centered in its remaining space.
-  // Hence the horizontal margin extends the overall width, while the vertical distance does not.
+  proposedSize.Height = toolstrip->Size.Height - toolstrip->Padding.Vertical;
+  if (proposedSize.Height < 0)
+    proposedSize.Height = 0;
 
   // Determine total width of all items so we can distribute any remaining space over
   // all expander items.
@@ -75,19 +73,22 @@ System::Drawing::Size MformsToolStripLayout::ComputeLayout(MformsToolStrip^ tool
       continue;
 
     if (item->GetType() != ToolStripSegmentedButton::typeid)
-      item->Size = item->GetPreferredSize(proposedSize);
-    itemsTotalWidth += item->Width+ item->Margin.Horizontal;
+      item->Size = item->GetPreferredSize(proposedSize); // The size includes the item's margin.
+    
     if (item->Height > maxHeight)
       maxHeight = item->Height;
     if (item->GetType() == ToolStripExpander::typeid)
-      expandCount++;
+      expandCount++; // Expanders have no width per se, but get all that's unused.
+    else
+      itemsTotalWidth += item->Width + item->Margin.Horizontal;
   }
   
-  // Adjust width of the container if it is too small.
+  // Adjust width of the container if it is too small, but don't make the toolbar smaller than
+  // the available space. It always stretches over the available space.
   if (proposedSize.Width < itemsTotalWidth)
     proposedSize.Width = itemsTotalWidth;
 
-  // Adjust height of the container if it is too small.
+  // Same for the height.
   if (proposedSize.Height < maxHeight)
     proposedSize.Height = maxHeight;
 
@@ -108,6 +109,9 @@ System::Drawing::Size MformsToolStripLayout::ComputeLayout(MformsToolStrip^ tool
 
     for each (ToolStripItem^ item in toolstrip->Items)
     {
+      if (!item->Visible)
+        continue;
+
       if (item->GetType() == ToolStripExpander::typeid)
       {
         offset += expanderFraction;
@@ -120,29 +124,19 @@ System::Drawing::Size MformsToolStripLayout::ComputeLayout(MformsToolStrip^ tool
       else
       {
         System::Drawing::Point itemLocation;
-        int remainingHeight = proposedSize.Height - item->Margin.Vertical;
+        offset += item->Margin.Left;
 
-        // Vertically center the item within the bounds that remain when we take off its margin.
-        itemLocation.Y = toolstrip->Padding.Top + item->Margin.Top + borderOffset / 2 +
-          (int) Math::Ceiling((remainingHeight - item->Height) / 2.0);
+        // Vertically center the item within the bounds of the toolstrip (less its padding).
+        itemLocation.Y = toolstrip->Padding.Top + (int) Math::Ceiling((proposedSize.Height - item->Height) / 2.0);
         itemLocation.X = offset;
         toolstrip->ApplyLocation(item, itemLocation);
-        offset += item->Width;
-
-        // No spacing between segmented buttons, but still spacing around the first and last button.
-        if (item->GetType() == ToolStripSegmentedButton::typeid)
-        {
-          // Check if this item is surrounded by other segmented buttons.
-          offset += 0;
-        }
-        else
-          offset += item->Margin.Horizontal;
+        offset += item->Width + item->Margin.Right;
       }
     }
   }
 
   proposedSize.Width += toolstrip->Padding.Horizontal + toolstrip->GripMargin.Horizontal;
-  proposedSize.Height += toolstrip->Padding.Vertical + borderOffset;
+  proposedSize.Height += toolstrip->Padding.Vertical;
 
   return proposedSize;
 }
@@ -151,9 +145,9 @@ System::Drawing::Size MformsToolStripLayout::ComputeLayout(MformsToolStrip^ tool
 
 bool MformsToolStripLayout::Layout(Object^ container, LayoutEventArgs^ arguments)
 {
-  MformsToolStrip^ toolstrip= (MformsToolStrip^) container;
-
-  ControlUtilities::SuspendDrawing(toolstrip);
+  MformsToolStrip^ toolstrip = (MformsToolStrip^) container;
+  if (!ViewImpl::can_layout(toolstrip, arguments->AffectedProperty))
+    return false;
 
   System::Drawing::Size toolStripSize = ComputeLayout(toolstrip, toolstrip->Size, false);
 
@@ -162,14 +156,17 @@ bool MformsToolStripLayout::Layout(Object^ container, LayoutEventArgs^ arguments
   if (parentLayoutNeeded && toolstrip->AutoSize)
     toolstrip->Size = toolStripSize;
 
-  ControlUtilities::ResumeDrawing(toolstrip);
-  toolstrip->Refresh();
-
   return parentLayoutNeeded;
 }
 
 //----------------- MformsToolStrip ----------------------------------------------------------------
 
+MformsToolStrip::MformsToolStrip()
+{
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void MformsToolStrip::ApplyLocation(ToolStripItem^ item, Point location)
 {
   // SetItemLocation is protected and its visibility cannot be increased nor is it possible
@@ -360,9 +357,9 @@ bool ToolBarImpl::create_tool_bar(mforms::ToolBar* item, mforms::ToolBarType typ
       native_toolbar = ViewImpl::create<MformsToolStrip>(item, toolbar);
       native_toolbar->Name = "MainToolBar";
       native_toolbar->Renderer = gcnew FlatMainToolStripRenderer();
-      native_toolbar->Padding = Padding(0);
+      native_toolbar->Padding = Padding(2);
       native_toolbar->GripMargin = Padding(4, 6, 4, 6);
-      native_toolbar->Margin = Padding(4, 8, 2, 8);
+      native_toolbar->Margin = Padding(4, 5, 2, 5); // Used only for separator items. Does not influence the height.
       native_toolbar->AutoSize = true;
       break;
 
@@ -375,7 +372,7 @@ bool ToolBarImpl::create_tool_bar(mforms::ToolBar* item, mforms::ToolBarType typ
       native_toolbar->GripMargin = Padding(2, 4, 2, 5);
       native_toolbar->Padding = Padding(5, 0, 0, 0);
       native_toolbar->AutoSize = false;
-      native_toolbar->Size = Size(100, 28);
+      native_toolbar->Size = Size(100, 23);
       try
       {
         native_toolbar->Font = ControlUtilities::GetFont("Microsoft Sans Serif", 8.25f);
@@ -386,7 +383,7 @@ bool ToolBarImpl::create_tool_bar(mforms::ToolBar* item, mforms::ToolBarType typ
         log_error("ToolBarImpl::create_tool_bar failed. %s\n", e->Message);
       }
 
-      native_toolbar->BackColor = ColorTranslator::FromHtml("#bcc7d8");
+      native_toolbar->BackColor = Conversions::GetApplicationColor(ApplicationColor::AppColorPanelToolbar, false);
       break;
 
     case mforms::OptionsToolBar:
@@ -401,11 +398,10 @@ bool ToolBarImpl::create_tool_bar(mforms::ToolBar* item, mforms::ToolBarType typ
     case mforms::ToolPickerToolBar:
       native_toolbar = ViewImpl::create<ToolStrip>(item, toolbar);
       native_toolbar->Name = "ToolPickerToolBar";
-      native_toolbar->BackColor = SystemColors::ButtonFace;
+      native_toolbar->BackColor = Conversions::GetApplicationColor(ApplicationColor::AppColorPanelToolbar, false);
       native_toolbar->GripStyle = ToolStripGripStyle::Hidden;
-      //native_toolbar->ImageScalingSize = System::Drawing::Size(24, 22);
       native_toolbar->LayoutStyle = ToolStripLayoutStyle::VerticalStackWithOverflow;
-      native_toolbar->Padding = Padding(0, 0, 0, 0);
+      native_toolbar->Padding = Padding(0);
       native_toolbar->RenderMode = ToolStripRenderMode::ManagerRenderMode;
       native_toolbar->AutoSize = true;
       break;
@@ -428,6 +424,11 @@ void ToolBarImpl::insert_item(mforms::ToolBar* toolbar, int index, mforms::ToolB
   {
     ToolStrip^ native_toolbar = toolbar_impl->get_control<ToolStrip>();
     ToolStripItem^ native_item = toolbar_item->get_control<ToolStripItem>();
+
+    // update the name of the control, so the accessibility support has something to work with
+    native_item->Name = CppStringToNative(item->get_name());
+    native_item->AccessibleName = native_item->Name;
+
     if (index < 0 || index >= native_toolbar->Items->Count)
     {
       index = native_toolbar->Items->Count;
@@ -467,25 +468,25 @@ bool ToolBarImpl::create_tool_item(mforms::ToolBarItem* item, mforms::ToolBarIte
   case mforms::ActionItem:
     native_item = ViewImpl::create<ToolStripButton>(item, toolbar_item, native_item);
     native_item->Click += gcnew System::EventHandler(toolbar_item, &ToolBarItemImpl::OnItemActivation);
-    native_item->Margin = Padding(1, 2, 1, 2);
+    native_item->Margin = Padding(2, 2, 2, 2);
     break;
 
   case mforms::TextActionItem:
     native_item = ViewImpl::create<ToolStripButton>(item, toolbar_item, native_item);
     native_item->Click += gcnew System::EventHandler(toolbar_item, &ToolBarItemImpl::OnItemActivation); 
-    native_item->ForeColor = ColorTranslator::FromHtml("#4a6184");
+    native_item->ForeColor = Conversions::GetApplicationColor(ApplicationColor::AppColorPanelToolbar, true);
     break;
 
   case mforms::LabelItem:
     native_item = ViewImpl::create<ToolStripLabel>(item, toolbar_item, native_item);
-    native_item->ForeColor = Color::Black;
+    native_item->ForeColor = Conversions::GetApplicationColor(ApplicationColor::AppColorPanelToolbar, true);
     break;
 
   case mforms::ToggleItem:
     native_item = ViewImpl::create<ToolStripButton>(item, toolbar_item, native_item);
     native_item->DisplayStyle = ToolStripItemDisplayStyle::Image;
     native_item->Click += gcnew System::EventHandler(toolbar_item, &ToolBarItemImpl::OnItemActivation); 
-    native_item->Margin = Padding(1, 2, 1, 2);
+    native_item->Margin = Padding(2, 2, 2, 2);
     ((ToolStripButton^) native_item)->CheckOnClick = true;
     break;
 
@@ -517,7 +518,7 @@ bool ToolBarImpl::create_tool_item(mforms::ToolBarItem* item, mforms::ToolBarIte
       selector->DropDownStyle = ComboBoxStyle::DropDownList;
       selector->FlatStyle = FlatStyle::Popup;
       selector->Font = ControlUtilities::GetFont("Microsoft Sans Serif", 8.25f);
-      selector->Margin = Padding(1, 3, 1, 3);
+      selector->Margin = Padding(2, 3, 2, 3);
       selector->SelectedIndexChanged += gcnew EventHandler(toolbar_item, &ToolBarItemImpl::OnItemActivation);
       native_item = selector;
       break;
@@ -547,10 +548,11 @@ bool ToolBarImpl::create_tool_item(mforms::ToolBarItem* item, mforms::ToolBarIte
     break;
 
   default:
-    throw std::runtime_error(base::strfmt("Internal error: unimplemented toolbaritem type requested (%i).", (int) type));
+    throw std::runtime_error(base::strfmt("Internal error: unimplemented toolbar item type requested (%i).", (int) type));
   }
 
   native_item->ImageScaling = ToolStripItemImageScaling::None;
+  native_item->Name = CppStringToNative(item->get_name());
 
   return true;
 }
@@ -589,25 +591,45 @@ void ToolBarImpl::set_item_text(mforms::ToolBarItem* item, const std::string& te
   if (toolbar_item != nullptr)
   {
     ToolStripItem^ native_item = toolbar_item->get_control<ToolStripItem>();
-    native_item->Text = CppStringToNative(text);
-    if (item->get_type() == mforms::ColorSelectorItem)
+    String ^native_text = CppStringToNative(text);
+
+    switch (item->get_type())
     {
-      // If this item is a color selector then find the entry with the corresponding color
-      // and use its image for the selector.
-      ToolStripDropDownButton^ selector = dynamic_cast<ToolStripDropDownButton^>(native_item);
-      bool found = false;
-      for each (ToolStripButton^ button in selector->DropDownItems)
+    case mforms::SelectorItem:
       {
-        if (button->Text == native_item->Text)
+        ToolStripComboBox ^selector = dynamic_cast<ToolStripComboBox^>(native_item);
+        int index = selector->FindStringExact(native_text);
+        if (index < 0)
+          index = 0;
+        selector->SelectedIndex = index;
+        
+        break;
+      }
+      
+    case mforms::ColorSelectorItem:
+      {
+        // If this item is a color selector then find the entry with the corresponding color
+        // and use its image for the selector.
+        ToolStripDropDownButton ^selector = dynamic_cast<ToolStripDropDownButton^>(native_item);
+        bool found = false;
+        for each (ToolStripButton^ button in selector->DropDownItems)
         {
-          native_item->Image = button->Image;
-          found = true;
-          break;
+          if (button->Text == native_text)
+          {
+            native_item->Image = button->Image;
+            found = true;
+            break;
+          }
         }
+
+        if (!found)
+          native_item->Image = create_color_image(native_item->Text);
+
+        break;
       }
 
-      if (!found)
-        native_item->Image = create_color_image(native_item->Text);
+    default:
+      native_item->Text = native_text;
     }
   }
 }
