@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,10 +18,23 @@
  */
 
 #include "stdafx.h"
+
+#include "mforms/mforms.h"
 #include "wf_view.h"
+#include "wf_code_editor.h"
+
+#include "wf_box.h"
+#include "wf_appview.h"
+#include "wf_app.h"
+
+#include "base/log.h"
+#include "base/string_utilities.h"
+
+#include "ConvUtils.h"
 
 using namespace Windows::Forms;
 using namespace System::Drawing;
+using namespace System::IO;
 
 using namespace MySQL;
 using namespace MySQL::Forms;
@@ -29,16 +42,18 @@ using namespace MySQL::Utilities;
 using namespace MySQL::Controls;
 
 using namespace Aga::Controls::Tree;
-using namespace ScintillaNet;
 
 DEFAULT_LOG_DOMAIN(DOMAIN_MFORMS_WRAPPER)
 
 //----------------- ViewImpl ----------------------------------------------------------------------
 
-ViewImpl::ViewImpl(::mforms::View *view)
+ViewImpl::ViewImpl(mforms::View *view)
   : ObjectImpl(view)
 {
-  tooltip= nullptr;
+  tooltip = nullptr;
+  backgroundImage = nullptr;
+  layoutSuspended = false;
+  _resize_mode = AutoResizeMode::ResizeBoth;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -65,9 +80,11 @@ bool ViewImpl::use_min_width_for_layout(Control^ control)
   if (!needsMinWidth)
   {
     System::Type^ type = control->GetType();
-    needsMinWidth= (type == TabControl::typeid) || (type == Panel::typeid) || (type == ComboBox::typeid)
+    needsMinWidth = (type == TabControl::typeid) || (type == Panel::typeid) || (type == ComboBox::typeid)
       || (type == TreeViewAdv::typeid) || (type == ListBox::typeid) || (type == TextBox::typeid)
-      || (type == SplitContainer::typeid) || (type == Scintilla::typeid)
+      || (type == SplitContainer::typeid) || (type == ScintillaControl::typeid)
+      || (type == Windows::Forms::WebBrowser::typeid) || (type == HtmlLabel::typeid) || (type == HtmlPanel::typeid)
+      || (type == Windows::Forms::ProgressBar::typeid)
       
       // TODO: investigate if we can remove this and override GetPreferredSize in FlatTabControl instead.
       || (type == FlatTabControl::typeid);
@@ -86,7 +103,8 @@ bool ViewImpl::use_min_height_for_layout(Control^ control)
   {
     System::Type^ type = control->GetType();
     needsMinHeight= (type == TabControl::typeid) || (type == Panel::typeid) || (type == ListBox::typeid)
-      || (type == TreeViewAdv::typeid) || (type == SplitContainer::typeid) || (type == Scintilla::typeid)
+      || (type == TreeViewAdv::typeid) || (type == SplitContainer::typeid) || (type == ScintillaControl::typeid)
+      || (type == Windows::Forms::WebBrowser::typeid)
 
     // TODO: investigate if we can remove this and override GetPreferredSize in FlatTabControl instead.
     || (type == FlatTabControl::typeid);
@@ -100,39 +118,35 @@ bool ViewImpl::use_min_height_for_layout(Control^ control)
 /**
  * Removes the given mode from the auto resize settings of the given control.
  */
-void ViewImpl::remove_auto_resize(Control^ control, mforms::AutoResizeMode mode)
+void ViewImpl::remove_auto_resize(Control^ control, AutoResizeMode mode)
 {
-  // Get backend object for the control, which allows to set individual auto resize modes.
-  mforms::View* view= ViewImpl::get_backend_control<mforms::View>(control);
+  ViewImpl ^wrapper = get_wrapper<ViewImpl>(control);
   switch (mode)
   {
-    case mforms::ResizeBoth:
-      view->set_resize_mode(mforms::ResizeNone);
+    case AutoResizeMode::ResizeBoth:
+      wrapper->_resize_mode = AutoResizeMode::ResizeNone;
       break;
-    case mforms::ResizeHorizontal:
-      if (view->get_resize_mode() == mforms::ResizeBoth)
-        view->set_resize_mode(mforms::ResizeVertical);
+    case AutoResizeMode::ResizeHorizontal:
+      if (wrapper->_resize_mode == AutoResizeMode::ResizeBoth)
+        wrapper->_resize_mode = AutoResizeMode::ResizeVertical;
       else
-        view->set_resize_mode(mforms::ResizeNone);
+        wrapper->_resize_mode = AutoResizeMode::ResizeNone;
       break;
-    case mforms::ResizeVertical:
-      if (view->get_resize_mode() == mforms::ResizeBoth)
-        view->set_resize_mode(mforms::ResizeHorizontal);
+    case AutoResizeMode::ResizeVertical:
+      if (wrapper->_resize_mode == AutoResizeMode::ResizeBoth)
+        wrapper->_resize_mode = AutoResizeMode::ResizeHorizontal;
       else
-        view->set_resize_mode(mforms::ResizeNone);
+        wrapper->_resize_mode = AutoResizeMode::ResizeNone;
       break;
   }
 }
 
 //-------------------------------------------------------------------------------------------------
 
-/**
- * Returns the current resize mode set for the associated back end object of control.
- */
-mforms::AutoResizeMode ViewImpl::get_auto_resize(Control^ control)
+AutoResizeMode ViewImpl::get_auto_resize(Control^ control)
 {
-  mforms::View* view= ViewImpl::get_backend_control<mforms::View>(control);
-  return view->get_resize_mode();
+  ViewImpl ^wrapper = get_wrapper<ViewImpl>(control);
+  return wrapper->_resize_mode;
 }
 
 //-------------------------------------------------------------------------------------------------
@@ -142,10 +156,10 @@ mforms::AutoResizeMode ViewImpl::get_auto_resize(Control^ control)
  */
 bool ViewImpl::can_auto_resize_horizontally(Control^ control)
 {
-  mforms::View* view= ViewImpl::get_backend_control<mforms::View>(control);
-  mforms::AutoResizeMode mode= view->get_resize_mode();
+  ViewImpl ^wrapper = get_wrapper<ViewImpl>(control);
+  AutoResizeMode mode = wrapper->_resize_mode;
 
-  return (mode == mforms::ResizeHorizontal) || (mode == mforms::ResizeBoth);
+  return (mode == AutoResizeMode::ResizeHorizontal) || (mode == AutoResizeMode::ResizeBoth);
 }
 
 //-------------------------------------------------------------------------------------------------
@@ -155,10 +169,10 @@ bool ViewImpl::can_auto_resize_horizontally(Control^ control)
  */
 bool ViewImpl::can_auto_resize_vertically(Control^ control)
 {
-  mforms::View* view= ViewImpl::get_backend_control<mforms::View>(control);
-  mforms::AutoResizeMode mode= view->get_resize_mode();
+  ViewImpl ^wrapper = get_wrapper<ViewImpl>(control);
+  AutoResizeMode mode = wrapper->_resize_mode;
 
-  return (mode == mforms::ResizeVertical) || (mode == mforms::ResizeBoth);
+  return (mode == AutoResizeMode::ResizeVertical) || (mode == AutoResizeMode::ResizeBoth);
 }
 
 //-------------------------------------------------------------------------------------------------
@@ -168,7 +182,8 @@ bool ViewImpl::can_auto_resize_vertically(Control^ control)
  */
 void ViewImpl::set_full_auto_resize(Control^ control)
 {
-  ViewImpl::get_backend_control<mforms::View>(control)->set_resize_mode(mforms::ResizeBoth);
+  ViewImpl ^wrapper = get_wrapper<ViewImpl>(control);
+  wrapper->_resize_mode = AutoResizeMode::ResizeBoth;
 }
 
 //-------------------------------------------------------------------------------------------------
@@ -176,9 +191,10 @@ void ViewImpl::set_full_auto_resize(Control^ control)
 /**
  * Sets resize mode whatever the caller specified.
  */
-void ViewImpl::set_auto_resize(Control^ control, mforms::AutoResizeMode mode)
+void ViewImpl::set_auto_resize(Control^ control, AutoResizeMode mode)
 {
-  ViewImpl::get_backend_control<mforms::View>(control)->set_resize_mode(mode);
+  ViewImpl ^wrapper = get_wrapper<ViewImpl>(control);
+  wrapper->_resize_mode = mode;
 }
 
 //-------------------------------------------------------------------------------------------------
@@ -199,7 +215,7 @@ void ViewImpl::set_layout_dirty(Control^ control, bool value)
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::destroy(::mforms::View *self)
+void ViewImpl::destroy(mforms::View *self)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   delete view;
@@ -207,28 +223,40 @@ void ViewImpl::destroy(::mforms::View *self)
 
 //--------------------------------------------------------------------------------------------------
 
-void ViewImpl::show(::mforms::View *self, bool show)
+void ViewImpl::show(mforms::View *self, bool show)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
 
   if (view != nullptr)
   {
     Control^ control= view->get_control<Control>();
-    control->Visible= show;
 
-    // Unfortunately, we cannot optimize handling here. Control::Visible not only returns the
-    // visibility of itself but its entire view hierarchy. So we cannot distinct between the two
-    // cases if the control itself is hidden or one of its parents. Setting Control::Visible
-    // does the right job, though.
-    //if (control->Visible != show)
-    if (control->IsHandleCreated && self->get_parent() != NULL)
-      self->get_parent()->set_layout_dirty(true);
+    // Create the control's window handle if not yet done. This is necessary because otherwise
+    // when setting a control to hidden and its window handle is not created, it will be moved to the end
+    // of it's parent Controls list, messing up our layout.
+    if (!control->IsHandleCreated)
+      control->Handle;
+
+    // Use reflection to get the true visible state. The Visible property returns the value for the
+    // control itself as well as all its parents.
+    bool visible = (bool)Control::typeid
+      ->GetMethod("GetState", System::Reflection::BindingFlags::Instance | System::Reflection::BindingFlags::NonPublic)
+      ->Invoke(control, gcnew array<Object^> { 2 });
+    if (show != visible && self->get_parent() != NULL)
+    {
+      if (view->layoutSuspended)
+        self->get_parent()->set_layout_dirty(true);
+      else
+        control->PerformLayout(control, "Visible");
+    }
+
+    control->Visible = show;
   };
 }
 
 //-------------------------------------------------------------------------------------------------
 
-int ViewImpl::get_width(::mforms::View *self)
+int ViewImpl::get_width(mforms::View *self)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -238,7 +266,7 @@ int ViewImpl::get_width(::mforms::View *self)
 
 //-------------------------------------------------------------------------------------------------
 
-int ViewImpl::get_height(::mforms::View *self)
+int ViewImpl::get_height(mforms::View *self)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -248,7 +276,7 @@ int ViewImpl::get_height(::mforms::View *self)
 
 //-------------------------------------------------------------------------------------------------
 
-int ViewImpl::get_preferred_width(::mforms::View *self)
+int ViewImpl::get_preferred_width(mforms::View *self)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -260,7 +288,7 @@ int ViewImpl::get_preferred_width(::mforms::View *self)
 
 //-------------------------------------------------------------------------------------------------
 
-int ViewImpl::get_preferred_height(::mforms::View *self)
+int ViewImpl::get_preferred_height(mforms::View *self)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -272,7 +300,7 @@ int ViewImpl::get_preferred_height(::mforms::View *self)
 
 //-------------------------------------------------------------------------------------------------
 
-int ViewImpl::get_x(::mforms::View *self)
+int ViewImpl::get_x(mforms::View *self)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -284,7 +312,7 @@ int ViewImpl::get_x(::mforms::View *self)
 
 //-------------------------------------------------------------------------------------------------
 
-int ViewImpl::get_y(::mforms::View *self)
+int ViewImpl::get_y(mforms::View *self)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -296,7 +324,7 @@ int ViewImpl::get_y(::mforms::View *self)
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::set_size(::mforms::View *self, int w, int h)
+void ViewImpl::set_size(mforms::View *self, int w, int h)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -305,7 +333,7 @@ void ViewImpl::set_size(::mforms::View *self, int w, int h)
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::set_padding(::mforms::View *self, int left, int top, int right, int bottom)
+void ViewImpl::set_padding(mforms::View *self, int left, int top, int right, int bottom)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -317,7 +345,7 @@ void ViewImpl::set_padding(::mforms::View *self, int left, int top, int right, i
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::set_position(::mforms::View *self, int x, int y)
+void ViewImpl::set_position(mforms::View *self, int x, int y)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -330,7 +358,7 @@ void ViewImpl::set_position(::mforms::View *self, int x, int y)
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::client_to_screen(::mforms::View *self, int& x, int& y)
+void ViewImpl::client_to_screen(mforms::View *self, int& x, int& y)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -361,12 +389,12 @@ void ViewImpl::relayout()
   if (control->InvokeRequired)
     control->BeginInvoke(gcnew MethodInvoker(this, &ViewImpl::relayout));
   else
-    control->PerformLayout();
+    control->PerformLayout(control, "Bounds");
 }
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::set_enabled(::mforms::View *self, bool flag)
+void ViewImpl::set_enabled(mforms::View *self, bool flag)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -375,7 +403,7 @@ void ViewImpl::set_enabled(::mforms::View *self, bool flag)
 
 //-------------------------------------------------------------------------------------------------
 
-bool ViewImpl::is_enabled(::mforms::View *self)
+bool ViewImpl::is_enabled(mforms::View *self)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -385,23 +413,26 @@ bool ViewImpl::is_enabled(::mforms::View *self)
 
 //-------------------------------------------------------------------------------------------------
 
-View *ViewImpl::find_subview(::mforms::View *self, std::string &name)
+View *ViewImpl::find_subview(mforms::View *self, std::string &name)
 {
   return 0;
 }
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::set_name(::mforms::View *self, const std::string& text)
+void ViewImpl::set_name(mforms::View *self, const std::string& text)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
+  {
     view->get_control<Control>()->Name = CppStringToNative(text);
+    view->get_control<Control>()->AccessibleName = CppStringToNative(text);
+  }
 }
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::relayout(::mforms::View *self)
+void ViewImpl::relayout(mforms::View *self)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -410,7 +441,7 @@ void ViewImpl::relayout(::mforms::View *self)
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::set_needs_repaint(::mforms::View *self)
+void ViewImpl::set_needs_repaint(mforms::View *self)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -419,18 +450,20 @@ void ViewImpl::set_needs_repaint(::mforms::View *self)
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::suspend_layout(::mforms::View *self, bool flag)
+void ViewImpl::suspend_layout(mforms::View *self, bool flag)
 {
-  ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
+  ViewImpl^ view = (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
   {
+    view->layoutSuspended = flag;
+    Control ^control = view->get_control();
     if (flag)
-      view->get_control<Control>()->SuspendLayout();
+      control->SuspendLayout();
     else
     {
-      view->get_control<Control>()->ResumeLayout();
+      control->ResumeLayout();
       if (self->is_layout_dirty())
-        view->get_control<Control>()->PerformLayout();
+        control->PerformLayout(control, "Bounds");
     }
   }
 }
@@ -475,17 +508,34 @@ std::string ViewImpl::get_back_color(mforms::View *self)
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::set_back_image(mforms::View *self, const std::string &path, mforms::ImageLayout layout)
+void ViewImpl::set_back_image(mforms::View *self, const std::string &path, mforms::Alignment align)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
-    view->set_back_image(CppStringToNative(path), layout);
+    view->set_back_image(CppStringToNative(path), align);
 }
 
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::set_tooltip(::mforms::View *self, const std::string& text)
+void ViewImpl::focus(mforms::View *self)
+{
+  ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
+  if (view != nullptr)
+  {
+    Control^ control = view->get_control();
+    Windows::Forms::Form ^form = control->FindForm();
+    if (form != nullptr)
+      form->ActiveControl = control;
+    else
+      if (control->CanSelect)
+        control->Select();
+  }
+}
+
+//-------------------------------------------------------------------------------------------------
+
+void ViewImpl::set_tooltip(mforms::View *self, const std::string& text)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -494,7 +544,7 @@ void ViewImpl::set_tooltip(::mforms::View *self, const std::string& text)
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::set_font(::mforms::View *self, const std::string& fontDescription)
+void ViewImpl::set_font(mforms::View *self, const std::string& fontDescription)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -503,7 +553,7 @@ void ViewImpl::set_font(::mforms::View *self, const std::string& fontDescription
 
 //-------------------------------------------------------------------------------------------------
 
-bool ViewImpl::is_shown(::mforms::View *self)
+bool ViewImpl::is_shown(mforms::View *self)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -519,7 +569,10 @@ void ViewImpl::initialize()
 
   // Can be null, e.g. for non-control objects like dialogs.
   if (control != nullptr)
-    control->Resize += gcnew EventHandler(this, &ViewImpl::OnResize);
+  {
+    control->Resize += gcnew EventHandler(this, &ViewImpl::Resize);
+    control->Enter += gcnew EventHandler(this, &ViewImpl::Enter);
+  }
 }
 
 //-------------------------------------------------------------------------------------------------
@@ -573,7 +626,7 @@ void ViewImpl::set_tooltip(const std::string& text)
 void ViewImpl::set_font(const std::string& fontDescription)
 {
   std::string font;
-  int size;
+  float size;
   bool bold;
   bool italic;
   base::parse_font_description(fontDescription, font, size, bold, italic);
@@ -586,8 +639,8 @@ void ViewImpl::set_font(const std::string& fontDescription)
 
   try
   {
-    get_control<Control>()->Font = ControlUtilities::GetFont(CppStringToNativeRaw(font),
-      (float) size, style);
+    // Font size in points.
+    get_control<Control>()->Font = ControlUtilities::GetFont(CppStringToNativeRaw(font), size, style);
   }
   catch (System::ArgumentException^ e)
   {
@@ -614,43 +667,38 @@ String^ ViewImpl::get_front_color()
 
 void ViewImpl::set_back_color(String ^color)
 {
-  get_control<Control>()->BackColor= System::Drawing::ColorTranslator::FromHtml(color);
+  if (color == "")
+    get_control<Control>()->BackColor = Color::Transparent;
+  else
+    get_control<Control>()->BackColor = System::Drawing::ColorTranslator::FromHtml(color);
 }
 
 //-------------------------------------------------------------------------------------------------
 
 String^ ViewImpl::get_back_color()
 {
+  if (get_control<Control>()->BackColor == Color::Transparent)
+    return "";
   return System::Drawing::ColorTranslator::ToHtml(get_control<Control>()->BackColor);
 }
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::set_back_image(String ^path, mforms::ImageLayout layout)
+void ViewImpl::set_back_image(String ^path, mforms::Alignment layout)
 {
-  Windows::Forms::ImageLayout native_layout;
-  switch (layout)
+  path = AppImpl::get_image_path(path);
+  if (File::Exists(path))
   {
-    case mforms::LayoutTile:
-      native_layout= Windows::Forms::ImageLayout::Tile;
-      break;
-    case mforms::LayoutCenter:
-      native_layout= Windows::Forms::ImageLayout::Center;
-      break;
-    case mforms::LayoutStretch:
-      native_layout= Windows::Forms::ImageLayout::Stretch;
-      break;
-    case mforms::LayoutZoom:
-      native_layout= Windows::Forms::ImageLayout::Zoom;
-      break;
-    default:
-        native_layout= Windows::Forms::ImageLayout::None;
+    backgroundImage = Image::FromFile(path, true);
+    backgroundImageAlignment = layout;
   }
+}
 
-  // An exception is raised if the image cannot be found.
-  Image^ image= Image::FromFile(path, true);
-  get_control<Control>()->BackgroundImage= image;
-  get_control<Control>()->BackgroundImageLayout= native_layout;
+//-------------------------------------------------------------------------------------------------
+
+void ViewImpl::set_resize_mode(AutoResizeMode mode)
+{
+  _resize_mode = mode;
 }
 
 //-------------------------------------------------------------------------------------------------
@@ -685,25 +733,59 @@ void ViewImpl::resize_with_docking(Control^ control, System::Drawing::Size& size
 void ViewImpl::adjust_auto_resize_from_docking(Control^ control)
 {
   if (control->Dock == DockStyle::Fill)
-    set_auto_resize(control, mforms::ResizeNone);
+    set_auto_resize(control, AutoResizeMode::ResizeNone);
   else
     if (control->Dock == DockStyle::Left || control->Dock == DockStyle::Right)
-      remove_auto_resize(control, mforms::ResizeHorizontal);
+      remove_auto_resize(control, AutoResizeMode::ResizeHorizontal);
     else
       if (control->Dock == DockStyle::Top || control->Dock == DockStyle::Bottom)
-        remove_auto_resize(control, mforms::ResizeVertical);
+        remove_auto_resize(control, AutoResizeMode::ResizeVertical);
 }
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::flush_events(::mforms::View *)
+/**
+ * Quick check if it makes sense to start layouting the given control.
+ */
+bool ViewImpl::can_layout(Control ^control, String ^reason)
+{
+  // Limit layouting to certain property changes.
+  if (reason != nullptr)
+  {
+    if (reason != "Padding" &&
+      reason != "Bounds" &&
+      reason != "Visible" &&
+      reason != "Parent"
+      )
+      return false;
+  }
+
+  /* Opting out based on a not created handle is a good optimization. Unfortunately, sometimes it
+     prevents some forms (e.g. wizards) from layouting properly. So we have to disable this for now.
+  if (!control->IsHandleCreated)
+    return false;
+  */
+
+  if (control->Tag != nullptr)
+  {
+    mforms::View *backend = ViewImpl::get_backend_control<mforms::View>(control);
+    if (backend->is_destroying())
+      return false;
+  }
+
+  return true;
+}
+
+//-------------------------------------------------------------------------------------------------
+
+void ViewImpl::flush_events(mforms::View *)
 {
   Application::DoEvents();
 }
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::OnResize(System::Object ^sender, EventArgs ^e)
+void ViewImpl::Resize(System::Object ^sender, EventArgs ^e)
 {
   Control^ control = (Control^)sender;
 
@@ -711,7 +793,70 @@ void ViewImpl::OnResize(System::Object ^sender, EventArgs ^e)
   {
     mforms::View* view = ViewImpl::get_backend_control<mforms::View>(control);
     if (!view->is_destroying())
-      (*view->signal_resized())();
+      view->resize();
+  }
+}
+
+//-------------------------------------------------------------------------------------------------
+
+void ViewImpl::Enter(System::Object ^sender, EventArgs ^e)
+{
+  Control^ control = (Control^)sender;
+
+  if (control->Tag != nullptr)
+  {
+    mforms::View* view = ViewImpl::get_backend_control<mforms::View>(control);
+    if (!view->is_destroying())
+      view->focus_changed();
+  }
+}
+
+//-------------------------------------------------------------------------------------------------
+
+/**
+ * Draws the background image with the earlier set layout, if there's one.
+ */
+void ViewImpl::draw_background(PaintEventArgs ^args)
+{
+  if (backgroundImage != nullptr)
+  {
+    int left = 0;
+    int top = 0;
+    Control ^control = get_control();
+
+    // Horizontal alignment.
+    switch (backgroundImageAlignment)
+    {
+    case mforms::BottomCenter:
+    case mforms::MiddleCenter:
+    case mforms::TopCenter:
+      left = (control->Width - backgroundImage->Width) / 2;
+      break;
+
+    case mforms::BottomRight:
+    case mforms::MiddleRight:
+    case mforms::TopRight:
+      left = control->Width - backgroundImage->Width;
+      break;
+    }
+
+    // Vertical alignment.
+    switch (backgroundImageAlignment)
+    {
+    case mforms::MiddleLeft:
+    case mforms::MiddleCenter:
+    case mforms::MiddleRight:
+      top = (control->Height - backgroundImage->Height) / 2;
+      break;
+
+    case mforms::BottomLeft:
+    case mforms::BottomCenter:
+    case mforms::BottomRight:
+      top = control->Height - backgroundImage->Height;
+      break;
+    }
+
+    args->Graphics->DrawImage(backgroundImage, Point(left, top));
   }
 }
 
