@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,14 +16,21 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
-#include "stdafx.h"
-#include "wf_wizard.h"
-#include "mforms/app.h"
 
 /**
  * Implementation of the mforms wizard form.
  */
 
+#include "stdafx.h"
+
+#include "mforms/mforms.h"
+#include "wf_view.h"
+#include "wf_form.h"
+#include "wf_wizard.h"
+
+#include "base/log.h"
+#include "ConvUtils.h"
+
 using namespace System::IO;
 using namespace Drawing::Drawing2D;
 
@@ -42,23 +49,12 @@ BufferedPanel::BufferedPanel()
 
 //----------------- WizardImpl static functions ----------------------------------------------------
 
-bool WizardImpl::create(::mforms::Wizard *self, ::mforms::Form* parent)
+bool WizardImpl::create(mforms::Wizard* self, mforms::Form* owner)
 {
-  WizardImpl ^wizard= gcnew WizardImpl(self);
+  WizardImpl^ wizard = gcnew WizardImpl(self, owner);
 
-  if (parent != NULL)
-  {
-    // Meant is the window parent here, not the real owner.
-    if (parent == mforms::Form::main_form())
-      wizard->_parent = Application::OpenForms[0];
-    else
-    {
-      FormImpl^ parent_form= (FormImpl^)ObjectImpl::FromUnmanaged(parent);
-      wizard->_parent = parent_form->get_control<Windows::Forms::Form>();
-    }
-  }
-  else
-    wizard->_parent = nullptr;
+  ViewImpl::create<Form>(self, wizard);
+  wizard->setupLayout(self);
 
   if (wizard != nullptr)
     return true;
@@ -67,7 +63,7 @@ bool WizardImpl::create(::mforms::Wizard *self, ::mforms::Form* parent)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_title(::mforms::Wizard* self, const string& title)
+void WizardImpl::set_title(mforms::Wizard* self, const std::string& title)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -76,7 +72,7 @@ void WizardImpl::set_title(::mforms::Wizard* self, const string& title)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::run_modal(::mforms::Wizard* self)
+void WizardImpl::run_modal(mforms::Wizard* self)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -85,7 +81,7 @@ void WizardImpl::run_modal(::mforms::Wizard* self)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::close(::mforms::Wizard* self)
+void WizardImpl::close(mforms::Wizard* self)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -94,7 +90,7 @@ void WizardImpl::close(::mforms::Wizard* self)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::flush_events(::mforms::Wizard* self)
+void WizardImpl::flush_events(mforms::Wizard* self)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -103,7 +99,7 @@ void WizardImpl::flush_events(::mforms::Wizard* self)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_content(::mforms::Wizard* self, ::mforms::View* view)
+void WizardImpl::set_content(mforms::Wizard* self, mforms::View* view)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -112,7 +108,7 @@ void WizardImpl::set_content(::mforms::Wizard* self, ::mforms::View* view)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_heading(::mforms::Wizard* self, const string& heading)
+void WizardImpl::set_heading(mforms::Wizard* self, const std::string& heading)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -121,7 +117,7 @@ void WizardImpl::set_heading(::mforms::Wizard* self, const string& heading)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_step_list(::mforms::Wizard* self, const vector<string>& steps)
+void WizardImpl::set_step_list(mforms::Wizard* self, const vector<string>& steps)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -130,7 +126,7 @@ void WizardImpl::set_step_list(::mforms::Wizard* self, const vector<string>& ste
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_allow_cancel(::mforms::Wizard* self, bool flag)
+void WizardImpl::set_allow_cancel(mforms::Wizard* self, bool flag)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -139,7 +135,7 @@ void WizardImpl::set_allow_cancel(::mforms::Wizard* self, bool flag)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_allow_back(::mforms::Wizard* self, bool flag)
+void WizardImpl::set_allow_back(mforms::Wizard* self, bool flag)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -148,7 +144,7 @@ void WizardImpl::set_allow_back(::mforms::Wizard* self, bool flag)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_allow_next(::mforms::Wizard* self, bool flag)
+void WizardImpl::set_allow_next(mforms::Wizard* self, bool flag)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -157,7 +153,7 @@ void WizardImpl::set_allow_next(::mforms::Wizard* self, bool flag)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_show_extra(::mforms::Wizard* self, bool flag)
+void WizardImpl::set_show_extra(mforms::Wizard* self, bool flag)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -166,7 +162,7 @@ void WizardImpl::set_show_extra(::mforms::Wizard* self, bool flag)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_extra_caption(::mforms::Wizard* self, const string& caption)
+void WizardImpl::set_extra_caption(mforms::Wizard* self, const std::string& caption)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -175,7 +171,7 @@ void WizardImpl::set_extra_caption(::mforms::Wizard* self, const string& caption
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_next_caption(::mforms::Wizard* self, const string& caption)
+void WizardImpl::set_next_caption(mforms::Wizard* self, const std::string& caption)
 {
   WizardImpl^ wizard= (WizardImpl^)ObjectImpl::FromUnmanaged(self);
   if (wizard != nullptr)
@@ -184,110 +180,9 @@ void WizardImpl::set_next_caption(::mforms::Wizard* self, const string& caption)
 
 //----------------- WizardImpl actual implementation -----------------------------------------------
 
-WizardImpl::WizardImpl(::mforms::Wizard* self)
-  : ObjectImpl(self)
+WizardImpl::WizardImpl(mforms::Wizard* self, mforms::Form* owner)
+  : FormImpl(self, owner, mforms::FormDialogFrame)
 {
-  _main= gcnew Form;
-  _sidebar= gcnew BufferedPanel;
-  _title= gcnew Label;
-  _extra_button= gcnew Button;
-  _footer= gcnew TableLayoutPanel;
-  _button_bar= gcnew FlowLayoutPanel;
-  _back_button= gcnew Button;
-  _next_button= gcnew Button;
-  _cancel_button= gcnew Button;
-  _content= gcnew Panel;
-
-  string icon_path= mforms::App::get()->get_resource_path("wb-wizard-vista-bg.png");
-  String^ native_path= gcnew String(icon_path.c_str());
-  if (File::Exists(native_path))
-    _sidebar_overlay= Image::FromFile(native_path);
-  else
-    _sidebar_overlay= nullptr;
-
-  // Main form, the dialog itself.
-  _main->BackColor= Color::White;
-  _main->FormBorderStyle = ::FormBorderStyle::SizableToolWindow;
-  _main->MaximizeBox = false;      
-  _main->MinimizeBox = false;
-  _main->StartPosition = FormStartPosition::CenterScreen;
-  _main->CancelButton= _cancel_button;
-  _main->Resize += gcnew EventHandler(resize);
-  _main->Tag= gcnew IntPtr(self);
-  _main->Icon= gcnew Icon("images/icons/MySQLWorkbench.ico", Size(16, 16));
-
-  // The footer with buttons.
-  _main->Controls->Add(_footer);
-  _footer->RowCount= 1;
-  _footer->ColumnCount= 2;
-  _footer->AutoSize= true;
-  _footer->Dock= DockStyle::Bottom;
-  _footer->Padding= Padding(10);
-  _footer->Controls->Add(_extra_button, 0, 0);
-
-  _button_bar->FlowDirection= FlowDirection::RightToLeft;
-  _button_bar->Controls->Add(_cancel_button);
-  _button_bar->Controls->Add(_next_button);
-  _button_bar->Controls->Add(_back_button);
-  _button_bar->Dock= DockStyle::Right;
-  _button_bar->AutoSize= true;
-  _button_bar->Margin= Padding(0);
-  _footer->Controls->Add(_button_bar, 1, 0);
-
-  // We need to keep a reference to the wizard in the buttons to be able
-  // to notify it when the buttons are clicked.
-  _back_button->Tag= gcnew IntPtr(self);
-  _back_button->Text= "&Back";
-  _back_button->UseMnemonic= true;
-  _back_button->Click += gcnew EventHandler(backClick);
-  _back_button->FlatStyle = FlatStyle::System;
-
-  _next_button->Tag= gcnew IntPtr(self);
-  _next_button->Text= "&Next";
-  _next_button->UseMnemonic= true;
-  _next_button->Click += gcnew EventHandler(nextClick);
-  _next_button->FlatStyle = FlatStyle::System;
-
-  _cancel_button->Tag= gcnew IntPtr(self);
-  _cancel_button->Text= "Cancel";
-  _cancel_button->Click += gcnew EventHandler(cancelClick);
-  _cancel_button->FlatStyle = FlatStyle::System;
-
-  _extra_button->Tag= gcnew IntPtr(self);
-  _extra_button->Text= "&Advanced";
-  _extra_button->UseMnemonic= true;
-  _extra_button->Click += gcnew EventHandler(extraClick);
-  _extra_button->FlatStyle = FlatStyle::System;
-
-  // The side bar with background painting and list of strings.
-  _main->Controls->Add(_sidebar);
-  _sidebar->Tag= gcnew IntPtr(self);
-  _sidebar->Dock= DockStyle::Left;
-  _sidebar->ForeColor= Color::White;
-  _sidebar->Paint += gcnew PaintEventHandler(this, &WizardImpl::sidebarPaint);
-
-  // Title
-  _main->Controls->Add(_title);
-  _title->AutoSize= false;
-  try
-  {
-    _title->Font= gcnew Font("Tahoma", 12.0, FontStyle::Bold, GraphicsUnit::Pixel);
-  }
-  catch (System::ArgumentException^ e)
-  {
-    // Argument exception pops up when the system cannot find the Regular font style (corrupt font).
-    log_error("WizardImpl::c_tor setting title font failed. %s\n", e->Message);
-  }
-
-  _title->ForeColor= ColorTranslator::FromHtml("#003392");
-  _title->Location= Point(_sidebar->Width, 0);
-  _title->Padding= Padding(15);
-
-  // Other initialization.
-  _main->Controls->Add(_content);
-
-  // Resize as last action. Triggers layouting of the elements.
-  _main->Size= Size(800, 600); 
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -316,13 +211,119 @@ void WizardImpl::sidebarPaint(Object^ sender, PaintEventArgs^ arguments)
 
 //--------------------------------------------------------------------------------------------------
 
+void WizardImpl::setupLayout(mforms::Wizard* wizard)
+{
+  _sidebar = gcnew BufferedPanel;
+  _title = gcnew Label;
+  _extra_button = gcnew Button;
+  _footer = gcnew TableLayoutPanel;
+  _button_bar = gcnew FlowLayoutPanel;
+  _back_button = gcnew Button;
+  _next_button = gcnew Button;
+  _cancel_button = gcnew Button;
+  _content = gcnew Panel;
+
+  std::string icon_path = mforms::App::get()->get_resource_path("wb-wizard-vista-bg.png");
+  String^ native_path = gcnew String(icon_path.c_str());
+  if (File::Exists(native_path))
+    _sidebar_overlay = Image::FromFile(native_path);
+  else
+    _sidebar_overlay = nullptr;
+
+  // Main form, the dialog itself.
+  Form^ form = get_control<Form>();
+  form->BackColor = Color::White;
+  form->FormBorderStyle = FormBorderStyle::SizableToolWindow;
+  form->MaximizeBox = false;      
+  form->MinimizeBox = false;
+  form->StartPosition = FormStartPosition::CenterScreen;
+  form->CancelButton = _cancel_button;
+  form->Resize += gcnew EventHandler(resize);
+  form->Icon = gcnew Icon("images/icons/MySQLWorkbench.ico", Size(16, 16));
+
+  // The footer with buttons.
+  form->Controls->Add(_footer);
+  _footer->RowCount = 1;
+  _footer->ColumnCount = 2;
+  _footer->AutoSize = true;
+  _footer->Dock = DockStyle::Bottom;
+  _footer->Padding = Padding(10);
+  _footer->Controls->Add(_extra_button, 0, 0);
+
+  _button_bar->FlowDirection = FlowDirection::RightToLeft;
+  _button_bar->Controls->Add(_cancel_button);
+  _button_bar->Controls->Add(_next_button);
+  _button_bar->Controls->Add(_back_button);
+  _button_bar->Dock = DockStyle::Right;
+  _button_bar->AutoSize = true;
+  _button_bar->Margin = Padding(0);
+  _footer->Controls->Add(_button_bar, 1, 0);
+
+  // We need to keep a reference to the wizard in the buttons to be able
+  // to notify it when the buttons are clicked.
+  _back_button->Tag = gcnew IntPtr(wizard);
+  _back_button->Text = "&Back";
+  _back_button->UseMnemonic = true;
+  _back_button->Click += gcnew EventHandler(backClick);
+  _back_button->FlatStyle = FlatStyle::System;
+
+  _next_button->Tag = gcnew IntPtr(wizard);
+  _next_button->Text = "&Next";
+  _next_button->UseMnemonic = true;
+  _next_button->Click += gcnew EventHandler(nextClick);
+  _next_button->FlatStyle = FlatStyle::System;
+
+  _cancel_button->Tag = gcnew IntPtr(wizard);
+  _cancel_button->Text = "Cancel";
+  _cancel_button->Click += gcnew EventHandler(cancelClick);
+  _cancel_button->FlatStyle = FlatStyle::System;
+
+  _extra_button->Tag = gcnew IntPtr(wizard);
+  _extra_button->Text = "&Advanced";
+  _extra_button->UseMnemonic = true;
+  _extra_button->Click += gcnew EventHandler(extraClick);
+  _extra_button->FlatStyle = FlatStyle::System;
+
+  // The side bar with background painting and list of strings.
+  form->Controls->Add(_sidebar);
+  _sidebar->Tag = gcnew IntPtr(wizard);
+  _sidebar->Dock = DockStyle::Left;
+  _sidebar->ForeColor = Color::White;
+  _sidebar->Paint += gcnew PaintEventHandler(this, &WizardImpl::sidebarPaint);
+
+  // Title
+  form->Controls->Add(_title);
+  _title->AutoSize= false;
+  try
+  {
+    _title->Font = gcnew Font("Tahoma", 12.0, FontStyle::Bold, GraphicsUnit::Pixel);
+  }
+  catch (System::ArgumentException^ e)
+  {
+    // Argument exception pops up when the system cannot find the Regular font style (corrupt font).
+    log_error("WizardImpl::c_tor setting title font failed. %s\n", e->Message);
+  }
+
+  _title->ForeColor = ColorTranslator::FromHtml("#003392");
+  _title->Location = Point(_sidebar->Width, 0);
+  _title->Padding = Padding(15);
+
+  // Other initialization.
+  form->Controls->Add(_content);
+
+  // Resize as last action. Triggers layouting of the elements.
+  form->Size = Size(800, 600); 
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void WizardImpl::nextClick(Object^ sender, EventArgs^ arguments)
 {
   Windows::Forms::Button^ btn= (Windows::Forms::Button^)sender;
 
   if (btn->Tag != nullptr)
   {
-    ::mforms::Wizard* wizard= ViewImpl::get_backend_control<::mforms::Wizard>(btn);
+    mforms::Wizard* wizard = ViewImpl::get_backend_control<mforms::Wizard>(btn);
     if (wizard != NULL)
       wizard->next_clicked();
   }
@@ -332,11 +333,11 @@ void WizardImpl::nextClick(Object^ sender, EventArgs^ arguments)
 
 void WizardImpl::backClick(Object^ sender, EventArgs^ arguments)
 {
-  Windows::Forms::Button^ btn= (Windows::Forms::Button^)sender;
+  Windows::Forms::Button^ btn = (Windows::Forms::Button^)sender;
 
   if (btn->Tag != nullptr)
   {
-    ::mforms::Wizard* wizard= ViewImpl::get_backend_control<::mforms::Wizard>(btn);
+    mforms::Wizard* wizard = ViewImpl::get_backend_control<mforms::Wizard>(btn);
     if (wizard != NULL)
       wizard->back_clicked();
   }
@@ -346,11 +347,11 @@ void WizardImpl::backClick(Object^ sender, EventArgs^ arguments)
 
 void WizardImpl::cancelClick(Object^ sender, EventArgs^ arguments)
 {
-  Windows::Forms::Button^ btn= (Windows::Forms::Button^)sender;
+  Windows::Forms::Button^ btn = (Windows::Forms::Button^)sender;
 
   if (btn->Tag != nullptr)
   {
-    ::mforms::Wizard* wizard= ViewImpl::get_backend_control<::mforms::Wizard>(btn);
+    mforms::Wizard* wizard = ViewImpl::get_backend_control<mforms::Wizard>(btn);
     if (wizard != NULL && wizard->_cancel_slot())
       ((Form^) btn->TopLevelControl)->Close();
   }
@@ -364,7 +365,7 @@ void WizardImpl::extraClick(Object^ sender, EventArgs^ arguments)
 
   if (btn->Tag != nullptr)
   {
-    ::mforms::Wizard* wizard= ViewImpl::get_backend_control<::mforms::Wizard>(btn);
+    mforms::Wizard* wizard= ViewImpl::get_backend_control<mforms::Wizard>(btn);
     if (wizard != NULL)
       wizard->extra_clicked();
   }
@@ -374,11 +375,12 @@ void WizardImpl::extraClick(Object^ sender, EventArgs^ arguments)
 
 void WizardImpl::resize(Object^ sender, EventArgs^ arguments)
 {
-  ::mforms::Wizard* wizard= ViewImpl::get_backend_control<::mforms::Wizard>((Form^) sender);
+  Form^ form = dynamic_cast<Form^>(sender);
+  mforms::Wizard* wizard= ViewImpl::get_backend_control<mforms::Wizard>(form);
   WizardImpl^ wizard_impl= (WizardImpl^)ObjectImpl::FromUnmanaged(wizard);
 
-  System::Drawing::Rectangle area= wizard_impl->_main->DisplayRectangle;
-  wizard_impl->_title->Size= Size(area.Width - wizard_impl->_sidebar->Width, 50);
+  System::Drawing::Rectangle area = form->DisplayRectangle;
+  wizard_impl->_title->Size = Size(area.Width - wizard_impl->_sidebar->Width, 50);
   wizard_impl->_title->Location= Point(wizard_impl->_sidebar->Width, 0);
 
   wizard_impl->_content->Size= Size(area.Width - wizard_impl->_sidebar->Width, 
@@ -388,23 +390,26 @@ void WizardImpl::resize(Object^ sender, EventArgs^ arguments)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_title(const string& title)
+void WizardImpl::set_title(const std::string& title)
 {
-  _main->Text= CppStringToNative(title);
+  Form^ form = get_control<Form>();
+  form->Text = CppStringToNative(title);
 }
 
 //--------------------------------------------------------------------------------------------------
 
 void WizardImpl::run_modal()
 {
-  _main->ShowDialog(_parent);
+  Form^ form = get_control<Form>();
+  form->ShowDialog(Owner);
 }
 
 //--------------------------------------------------------------------------------------------------
 
 void WizardImpl::close()
 {
-  _main->Close();
+  Form^ form = get_control<Form>();
+  form->Close();
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -416,20 +421,20 @@ void WizardImpl::flush_events()
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_content(::mforms::View *view)
+void WizardImpl::set_content(mforms::View *view)
 {
   // Remove old stuff if there is some.
   if (_content->Controls->Count > 0)
   {
-    _content->Controls[0]->Dock= DockStyle::None;
+    _content->Controls[0]->Dock = DockStyle::None;
     _content->Controls->Clear();
   }
 
   if (view != NULL)
   {
-    ViewImpl^ new_content_view= (ViewImpl^)ObjectImpl::FromUnmanaged(view);
-    view->set_resize_mode(::mforms::ResizeNone);
-    Control^ new_content= new_content_view->get_control<Control>();
+    ViewImpl^ new_content_view = (ViewImpl^)ObjectImpl::FromUnmanaged(view);
+    new_content_view->set_resize_mode(AutoResizeMode::ResizeNone);
+    Control^ new_content = new_content_view->get_control<Control>();
 
     _content->Controls->Add(new_content);
     new_content->Dock = DockStyle::Fill;
@@ -441,7 +446,7 @@ void WizardImpl::set_content(::mforms::View *view)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_heading(const string& heading)
+void WizardImpl::set_heading(const std::string& heading)
 {
   _title->Text= CppStringToNative(heading);
 }
@@ -450,8 +455,10 @@ void WizardImpl::set_heading(const string& heading)
 
 void WizardImpl::set_step_list(const vector<string>& steps)
 {
+  Form^ form = get_control<Form>();
+
   // Remove old list.
-  _main->SuspendLayout();
+  form->SuspendLayout();
   try
   {
     _sidebar->Controls->Clear();
@@ -493,7 +500,7 @@ void WizardImpl::set_step_list(const vector<string>& steps)
   }
   finally
   {
-    _main->ResumeLayout();
+    form->ResumeLayout();
   }
 }
 
@@ -527,14 +534,14 @@ void WizardImpl::set_show_extra(bool flag)
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_extra_caption(const string& caption)
+void WizardImpl::set_extra_caption(const std::string& caption)
 {
   _extra_button->Text= CppStringToNative(caption);
 }
 
 //--------------------------------------------------------------------------------------------------
 
-void WizardImpl::set_next_caption(const string& caption)
+void WizardImpl::set_next_caption(const std::string& caption)
 {
   if (caption.size() == 0)
     _next_button->Text= "&Next";
