@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -41,7 +41,6 @@ namespace MySQL {
       virtual void OnMouseEnter(EventArgs ^e) override;
       virtual void OnMouseLeave(EventArgs ^e) override;
 
-      virtual void OnKeyPress(KeyPressEventArgs^ e) override;
     public:
       PopupControl();
 
@@ -62,22 +61,52 @@ namespace MySQL {
         }
       }
 
-      property int ModalResult
+      virtual property int ModalResult
       {
         int get() { return modalResult; }
         void set(int value) { modalResult = value; }
       }
     };
 
-    private ref class StandardPopupControl: PopupControl
+    /**
+     * A simple transparent and rectangular window.
+     */
+    private ref class TransparentPopupControl: PopupControl
     {
+    private:
+      System::Drawing::Bitmap^ contentBitmap;
     protected:
-      virtual void OnPaint(PaintEventArgs ^e) override;
+      void PrepareBitmap();
+
+      virtual void OnDeactivate(EventArgs ^e) override;
+      virtual void OnKeyPress(KeyPressEventArgs ^e) override;
     public:
-      StandardPopupControl();
+      TransparentPopupControl();
+
+      virtual void DoRepaint() override;
+      virtual int Show(int x, int y) override;
+
+      virtual property int ModalResult
+      {
+        void set(int value) override;
+      }
+
+      virtual property Windows::Forms::CreateParams^ CreateParams
+      {
+        Windows::Forms::CreateParams^ get() override
+        {
+          Windows::Forms::CreateParams^ cp = PopupControl::CreateParams;
+          cp->ExStyle |= WS_EX_LAYERED | WS_EX_NOACTIVATE;
+          return cp;
+        }
+      }
     };
 
-    private ref class TransparentPopupControl: PopupControl
+    /**
+     * A transparent window with rounded corners, a white border, a shadow and a semitransparent black background.
+     * It can optionally use a fade animation when showing and hiding.
+     */
+    private ref class BezelPopupControl: PopupControl
     {
     private:
       System::Drawing::Bitmap^ contentBitmap;
@@ -102,7 +131,7 @@ namespace MySQL {
       virtual void OnLostFocus(EventArgs^ e) override;
       virtual void OnMouseDown(MouseEventArgs^ e) override;
     public:
-      TransparentPopupControl();
+      BezelPopupControl();
 
       virtual void DoRepaint() override;
       virtual int Show(int x, int y) override;
@@ -126,7 +155,7 @@ namespace MySQL {
         bool get() override { return true; }
       }
 
-      virtual property Windows::Forms::CreateParams^ CreateParams	
+      virtual property Windows::Forms::CreateParams^ CreateParams
       {
         Windows::Forms::CreateParams^ get() override
         {
@@ -146,6 +175,7 @@ namespace MySQL {
       ~PopupImpl();
 
       static bool create(mforms::Popup *self, mforms::PopupStyle style);
+      static void destroy(mforms::Popup *self);
       static void set_needs_repaint(mforms::Popup *self);
       static void set_size(mforms::Popup *self, int width, int height);
       static int show(mforms::Popup *self, int spot_x, int spot_y);
@@ -158,6 +188,7 @@ namespace MySQL {
         mforms::ControlFactory *f= mforms::ControlFactory::get_instance();
 
         DEF_CALLBACK2(bool, mforms::Popup*, mforms::PopupStyle, mgr, f->_popup_impl, PopupImpl, create);
+        DEF_CALLBACK1(void, mforms::Popup*, mgr, f->_popup_impl, PopupImpl, destroy);
         DEF_CALLBACK1(void, mforms::Popup*, mgr, f->_popup_impl, PopupImpl, set_needs_repaint);
         DEF_CALLBACK3(void, mforms::Popup*, int, int, mgr, f->_popup_impl, PopupImpl, set_size);
         DEF_CALLBACK3(int, mforms::Popup*, int, int, mgr, f->_popup_impl, PopupImpl, show);
