@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -80,7 +80,10 @@ typedef __gnu_cxx::hash_set<std::string, string_hash> string_hash_set;
 
 #define GRT_VERSION "4.1.0"
 
-
+namespace sql
+{
+  class SQLException;
+};
 
 namespace grt {
  
@@ -145,7 +148,23 @@ namespace grt {
   public:
     user_cancelled(const std::string &exc) : std::runtime_error(exc) {}
   };  
-  
+
+  class MYSQLGRT_PUBLIC db_error : public std::runtime_error
+  {
+    int _error;
+  public:
+    db_error(const sql::SQLException &exc);
+    db_error(const std::string &exc, int error) : std::runtime_error(exc), _error(error) {}
+
+    int error() const { return _error; };
+  };
+
+  class MYSQLGRT_PUBLIC db_not_connected : public std::runtime_error
+  {
+  public:
+    db_not_connected(const std::string &exc) : std::runtime_error(exc) {}
+  };
+
   class MYSQLGRT_PUBLIC db_login_error : public std::runtime_error
   {
   public:
@@ -169,9 +188,15 @@ namespace grt {
 
 
 #include "grtpp_value.h"
-
+//this is too unstable and overcomplicated
+//#define USE_EXPRERIMENTAL_REFS
+#ifdef USE_EXPRERIMENTAL_REFS
+#include "grtpp_references.h"
+#endif
 
 namespace grt {
+
+#ifndef USE_EXPRERIMENTAL_REFS
   class GRT;
   
   namespace internal
@@ -346,7 +371,7 @@ namespace grt {
     Ref(const Ref<Class> &ref)
       : ValueRef(ref)
     {
-#ifdef ENABLE_DEBUG
+#if defined(ENABLE_DEBUG) || defined(_DEBUG)
 #if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ > 2) // this causes errors in mac, with gcc 4.2
       // just to give an error if Class is not an object
       Class::static_class_name();
@@ -409,7 +434,7 @@ namespace grt {
     Class *operator->() const { return static_cast<Class*>(_value); }
 
     ValueRef get_member(const std::string &m) const { return content().get_member(m); }
-    ValueRef get_member(const struct ClassMember *m) const;
+
     void set_member(const std::string &m, const ValueRef &new_value) { content().set_member(m, new_value); }
 
     std::string get_string_member(const std::string &member) const { return content().get_string_member(member); }
@@ -1628,6 +1653,11 @@ namespace grt {
       return DictRef(ivalue);
     }
 
+    static bool can_wrap(const ValueRef& ivalue)
+    {
+      return ivalue.type() == DictType;
+    }
+
     inline Type content_type() const { return content().content_type(); };
     inline std::string content_class_name() const { return content().content_class_name(); }
 
@@ -1840,7 +1870,7 @@ namespace grt {
         throw type_error(DictType, content().content_type(), ListType);
     }
   };
-
+#endif
 
 //--------------------------------------------------------------------------------------------------
 
@@ -2525,8 +2555,9 @@ namespace grt {
     void serialize(const ValueRef &value, const std::string &path, 
                    const std::string &doctype="", const std::string &version="",
                    bool list_objects_as_links= false);
-    ValueRef unserialize(const std::string &path);
+    ValueRef unserialize(const std::string &path, boost::shared_ptr<grt::internal::Unserializer> unserializer = boost::shared_ptr<grt::internal::Unserializer>());
     ValueRef unserialize(const std::string &path, std::string &doctype_ret, std::string &version_ret);
+    boost::shared_ptr<grt::internal::Unserializer> get_unserializer();
 
     xmlDocPtr load_xml(const std::string &path);
     void get_xml_metainfo(xmlDocPtr doc, std::string &doctype_ret, std::string &version_ret);
@@ -2740,6 +2771,7 @@ namespace grt {
     friend class MetaClass;
 
     //std::map<std::string,internal::Object*> _objects;
+    std::map<std::string, ObjectRef > _objects_cache;
 
     std::vector<MessageSlot> _message_slot_stack;
     std::vector<StatusQuerySlot> _status_query_slot_stack;
