@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,7 +24,9 @@
 #include "grtpp_util.h"
 
 #include "base/file_functions.h"
+#include "base/file_utilities.h"
 #include "base/util_functions.h"
+#include "base/string_utilities.h"
 
 #include <stdio.h>
 #include <errno.h>
@@ -142,18 +144,18 @@ static std::string format_signal_names(const std::vector<MetaClass::SignalArg> &
 }
 
 
-static std::string format_argname_list(const std::vector<ArgSpec> &args)
-{
-  std::string s;
-  for (std::vector<ArgSpec>::const_iterator iter= args.begin();
-       iter != args.end(); ++iter)
-  {
-    if (!s.empty())
-      s.append(", ");
-    s.append(iter->name);
-  }
-  return s;  
-}
+//static std::string format_argname_list(const std::vector<ArgSpec> &args)
+//{
+//  std::string s;
+//  for (std::vector<ArgSpec>::const_iterator iter= args.begin();
+//       iter != args.end(); ++iter)
+//  {
+//    if (!s.empty())
+//      s.append(", ");
+//    s.append(iter->name);
+//  }
+//  return s;  
+//}
 
 static std::string format_wraparg_list(const std::vector<ArgSpec> &args)
 {
@@ -245,7 +247,9 @@ struct ClassImplGenerator
       default: break;
       }
     }
-    if (needs_body && gstruct->impl_data())
+    if (!gstruct->get_attribute("simple-impl-data").empty())
+      fprintf(f, ",\n    _data(0), _release_data(NULL)");
+    else if ((needs_body && gstruct->impl_data()))
       fprintf(f, ",\n    _data(0)");
     fprintf(f, "\n");
   }
@@ -317,10 +321,16 @@ struct ClassImplGenerator
     fprintf(f, "{\n");
     fprintf(f, "  typedef %s super;\n", pname.c_str());
     fprintf(f, "public:\n");
-    if (gstruct->impl_data())
+    std::string klass = gstruct->get_attribute("simple-impl-data");
+    if (gstruct->impl_data() || !klass.empty())
     {
-      fprintf(f, "  class ImplData;\n");
-      fprintf(f, "  friend class ImplData;\n");
+      if (klass.empty())
+      {
+        fprintf(f, "  class ImplData;\n");
+        fprintf(f, "  friend class ImplData;\n");
+      }
+      else
+        fprintf(f, "  typedef %s ImplData;\n", klass.c_str());
     }
     // generate constructors
     bool default_ctor_created= false;
@@ -353,9 +363,14 @@ struct ClassImplGenerator
       fprintf(f, "\n");
     }
 
-    if (needs_body)
+    if (needs_body || !gstruct->get_attribute("simple-impl-data").empty())
     {
-      fprintf(f, "  virtual ~%s();\n\n", cname.c_str());
+      if (gstruct->get_attribute("simple-impl-data").empty())
+        fprintf(f, "  virtual ~%s();\n\n", cname.c_str());
+      else
+      {
+        fprintf(f, "  virtual ~%s() { if (_release_data && _data) _release_data(_data);  }\n\n", cname.c_str());
+      }
     }
    
     fprintf(f, "  static std::string static_class_name() { return \"%s\"; }\n", 
@@ -511,15 +526,34 @@ struct ClassImplGenerator
                 format_arg_list(iter->second.arg_types).c_str());
     }
     
-    if (needs_body)
+    if (needs_body || !gstruct->get_attribute("simple-impl-data").empty())
     {
+      bool needs_init = true;
       if (gstruct->impl_data())
       {
-        fprintf(f, "\n  ImplData *get_data() const { return _data; }\n\n");
-        fprintf(f, "  void set_data(ImplData *data);\n");
+        if (gstruct->get_attribute("simple-impl-data").empty())
+        {
+          fprintf(f, "\n  ImplData *get_data() const { return _data; }\n\n");
+          fprintf(f, "  void set_data(ImplData *data);\n");
+        }
+        else
+        {
+          fprintf(f, "\n  ImplData *get_data() const { return _data; }\n\n");
+          fprintf(f, "  void set_data(ImplData *data, void (*release)(ImplData*))\n");
+          fprintf(f, "  {\n");
+          fprintf(f, "    if (_data == data) return;\n");
+          fprintf(f, "    if (_data && _release_data) _release_data(_data);\n");
+          fprintf(f, "    _data= data;\n");
+          fprintf(f, "    _release_data= release;\n");
+          fprintf(f, "  }\n");
+          needs_init = false;
+        }
+      }
+      if (needs_init)
+      {
+        fprintf(f, "  // default initialization function. auto-called by Ref<Object> constructor\n");
+        fprintf(f, "  virtual void init();\n\n");
       }
-      fprintf(f, "  // default initialization function. auto-called by Ref<Object> constructor\n");
-      fprintf(f, "  virtual void init();\n\n");
     }
 
     fprintf(f, "protected:\n");
@@ -559,8 +593,13 @@ struct ClassImplGenerator
 
    
     fprintf(f, "private: // wrapper methods for use by grt\n");
-    if (needs_body && gstruct->impl_data())
-      fprintf(f, "  ImplData *_data;\n\n");
+    if ((needs_body && gstruct->impl_data()) || !gstruct->get_attribute("simple-impl-data").empty())
+    {
+      fprintf(f, "  ImplData *_data;\n");
+      if (!gstruct->get_attribute("simple-impl-data").empty())
+        fprintf(f, "  void (*_release_data)(ImplData *);\n");
+      fprintf(f, "\n");
+    }
     // function to create the object
     if (!gstruct->is_abstract())
     {
@@ -742,13 +781,13 @@ struct ClassImplGenerator
         if (iter->second.owned_object)
         {
           fprintf(f, "  // this member is owned by this object\n");
-          //fprintf(f, "  if (_is_global && _%s.is_valid()) _%s.valueptr()->unmark_global();\n",
+          //fprintf(f, "  if (_is_global && _%s.is_valid()) _%s.unmark_global();\n",
           //        iter->second.name.c_str(), iter->second.name.c_str());
 
           fprintf(f, "// add code here\n");
           fprintf(f, "  _%s= value;\n", iter->second.name.c_str());
 
-          //fprintf(f, "  if (_is_global && _%s.is_valid()) _%s.valueptr()->mark_global();\n",
+          //fprintf(f, "  if (_is_global && _%s.is_valid()) _%s.mark_global();\n",
           //        iter->second.name.c_str(), iter->second.name.c_str());
           fprintf(f, "  owned_member_changed(\"%s\", ovalue, value);\n", iter->second.name.c_str());
         }
@@ -932,20 +971,18 @@ void grt::helper::generate_struct_code(GRT *grt, const std::string &target_file,
     if (files.find((*iter)->source()) == files.end())
     {
       std::string file= (*iter)->source();
-      char *outfile= g_path_get_basename(file.c_str());
+      std::string outfile = base::basename(file);
       char *path;
-      char *tmp;
 
-      tmp= strstr(outfile, ".xml");
-      if (!tmp)
-      {
-        g_free(outfile);
+      std::size_t found = outfile.find(".xml");
+      if (found == std::string::npos)
         continue;
-      }
-      strcpy(tmp, ".h");
+
+      outfile = base::strip_extension(outfile).append(".h");
+
       header_file= outfile;
 
-      path= g_build_path("/", outpath.c_str(), outfile, NULL);
+      path= g_build_path("/", outpath.c_str(), outfile.c_str(), NULL);
 
       fhdr= base_fopen(path, "w+");
       if (!fhdr)
@@ -992,6 +1029,7 @@ void grt::helper::generate_struct_code(GRT *grt, const std::string &target_file,
         }
       }
       fprintf(fhdr, "\n\n");
+      
       if (foreign_classes.find((*iter)->source()) != foreign_classes.end())
       {
          std::set<std::string> classes(foreign_classes[(*iter)->source()]);
@@ -1003,6 +1041,28 @@ void grt::helper::generate_struct_code(GRT *grt, const std::string &target_file,
          fprintf(fhdr, "\n\n");
       }
       
+      // forward declare simple-impl-data classes
+      for (std::list<MetaClass*>::const_iterator jter= meta.begin();
+           jter != meta.end(); ++jter)
+      {
+        std::string klass = (*jter)->get_attribute("simple-impl-data");
+        if (!klass.empty())
+        {
+          std::vector<std::string> parts(base::split(klass, "::"));
+
+          for (size_t i = 0; i < parts.size()-1; i++)
+          {
+            fprintf(fhdr, "namespace %s { ", parts[i].c_str());
+          }
+          fprintf(fhdr, "\n\tclass %s;\n", parts.back().c_str());
+          for (size_t i = 0; i < parts.size()-1; i++)
+          {
+            fprintf(fhdr, "}; ");
+          }
+          fprintf(fhdr, "\n\n");
+        }
+      }        
+      
       g_free(path);
 
       files[(*iter)->source()]= fhdr;
@@ -1266,14 +1326,14 @@ static void export_module_function(FILE *f, const Module::Function &function)
 void grt::helper::generate_module_wrappers(GRT *grt, const std::string &outpath,
                                            const std::vector<Module*> &modules)
 {
-  char *path= g_path_get_basename(outpath.c_str());
+  std::string path = base::basename(outpath);
   FILE *f= base_fopen(outpath.c_str(), "w+");
   char *header_name;
 
   if (!f)
     throw grt::os_error(errno);
 
-  header_name= str_g_subst(path, ".", "_");
+  header_name= str_g_subst(path.c_str(), ".", "_");
 
   fprintf(f, "#ifndef __%s_\n", header_name);
   fprintf(f, "#define __%s_\n", header_name);
