@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,6 +27,8 @@
 #include <time.h>
 #include <boost/bind.hpp>
 
+#include "base/log.h"
+
 #ifdef _WIN32
 #undef max
 #endif
@@ -628,9 +630,9 @@ UndoManager::UndoManager(GRT *grt)
 
 UndoManager::~UndoManager()
 {
-  g_static_rec_mutex_free(&_mutex);
   _changed_signal.disconnect_all_slots(); // prevent emission in reset()
   reset();
+  g_static_rec_mutex_free(&_mutex);
 }
 
 
@@ -867,6 +869,8 @@ bool UndoManager::end_undo_group(const std::string &description, bool trim)
     if (!group->is_open() && _undo_log && _undo_log->good())
       group->dump(*_undo_log);
 
+    if (description != "cancelled")
+      _changed_signal();
     /* have to 1st merge or check for signal_apply from the deleted groups
     if (!getenv("DEBUG_UNDO"))
     {
@@ -888,7 +892,8 @@ bool UndoManager::end_undo_group(const std::string &description, bool trim)
       }
     }
      */
-    
+    static const char* const default_log_domain = "Undo Manager";
+    log_debug("end undo group: %s\n", description.c_str());
     return true;
   }
 }
@@ -903,7 +908,7 @@ void UndoManager::cancel_undo_group()
   else
     stack= &_undo_stack;
 
-  UndoGroup *group;
+  UndoGroup *group= 0;
   UndoGroup *parent= 0;
   UndoGroup *subgroup= 0;
   
@@ -925,6 +930,7 @@ void UndoManager::cancel_undo_group()
       // undo the deepest open undo group
       subgroup->undo(this);
       
+      lock();
       // if this was the top-level undo group, delete it from the stack
       if (subgroup == group)
       {
@@ -937,6 +943,7 @@ void UndoManager::cancel_undo_group()
         delete subgroup;
         parent->get_actions().pop_back();
       }
+      unlock();
     }
     enable();
   }
@@ -1099,7 +1106,9 @@ void UndoManager::add_undo(UndoAction *cmd)
   }
   unlock();
 
-  _changed_signal();
+  UndoGroup *ugrp = dynamic_cast<UndoGroup*>(cmd);
+  if (ugrp && !ugrp->is_open())
+    _changed_signal();
 }
 
 
