@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,7 +25,7 @@
 #include "grtpp_util.h"
 #include "base/string_utilities.h"
 
-#include <glib/gbase64.h>
+#include <glib.h>
 #include <functional>
 
 using namespace grt;
@@ -72,7 +72,7 @@ std::string grt::get_guid()
   str= CFUUIDCreateString(NULL, uid);
   
 #ifdef ENABLE_DEBUG
-  assert(sizeof(data) > CFStringGetLength(str));
+  assert((int)sizeof(data) > CFStringGetLength(str));
 #endif
   //data= (char*)g_malloc(sizeof(gchar)*(CFStringGetLength(str)+1));
   
@@ -180,6 +180,8 @@ ValueRef grt::get_value_by_path(const ValueRef &root, const std::string &path)
     return value;
 
   suffix= path.substr(1); // skip the /
+  // fix possible typos
+  base::replace(suffix, "//", "/");
 
   while (!suffix.empty() && value.is_valid())
   {
@@ -588,7 +590,7 @@ void CopyContext::copy_dict(DictRef &dict, const DictRef &source, bool dontfollo
 
 
 static void fixup_object_copied_references(ObjectRef copy, 
-                                           std::map<internal::Value*, ValueRef> &object_copies)
+                                           std::map<std::string, ValueRef> &object_copies)
 {
   MetaClass *metac(copy.get_metaclass());
     
@@ -622,9 +624,10 @@ static void fixup_object_copied_references(ObjectRef copy,
           
           if (dontfollow)
           {
-            if (object_copies.find(value.valueptr()) != object_copies.end())
+            ObjectRef obj(ObjectRef::cast_from(value));
+            if (object_copies.find(obj.id()) != object_copies.end())
             {
-              list.gset(i, object_copies[value.valueptr()]);
+              list.gset(i, object_copies[obj.id()]);
             }
           }
           else
@@ -643,8 +646,9 @@ static void fixup_object_copied_references(ObjectRef copy,
           
           if (dontfollow)
           {
-            if (object_copies.find(value.valueptr()) != object_copies.end())
-              dict[k]= object_copies[value.valueptr()];
+            ObjectRef obj(ObjectRef::cast_from(value));
+            if (object_copies.find(obj.id()) != object_copies.end())
+              dict[k]= object_copies[obj.id()];
           }
           else
             fixup_object_copied_references(ObjectRef::cast_from(value), object_copies);
@@ -656,8 +660,9 @@ static void fixup_object_copied_references(ObjectRef copy,
         // point to the new copy
         if (dontfollow)
         {
-          if (object_copies.find(v.valueptr()) != object_copies.end())
-            copy.set_member(k, object_copies[v.valueptr()]);  
+          ObjectRef obj(ObjectRef::cast_from(v));
+          if (object_copies.find(obj.id()) != object_copies.end())
+            copy.set_member(k, object_copies[obj.id()]);
         }
         else
           fixup_object_copied_references(ObjectRef::cast_from(v), object_copies);
@@ -669,7 +674,7 @@ static void fixup_object_copied_references(ObjectRef copy,
   while (metac != 0);
 }
 
-ObjectRef CopyContext::duplicate_object(ObjectRef object, std::set<std::string> skip_members)
+ObjectRef CopyContext::duplicate_object(ObjectRef object, std::set<std::string> skip_members, bool _dontfollow)
 {
   if (object.is_valid())
   {
@@ -677,7 +682,7 @@ ObjectRef CopyContext::duplicate_object(ObjectRef object, std::set<std::string>
     ObjectRef copy= metac->allocate();
 
     // save a mapping from the original value to its copy
-    object_copies[object.valueptr()]= copy;
+    object_copies[object.id()]= copy;
     
     do
     {
@@ -695,7 +700,7 @@ ObjectRef CopyContext::duplicate_object(ObjectRef object, std::set<std::string>
 
         Type type= mem->second.type.base.type;
         
-        bool dontfollow= !mem->second.owned_object;
+        bool dontfollow= _dontfollow || !mem->second.owned_object;
         
         if (is_simple_type(type))
         {
@@ -719,15 +724,16 @@ ObjectRef CopyContext::duplicate_object(ObjectRef object, std::set<std::string>
           // point to the new copy
           if (dontfollow)
           {
-            if (object_copies.find(v.valueptr()) != object_copies.end())
-              copy.set_member(k, object_copies[v.valueptr()]);
+            ObjectRef obj(ObjectRef::cast_from(v));
+            if (obj.is_valid() && object_copies.find(obj.id()) != object_copies.end())
+              copy.set_member(k, object_copies[obj.id()]);
             else
               copy.set_member(k, v);
           }
           else
           {
             if (k == "owner") throw; // consistency check
-            ObjectRef vcopy(duplicate_object(ObjectRef::cast_from(v), std::set<std::string>()));
+            ObjectRef vcopy(duplicate_object(ObjectRef::cast_from(v), std::set<std::string>(), false));
             copy.set_member(k, vcopy);
           }
         }
@@ -915,39 +921,57 @@ void grt::merge_contents(ObjectRef target, ObjectRef source)
 }
 
 
-
-ObjectRef grt::copy_object(GRT *grt, ObjectRef object, std::set<std::string> skip_members)
+std::string grt::join_string_list(const StringListRef &list, const std::string &separator)
 {
-  CopyContext copier(grt);
-  ObjectRef copy;
-  
-  copy= copier.copy(object, skip_members);
-  copier.finish();
-
-  return copy;
+  std::string result;
+  for (StringListRef::const_iterator i = list.begin(); i != list.end(); ++i)
+  {
+    if (i != list.begin())
+      result.append(separator);
+    result.append(*i);
+  }
+  return result;
 }
 
 
 
+
 ObjectRef CopyContext::copy(const ObjectRef &object, std::set<std::string> skip_members)
 {
-  ObjectRef copy= duplicate_object(object, skip_members);
+  ObjectRef copy= duplicate_object(object, skip_members, false);
   if (copy.is_valid())
     copies.push_back(copy);
 
   return copy;
 }
 
+ObjectRef CopyContext::shallow_copy(const ObjectRef &object)
+{
+  ObjectRef copy= duplicate_object(object, std::set<std::string>(), true);
+  if (copy.is_valid())
+    copies.push_back(copy);
+  
+  return copy;
+}
+
 
 void CopyContext::update_references()
 {
   // go through everything that was copied, replacing copied object references
   // if needed
-  for (std::list<ObjectRef >::iterator iter= copies.begin(); iter != copies.end(); ++iter)
+  for (std::list<ObjectRef>::iterator iter= copies.begin(); iter != copies.end(); ++iter)
     fixup_object_copied_references(*iter, object_copies);
 }
 
 
+ValueRef CopyContext::copy_for_object(ValueRef object)
+{
+  ObjectRef obj(ObjectRef::cast_from(object));
+  if (object_copies.find(obj.id()) != object_copies.end())
+    return object_copies[obj.id()];
+  return ValueRef();
+}
+
 static ValueRef copy_value(ValueRef value, bool deep, internal::Object *owner)
 {
   switch (value.type())
@@ -999,7 +1023,7 @@ static ValueRef copy_value(ValueRef value, bool deep, internal::Object *owner)
     {
       ObjectRef obj(ObjectRef::cast_from(value));
 
-      return copy_object(obj.content().get_grt(), obj);
+      return copy_object(obj);
     }
   }
   
@@ -1072,6 +1096,81 @@ void grt::add_python_module_dir(grt::GRT *grt, const std::string &python_module_
 #ifdef ENABLE_PYTHON_MODULES
   PythonModuleLoader *loader= dynamic_cast<PythonModuleLoader *>(grt->get_module_loader("python"));
   if (loader && !python_module_path.empty())
-    loader->get_python_context()->add_module_path(python_module_path);
+    loader->get_python_context()->add_module_path(python_module_path, true);
 #endif
 }
+
+static void dump_value(const grt::ValueRef &value, int level, bool skip_spacing=false);
+
+static bool dump_member(grt::ObjectRef object, const grt::MetaClass::Member *member, int level)
+{
+  if (!object.get_member(member->name).is_valid())
+    printf("%*s%s = NULL", level, "  ", member->name.c_str());
+  else if (member->type.base.type == grt::ObjectType && !member->owned_object)
+    printf("%*s%s = <<%s>>", level, "  ", member->name.c_str(), grt::ObjectRef::cast_from(object.get_member(member->name))->get_string_member("name").c_str());
+  else
+  {
+    printf("%*s%s = ", level, "  ", member->name.c_str());
+    ::dump_value(object.get_member(member->name), level+1, true);
+  }
+  printf(";\n");
+  return true;
+}
+
+static void dump_value(const grt::ValueRef &value, int level, bool skip_spacing)
+{
+  int s = skip_spacing ? 0 : 1;
+  switch (value.type())
+  {
+    case grt::ListType:
+    {
+      grt::BaseListRef list(grt::BaseListRef::cast_from(value));
+      printf("%*s%s", level*s, "  ", "[\n");
+      for (size_t i = 0; i < list.count(); i++)
+      {
+        if (i > 0)
+          printf(",\n");
+        ::dump_value(list[i], level+1);
+      }
+      printf("%*s%s", level, "  ", "]");
+      break;
+    }
+
+    case grt::DictType:
+    {
+      grt::DictRef dict(grt::DictRef::cast_from(value));
+      printf("%*s%s", level, "  ", "  {\n");
+      for (grt::DictRef::const_iterator iter = dict.begin(); iter != dict.end(); ++iter)
+      {
+        if (iter != dict.begin())
+          printf(",\n");
+        printf("%*s%s: ", level, "  ", iter->first.c_str());
+        ::dump_value(iter->second, level+1, true);
+      }
+      printf("%*s%s", level, "  ", "}");
+      break;
+    }
+
+    case grt::ObjectType:
+    {
+      grt::ObjectRef object(grt::ObjectRef::cast_from(value));
+      grt::MetaClass *mc = object.get_metaclass();
+      printf("%*s%s", level, "  ", "  {\n");
+      mc->foreach_member(boost::bind(::dump_member, object, _1, level+1));
+      printf("%*s%s", level, "  ", "}");
+      break;
+    }
+
+    default:
+      printf("%*s%s", level, "  ", value.repr().c_str());
+      break;
+  }
+}
+
+void grt::dump_value(const grt::ValueRef &value)
+{
+  ::dump_value(value, 0);
+  printf("\n");
+}
+
+
