@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -135,7 +135,7 @@ namespace grt
         {
           Ref<O> tmp= list[i];
           
-          if (tmp.is_valid() && g_strcasecmp(tmp->get_string_member(name).c_str(), value.c_str())==0)
+          if (tmp.is_valid() && g_ascii_strcasecmp(tmp->get_string_member(name).c_str(), value.c_str())==0)
             return tmp;
         }
       }
@@ -226,7 +226,7 @@ namespace grt
   struct MYSQLGRT_PUBLIC CopyContext
   {
     GRT *grt;
-    std::map<internal::Value*, ValueRef> object_copies;
+    std::map<std::string, ValueRef> object_copies;
     std::list<ObjectRef> copies;
     
     CopyContext(GRT *agrt) : grt(agrt) {}
@@ -263,11 +263,14 @@ namespace grt
       OType copy;
 
       copy= OType::cast_from(copier.shallow_copy(object));
-      copier.finish();
+      //this is a shallow copy so changing the references in it's sub-values would mean modifying
+      //their originals
+      //copier.finish();
 
       return copy;
   }
 
+  MYSQLGRT_PUBLIC void dump_value(const grt::ValueRef &value);
  
   // temporary code
   MYSQLGRT_PUBLIC bool init_python_support(grt::GRT *grt, const std::string &python_module_path);
@@ -327,7 +330,7 @@ namespace grt
 
 
   MYSQLGRT_PUBLIC
-    boost::shared_ptr<DiffChange> diff_make(const ValueRef &source, const ValueRef &target, const Omf* omf);
+    boost::shared_ptr<DiffChange> diff_make(const ValueRef &source, const ValueRef &target, const Omf* omf, bool dont_clone_values = false);
   
 };
 
