@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -629,9 +629,12 @@ TEST_FUNCTION(25)
       {
         case 3: { refcount= 1; break; } // was replaced in list by iv[5]
         case 5: { refcount= 3; break; } // was also assigned to iv[3]
-        default: { refcount= (i>5?1:2); break; } // beggining from 6 were not used
+        default: { refcount= (i>5?1:2); break; } // beginning from 6 were not used
       }
-      tut::ensure_equals(format_msg("var refcount after misc overwriting", NULL, AnyType, i), iv[i].refcount(), refcount);
+      tut::ensure_equals(
+        format_msg("Reference counts differ", NULL, AnyType, i).c_str(),
+        iv[i].refcount(),
+        refcount);
     }
 
     exception_thrown= false;
