@@ -29,7 +29,7 @@ static inline char *my_once_memdup(const char *src, unsigned int len, int unused
 static inline char *my_malloc(unsigned int Size, int unused) { return (gptr)malloc(Size); }
 static inline void my_no_flags_free(char* ptr) { free(ptr); }
 static inline void my_free(char* ptr, int unused) { free(ptr); }
-static inline char *get_charsets_dir(char *buf);
+//static inline char *get_charsets_dir(char *buf);
 static inline int my_error(int nr, int MyFlags, ...) { printf("my_error called: %d\n", nr); return 0; }
 } // namespace mysql_parser
 
@@ -934,7 +934,7 @@ extern CHARSET_INFO *get_charset_by_name(const char *cs_name, myf flags);
 extern CHARSET_INFO *get_charset_by_csname(const char *cs_name,
 					   uint cs_flags, myf my_flags);
 extern void free_charsets(void);
-extern char *get_charsets_dir(char *buf);
+//extern char *get_charsets_dir(char *buf);
 extern my_bool my_charset_same(CHARSET_INFO *cs1, CHARSET_INFO *cs2);
 extern my_bool init_compiled_charsets(myf flags);
 extern void add_compiled_collation(CHARSET_INFO *cs);
