@@ -349,16 +349,22 @@ inline SqlAstNode * new_ast_terminal_node(LEX *lex, const char* value, int value
   }
   else
   {
-    lex->last_item= &(SqlAstStatics::last_terminal_node=
-      SqlAstTerminalNode(
+    boost::shared_ptr<SqlAstTerminalNode> node(new SqlAstTerminalNode(
         value,
         value_length,
         token_start_lineno,
         /*stmt_boffset*/(lex->tok_start - lex->buf),
         /*stmt_eoffset*/(lex->ptr - lex->buf)));
+    
+    lex->last_item = node.get();
+    SqlAstStatics::last_terminal_node(node);
+    
     if (!lex->first_item)
-      lex->first_item= &(SqlAstStatics::first_terminal_node=
-        SqlAstStatics::last_terminal_node);
+    {
+      lex->first_item = node.get();
+      SqlAstStatics::first_terminal_node(node);
+    }
+    
     free(lex_string_to_free);
     return NULL;
   }
@@ -737,7 +743,7 @@ int MYSQLlex(void **arg, void *yyl)
     case MY_LEX_OPERATOR_OR_IDENT:	// Next is operator or keyword
     case MY_LEX_START:			// Start of token
       // Skip startspace
-      for (c=yyGet() ; (state_map[c] == MY_LEX_SKIP) ; c= yyGet()) ;
+      for (c=yyGet() ; state_map[c] == MY_LEX_SKIP ; c= yyGet()) ;
       lex->tok_start=lex->ptr-1;	// Start of real token
       state= (enum my_lex_states) state_map[c];
       token_start_lineno= lex->yylineno;
@@ -788,14 +794,14 @@ int MYSQLlex(void **arg, void *yyl)
 	state= MY_LEX_IDENT;
 	break;
       }
-      yyGet();				// Skip '
+      (void) yyGet();				// Skip '
       while ((c = yyGet()) && (c !='\'')) ;
       length=(lex->ptr - lex->tok_start);	// Length of hexnum+3
       if (c != '\'')
       {
 	return(ABORT_SYM);		// Illegal hex constant
       }
-      yyGet();				// get_token makes an unget
+      (void) yyGet();				// get_token makes an unget
       tmp_lex_string= get_token(lex,length);
       new_ast_terminal_node(lex, /*tmp_lex_string.str+2, */tmp_lex_string.length-3, tmp_lex_string.str);
 #if 0
@@ -1118,14 +1124,14 @@ int MYSQLlex(void **arg, void *yyl)
       return(DECIMAL_NUM);
 
     case MY_LEX_HEX_NUMBER:		// Found x'hexstring'
-      yyGet();				// Skip '
+      (void) yyGet();				// Skip '
       while (my_isxdigit(cs,(c = yyGet()))) ;
       length=(lex->ptr - lex->tok_start);	// Length of hexnum+3
       if (!(length & 1) || c != '\'')
       {
 	return(ABORT_SYM);		// Illegal hex constant
       }
-      yyGet();				// get_token makes an unget
+      (void) yyGet();				// get_token makes an unget
       tmp_lex_string= get_token(lex,yyLength());
       new_ast_terminal_node(lex, /*tmp_lex_string.str + 2, */tmp_lex_string.length - 3, tmp_lex_string.str);
 #if 0
@@ -1137,12 +1143,12 @@ int MYSQLlex(void **arg, void *yyl)
       return (HEX_NUM);
 
     case MY_LEX_BIN_NUMBER:           // Found b'bin-string'
-      yyGet();                                // Skip '
+      (void) yyGet();                                // Skip '
       while ((c= yyGet()) == '0' || c == '1') ;
       length= (lex->ptr - lex->tok_start);    // Length of bin-num + 3
       if (c != '\'')
       return(ABORT_SYM);              // Illegal hex constant
-      yyGet();                        // get_token makes an unget
+      (void) yyGet();                        // get_token makes an unget
       tmp_lex_string= get_token(lex,yyLength());
       new_ast_terminal_node(lex, /*tmp_lex_string.str + 2, */tmp_lex_string.length - 3, tmp_lex_string.str);
 #if 0
@@ -1527,3 +1533,4 @@ bool st_lex::need_correct_ident()
 }
 
 } // namespace mysql_parser
+
