@@ -1,4 +1,19 @@
-#!/usr/bin/python
+# Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
+#
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License as
+# published by the Free Software Foundation; version 2 of the
+# License.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+# 02110-1301  USA
 
 from __future__ import with_statement
 
@@ -10,8 +25,11 @@ import socket
 import select
 import sys
 import time
+import os
 
 import paramiko
+from grt import log_error, log_warning, log_debug, log_debug2
+_this_file = os.path.basename(__file__)
 
 SSH_PORT = 22
 REMOTE_PORT = 3306
@@ -25,7 +43,8 @@ if hasattr(paramiko, "WarningPolicy"):
 else:
     class WarningPolicy(paramiko.MissingHostKeyPolicy):
         def missing_host_key(self, client, hostname, key):
-            print 'WARNING: Unknown %s host key for %s: %s' % (key.get_name(), hostname, hexlify(key.get_fingerprint()))
+            import binascii
+            log_warning(_this_file, 'WARNING: Unknown %s host key for %s: %s\n' % (key.get_name(), hostname, binascii.hexlify(key.get_fingerprint())))
 
 
 class Tunnel(threading.Thread):
@@ -76,7 +95,7 @@ class Tunnel(threading.Thread):
     
     def run(self):
         sys.stdout.write('Thread started\n')  # sys.stdout.write is thread safe while print isn't
-        
+        log_debug2(_this_file,"SSH Tunel thread started\n")
         # Create a socket and pick a random port number for it:
         self._listen_sock = socket.socket()
         while True:
@@ -92,7 +111,7 @@ class Tunnel(threading.Thread):
                     err, msg = exc.args
                     if err == 22:
                         continue # retry
-                    self.notify('ERROR', sys.exc_info())
+                    self.notify_exception_error('ERROR',"Error initializing server end of tunnel", sys.exc_info())
                     raise exc
                 finally:
                     self.port_is_set.set()
@@ -125,7 +144,7 @@ class Tunnel(threading.Thread):
                 r, w, x = select.select(socks, [], [], TUNNEL_TIMEOUT)
             except Exception, e:
                 if not self._shutdown:
-                    self.notify('ERROR', 'Error while forwarding data: %r' % e)
+                    self.notify_exception_error('ERROR', 'Error while forwarding data: %r' % e, sys.exc_info())
                 break
 
             if not r and len(socks) <= 1:
@@ -152,7 +171,7 @@ class Tunnel(threading.Thread):
                     else:
                         sock.send(data)
 
-            for item in set(closed):  # FIXME: Why a set is needed here?
+            for item in set(closed):  # set() will remove duplicates from closed list
                 sock, chan = item
                 try:
                     sock.close()
@@ -180,7 +199,12 @@ class Tunnel(threading.Thread):
         self._client.close()
         
     def notify(self, msg_type, msg_object):
+        log_debug2(_this_file, "tunnel_%i: %s %s\n" % (self.local_port, msg_type, msg_object))
         self.q.put((msg_type, msg_object))
+        
+    def notify_exception_error(self, msg_type, msg_txt, msg_obj = None):
+        self.notify(msg_type, msg_txt)
+        log_error(_this_file, traceback.format_exc())
 
     def match(self, server, username, target):
         with self.lock:
@@ -195,16 +219,37 @@ class Tunnel(threading.Thread):
         paramiko.AuthenticationException   --- raised when authentication failed for some reason
         paramiko.PasswordRequiredException --- raised when a password is needed to unlock a private key file;
                                                this is a subclass of paramiko.AuthenticationException
-        paramiko.PasswordRequiredException --- raised when a password is needed to unlock a private key file;
-                                               also a subclass of paramiko.AuthenticationException
         """
         try:
+            
             self._client = paramiko.SSHClient()
             self._client.load_system_host_keys()
             self._client.set_missing_host_key_policy(WarningPolicy())
-            self._client.connect(self._server[0], self._server[1], username=self._username, key_filename=self._keyfile, password=self._password)
-        except:
-            self.notify('ERROR', sys.exc_info())
+            has_key = bool(self._keyfile)
+            self._client.connect(self._server[0], self._server[1], username=self._username,
+                                 key_filename=self._keyfile, password=self._password,
+                                 look_for_keys=has_key, allow_agent=has_key)
+        except paramiko.BadHostKeyException, exc:
+            if sys.platform == "win32":
+                self.notify_exception_error('ERROR', "Delete entries for the host from the SSH known hosts file", sys.exc_info())
+            else:
+                self.notify_exception_error('ERROR', "Delete entries for the host from the ~/.ssh/known_hosts file", sys.exc_info())
+            return False
+        except paramiko.BadAuthenticationType, exc:
+            self.notify_exception_error('ERROR', "Bad authentication type, the server is not accepting this type of authentication.\nAllowed ones are:\n %s" % exc.allowed_types, sys.exc_info());
+            return False
+        except paramiko.AuthenticationException, exc:
+            self.notify_exception_error('ERROR', "Authentication failed, please check credentials.\nPlease refer to logs for details", sys.exc_info())
+            return False
+        except socket.gaierror, exc:
+            self.notify_exception_error('ERROR', "Error connecting to SSH server: %s\nPlease refer to logs for details." % str(exc))
+            return False
+        except paramiko.ChannelException, exc:
+            self.notify_exception_error('ERROR', "Error connecting SSH channel.\nPlease refer to logs for details: %s" % str(exc), sys.exc_info())
+            return False
+        except Exception, exc:
+            print exc
+            self.notify_exception_error('ERROR', "Authentication error, unhandled exception caught in tunnel manager, please refer to logs for details", sys.exc_info())
             return False
         else:
             return True
@@ -218,7 +263,7 @@ class Tunnel(threading.Thread):
         try:
             local_sock, peeraddr = self._listen_sock.accept()
         except Exception, e:
-            self.notify('ERROR', 'Error accepting new tunnel client: %r' % e)
+            self.notify_exception_error('ERROR', 'Error accepting new tunnel client: %r' % e,sys.exc_info())
             return
         self.notify('INFO', 'Client connection established')
 
@@ -226,13 +271,17 @@ class Tunnel(threading.Thread):
 
         try:
             sshchan = transport.open_channel('direct-tcpip', self._target, local_sock.getpeername())
+        except paramiko.ChannelException, exc:
+            self.notify_exception_error('ERROR', 'Could not open port forwarding SSH channel: %s' % exc)
+            local_sock.close()
+            return
         except Exception, e:
-            self.notify('ERROR', 'Remote connection to %s:%d failed: %r' % (self._target[0], self._target[1], e) )
+            self.notify_exception_error('ERROR', 'Remote connection to %s:%d failed: %r' % (self._target[0], self._target[1], e), sys.exc_info())
             local_sock.close()
             return
 
         if sshchan is None:
-            self.notify('ERROR', 'Remote connection to %s:%d was rejected by the SSH server.' % (self._target[0], self._target[1]))
+            self.notify_exception_error('ERROR', 'Remote connection to %s:%d was rejected by the SSH server.' % (self._target[0], self._target[1]), sys.exc_info())
             local_sock.close()
             return
 
@@ -274,10 +323,9 @@ class TunnelManager:
     def open_tunnel(self, server, username, password, keyfile, target):
         try:
             port = self.open_ssh(server, username, password, keyfile, target)
-        except Exception, exc:
-            import traceback
+        except Exception:
             traceback.print_exc()
-            return (False, str(exc))
+            return (False, str(traceback.format_exc()))
         return (True, port)
    
     def open_ssh(self, server, username, password, keyfile, target):
@@ -325,7 +373,7 @@ class TunnelManager:
                     if type(msg) is tuple:
                         msg = '\n' + ''.join(traceback.format_exception(*msg))
                         _msg = str(_msg[1])
-                    print '%s: %s' % (msg_type, msg)
+                    log_debug(_this_file, "%s: %s\n" % (msg_type, msg))
                     if msg_type == 'ERROR':
                         error = _msg
                         break  # Exit returning the error message
@@ -333,6 +381,7 @@ class TunnelManager:
                 if not tunnel.is_connecting() or not tunnel.isAlive():
                     break
                 time.sleep(0.3)
+        log_debug(_this_file, "returning from wait_connection(%s): %s\n" % (port, error))
         return error
 
 
@@ -374,7 +423,6 @@ class TunnelManager:
             try:
                 cmd, args = eval(request, {}, {})
             except:
-                print "Ignoring invalid request"
                 self.send("ERROR", "Invalid request")
                 continue
             if cmd == "LOOKUP":
@@ -409,7 +457,7 @@ class TunnelManager:
                 else:
                     self.send("NONE")
             else:
-                print "Invalid request %s" % request
+                log_error(_this_file, "Invalid request %s\n" % request)
                 self.send("ERROR", "Invalid request")
 
 """
