@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -295,21 +295,23 @@ int DbMySQLQueryImpl::openConnectionP(const db_mgmt_ConnectionRef &info, const g
   CLEAR_ERROR();
   try
   {
-    Lock lock(_mutex);
-    new_connection_id = ++_connection_id;
+    {
+      Lock lock(_mutex);
+      new_connection_id = ++_connection_id;
+    }
+    sql::ConnectionWrapper conn;
     if (password.is_valid())
     {
       sql::Authentication::Ref auth = sql::Authentication::create(info, "");
       auth->set_password(password.c_str());
 
-      sql::ConnectionWrapper conn(dm->getConnection(info, boost::shared_ptr<sql::TunnelConnection>(), auth));
-      _connections[new_connection_id] = ConnectionInfo::Ref(new ConnectionInfo(conn));
+       conn = dm->getConnection(info, boost::shared_ptr<sql::TunnelConnection>(), auth);
     }
     else
-    {
-      sql::ConnectionWrapper conn(dm->getConnection(info));
-      _connections[new_connection_id] = ConnectionInfo::Ref(new ConnectionInfo(conn));
-    }
+        conn = dm->getConnection(info);
+
+    Lock lock(_mutex);
+    _connections[new_connection_id] = ConnectionInfo::Ref(new ConnectionInfo(conn));
   }
   catch (sql::SQLException &exc)
   {
@@ -341,7 +343,6 @@ int DbMySQLQueryImpl::closeConnection(int conn)
   Lock lock(_mutex);
   if (_connections.find(conn) == _connections.end())
     throw std::invalid_argument("Invalid connection");
-
   _connections.erase(conn);
 
   return 0;
@@ -542,7 +543,7 @@ grt::IntegerRef DbMySQLQueryImpl::resultFieldIntValue(int result, int field)
   sql::ResultSet *res = _resultsets[result];
 
   if (res->isNull(field))
-    return grt::IntegerRef();
+    return grt::IntegerRef(0);
   else
     return grt::IntegerRef(res->getInt(field));
 }
@@ -581,7 +582,7 @@ grt::IntegerRef  DbMySQLQueryImpl::resultFieldIntValueByName(int result, const s
   sql::ResultSet *res = _resultsets[result];
 
   if (res->isNull(field))
-    return grt::IntegerRef();
+    return grt::IntegerRef(0);
   else
     return grt::IntegerRef(res->getInt(field));
 }
