@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -72,7 +72,7 @@ int Mysql_sql_statement_decomposer::process_sql_statement(const std::string &sql
   _do_process_sql_statement= do_process_sql_statement_cb;
   _process_sql_statement= boost::bind(&Mysql_sql_statement_decomposer::do_process_sql_statement, this, _1);
 
-  Mysql_sql_parser_fe sql_parser_fe(_grtm->get_grt());
+  Mysql_sql_parser_fe sql_parser_fe(_grtm->get_app_option_string("SqlMode"));
   sql_parser_fe.ignore_dml= false;
 
   return process_sql_statement(sql, select_statement, sql_parser_fe);
