@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "wb_config.h"
-
 #include <glib/gstdio.h>
 
 #include "wb_model.h"
@@ -65,8 +63,17 @@ DEFAULT_LOG_DOMAIN("Reporting")
 LexerDocument::LexerDocument(const std::string& text)
 : _text(text)
 {
-  _style_position= 0;
-  _style_buffer= new char[_text.size()];
+  _style_position = 0;
+  _style_buffer = new char[_text.size()];
+
+  // Split the text into lines and store the index where a line starts.
+  std::vector<std::string> lines = base::split(text, "\n");
+  int start = 0;
+  for (size_t i = 0; i < lines.size(); ++i)
+  {
+    _line_starts.push_back(start);
+    start += lines[i].size() + 1;
+  }
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -81,9 +88,7 @@ LexerDocument::~LexerDocument()
 // IDocument implementation.
 int LexerDocument::Version() const
 {
-  throw std::logic_error(std::string("Internal error. Unexpected use of unimplemented function ").
-    append(__FUNCTION__).append(" in LexerDocument (").
-    append(__FILE__).append(")."));
+  return 0; // Indicates old style lexer document.
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -133,12 +138,9 @@ int LexerDocument::LineFromPosition(int position) const
 
 int LexerDocument::LineStart(int line) const
 {
-  if (line == 0)
+  if (line < 0 || line > (int)_line_starts.size())
     return 0;
-
-  throw std::logic_error(std::string("Internal error. Unexpected use of unimplemented function ").
-    append(__FUNCTION__).append(" in LexerDocument (").
-    append(__FILE__).append(")."));
+  return _line_starts[line];
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -448,6 +450,7 @@ void fillTablePropertyDict(const db_mysql_TableRef& table, TemplateDictionary* t
   assignValueOrNA(table_dict, REPORT_TABLE_PACK_KEYS, *table->packKeys());
   table_dict->SetValue(REPORT_TABLE_HAS_PASSWORD, ((*table->password()).size() == 0) ? "no" : "yes");
   assignValueOrNA(table_dict, REPORT_TABLE_ROW_FORMAT, *table->rowFormat());
+  assignValueOrNA(table_dict, REPORT_TABLE_KEY_BLOCK_SIZE, *table->keyBlockSize());
   assignValueOrNA(table_dict, REPORT_TABLE_DATA_DIR, *table->tableDataDir());
   assignValueOrNA(table_dict, REPORT_TABLE_INDEX_DIR, *table->tableIndexDir());
   assignValueOrNA(table_dict, REPORT_TABLE_ENGINE, *table->tableEngine());
