@@ -1,5 +1,5 @@
-/* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+/*
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,6 +27,8 @@
 
 #include "base/string_utilities.h"
 
+#include "mforms/code_editor.h"
+
 using namespace bec;
 using namespace base;
 
@@ -45,9 +47,13 @@ bool MySQLTableColumnsListBE::set_field(const NodeId &node, int column, const st
     if (!col.is_valid())
       return false;
 
-    db_SimpleDatatypeRef columnType;
-    switch ((MySQLColumnListColumns)column)
+    switch ((ColumnListColumns)column)
     {
+    case Type:
+      // Remove auto increment for non integer types
+      if (is_int_datatype(value) == false)
+        col->autoIncrement(false);
+      break;
     case Default:
       // If a default value is set then auto increment for a column doesn't make sense.
       if (!base::trim(value).empty())
@@ -64,7 +70,8 @@ bool MySQLTableColumnsListBE::set_field(const NodeId &node, int column, const st
 
         return result;
       }
-
+      break;
+    default:
       break;
     }
   }
@@ -97,7 +104,7 @@ bool MySQLTableColumnsListBE::set_field(const ::bec::NodeId &node, int column, i
         // Allow removing the auto inc setting even for non-numeric columns so we can
         // switch that off *after* we changed the column type or for invalid/old models
         // which have an auto inc set for non-numeric columns.
-        if (columnType->group()->name() == "numeric" || value == 0)
+        if (is_int_datatype(columnType->name()))
         {
           AutoUndoEdit undo(_owner);
 
@@ -149,6 +156,8 @@ bool MySQLTableColumnsListBE::set_field(const ::bec::NodeId &node, int column, i
             strfmt(_("Set Auto Increment '%s.%s'"), _owner->get_name().c_str(), col->name().c_str()) : 
             strfmt(_("Unset Auto Increment '%s.%s'"), _owner->get_name().c_str(), col->name().c_str()));
         }
+        else
+          col->autoIncrement(false);
       }
       return true;
     case IsAutoIncrementable:
@@ -166,32 +175,31 @@ bool MySQLTableColumnsListBE::get_field_grt(const ::bec::NodeId &node, int colum
   {
     if (node[0] < real_count())
       col= static_cast<MySQLTableEditorBE*>(_owner)->table()->columns().get(node[0]);
-
-    switch ((MySQLColumnListColumns)column)
+    if (col.is_valid())
     {
-    case IsAutoIncrement:
-      if (col.is_valid())
+      switch (column)
+      {
+      case IsAutoIncrement:
         value= col->autoIncrement();
-      else
+        return true;
+      case IsAutoIncrementable:
+        value = grt::IntegerRef(0);
+        if (col->simpleType().is_valid() && col->simpleType()->group().is_valid())
+        {
+          if (col->simpleType()->group()->name() == "numeric")
+            value= grt::IntegerRef(1);
+        }
+        return true;
+      case HasCharset:
         value= grt::IntegerRef(0);
-      return true;
-    case IsAutoIncrementable:
-      value= grt::IntegerRef(0);
-      if (col.is_valid() && col->simpleType().is_valid() && col->simpleType()->group().is_valid())
-      {
-        if (col->simpleType()->group()->name() == "numeric")
-          value= grt::IntegerRef(1);
-      }
-      return true;
-    case HasCharset:
-      value= grt::IntegerRef(0);
-      if (col.is_valid() && col->simpleType().is_valid())
-      {
-        if (col->simpleType()->group()->name() == "string" || col->simpleType()->group()->name() == "text"
+        if (col->simpleType().is_valid())
+        {
+          if (col->simpleType()->group()->name() == "string" || col->simpleType()->group()->name() == "text"
             || col->simpleType()->name() == "ENUM")
-          value= grt::IntegerRef(1);
+            value= grt::IntegerRef(1);
+        }
+        return true;          
       }
-      return true;          
     }
   }
   return TableColumnsListBE::get_field_grt(node, column, value);
@@ -227,30 +235,44 @@ bec::MenuItemList MySQLTableColumnsListBE::get_popup_items_for_nodes(const std::
     if (col.is_valid() && col->simpleType().is_valid())
     {
       std::string type = col->simpleType()->name();
+      bool improved_timestamp_support = false;
+      GrtVersionRef target_version = _owner->get_rdbms_target_version();
+      // in MySQL 5.6, CURRENT_TIMESTAMP works for TIMESTAMP and DATETIME
+      // and for any number of colums
+      if (target_version.is_valid() &&
+         (*target_version->majorNumber() > 5 ||
+            (*target_version->majorNumber() == 5 &&
+             *target_version->minorNumber() >= 6)))
+          improved_timestamp_support = true;
       
-      if (type == "TIMESTAMP")
+      if (type == "TIMESTAMP" || (improved_timestamp_support && type == "DATETIME"))
       {
         bool seen_current_ts = false;
-        bool is_first_ts = false; // only the 1st TIMESTAMP column can have CURRENT_TIMESTAMP, unless all 
+        bool current_timestamp_allowed = false; // only the 1st TIMESTAMP column can have CURRENT_TIMESTAMP, unless all
         // previous ones are set to 0 or a constant
         bool flag = false;
         
-        GRTLIST_FOREACH(db_Column, columns, c)
+        if (improved_timestamp_support)
+          current_timestamp_allowed = true;
+        else
         {
-          if ((*c)->simpleType().is_valid() && (*c)->simpleType()->name() == "TIMESTAMP")
+          GRTLIST_FOREACH(db_Column, columns, c)
           {
-            if (*c == col)
+            if ((*c)->simpleType().is_valid() && (*c)->simpleType()->name() == "TIMESTAMP")
             {
-              is_first_ts= !seen_current_ts;
-              flag = true;
-            }
-            
-            if (!((*c)->defaultValue() == "0" || can_be_timestamp((*c)->defaultValue().c_str())))
-              seen_current_ts = true;
+              if (*c == col)
+              {
+                current_timestamp_allowed= !seen_current_ts;
+                flag = true;
+              }
+              
+              if (!((*c)->defaultValue() == "0" || can_be_timestamp((*c)->defaultValue().c_str())))
+                seen_current_ts = true;
 
-            // a column before some other TS column that is already marked as CURRENT_TIMESTAMP cannot become CURRENT_TS
-            if (*c != col && flag && strstr((*c)->defaultValue().c_str(), "TIMESTAMP"))
-              is_first_ts = false;
+              // a column before some other TS column that is already marked as CURRENT_TIMESTAMP cannot become CURRENT_TS
+              if (*c != col && flag && strstr((*c)->defaultValue().c_str(), "TIMESTAMP"))
+                current_timestamp_allowed = false;
+            }
           }
         }
 
@@ -261,13 +283,18 @@ bec::MenuItemList MySQLTableColumnsListBE::get_popup_items_for_nodes(const std::
 
         item.caption = "Default CURRENT_TIMESTAMP";
         item.name    = "currentTSToolStripMenuItem";
-        item.enabled = is_first_ts;
+        item.enabled = current_timestamp_allowed;
         items.push_back(item);
-        
+
+        item.caption = "Default NULL ON UPDATE CURRENT_TIMESTAMP";
+        item.name    = "currentTSNullOnUpdateToolStripMenuItem";
+        item.enabled = current_timestamp_allowed;
+        items.push_back(item);
+
         item.caption = "Default CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP";
         item.name    = "currentTSOnUpdateToolStripMenuItem";
-        item.enabled = is_first_ts;
-        items.push_back(item);        
+        item.enabled = current_timestamp_allowed;
+        items.push_back(item);
       }
       else if (col->simpleType()->group()->name() == "numeric" ||
                col->simpleType()->group()->name() == "datetime")
@@ -303,6 +330,8 @@ bool MySQLTableColumnsListBE::activate_popup_item_for_nodes(const std::string &n
     value = "''";
   else if (name == "currentTSToolStripMenuItem")
     value = "CURRENT_TIMESTAMP";
+  else if (name == "currentTSNullOnUpdateToolStripMenuItem")
+    value = "NULL ON UPDATE CURRENT_TIMESTAMP";
   else if (name == "currentTSOnUpdateToolStripMenuItem")
     value = "CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP";
 
@@ -334,12 +363,351 @@ bool MySQLTableColumnsListBE::activate_popup_item_for_nodes(const std::string &n
   return TableColumnsListBE::activate_popup_item_for_nodes(name, orig_nodes);
 }  
 
-//--------------------------------------------------------------------------------
 
+//--------------------------------------------------------------------------------------------------
 
-MySQLTableEditorBE::MySQLTableEditorBE(::bec::GRTManager *grtm, const db_mysql_TableRef &table, const db_mgmt_RdbmsRef &rdbms)
-  : TableEditorBE(grtm, table, rdbms), _table(table), _columns(this), _partitions(this), _indexes(this)
+#include <mforms/table.h>
+#include <mforms/treenodeview.h>
+#include <mforms/textentry.h>
+#include <mforms/button.h>
+#include <mforms/box.h>
+#include <mforms/label.h>
+#include <mforms/toolbar.h>
+
+class MySQLTriggerPanel : public mforms::Box
 {
+public:
+  MySQLTriggerPanel(MySQLTableEditorBE *editor)
+  : mforms::Box(true), _editor(editor), _list(mforms::TreeFlatList|mforms::TreeSizeSmall), _button(mforms::SmallButton), _refreshing(false)
+  {
+    set_spacing(8);
+    set_padding(4);
+    
+    _list.set_size(230, -1);
+    _list.set_name("triggers list");
+    add(&_list, false, true);
+    _rtable.set_name("trigger right pane");
+    add(&_rtable, true, true);
+    
+    _rtable.set_row_count(2);
+    _rtable.set_column_count(5);
+    
+    _button.set_text("Delete Trigger");
+    _rtable.set_row_spacing(8);
+    _rtable.set_column_spacing(6);
+    
+    _name.set_name("trigger name");
+    _name.signal_changed()->connect(boost::bind(&MySQLTriggerPanel::name_changed, this));
+    _definer.set_name("trigger definer");
+    _definer.signal_changed()->connect(boost::bind(&MySQLTriggerPanel::definer_changed, this));
+
+    _namel.set_style(mforms::SmallStyle);
+    _definerl.set_style(mforms::SmallStyle);
+    _namel.set_text("Trigger Name:");
+    _definerl.set_text("Definer:");
+    _rtable.add(&_namel, 0, 1, 0, 1, mforms::HFillFlag);
+    _rtable.add(&_name, 1, 2, 0, 1, mforms::HExpandFlag|mforms::HFillFlag);
+    _rtable.add(&_definerl, 2, 3, 0, 1, mforms::HFillFlag);
+    _rtable.add(&_definer, 3, 4, 0, 1, mforms::HExpandFlag|mforms::HFillFlag);
+    _rtable.add(&_button, 4, 5, 0, 1, mforms::HFillFlag);
+
+    _list.add_column(mforms::StringColumnType, _("Name"), 110, false);
+    _list.add_column(mforms::StringColumnType, _("Timing/Event"), 100, false);
+    _list.end_columns();
+    _list.signal_changed()->connect(boost::bind(&MySQLTriggerPanel::selection_changed, this));
+
+    _rtable.add(editor->get_sql_editor()->get_container(), 0, 5, 1, 2,
+                mforms::HExpandFlag|mforms::VExpandFlag|mforms::HFillFlag|mforms::VFillFlag);
+    editor->get_sql_editor()->set_sql_check_enabled(true);
+    editor->get_sql_editor()->get_editor_control()->set_language(mforms::LanguageMySQL);
+    
+    _button.signal_clicked()->connect(boost::bind(&MySQLTriggerPanel::clicked, this));
+    _code_editor = _editor->get_sql_editor()->get_editor_control();
+    _code_editor->signal_lost_focus()->connect(boost::bind(&MySQLTriggerPanel::code_edited, this));
+    refresh();
+    selection_changed();
+  }
+
+  ~MySQLTriggerPanel()
+  {
+    _rtable.remove(_editor->get_sql_editor()->get_container());
+  }
+
+  static bool compare_order(db_TriggerRef a, db_TriggerRef b)
+  {
+    return a->order() > b->order();
+  }
+  
+  void refresh()
+  {
+    _refreshing = true;
+
+    std::set<std::string> leftover;
+    leftover.insert("BEFORE INSERT");
+    leftover.insert("AFTER INSERT");
+    leftover.insert("BEFORE UPDATE");
+    leftover.insert("AFTER UPDATE");
+    leftover.insert("BEFORE DELETE");
+    leftover.insert("AFTER DELETE");
+
+    mforms::TreeNodeRef selected = _list.get_selected_node();
+    int old_selected = 0;
+    if (selected)
+      old_selected = _list.row_for_node(selected);
+    
+    _list.clear();
+
+    grt::ListRef<db_Trigger> triggers(_editor->get_table()->triggers());
+    std::map<std::string, db_TriggerRef> trigmap;
+    GRTLIST_FOREACH(db_Trigger, triggers, trig)
+    {
+      std::string t = (*trig)->timing();
+      t.append(" ").append((*trig)->event());
+      trigmap.insert(std::make_pair(t, *trig));
+    }
+//    std::sort(trigvec.begin(), trigvec.end(), &MySQLTriggerPanel::compare_order);
+
+    mforms::TreeNodeRef node;
+    std::map<std::string, db_TriggerRef>::iterator it;
+    for (std::set<std::string>::const_iterator t = leftover.begin(); t != leftover.end(); ++t)
+    {
+      node = _list.add_node();
+      if ((it = trigmap.find(*t)) != trigmap.end())
+        node->set_string(0, it->second->name());
+      else
+        node->set_string(0, "-");
+
+      node->set_string(1, *t);
+    }
+
+    _list.select_node(_list.node_at_row(old_selected));
+
+    _refreshing = false;
+  }
+  
+  void code_edited()
+  {
+    // Don't run commit_changes() or the parser will kill your current trigger if it has an error.
+    if (_selected_trigger.is_valid())
+    {
+      if (_code_editor->is_dirty() && _selected_trigger->sqlDefinition() != _code_editor->get_string_value())
+      {
+        AutoUndoEdit undo(_editor, _selected_trigger, "sql");
+        grt::IntegerRef res = _editor->_sql_parser->parse_trigger(_selected_trigger, _code_editor->get_string_value().c_str());
+        _name.set_value(_selected_trigger->name());
+        _definer.set_value(_selected_trigger->definer());
+        undo.end(strfmt(_("Edit trigger `%s` of `%s`.`%s`"), _selected_trigger->name().c_str(), _editor->get_schema_name().c_str(), _editor->get_name().c_str()));
+      }
+    }
+    else
+    {
+      _editor->commit_changes();
+      refresh();
+    }
+  }
+  
+  void name_changed()
+  {
+    if (_selected_trigger.is_valid())
+    {
+      AutoUndoEdit undo(_editor, _selected_trigger, "name");
+      _selected_trigger->name(_name.get_string_value());
+      undo.end(base::strfmt("Rename trigger to %s", _name.get_string_value().c_str()));
+      mforms::TreeNodeRef node = _list.get_selected_node();
+      if (node)
+        node->set_string(0, _name.get_string_value());
+      selection_changed();
+    }
+  }
+
+  void definer_changed()
+  {
+    if (_selected_trigger.is_valid())
+    {
+      AutoUndoEdit undo(_editor, _selected_trigger, "definer");
+      _selected_trigger->definer(_definer.get_string_value());
+      undo.end(base::strfmt("Change trigger definer to %s", _definer.get_string_value().c_str()));
+      selection_changed();
+    }
+  }
+
+  void clicked()
+  {
+    std::string timing, event;
+    mforms::TreeNodeRef node = _list.get_selected_node();
+    
+    if (base::partition(node->get_string(1), " ", timing, event))
+    {
+      grt::ListRef<db_Trigger> triggers(_editor->get_table()->triggers());
+      db_TriggerRef trigger;
+      GRTLIST_FOREACH(db_Trigger, triggers, trig)
+      {
+        if ((*trig)->timing() == timing && (*trig)->event() == event)
+        {
+          trigger = *trig;
+          break;
+        }
+      }
+      
+      if (trigger.is_valid())
+      {
+        _editor->freeze_refresh_on_object_change();
+        AutoUndoEdit undo(_editor);
+        triggers.remove_value(trigger);
+        undo.end(base::strfmt("Delete trigger %s", trigger->name().c_str()));
+        
+        node->set_string(0, "-");
+        _editor->thaw_refresh_on_object_change(true);
+      }
+      else
+      {
+        _editor->freeze_refresh_on_object_change();
+        AutoUndoEdit undo(_editor);
+        trigger = db_mysql_TriggerRef(_editor->get_grt());
+        trigger->owner(_editor->get_table());
+        trigger->name(base::strfmt("%s_%c%s", _editor->get_name().c_str(),
+                                   timing[0], event.substr(0, 3).c_str()));
+        trigger->event(event);
+        trigger->timing(timing);
+        triggers.insert(trigger);
+        undo.end(base::strfmt("Added trigger to %s.%s", _editor->get_schema_name().c_str(), _editor->get_name().c_str()));
+        node->set_string(0, trigger->name());
+        _editor->thaw_refresh_on_object_change(true);
+      }
+      update_editor();
+    }
+  }
+  
+  void selection_changed()
+  {
+    if (_refreshing)
+      return;
+    if (_code_editor->is_dirty())
+      code_edited();
+
+    update_editor();
+  }
+
+
+  void update_editor()
+  {
+    mforms::TreeNodeRef node = _list.get_selected_node();
+    if (!node)
+    {
+      _list.select_node(_list.node_at_row(0));
+      return;
+    }
+
+    std::string timing, event;
+    std::string sql;
+    bool editor_enabled = true;
+    base::partition(node->get_string(1), " ", timing, event);
+    grt::ListRef<db_Trigger> triggers(_editor->get_table()->triggers());
+    db_TriggerRef trigger;
+    GRTLIST_FOREACH(db_Trigger, triggers, trig)
+    {
+      if ((*trig)->timing() == timing && (*trig)->event() == event)
+      {
+        trigger = *trig;
+        break;
+      }
+    }
+
+    bool empty_trigger = false;
+    if (_selected_trigger != trigger)
+    {
+      _selected_trigger = trigger;
+
+      if (trigger.is_valid())
+      {
+        _button.set_text("Delete Trigger");
+        _name.set_value(trigger->name());
+        _definer.set_value(trigger->definer());
+        _name.set_enabled(true);
+        _definer.set_enabled(true);
+      
+        sql.append(base::strfmt("USE `%s`;\nDELIMITER $$\n", _editor->get_schema_name().c_str()));
+        if (trigger->sqlDefinition().empty())
+        {
+          sql.append(base::strfmt("CREATE TRIGGER `%s` %s %s ON `%s` FOR EACH ROW\n", trigger->name().c_str(),
+            timing.c_str(), event.c_str(), _editor->get_name().c_str()));
+          
+          // No longer necessary. We can now parse the entire sql (if the user types complete nonsense
+          // we are all lost however, but that's no news).
+          //sql.append("-- Edit trigger body code below this line. Do not edit lines above this one\n");
+        }
+        else
+          sql.append(trigger->sqlDefinition());
+
+        _code_editor->set_text_keeping_state(sql.c_str());
+      }
+      else
+      {
+        empty_trigger = true;
+      }
+    }
+
+    if (empty_trigger || !_selected_trigger.is_valid())
+    {
+      _name.set_value("");
+      _definer.set_value("");
+      _name.set_enabled(false);
+      _definer.set_enabled(false);
+      _button.set_text("Add Trigger");
+      editor_enabled = false;
+
+      sql = "-- Trigger not defined. Click Add Trigger to create it.\n";
+      _code_editor->set_text_keeping_state(sql.c_str());
+    }
+    _button.set_enabled(true);
+    _code_editor->reset_dirty();
+    _code_editor->set_enabled(editor_enabled);
+  }
+  
+private:
+  MySQLTableEditorBE *_editor;
+  mforms::Table _rtable;
+  mforms::TreeNodeView _list;
+  mforms::TextEntry _name;
+  mforms::TextEntry _definer;
+  mforms::Label _namel;
+  mforms::Label _definerl;
+  mforms::Button _button;
+  mforms::CodeEditor *_code_editor;
+  db_TriggerRef _selected_trigger;
+  bool _refreshing;
+};
+
+//--------------------------------------------------------------------------------------------------
+
+MySQLTableEditorBE::MySQLTableEditorBE(::bec::GRTManager *grtm, db_mysql_TableRef table, const db_mgmt_RdbmsRef &rdbms)
+  : TableEditorBE(grtm, table, rdbms), _table(table), _columns(this), _partitions(this), _indexes(this), _trigger_panel(0)
+{
+  _updating_triggers = false;
+  if (_table->isStub() == 1)
+  {
+    int rc;
+    rc = mforms::Utilities::show_warning(
+      _("Edit Stub Table"),
+      _("The table you are trying to edit is a model-only stub, created to represent missing external tables referenced by foreign keys.\n"
+        "Such tables are ignored by forward engineering and synchronization.\n\n"
+        "You may convert this table to a real one that appears also in the generated SQL or keep it as stub."),
+                                         _("Convert to real table"), _("Edit as is"));
+    if (rc == mforms::ResultOk)
+      db_mysql_TableRef(table)->isStub(0);
+  }
+}
+
+MySQLTableEditorBE::~MySQLTableEditorBE()
+{
+  delete _trigger_panel;
+}
+
+mforms::View *MySQLTableEditorBE::get_trigger_panel()
+{
+  if (!_trigger_panel)
+    _trigger_panel = new MySQLTriggerPanel(this);
+  return _trigger_panel;
 }
 
 
@@ -349,8 +717,13 @@ std::vector<std::string> MySQLTableEditorBE::get_index_types()
 
   index_types.push_back("INDEX");
   index_types.push_back("UNIQUE");
-  index_types.push_back("FULLTEXT");
-  index_types.push_back("SPATIAL");
+  // FULLTEXT exists only in MyISAM prior to 5.6. in 5.6+ InnoDB also supports it
+  if (_table->tableEngine() == "MyISAM" || (_table->tableEngine() == "InnoDB" && is_server_version_at_least(5, 6)))
+    index_types.push_back("FULLTEXT");
+  // SPATIAL is not supported by InnoDB
+  if (_table->tableEngine() == "MyISAM")
+    index_types.push_back("SPATIAL");
+
   // these are special types for PK and FK
   index_types.push_back("PRIMARY");
 //  index_types.push_back("FOREIGN");
@@ -361,9 +734,11 @@ std::vector<std::string> MySQLTableEditorBE::get_index_storage_types()
 {
   std::vector<std::string> index_types;
 
-  index_types.push_back("BTREE");
-  index_types.push_back("RTREE");
-  index_types.push_back("HASH");
+  index_types.push_back("BTREE"); // BTREE is supported by all engines
+  if (_table->tableEngine() == "MyISAM")
+    index_types.push_back("RTREE"); // as of 5.7, RTREE is recognized by parser but not supported
+  if (_table->tableEngine() == "MEMORY" || _table->tableEngine() == "HEAP" || _table->tableEngine() == "ndbcluster")
+    index_types.push_back("HASH");
   
   return index_types;
 }
@@ -423,6 +798,7 @@ static struct TableOption
   {"AUTO_INCREMENT",  "nextAutoInc", true},
   {"DELAY_KEY_WRITE", "delayKeyWrite", false},
   {"ROW_FORMAT",      "rowFormat", true},
+  {"KEY_BLOCK_SIZE",  "keyBlockSize", true},
   {"AVG_ROW_LENGTH",  "avgRowLength", true},
   {"MAX_ROWS",        "maxRows", true},
   {"MIN_ROWS",        "minRows", true},
@@ -524,6 +900,8 @@ std::string MySQLTableEditorBE::get_table_option_by_name(const std::string& name
     return _table->delayKeyWrite().repr();
   else if(name.compare("ROW_FORMAT") == 0)
     return _table->rowFormat();
+  else if(name.compare("KEY_BLOCK_SIZE") == 0)
+    return _table->keyBlockSize();
   else if(name.compare("AVG_ROW_LENGTH") == 0)
     return _table->avgRowLength();
   else if(name.compare("MAX_ROWS") == 0)
@@ -558,7 +936,10 @@ std::string MySQLTableEditorBE::get_all_triggers_sql() const
 {
   std::string retval;
 
-  retval.append("-- Trigger DDL Statements\n").append(
+  retval.append("-- Full Trigger DDL Statements\n");
+  retval.append("-- Note: Only CREATE TRIGGER statements are allowed\n");
+
+  retval.append(
     strfmt("DELIMITER %s\n\n", _non_std_sql_delimiter.c_str())).append(
     "USE `").append(_table->owner()->name()).append("`").
     append(_non_std_sql_delimiter.c_str()).append("\n\n");
@@ -590,8 +971,47 @@ std::string MySQLTableEditorBE::get_all_triggers_sql() const
   return retval;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Loads the current trigger sql text into the editor control and marks that as not dirty.
+ */
+void MySQLTableEditorBE::load_trigger_sql()
+{
+  if (_trigger_panel && !_updating_triggers)
+  {
+    _updating_triggers = true;
+    _trigger_panel->refresh();
+    _trigger_panel->update_editor();
+    _updating_triggers = false;
+  }
+}
 
 
+//--------------------------------------------------------------------------------------------------
+
+bool MySQLTableEditorBE::can_close()
+{
+  _trigger_panel->code_edited(); // Same handling as for focus-lost. Might not be necessary but better safe than sorry.
+  return TableEditorBE::can_close();
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void MySQLTableEditorBE::commit_changes()
+{
+  mforms::CodeEditor* editor = get_sql_editor()->get_editor_control();
+  if (editor->is_dirty())
+  {
+    // TODO: no longer needed since we don't set all triggers at once anymore.
+    set_triggers_sql(get_all_triggers_sql(), true); //we need to always get all editors code not just the currently visible
+    // Note: don't reset the dirty state of the editor to be able to undo beyond this commit.
+  }
+
+  // TODO: anything else to store?
+}
+
+//--------------------------------------------------------------------------------------------------
 
 bool MySQLTableEditorBE::set_partition_type(const std::string &type)
 {
@@ -675,6 +1095,11 @@ int MySQLTableEditorBE::get_partition_count()
 }
 
 
+bool MySQLTableEditorBE::subpartition_count_allowed()
+{
+  return (*table()->partitionType() == "RANGE" || *table()->partitionType() == "LIST");
+}
+
 bool MySQLTableEditorBE::set_subpartition_type(const std::string &type)
 {
   if (*table()->partitionType() == "RANGE" || *table()->partitionType() == "LIST")
@@ -843,6 +1268,36 @@ void MySQLTableEditorBE::reset_partition_definitions(int parts, int subparts)
   undo.end("Reset Partitioning");
 }
 
+
+db_TableRef MySQLTableEditorBE::create_stub_table(const std::string &schema, const std::string &table)
+{
+  db_SchemaRef dbschema = grt::find_named_object_in_list(get_catalog()->schemata(), schema, false);
+  db_TableRef dbtable;
+
+  if (dbschema.is_valid())
+    dbtable = grt::find_named_object_in_list(dbschema->tables(), table);
+  else
+  {
+    dbschema = db_mysql_SchemaRef(get_grt());
+    dbschema->owner(get_catalog());
+    dbschema->name(schema);
+    dbschema->comment("This schema was created for a stub table");
+    get_catalog()->schemata().insert(dbschema);
+  }
+
+  if (!dbtable.is_valid())
+  {
+    dbtable = db_mysql_TableRef(get_grt());
+    dbtable->owner(dbschema);
+    dbtable->name(table);
+    dbtable->isStub(1);
+    dbschema->tables().insert(dbtable);
+  }
+
+  return dbtable;
+}
+
+
 static db_SimpleDatatypeRef get_simple_datatype(const db_ColumnRef &column)
 {
   if (column->simpleType().is_valid())
@@ -892,7 +1347,6 @@ bool MySQLTableEditorBE::check_column_referenceable_by_fk(const db_ColumnRef &co
 
 //--------------------------------------------------------------------------------
 
-
 MySQLTablePartitionTreeBE::MySQLTablePartitionTreeBE(MySQLTableEditorBE *owner)
 : _owner(owner)
 {
