@@ -1,4 +1,19 @@
-#include "wb_config.h"
+/* 
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * 
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+
 #include "linux_utilities/plugin_editor_base.h"
 #include "../backend/mysql_routine_editor.h"
 #include "grtdb/db_object_helpers.h"
@@ -18,7 +33,6 @@ class DbMySQLRoutineEditor : public PluginEditorBase
   
   virtual bool can_close() { return _be->can_close(); }
 
-  void set_comment(const std::string &s);
  public:
   DbMySQLRoutineEditor(grt::Module *m, bec::GRTManager *grtm, const grt::BaseListRef &args);
   
@@ -26,12 +40,15 @@ class DbMySQLRoutineEditor : public PluginEditorBase
   virtual void do_refresh_form_data();
 
   virtual bool switch_edited_object(bec::GRTManager *grtm, const grt::BaseListRef &args);
+
+  bool comment_lost_focus(GdkEventFocus *ev, Gtk::TextView *view);
 };
 
 DbMySQLRoutineEditor::DbMySQLRoutineEditor(grt::Module *m, bec::GRTManager *grtm, const grt::BaseListRef &args)
     : PluginEditorBase(m, grtm, args, "modules/data/editor_routine.glade")
     , _be(new MySQLRoutineEditorBE(grtm, db_mysql_RoutineRef::cast_from(args[0]), get_rdbms_for_db_object(args[0])))
 {
+
   xml()->get_widget("mysql_routine_editor_notebook", _editor_notebook);
 
   Gtk::Image *image;
@@ -47,7 +64,7 @@ DbMySQLRoutineEditor::DbMySQLRoutineEditor(grt::Module *m, bec::GRTManager *grtm
 
   Gtk::VBox *ddl_win;
   xml()->get_widget("routine_ddl", ddl_win);
-  embed_code_editor(_be->get_sql_editor()->get_editor_control(), ddl_win);
+  embed_code_editor(_be->get_sql_editor()->get_container(), ddl_win);
   _be->load_routine_sql();
 
   if (!is_editing_live_object())
@@ -57,7 +74,9 @@ DbMySQLRoutineEditor::DbMySQLRoutineEditor(grt::Module *m, bec::GRTManager *grtm
 
     Gtk::TextView *tview(0);
     xml()->get_widget("comment", tview);
-    add_text_change_timer(tview, sigc::mem_fun(this, &DbMySQLRoutineEditor::set_comment));
+    tview->get_buffer()->set_text(_be->get_comment());
+
+    tview->signal_focus_out_event().connect(sigc::bind(sigc::mem_fun(this, &DbMySQLRoutineEditor::comment_lost_focus), tview), false);
   }
   else
   {
@@ -67,6 +86,8 @@ DbMySQLRoutineEditor::DbMySQLRoutineEditor(grt::Module *m, bec::GRTManager *grtm
 
   refresh_form_data();
   
+  _be->reset_editor_undo_stack();
+
   show_all();
 }
 
@@ -77,11 +98,12 @@ DbMySQLRoutineEditor::~DbMySQLRoutineEditor()
   delete _be;
 }
 
-//------------------------------------------------------------------------------
-void DbMySQLRoutineEditor::set_comment(const std::string &s)
-{
-  if (_be)
-    _be->set_comment(s);
+bool DbMySQLRoutineEditor::comment_lost_focus(GdkEventFocus *ev, Gtk::TextView *view) {
+  if(_be)
+  {
+    _be->set_comment(view->get_buffer()->get_text());
+  }
+  return false;
 }
 
 //------------------------------------------------------------------------------
@@ -94,7 +116,7 @@ bool DbMySQLRoutineEditor::switch_edited_object(bec::GRTManager *grtm, const grt
 
   _be = new MySQLRoutineEditorBE(grtm, db_mysql_RoutineRef::cast_from(args[0]), get_rdbms_for_db_object(args[0]));
 
-  embed_code_editor(_be->get_sql_editor()->get_editor_control(), ddl_win);
+  embed_code_editor(_be->get_sql_editor()->get_container(), ddl_win);
   _be->load_routine_sql();
 
   if (!_be->is_editing_live_object())
