@@ -21,7 +21,7 @@ Db_frw_eng::Db_frw_eng(bec::GRTManager *grtm)
 {
   {
     workbench_DocumentRef doc= workbench_DocumentRef::cast_from(grtm->get_grt()->get("/wb/doc"));
-    Db_frw_eng::grtm(grtm);
+    Db_frw_eng::grtm(grtm, false);
   }
 
   _catalog= db_mysql_CatalogRef::cast_from(_grtm->get_grt()->get("/wb/doc/physicalModels/0/catalog"));
