@@ -1,3 +1,22 @@
+/* 
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
+
 #include "stdafx.h"
 
 #include "grtdb/db_object_helpers.h"
@@ -7,14 +26,14 @@
 #include "grts/structs.db.mysql.h"
 
 #include "grtpp.h"
+#include "grt/grt_manager.h"
 #include "grt/grt_reporter.h"
 #include "grtdb/diff_dbobjectmatch.h"
+#include "base/log.h"
 
 using namespace grt;
 
-#include "grti/sqlgenerator.h"
-//QQQ#include "grti/wbvalidation.h"
-//QQQ#include "grti/wbvalidationmysql.h"
+#include "interfaces/sqlgenerator.h"
 
 #include "db_mysql_sql_export.h"
 #include "diff_tree.h"
@@ -23,45 +42,33 @@ using namespace grt;
 #include "db.mysql/src/module_db_mysql_shared_code.h"
 #include "grtsqlparser/sql_facade.h"
 
-void DbMySQLSQLExport::init_from_ctor(bec::GRTManager *grtm, db_mysql_CatalogRef catalog)
-{
-  //_manager= grtm;
+DEFAULT_LOG_DOMAIN("DbMySQLSQLExport");
 
+DbMySQLSQLExport::DbMySQLSQLExport(bec::GRTManager *grtm, db_mysql_CatalogRef catalog)
+  : DbMySQLValidationPage(grtm)
+{
   _tables_are_selected= true;
   _triggers_are_selected= true;
   _routines_are_selected= true;
   _views_are_selected= true;
   _users_are_selected= true;
   _catalog= catalog;
+  _case_sensitive = true;
 
   if(!_catalog.is_valid())
     _catalog= get_model_catalog();  // call own version
 
-  _users_model= new bec::GrtStringListModel();
-  _users_exc_model= new bec::GrtStringListModel();
+  _users_model= boost::shared_ptr<bec::GrtStringListModel>(new bec::GrtStringListModel());
+  _users_exc_model= boost::shared_ptr<bec::GrtStringListModel>(new bec::GrtStringListModel());
  
-  _tables_model= new bec::GrtStringListModel();
-  _tables_exc_model= new bec::GrtStringListModel();
-  _views_model= new bec::GrtStringListModel();
-  _views_exc_model= new bec::GrtStringListModel();
-  _routines_model= new bec::GrtStringListModel();
-  _routines_exc_model= new bec::GrtStringListModel();
-  _triggers_model= new bec::GrtStringListModel();
-  _triggers_exc_model= new bec::GrtStringListModel();
-}
-
-
-DbMySQLSQLExport::DbMySQLSQLExport(bec::GRTManager *grtm, db_mysql_CatalogRef catalog)
-  : DbMySQLValidationPage(grtm)
-{
-  init_from_ctor(grtm, catalog);
-}
-
-DbMySQLSQLExport::DbMySQLSQLExport(bec::GRTManager *grtm)
-  : DbMySQLValidationPage(grtm)
-//  : messages_list(NULL)
-{
-  init_from_ctor(grtm, db_mysql_CatalogRef());
+  _tables_model= boost::shared_ptr<bec::GrtStringListModel>(new bec::GrtStringListModel());
+  _tables_exc_model= boost::shared_ptr<bec::GrtStringListModel>(new bec::GrtStringListModel());
+  _views_model= boost::shared_ptr<bec::GrtStringListModel>(new bec::GrtStringListModel());
+  _views_exc_model= boost::shared_ptr<bec::GrtStringListModel>(new bec::GrtStringListModel());
+  _routines_model= boost::shared_ptr<bec::GrtStringListModel>(new bec::GrtStringListModel());
+  _routines_exc_model= boost::shared_ptr<bec::GrtStringListModel>(new bec::GrtStringListModel());
+  _triggers_model= boost::shared_ptr<bec::GrtStringListModel>(new bec::GrtStringListModel());
+  _triggers_exc_model= boost::shared_ptr<bec::GrtStringListModel>(new bec::GrtStringListModel());
 }
 
 db_mysql_CatalogRef DbMySQLSQLExport::get_model_catalog()
@@ -79,10 +86,16 @@ void DbMySQLSQLExport::set_option(const std::string& name, bool value)
     _gen_warnings= value;
   else if(name.compare("GenerateCreateIndex") == 0)
     _gen_create_index= value;
-  else if(name.compare("NoUsersJustPrivileges") == 0)
+  else if(name.compare("NoViewPlaceholders") == 0)
     _no_users_just_privileges=value;
+  else if(name.compare("NoUsersJustPrivileges") == 0)
+    _no_view_placeholders=value;
   else if(name.compare("GenerateInserts") == 0)
     _gen_inserts=value;
+  else if(name.compare("NoFKForInserts") == 0)
+    _no_FK_for_inserts=value;
+  else if(name.compare("TriggersAfterInserts") == 0)
+    _triggers_after_inserts=value;
   else if(name.compare("TablesAreSelected") == 0)
     _tables_are_selected= value;
   else if(name.compare("TriggersAreSelected") == 0)
@@ -119,15 +132,15 @@ grt::StringListRef convert_string_vector_to_grt_list(grt::GRT *grt, const std::v
   return grt_list;
 }
 
-static void parse_string_from_filter(const std::string& source, 
-                                     std::string& out_prefix,
-                                     std::string& out_suffix)
-{
-  std::string::size_type period= source.find('.');
-  out_prefix.assign(source.substr(0, period));
-  if(period != std::string::npos)
-    out_suffix.assign(source.substr(period+1));
-}
+//static void parse_string_from_filter(const std::string& source, 
+//                                     std::string& out_prefix,
+//                                     std::string& out_suffix)
+//{
+//  std::string::size_type period= source.find('.');
+//  out_prefix.assign(source.substr(0, period));
+//  if(period != std::string::npos)
+//    out_suffix.assign(source.substr(period+1));
+//}
 
 typedef std::map<std::string, std::list<std::string> > StringListMap;
 
@@ -161,7 +174,10 @@ grt::DictRef DbMySQLSQLExport::get_options_as_dict(grt::GRT *grt)
   options.set("GenerateWarnings", grt::IntegerRef(_gen_warnings ? 1 : 0));
   options.set("GenerateCreateIndex", grt::IntegerRef(_gen_create_index ? 1 : 0));
   options.set("NoUsersJustPrivileges", grt::IntegerRef(_no_users_just_privileges ? 1 : 0));
+  options.set("NoViewPlaceholders", grt::IntegerRef(_no_users_just_privileges ? 1 : 0));
   options.set("GenerateInserts", grt::IntegerRef(_gen_inserts ? 1 : 0));
+  options.set("NoFKForInserts", grt::IntegerRef(_no_FK_for_inserts ? 1 : 0));
+  options.set("TriggersAfterInserts", grt::IntegerRef(_triggers_after_inserts ? 1 : 0));
   options.set("UseShortNames", grt::IntegerRef(_omit_schemata ? 1 : 0));
   options.set("GenerateUse", grt::IntegerRef(_generate_use ? 1 : 0));
   options.set("SkipForeignKeys", grt::IntegerRef(_skip_foreign_keys ? 1 : 0));
@@ -179,19 +195,19 @@ grt::DictRef DbMySQLSQLExport::get_options_as_dict(grt::GRT *grt)
 
   // filtering options
   options.set("TableFilterList", _tables_are_selected ? 
-    convert_string_vector_to_grt_list(grt,get_names(_tables_model,_tables_map, schemas, _case_sensitive)) : grt::StringListRef(grt));
+    convert_string_vector_to_grt_list(grt,get_names(_tables_model.get(), _tables_map, schemas, _case_sensitive)) : grt::StringListRef(grt));
 
   options.set("ViewFilterList", _views_are_selected ? 
-    convert_string_vector_to_grt_list(grt, get_names(_views_model,_views_map, schemas, _case_sensitive)) : grt::StringListRef(grt));
+    convert_string_vector_to_grt_list(grt, get_names(_views_model.get(), _views_map, schemas, _case_sensitive)) : grt::StringListRef(grt));
 
   options.set("RoutineFilterList", _routines_are_selected ? 
-    convert_string_vector_to_grt_list(grt, get_names(_routines_model,_routines_map, schemas, _case_sensitive)) : grt::StringListRef(grt));
+    convert_string_vector_to_grt_list(grt, get_names(_routines_model.get(), _routines_map, schemas, _case_sensitive)) : grt::StringListRef(grt));
 
   options.set("TriggerFilterList", _triggers_are_selected ? 
-    convert_string_vector_to_grt_list(grt, get_names(_triggers_model,_triggers_map, schemas, _case_sensitive)) : grt::StringListRef(grt));
+    convert_string_vector_to_grt_list(grt, get_names(_triggers_model.get(), _triggers_map, schemas, _case_sensitive)) : grt::StringListRef(grt));
 
   options.set("UserFilterList", _users_are_selected ? 
-    convert_string_vector_to_grt_list(grt, get_names(_users_model,_users_map, schemas, _case_sensitive)) : grt::StringListRef(grt));
+      convert_string_vector_to_grt_list(grt, get_names(_users_model.get(), _users_map, schemas, _case_sensitive)) : grt::StringListRef(grt));
 
   grt::StringListRef schema_names_list(grt);
   for (std::set<db_mysql_SchemaRef>::const_iterator It = schemas.begin(); It != schemas.end(); ++It)
@@ -199,6 +215,8 @@ grt::DictRef DbMySQLSQLExport::get_options_as_dict(grt::GRT *grt)
   
   options.set("SchemaFilterList", schema_names_list);
 
+  options.set("CaseSensitive", grt::IntegerRef(_case_sensitive));
+
   return options;
 }
 
@@ -232,55 +250,48 @@ ValueRef DbMySQLSQLExport::export_task(grt::GRT* grt, grt::StringRef)
 
   try
   {
-    SQLGeneratorInterfaceWrapper *diffsql_module= 
-      //grt->get_module_wrapper<SQLGeneratorInterfaceWrapper>(grt->get_module("MySQLModuleDbMySQL"));
-      grt->get_module_wrapper<SQLGeneratorInterfaceWrapper>(grt->get_module("DbMySQL"));
+    SQLGeneratorInterfaceImpl *diffsql_module= dynamic_cast<SQLGeneratorInterfaceImpl*>(grt->get_module("DbMySQL"));
   
     if(diffsql_module == NULL)
-      return grt::StringRef("\nSQL Script Export Error: Not able to load 'MySQLModuleDbMySQL' module");
+      return grt::StringRef("\nSQL Script Export Error: Not able to load 'DbMySQL' module");
 
-    grt::ValueRef e(NULL);
-
-    default_omf omf;
-    grt::NormalizedComparer normalizer(grt);
-    normalizer.init_omf(&omf);
-    boost::shared_ptr<DiffChange> create_change= diff_make(e, _catalog, &omf);
-    boost::shared_ptr<DiffChange> drop_change= diff_make(_catalog, e, &omf);
-    DictRef create_map(grt);
-    DictRef drop_map(grt);
-
-    char sprintf_buf[32];
-    
-    std::string change_ptr;
-    std::string log_ptr;
-
-    sprintf(sprintf_buf, "%p", create_change.get());
-    change_ptr.assign(sprintf_buf);
+    DictRef create_map;
+    DictRef drop_map;
 
     grt::DictRef options= get_options_as_dict(grt);
 
-    options.set("OutputContainer", create_map);
+    options.set("SQL_MODE", _manager->get_app_option("SqlGenerator.Mysql:SQL_MODE"));
+    options.gset("UseFilteredLists", 1);
 
-    diffsql_module->generateSQL(GrtNamedObjectRef::cast_from(ValueRef(NULL)), options, change_ptr);
-    //delete create_change;
-
-    sprintf(sprintf_buf, "%p", drop_change.get());
-    change_ptr.assign(sprintf_buf);
-
-    if(_gen_drops)
+    if(_db_options.is_valid())
     {
-      options.set("OutputContainer", drop_map);
-      diffsql_module->generateSQL(GrtNamedObjectRef::cast_from(_catalog), options, change_ptr);
-      //delete drop_change;
+        _db_options.set("CaseSensitive", grt::IntegerRef(1));
+        options.set("DBSettings",_db_options);
+    }else
+    {
+        grt::DictRef dboptions = diffsql_module->getDefaultTraits();
+        dboptions.set("CaseSensitive", grt::IntegerRef(1));
+        options.set("DBSettings", dboptions );
     }
+    //grt::DictRef::cast_from(options.get("DBSettings", getDefaultTraits())),
+    create_map = diffsql_module->generateSQLForDifferences(GrtNamedObjectRef(), _catalog, options);
 
+    if (_gen_drops)
+      drop_map = diffsql_module->generateSQLForDifferences(_catalog, GrtNamedObjectRef(), options);
+    if (!drop_map.is_valid())
+      drop_map = grt::DictRef(grt);
+    
     grt::StringListRef strlist= grt::StringListRef::cast_from(options.get("ViewFilterList"));
 
-    if(diffsql_module->makeSQLExportScript(_catalog, options, create_map, drop_map))
+    // generateSQLForDifferences() will set DiffCaseSensitiveness to the used value
+    _case_sensitive = options.get_int("DiffCaseSensitiveness", _case_sensitive);
+    options.set("CaseSensitive", grt::IntegerRef(_case_sensitive));
+
+    if (diffsql_module->makeSQLExportScript(_catalog, options, create_map, drop_map))
     {
       return grt::StringRef("\nSQL Script Export Error: SQL Script Export Module Returned Error");
     }
-
+    
     _export_sql_script= options.get_string("OutputScript");
     if (!_output_filename.empty())
     {
@@ -385,11 +396,11 @@ void DbMySQLSQLExport::setup_grt_string_list_models_from_catalog(bec::GrtStringL
   _routines_exc_model->reset(empty_list);
   _triggers_exc_model->reset(empty_list);
 
-  _users_model->items_val_masks(_users_exc_model);
-  _tables_model->items_val_masks(_tables_exc_model);
-  _views_model->items_val_masks(_views_exc_model);
-  _routines_model->items_val_masks(_routines_exc_model);
-  _triggers_model->items_val_masks(_triggers_exc_model);
+  _users_model->items_val_masks(_users_exc_model.get());
+  _tables_model->items_val_masks(_tables_exc_model.get());
+  _views_model->items_val_masks(_views_exc_model.get());
+  _routines_model->items_val_masks(_routines_exc_model.get());
+  _triggers_model->items_val_masks(_triggers_exc_model.get());
 
   _users_model->reset(users_list);
   _tables_model->reset(tables_list);  
@@ -397,16 +408,16 @@ void DbMySQLSQLExport::setup_grt_string_list_models_from_catalog(bec::GrtStringL
   _routines_model->reset(routines_list);
   _triggers_model->reset(triggers_list);
 
-  *users_model= _users_model;
-  *users_exc_model= _users_exc_model;
-  *tables_model= _tables_model;
-  *tables_exc_model= _tables_exc_model;
-  *views_model= _views_model;
-  *views_exc_model= _views_exc_model;
-  *routines_model= _routines_model;
-  *routines_exc_model= _routines_exc_model;
-  *triggers_model= _triggers_model;
-  *triggers_exc_model= _triggers_exc_model;
+  *users_model= _users_model.get();
+  *users_exc_model= _users_exc_model.get();
+  *tables_model= _tables_model.get();
+  *tables_exc_model= _tables_exc_model.get();
+  *views_model= _views_model.get();
+  *views_exc_model= _views_exc_model.get();
+  *routines_model= _routines_model.get();
+  *routines_exc_model= _routines_exc_model.get();
+  *triggers_model= _triggers_model.get();
+  *triggers_exc_model= _triggers_exc_model.get();
 }
 
 //void DbMySQLSQLExport::validation_finished(grt::ValueRef res)
