@@ -16,12 +16,11 @@ using namespace grt;
 #include "db_mysql_sql_sync.h"
 
 #include "diff/diffchange.h"
-#include "grti/sqlgenerator.h"
 
 DbMySQLSync::DbMySQLSync(bec::GRTManager *grtm)
   : Db_plugin(), DbMySQLValidationPage(grtm)
 {
-  DbMySQLSync::grtm(grtm);
+  DbMySQLSync::grtm(grtm, false);
   _catalog= db_mysql_CatalogRef::cast_from(_grtm->get_grt()->get("/wb/doc/physicalModels/0/catalog"));
 }
 
