@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,8 +19,6 @@
 
 #include "stdafx.h"
 
-#include "wb_config.h"
-
 #include <ostream>
 #include <sstream>
 #include <memory>
@@ -30,10 +28,11 @@
 #include "grt/icon_manager.h"
 #include "grts/structs.db.h"
 #include "base/string_utilities.h"
+#include "db.mysql/src/module_db_mysql.h"
 
 #include <glib.h>
 
-void Db_plugin::grtm(bec::GRTManager *grtm)
+void Db_plugin::grtm(bec::GRTManager *grtm, bool reveng)
 {
   Wb_plugin::grtm(grtm);
 
@@ -44,7 +43,8 @@ void Db_plugin::grtm(bec::GRTManager *grtm)
     _doc= workbench_DocumentRef::cast_from(_grtm->get_grt()->get("/wb/doc"));
 
     db_mgmt_ManagementRef mgmt= workbench_WorkbenchRef::cast_from(_doc->owner())->rdbmsMgmt();
-    _db_conn.init(mgmt, true);
+    // don't need schema box for reverse engineer, but need it for fwd/sync (in case OmitQualifiers is on)
+    _db_conn = new DbConnection(mgmt, db_mgmt_DriverRef(), reveng ? true : false); 
 
     _tables.icon_id(table_icon_id(bec::Icon16));
     _views.icon_id(view_icon_id(bec::Icon16));
@@ -63,15 +63,14 @@ std::string Db_plugin::task_desc()
 }
 
 
-void Db_plugin::model_catalog(db_CatalogRef catalog)
+db_mgmt_RdbmsRef Db_plugin::selected_rdbms()
 {
-  _catalog= catalog;
+  return db_mgmt_RdbmsRef::cast_from(_db_conn->get_connection()->driver()->owner());
 }
 
-
 db_CatalogRef Db_plugin::model_catalog()
 {
-  db_mgmt_RdbmsRef rdbms= _db_conn.get_rdbms();
+  db_mgmt_RdbmsRef rdbms= selected_rdbms();
 
   // find first appropriate model catalog for selected rdbms
   grt::ListRef<workbench_physical_Model> physicalModels= _doc->physicalModels();
@@ -129,28 +128,38 @@ std::string Db_plugin::db_objects_struct_name_by_type(Db_object_type db_object_t
   return obj.get_metaclass()->get_member_type(attr_name).content.object_class;
 }
 
-
 void Db_plugin::load_schemata(std::vector<std::string> &schemata)
 {
   _schemata.clear();
   _schemata_ddl.clear();
 
-  sql::ConnectionWrapper dbc_conn= _db_conn.get_dbc_connection();
+  sql::ConnectionWrapper dbc_conn= _db_conn->get_dbc_connection();
   sql::DatabaseMetaData *dbc_meta(dbc_conn->getMetaData());
 
   _grtm->get_grt()->send_info(_("Fetching schema list."));
   _grtm->get_grt()->send_progress(0.0, _("Fetching schema list..."));
 
+  const unsigned int major = dbc_meta->getDatabaseMajorVersion();
+  const unsigned int minor = dbc_meta->getDatabaseMinorVersion();
+  const unsigned int revision = dbc_meta->getDatabasePatchVersion();
+
+  DbMySQLImpl *diffsql_module= _grtm->get_grt()->find_native_module<DbMySQLImpl>("DbMySQL");
+  _db_options = diffsql_module->getTraitsForServerVersion(major, minor, revision);
+  _db_options.set("CaseSensitive", grt::IntegerRef(dbc_meta->storesMixedCaseIdentifiers()));
+
   std::auto_ptr<sql::ResultSet> rset(dbc_meta->getSchemaObjects("", "", "schema"));
   _schemata.reserve(rset->rowsCount());
   float total= (float)rset->rowsCount();
   int current= 0;
   while (rset->next())
   {
-    _schemata.push_back(rset->getString("name"));
-    _schemata_ddl[rset->getString("name")]= rset->getString("ddl");
-
-    _grtm->get_grt()->send_progress(current++/total, rset->getString("name"), "");
+    std::string name = rset->getString("name");
+    if (name != "mysql" && name != "information_schema" && name != "performance_schema")
+    {
+      _schemata.push_back(name);
+      _schemata_ddl[name]= rset->getString("ddl");
+    }
+    _grtm->get_grt()->send_progress(current++/total, name, "");
   }
 
   _grtm->get_grt()->send_progress(1.0, _("Fetch finished."));
@@ -159,26 +168,26 @@ void Db_plugin::load_schemata(std::vector<std::string> &schemata)
   schemata= _schemata;
 }
 
-
-void Db_plugin::default_schemata_selection(std::vector<std::string> &selection)
-{
-  grt::ListRef<db_Schema> model_schemata= model_catalog()->schemata();
-
-  for (grt::ListRef<db_Schema>::const_iterator It= model_schemata.begin(); It != model_schemata.end(); ++It)
-  {
-    selection.push_back((*It)->name());  
-  }
-  return;
-/*
-//select only schemas thet exists both in  DB and model
-  for (size_t n= 0, count= _schemata.size(); n < count; ++n)
-  {
-    db_SchemaRef model_schema= grt::find_named_object_in_list(model_schemata, _schemata[n].c_str());
-    if (model_schema.is_valid())
-      selection.push_back(_schemata[n]);
-  }
-  */
-}
+//
+//void Db_plugin::default_schemata_selection(std::vector<std::string> &selection)
+//{
+//  grt::ListRef<db_Schema> model_schemata= model_catalog()->schemata();
+//
+//  for (grt::ListRef<db_Schema>::const_iterator It= model_schemata.begin(); It != model_schemata.end(); ++It)
+//  {
+//    selection.push_back((*It)->name());  
+//  }
+//  return;
+///*
+////select only schemas thet exists both in  DB and model
+//  for (size_t n= 0, count= _schemata.size(); n < count; ++n)
+//  {
+//    db_SchemaRef model_schema= grt::find_named_object_in_list(model_schemata, _schemata[n].c_str());
+//    if (model_schema.is_valid())
+//      selection.push_back(_schemata[n]);
+//  }
+//  */
+//}
 
 
 void Db_plugin::schemata_selection(const std::vector<std::string> &selection, bool sel_none_means_sel_all)
@@ -199,7 +208,7 @@ void Db_plugin::load_db_objects(Db_object_type db_object_type)
   
   _grtm->get_grt()->send_progress(0.0, std::string("Fetching ").append(db_objects_type_to_string(db_object_type)).append(" list."));
   
-  sql::ConnectionWrapper dbc_conn= _db_conn.get_dbc_connection();
+  sql::ConnectionWrapper dbc_conn= _db_conn->get_dbc_connection();
   sql::DatabaseMetaData *dbc_meta(dbc_conn->getMetaData());
   std::string db_object_type_name= db_objects_type_to_string(db_object_type);
   std::list<Db_obj_handle> db_objects;
@@ -218,23 +227,26 @@ void Db_plugin::load_db_objects(Db_object_type db_object_type)
     _grtm->get_grt()->send_progress((current_schema / total_schemas), 
                                     std::string("Fetch ").append(db_objects_type_to_string(db_object_type)).append(" objects from ").append(schema_name));
 
-    std::auto_ptr<sql::ResultSet> rset(dbc_meta->getSchemaObjects("", schema_name, db_object_type_name));
-    total_objects= (float)rset->rowsCount();
-    while (rset->next())
+    if (!schema_name.empty())
     {
-      Db_obj_handle db_obj;
-      db_obj.schema= schema_name;
-      db_obj.name= rset->getString("name");
-      db_obj.ddl= rset->getString("ddl");
-      setup->all.push_back(db_obj);
-
-      // prefixed by schema name
-      db_obj_names.push_back(std::string(schema_name).append(".").append(db_obj.name));
-
-      _grtm->get_grt()->send_progress((current_schema / total_schemas) + (count / total_objects)/total_schemas,
-                                      db_obj_names.back());
-      
-      count++;
+      std::auto_ptr<sql::ResultSet> rset(dbc_meta->getSchemaObjects("", schema_name, db_object_type_name));
+      total_objects= (float)rset->rowsCount();
+      while (rset->next())
+      {
+        Db_obj_handle db_obj;
+        db_obj.schema= schema_name;
+        db_obj.name= rset->getString("name");
+        db_obj.ddl= rset->getString("ddl");
+        setup->all.push_back(db_obj);
+        
+        // prefixed by schema name
+        db_obj_names.push_back(std::string(schema_name).append(".").append(db_obj.name));
+        
+        _grtm->get_grt()->send_progress((current_schema / total_schemas) + (count / total_objects)/total_schemas,
+                                        db_obj_names.back());
+        
+        count++;
+      }
     }
 
     current_schema++;
@@ -255,6 +267,78 @@ void Db_plugin::load_db_objects(Db_object_type db_object_type)
 }
 
 
+/** read_back_view_ddl()
+ 
+ Load back VIEW code from the database for everything we created, so that we know how it was normalized.
+
+ This will go through the whole catalog and read the SQL definition for each view. This is needed because
+ the server stores the view definition in a canonical form, which is quite different from what the user
+ types (not only formatting is stripped, but object names are expanded and expressions are rewritten).
+ That makes it impossible for us to synchronize by comparing the definition in the model with the
+ definition in the server. So we need to store the server version of the view right after we create it in
+ there and a snapshot of the view that was used to create that view. 
+ 
+ During synchronization, we can detect changes in the model by comparing the model version of the snapshot with 
+ the model SQL definition and detect server changes by comparing the server version of the definition with 
+ the snapshot for the server. We need to do that separately for every target server that synchronization is 
+ used with (using the db.SyncProfile object).
+ */
+void Db_plugin::read_back_view_ddl()
+{
+  Db_objects_setup *setup= db_objects_setup_by_type(dbotView);
+  setup->reset();
+
+  _grtm->get_grt()->send_info(std::string("Fetching back view definitions in final form."));
+
+  _grtm->get_grt()->send_progress(0.0, std::string("Fetching back view definitions in final form."));
+
+  sql::ConnectionWrapper dbc_conn= _db_conn->get_dbc_connection();
+  sql::DatabaseMetaData *dbc_meta(dbc_conn->getMetaData());
+  std::list<Db_obj_handle> db_objects;
+
+  float total_views = 0;
+
+  for (size_t sc = model_catalog()->schemata().count(), s = 0; s < sc; ++s)
+  {
+    db_SchemaRef schema(model_catalog()->schemata()[s]);
+    total_views += schema->views().count();
+  }
+  if (total_views == 0)
+  {
+    _grtm->get_grt()->send_progress(1.0, "Finished.");
+    _grtm->get_grt()->send_info("Nothing to fetch");
+    return;
+  }
+
+  int current_view = 0;
+  for (size_t sc = model_catalog()->schemata().count(), s = 0; s < sc; ++s)
+  {
+    db_SchemaRef schema(model_catalog()->schemata()[s]);
+    for (size_t vc = schema->views().count(), v = 0; v < vc; ++v)
+    {
+      db_ViewRef view(schema->views()[v]);
+
+      _grtm->get_grt()->send_progress((current_view / total_views),
+                                      std::string("Fetch back database view code for ").append(schema->name()).append(".").append(view->name()));
+      std::auto_ptr<sql::ResultSet> rset(dbc_meta->getSchemaObjects("", *schema->name(), "view", true, *view->name()));
+
+      // take a snapshot of the server version of the SQL
+      if (rset->next())
+        view->oldServerSqlDefinition(grt::StringRef(rset->getString("ddl")));
+      else
+        _grtm->get_grt()->send_info(base::strfmt("Could not get definition for %s.%s from server", schema->name().c_str(), view->name().c_str()));
+
+      // take a snapshot of the model version of the SQL
+      view->oldModelSqlDefinition(view->sqlDefinition());
+
+      current_view++;
+    }
+  }
+  _grtm->get_grt()->send_progress(1.0, "Finished.");
+  _grtm->get_grt()->send_info(base::strfmt("%i views were read back.", current_view));
+}
+
+
 bool Db_plugin::validate_db_objects_selection(std::list<std::string> *messages)
 {
   // check if there are selected triggers without selected owner table
@@ -305,7 +389,7 @@ void Db_plugin::dump_ddl(Db_object_type db_object_type, std::string &sql_script)
 {
   std::string _non_std_sql_delimiter;
   {
-    SqlFacade::Ref sql_facade= SqlFacade::instance_for_rdbms(_db_conn.get_rdbms());
+    SqlFacade::Ref sql_facade= SqlFacade::instance_for_rdbms(selected_rdbms());
     Sql_specifics::Ref sql_specifics= sql_facade->sqlSpecifics();
     _non_std_sql_delimiter= sql_specifics->non_std_sql_delimiter();
   }
@@ -388,7 +472,9 @@ db_CatalogRef Db_plugin::db_catalog()
   catalog->oldName(catalog->name());
 
   SqlFacade::Ref sql_parser= SqlFacade::instance_for_rdbms(pm->rdbms());
-  sql_parser->parseSqlScriptString(catalog, sql_input_script);
+  grt::DictRef parse_options(_grtm->get_grt());
+  parse_options.set("case_sensitive_identifiers", _db_options.get("CaseSensitive", grt::IntegerRef(1)));
+  sql_parser->parseSqlScriptStringEx(catalog, sql_input_script, parse_options);
 
   return catalog;
 }
@@ -408,7 +494,7 @@ grt::StringRef Db_plugin::apply_script_to_db(grt::GRT *grt)
   grt->send_info(_("Executing SQL script in server"));
   
   std::list<std::string> statements;
-  SqlFacade::Ref sql_splitter= SqlFacade::instance_for_rdbms(_db_conn.get_rdbms());
+  SqlFacade::Ref sql_splitter= SqlFacade::instance_for_rdbms(selected_rdbms());
   sql_splitter->splitSqlScript(_sql_script, statements);
 
   sql::SqlBatchExec sql_batch_exec;
@@ -426,7 +512,7 @@ grt::StringRef Db_plugin::apply_script_to_db(grt::GRT *grt)
 int Db_plugin::process_sql_script_error(long long err_no, const std::string &err_msg, const std::string &statement)
 {
   std::ostringstream oss;
-  oss << _("Error ") << err_no << _(": ") << err_msg << std::endl << statement << std::endl;
+  oss << _("Error ") << err_no << ": " << err_msg << std::endl << statement << std::endl;
   _grtm->get_grt()->send_error(oss.str());
   return 0;
 }
@@ -434,7 +520,7 @@ int Db_plugin::process_sql_script_error(long long err_no, const std::string &err
 
 int Db_plugin::process_sql_script_progress(float progress_state)
 {
-  _grtm->get_grt()->send_progress(progress_state, _(""));
+  _grtm->get_grt()->send_progress(progress_state, "");
   return 0;
 }
 
@@ -444,7 +530,7 @@ int Db_plugin::process_sql_script_statistics(long success_count, long err_count)
   std::ostringstream oss;
   oss << _("SQL script execution finished: statements: ") << success_count << _(" succeeded, ")
     << err_count << _(" failed") << std::endl;
-  _grtm->get_grt()->send_progress(1.f, _(""));
+  _grtm->get_grt()->send_progress(1.f, "");
   _grtm->get_grt()->send_info(oss.str());
   return 0;
 }
