@@ -1,5 +1,26 @@
+/* 
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
+
 #include "stdafx.h"
 
+#include "base/log.h"
+
 #include "diff/diffchange.h"
 #include "diff/grtdiff.h"
 #include "diff/changeobjects.h"
@@ -12,6 +33,9 @@
 #include <set>
 
 #include "grtdb/catalog_templates.h"
+#include "module_db_mysql_shared_code.h"
+
+DEFAULT_LOG_DOMAIN("difftree");
 
 //using namespace wb;
 using namespace bec;
@@ -62,36 +86,91 @@ namespace
   struct CompareName
   {
     std::string name;
+    const bool _case_sensitive;
 
-    CompareName(const std::string& n)
-      : name(utf_to_upper(n.c_str())) {}
+    CompareName(const std::string& n, const bool case_sensitive)
+      : name(n.c_str()),_case_sensitive(case_sensitive) {}
 
     bool operator () (const DiffNode *node) 
     { 
-      if(!node->get_db_part().is_valid_object())
-        return false;
-
+      if(node->get_db_part().is_valid_object())
+      {
       std::string node_name(
-        utf_to_upper(node->get_db_part().get_name().c_str()));
+        get_old_object_name_for_key(node->get_db_part().get_object(), _case_sensitive));
 
-      return (name.compare(node_name) == 0);     }
+      return (name.compare(node_name) == 0);
+      }
+      else if(node->get_model_part().is_valid_object())
+      {
+      std::string node_name(
+        get_old_object_name_for_key(node->get_model_part().get_object(), _case_sensitive));
+
+      return (name.compare(node_name) == 0);
+      }
+      return false;
+    }
   };
 }
 
+
+void DiffNode::dump(int depth)
+{
+  const char *dir = NULL;
+  switch (applyDirection)
+  {
+    case ApplyToModel: dir = "model->";
+    case ApplyToDb: dir = "<-db";
+    case DontApply: dir = "ignore";
+    case CantApply: dir = "n/a";
+  }
+
+  log_debug("%*s: %s: %s | %s | %s\n", depth, "-", change ? change->get_type_name().c_str() : "nil",
+            db_part.is_valid_object() ? db_part.get_name().c_str() : "nil",
+            dir,
+            model_part.is_valid_object() ? model_part.get_name().c_str() : "nil");
+  for (DiffNodeVector::iterator It = children.begin();  It != children.end(); ++It)
+    (*It)->dump(depth+1);
+}
+
+
 DiffNode *DiffNode::find_child_by_db_part_name(const std::string& name)
 {
 
-  if(this == NULL)
-    throw std::logic_error("invalid object name");
+    if(this == NULL)
+        throw std::logic_error("invalid object name");
 
-  DiffNodeVector::const_iterator b= children.begin();
-  DiffNodeVector::const_iterator e= children.end();
-  DiffNodeVector::const_iterator it= std::find_if(b, e, CompareName(name));
-  if(it == e)
-    return NULL;
-  return *it;
+    DiffNodeVector::const_iterator b= children.begin();
+    DiffNodeVector::const_iterator e= children.end();
+    DiffNodeVector::const_iterator it= std::find_if(b, e, CompareName(name, true));
+    if(it == e)
+    {
+        it= std::find_if(b, e, CompareName(base::toupper(name), false));
+        if(it == e)
+            return NULL;
+    }    
+    return *it;
 }
 
+DiffNode *DiffNode::find_node_for_object(const grt::ObjectRef obj)
+{
+    if(get_db_part().is_valid_object())
+    {
+        if(get_db_part().get_object()->id() == obj->id())
+            return this;    
+    }else if(get_model_part().is_valid_object())
+    {
+        if(get_model_part().get_object()->id() == obj->id())
+            return this;    
+    }
+    for(DiffNodeVector::const_iterator It = children.begin(); It != children.end(); ++It)
+    {
+        DiffNode* node = (*It)->find_node_for_object(obj);
+        if(node)
+            return node;
+    }
+    return NULL;
+};
+
 DiffNodeController::DiffNodeController()
 {
   _directions_map[DiffNode::ApplyToModel] = DiffNode::ApplyToDb;
@@ -104,19 +183,19 @@ DiffNodeController::DiffNodeController(const std::map<DiffNode::ApplicationDirec
 
 };
 
-void DiffNodeController::set_next_apply_direction(DiffNode *node)const
+void DiffNodeController::set_next_apply_direction(DiffNode *node) const
 {
   std::map<DiffNode::ApplicationDirection,DiffNode::ApplicationDirection>::const_iterator newdir = _directions_map.find(node->apply_direction());
   node->apply_direction(newdir == _directions_map.end()?DiffNode::DontApply:newdir->second);
 };
 
-void DiffNodeController::set_apply_direction(DiffNode *node, DiffNode::ApplicationDirection dir, bool recursive)const
+void DiffNodeController::set_apply_direction(DiffNode *node, DiffNode::ApplicationDirection dir, bool recursive) const
 {
   node->apply_direction(dir);
 
   if (recursive)
     for(DiffNode::DiffNodeVector::const_iterator It = node->get_children_begin(); It != node->get_children_end(); ++It)
-      set_apply_direction(*It,dir,recursive);
+      set_apply_direction(*It, dir, recursive);
 };
 
 
@@ -196,8 +275,9 @@ void DiffNode::get_object_list_to_apply_to_model(std::vector<grt::ValueRef>& vec
   std::for_each(b, e, GetObjectListToApplyToModel(vec,removal_vec));
 }
 
-void DiffNode::set_modified_and_update_dir(bool m)
+void DiffNode::set_modified_and_update_dir(bool m, boost::shared_ptr<DiffChange> c)
 {
+  change = c;
   modified= m;
   applyDirection= m ? ApplyToDb : CantApply;
 }
@@ -207,7 +287,7 @@ std::string get_old_name_or_name(GrtNamedObjectRef obj)
   if(!obj.is_valid())
     return "";
 
-  if(strlen(obj->oldName().c_str()))
+  if(strlen(obj->oldName().c_str()) && !db_mysql_SchemaRef::can_wrap(obj))
     return std::string(obj->oldName().c_str());
   return std::string(obj->name().c_str());
 }
@@ -359,233 +439,121 @@ void DiffTreeBE::fill_tree(DiffNode *root, db_mysql_CatalogRef model_catalog, co
   }
 }
 
-void DiffTreeBE::build_diff_item_list(const DiffChange* diffchange, DiffTreeBE::DiffItemList& list, DiffTreeBE::ModifiedObjectStack& workstack)
+bool is_node_object(const grt::ValueRef v)
 {
-  switch(diffchange->get_change_type())
-  {
-  case SimpleValue:
-    {
-      if(!workstack.empty())
-        list.push_back(DiffItem(workstack.top(), DiffTreeBE::DiffItemModified));
-    }
-    break;
-
-  case grt::ListItemAdded:
-    {
-      grt::ValueRef v= static_cast<const grt::ListItemAddedChange *>(diffchange)->get_value();
-
-      bool dont_push= 
-        (db_SchemaRef::can_wrap(v) || db_TableRef::can_wrap(v) 
-        || db_ViewRef::can_wrap(v) || db_RoutineRef::can_wrap(v) 
-        || db_TriggerRef::can_wrap(v));
-
-      if(!dont_push && !workstack.empty())
-        list.push_back(DiffItem(workstack.top(), DiffTreeBE::DiffItemModified));
-    }
-    break;
-
-  case grt::ListItemRemoved:
-    list.push_back(
-      DiffItem(
-        grt::ValueRef(static_cast<const grt::ListItemRemovedChange *>(diffchange)->get_value()), 
-        DiffTreeBE::DiffItemRemoved));
-    break;
-
-  case grt::ObjectModified:
-    {
-      for(grt::ChangeSet::const_iterator e= diffchange->subchanges()->end(), it= diffchange->subchanges()->begin(); it != e; it++)
-          build_diff_item_list(it->get(), list, workstack);
-    }
-    break;
-  
-  case grt::ObjectAttrModified:
-    build_diff_item_list(static_cast<const grt::ObjectAttrModifiedChange *>
-      (diffchange)->get_subchange(), list, workstack);
-    break;
-
-  case grt::ListModified:
-    {
-      for(grt::ChangeSet::const_iterator e= diffchange->subchanges()->end(), 
-        it= diffchange->subchanges()->begin(); it != e; it++)
-        build_diff_item_list(it->get(), list, workstack);
-    }
-    break;
-
-  case grt::ListItemModified:
-    {
-      grt::ValueRef v(static_cast<const grt::ListItemModifiedChange *>
-        (diffchange)->get_old_value());
-
-      bool push= 
-        (db_SchemaRef::can_wrap(v) || db_TableRef::can_wrap(v) 
-        || db_ViewRef::can_wrap(v) || db_RoutineRef::can_wrap(v) 
-        || db_TriggerRef::can_wrap(v));
-
-      if(push)
-        workstack.push(v);
-      build_diff_item_list((diffchange)->subchanges()->begin()->get(), list, workstack);
-      if(push)
-        workstack.pop();
-    }
-    break;
-
-  case grt::ListItemOrderChanged:
-    {
-      const grt::ListItemOrderChange *oc= 
-        static_cast<const grt::ListItemOrderChange *>(diffchange);
-      grt::ValueRef v(oc->get_old_value());
-      if(db_ColumnRef::can_wrap(v) || db_IndexColumnRef::can_wrap(v))
-        list.push_back(DiffItem(workstack.top(), DiffTreeBE::DiffItemModified));
-      // the list item order change action can have a nested
-      // object-modified action
-      else if(db_TableRef::can_wrap(v) && !oc->subchanges()->empty())
-        list.push_back(DiffItem(v, DiffTreeBE::DiffItemModified));
-      else if(db_SchemaRef::can_wrap(v) && !oc->subchanges()->empty())
-        build_diff_item_list(oc->subchanges()->begin()->get(), list, workstack);
-      else if(oc->get_subchange())
-          build_diff_item_list(oc->get_subchange(), list, workstack);
-    }
-    break;
-
-  default:
-    break;
-  }
+  return (db_SchemaRef::can_wrap(v) || db_TableRef::can_wrap(v)
+          || db_ViewRef::can_wrap(v) || db_RoutineRef::can_wrap(v)
+          || db_TriggerRef::can_wrap(v));
 }
 
-void DiffTreeBE::update_tree_from_diff_item_list(DiffNode *root, const DiffTreeBE::DiffItemList& worklist)
+bool DiffTreeBE::update_tree_with_changes(const boost::shared_ptr<DiffChange> diffchange)
 {
-  for(DiffTreeBE::DiffItemList::const_iterator e= worklist.end(), it= worklist.begin(); it != e; it++)
-  {
-    grt::ValueRef value= it->object;
+    if(!diffchange)
+        return false;
 
-    if(db_mysql_SchemaRef::can_wrap(value))
+    switch(diffchange->get_change_type())
     {
-      db_mysql_SchemaRef schema= db_mysql_SchemaRef::cast_from(value);
-      DiffNode *schema_node= NULL;
-      //if(it->action == DiffItemAdded)
-      if(it->action == DiffItemRemoved)
-      {
-        schema_node= new DiffNode(db_mysql_SchemaRef(), schema, false);
-        root->append(schema_node);
-        fill_tree(schema_node, schema, CatalogMap(), true);
-      }
-      else if(it->action == DiffItemModified)
-      {
-        schema_node= root->find_child_by_db_part_name(std::string(schema->oldName().c_str()));
-        schema_node->set_modified_and_update_dir(true);
-      }
-    }
-    else if(db_mysql_TableRef::can_wrap(value))
+    case SimpleValue:
+        return true;
+        break;
+
+    case grt::ListItemAdded:
+        {
+            grt::ValueRef v= static_cast<const grt::ListItemAddedChange *>(diffchange.get())->get_value();
+            if(is_node_object(v))
+              apply_change(GrtObjectRef::cast_from(v), diffchange);
+            else
+              return true;
+        }
+        break;
+
+    case grt::ListItemRemoved:
+        {
+            grt::ValueRef v= static_cast<const grt::ListItemRemovedChange *>(diffchange.get())->get_value();
+            if(is_node_object(v))
+              apply_change(GrtObjectRef::cast_from(v), diffchange);
+            else
+              return true;
+        }
+        break;
+
+    case grt::ListModified:
+    case grt::ObjectModified:
     {
-      db_mysql_TableRef table= db_mysql_TableRef::cast_from(value);
-      DiffNode *schema_node= root->find_child_by_db_part_name(std::string(db_mysql_SchemaRef::cast_from(table->owner())->oldName().c_str()));
-      DiffNode *table_node= NULL;
-      //if(it->action == DiffItemAdded)
-      if(it->action == DiffItemRemoved)
-      {
-        table_node= new DiffNode(db_mysql_TableRef(), table, false);
-        schema_node->append(table_node);
-        fill_tree(table_node, table, CatalogMap(), true);
-      }
-      else if(it->action == DiffItemModified)
-      {
-        table_node= schema_node->find_child_by_db_part_name(std::string(table->oldName().c_str()));
-        if (table_node)
-          table_node->set_modified_and_update_dir(true);
-      }
+        bool flag = false;
+        for(grt::ChangeList::const_iterator it = diffchange->subchanges()->begin(); it < diffchange->subchanges()->end(); ++it)
+        {
+          if(update_tree_with_changes(*it))
+            flag = true;
+        }
+        return flag;
     }
-    else if(db_mysql_ViewRef::can_wrap(value))
-    {
-      db_mysql_ViewRef view= db_mysql_ViewRef::cast_from(value);
-      DiffNode *schema_node= root->find_child_by_db_part_name(std::string(db_mysql_SchemaRef::cast_from(view->owner())->oldName().c_str()));
-      DiffNode *view_node= NULL;
-      //if(it->action == DiffItemAdded)
-      if(it->action == DiffItemRemoved)
-      {      
-        view_node= new DiffNode(db_mysql_ViewRef(), view, false);
-        schema_node->append(view_node);
-      }
-      else if(it->action == DiffItemModified)
-      {
-        view_node= schema_node->find_child_by_db_part_name(std::string(view->oldName().c_str()));
-        view_node->set_modified_and_update_dir(true);
-      }
+    case grt::ObjectAttrModified:
+        return update_tree_with_changes(static_cast<const grt::ObjectAttrModifiedChange *>
+            (diffchange.get())->get_subchange());
+
+    case grt::ListItemModified:
+        {
+            grt::ValueRef v(static_cast<const grt::ListItemModifiedChange *>
+                (diffchange.get())->get_old_value());
+
+            bool push = is_node_object(v); 
+
+            bool flag = update_tree_with_changes(static_cast<const grt::ListItemModifiedChange *>(diffchange.get())->get_subchange());
+            if(push && flag)
+            {
+                apply_change(GrtObjectRef::cast_from(v), diffchange);
+                flag = false;
+            }
+            return flag;
+        }
+        break;
+
+    case grt::ListItemOrderChanged:
+        {
+            const grt::ListItemOrderChange *oc= 
+                static_cast<const grt::ListItemOrderChange *>(diffchange.get());
+            grt::ValueRef v(oc->get_old_value());
+            // order changes matter for colums and indexcolumn... in anything else, it doesn't
+            if (db_ColumnRef::can_wrap(v) || db_IndexColumnRef::can_wrap(v))
+                return true;
+            // the list item order change action can have a nested
+            // object-modified action
+            else if (oc->get_subchange())
+                return update_tree_with_changes(oc->get_subchange());
+        }
+        break;
+
+    default:
+        break;
     }
-    else if(db_mysql_RoutineRef::can_wrap(value))
-    {
-      db_mysql_RoutineRef routine= db_mysql_RoutineRef::cast_from(value);
-      DiffNode *routine_node= NULL;
-      DiffNode *schema_node= root->find_child_by_db_part_name(std::string(db_mysql_SchemaRef::cast_from(routine->owner())->oldName().c_str()));
-      //if(it->action == DiffItemAdded)
-      if(it->action == DiffItemRemoved)
-      {
-        routine_node= new DiffNode(db_mysql_RoutineRef(), routine, false);
-        schema_node->append(routine_node);
-      }
-      else if(it->action == DiffItemModified)
-      {
-        routine_node= schema_node->find_child_by_db_part_name(std::string(routine->oldName().c_str()));
-        routine_node->set_modified_and_update_dir(true);
-      }
-    }
-    else if(db_ColumnRef::can_wrap(value))
-    {
-      db_ColumnRef column= db_ColumnRef::cast_from(value);
-      DiffNode *schema_node= root->find_child_by_db_part_name(std::string(db_mysql_SchemaRef::cast_from(column->owner()->owner())->oldName().c_str()));
-      DiffNode *table_node= schema_node->find_child_by_db_part_name(std::string(db_mysql_TableRef::cast_from(column->owner())->oldName().c_str()));
-      if (table_node)
-          table_node->set_modified_and_update_dir(true);
-    }
-    else if(db_mysql_IndexRef::can_wrap(value))
-    {
-      db_mysql_IndexRef index= db_mysql_IndexRef::cast_from(value);
-      DiffNode *schema_node= root->find_child_by_db_part_name(std::string(db_mysql_SchemaRef::cast_from(index->owner()->owner())->oldName().c_str()));
-      DiffNode *table_node= schema_node->find_child_by_db_part_name(std::string(db_mysql_TableRef::cast_from(index->owner())->oldName().c_str()));
-      if (table_node)
-          table_node->set_modified_and_update_dir(true);
-    }
-    else if(db_mysql_IndexColumnRef::can_wrap(value))
-    {
-      db_mysql_IndexColumnRef index_col= db_mysql_IndexColumnRef::cast_from(value);
-      db_mysql_IndexRef index= db_mysql_IndexRef::cast_from(index_col->owner());
-      DiffNode *schema_node= root->find_child_by_db_part_name(std::string(db_mysql_SchemaRef::cast_from(index->owner()->owner())->oldName().c_str()));
-      DiffNode *table_node= schema_node->find_child_by_db_part_name(std::string(db_mysql_TableRef::cast_from(index->owner())->oldName().c_str()));
-      if (table_node)
-          table_node->set_modified_and_update_dir(true);
-    }
-    else if(db_mysql_ForeignKeyRef::can_wrap(value))
-    {
-      db_mysql_ForeignKeyRef fk= db_mysql_ForeignKeyRef::cast_from(value);
-      DiffNode *schema_node= root->find_child_by_db_part_name(std::string(db_mysql_SchemaRef::cast_from(fk->owner()->owner())->oldName().c_str()));
-      DiffNode *table_node= schema_node->find_child_by_db_part_name(std::string(db_mysql_TableRef::cast_from(fk->owner())->oldName().c_str()));
-      if (table_node)
-          table_node->set_modified_and_update_dir(true);
-    }
-    else if(db_mysql_TriggerRef::can_wrap(value))
-    {
-      db_mysql_TriggerRef trigger= db_mysql_TriggerRef::cast_from(value);
-      DiffNode *trigger_node= NULL;
-      DiffNode *schema_node= root->find_child_by_db_part_name(std::string(db_mysql_SchemaRef::cast_from(trigger->owner()->owner())->oldName().c_str()));
-      DiffNode *table_node= schema_node->find_child_by_db_part_name(std::string(db_mysql_TableRef::cast_from(trigger->owner())->oldName().c_str()));
-      if(it->action == DiffItemRemoved)
-      //if(it->action == DiffItemAdded)
-      {
-        trigger_node= new DiffNode(db_mysql_TriggerRef(), trigger, false);
-        table_node->append(trigger_node);
-      }
-      else if(it->action == DiffItemModified)
-      {
-        trigger_node= table_node->find_child_by_db_part_name(std::string(trigger->oldName().c_str()));
-        trigger_node->set_modified_and_update_dir(true);
-      }
-    }
-  }
+  return false;
 }
 
-#ifdef _WIN32
-#include <windows.h>
+//TODO check how new DiffNode being deleted
+void DiffTreeBE::apply_change(GrtObjectRef obj, boost::shared_ptr<DiffChange> change)
+{
+  DiffNode *obj_node = _root->find_node_for_object(obj);
+
+#if 0
+  // Uncomment to see what are the differences detected for a given object
+  if (obj->name() == "")
+    change->dump_log(0);
 #endif
 
+  if(obj_node)
+    obj_node->set_modified_and_update_dir(true, change);
+  else
+  {
+    DiffNode *owner_node = _root->find_node_for_object(obj->owner() );
+    if(!owner_node)
+      owner_node = _root;
+
+    DiffNode * newnode = new DiffNode(GrtNamedObjectRef(), GrtNamedObjectRef::cast_from(obj), false, change);
+    owner_node->append(newnode);
+  }
+}
+
 DiffTreeBE::DiffTreeBE(const std::vector<std::string>& schemata,
                        db_mysql_CatalogRef model_catalogRef, 
                        db_mysql_CatalogRef external_catalog, 
@@ -597,23 +565,18 @@ _node_controller(controller)
   _schemata.assign(schemata.begin(), schemata.end());
   
   CatalogMap map;
-  build_catalog_map(external_catalog, map);
+  if(external_catalog.is_valid())
+    build_catalog_map(external_catalog, map);
   fill_tree(_root, model_catalogRef, map, false);
-  
-  DiffItemList diffitemlist;
-  ModifiedObjectStack workstack;
-
-  if(diffchange)
-  {
-    build_diff_item_list(diffchange.get(), diffitemlist, workstack);
-    update_tree_from_diff_item_list(_root, diffitemlist);
-  }
+  update_tree_with_changes(diffchange);
 
   change_nothing_icon= bec::IconManager::get_instance()->get_icon_id("change_nothing.png");
   change_backward_icon= bec::IconManager::get_instance()->get_icon_id("change_backward.png");
   change_forward_icon= bec::IconManager::get_instance()->get_icon_id("change_forward.png");
   change_ignore_icon= bec::IconManager::get_instance()->get_icon_id("change_ignore.png");
   alert_icon= bec::IconManager::get_instance()->get_icon_id("change_alert_thin.png");
+  create_alert_icon= bec::IconManager::get_instance()->get_icon_id("change_alert_create.png");
+  drop_alert_icon= bec::IconManager::get_instance()->get_icon_id("change_alert_drop.png");
 }
 
 DiffNode *DiffTreeBE::get_node_with_id(const NodeId &nodeid)
@@ -674,7 +637,16 @@ bool DiffTreeBE::get_field(const bec::NodeId &node_id, int column, std::string &
   {
   case ModelObjectName:
     if(node->get_model_part().is_valid_object())
+    {
       value= node->get_model_part().get_name();
+      if (db_SchemaRef::can_wrap(node->get_model_part().get_object()))
+      {
+        // put the actual name of the schema if it's being synchronized with a differently named schema
+        std::string real_name = db_SchemaRef::cast_from(node->get_model_part().get_object())->customData().get_string("db.mysql.synchronize:originalName");
+        if (!real_name.empty())
+          value.append(" ("+real_name+")");
+      }
+    }
     else
       value= "N/A";
     break;
@@ -715,27 +687,86 @@ bec::IconId DiffTreeBE::get_field_icon(const bec::NodeId &node_id, int column, I
 
   switch(column)
   {
-  case ModelChanged:
-    return (node->is_modified()) ? alert_icon : 0;
-  case ModelObjectName:
-    return object_icon;
-  case ApplyDirection:
-    switch(node->get_application_direction())
-    {
-    case DiffNode::ApplyToModel:
-      return change_backward_icon;
-    case DiffNode::ApplyToDb:
-      return change_forward_icon;
-    case DiffNode::DontApply:
-      return change_ignore_icon;
-    case DiffNode::CantApply:
-      return change_nothing_icon;
-    }
-    return -1;
-  case DbObjectName:
-    return (node->is_modified()) ? alert_icon : 0;
-  default:
-    return -1;
+    case ModelChanged:
+      if (node->is_modified_recursive())
+      {
+        bool model_valid = node->get_model_part().is_valid_object();
+        bool db_valid = node->get_db_part().is_valid_object();
+        DiffNode::ApplicationDirection dir = node->get_application_direction();
+
+        if (model_valid && !db_valid)
+        {
+          // exists in model, doesn't in DB
+          if (dir == DiffNode::ApplyToModel)
+            return drop_alert_icon;
+          else
+            return alert_icon;
+        }
+        else if (!model_valid && db_valid)
+        {
+          // exists in model, doesn't in DB
+          if (dir == DiffNode::ApplyToDb)
+            return create_alert_icon;
+          else
+            return alert_icon;
+        }
+        else
+          return alert_icon;
+      }
+      else
+        return 0;
+      //return (node->is_modified()) ? ((!node->get_db_part().is_valid_object() || !node->get_model_part().is_valid_object())? drop_alert_icon : alert_icon) : 0;
+    case ModelObjectName:
+      return object_icon;
+    case ApplyDirection:
+      if (node->is_modified())
+      {
+        switch(node->get_application_direction())
+        {
+          case DiffNode::ApplyToModel:
+            return change_backward_icon;
+          case DiffNode::ApplyToDb:
+            return change_forward_icon;
+          case DiffNode::DontApply:
+            return change_ignore_icon;
+          case DiffNode::CantApply:
+            return change_nothing_icon;
+        }
+      }
+      else
+        return change_nothing_icon;
+      return -1;
+    case DbObjectName:
+      if (node->is_modified())
+      {
+        bool model_valid = node->get_model_part().is_valid_object();
+        bool db_valid = node->get_db_part().is_valid_object();
+        DiffNode::ApplicationDirection dir = node->get_application_direction();
+
+        if (model_valid && !db_valid)
+        {
+          // exists in model, doesn't in DB
+          if (dir == DiffNode::ApplyToDb)
+            return create_alert_icon;
+          else
+            return alert_icon;
+        }
+        else if (!model_valid && db_valid)
+        {
+          // exists in model, doesn't in DB
+          if (dir == DiffNode::ApplyToDb)
+            return drop_alert_icon;
+          else
+            return alert_icon;
+        }
+        else
+          return alert_icon;
+        //return (node->is_modified()) ? ((!node->get_db_part().is_valid_object() || !node->get_model_part().is_valid_object())? drop_alert_icon : alert_icon) : 0;
+      }
+      else
+        return 0;
+    default:
+      return -1;
   }
 }
 
@@ -753,6 +784,13 @@ void DiffTreeBE::set_apply_direction(const bec::NodeId &node_id, DiffNode::Appli
     _node_controller.set_apply_direction(node, dir, recursive);
 }
 
+DiffNode::ApplicationDirection DiffTreeBE::get_apply_direction(const bec::NodeId &node_id)
+{
+  DiffNode *node= dynamic_cast<DiffNode *>(get_node_with_id(node_id));
+  if (node)
+    return node->get_application_direction();
+  return DiffNode::CantApply;
+}
 
 void DiffTreeBE::get_object_list_for_script(std::vector<grt::ValueRef>& vec) const
 {
