@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -100,10 +100,16 @@ public:
     _options_box.add(&_generate_create_index_check, false, false);
     _generate_show_warnings_check.set_text(_("Add SHOW WARNINGS After Every DDL Statement"));
     _options_box.add(&_generate_show_warnings_check, false, false);
+    _no_view_placeholders.set_text(_("Don't create view placehoder tables."));
+    _options_box.add(&_no_view_placeholders, false, false);
     _skip_users_check.set_text(_("Do Not Create Users. Only Create Privileges (GRANTs)"));
     _options_box.add(&_skip_users_check, false, false);
     _generate_insert_check.set_text(_("Generate INSERT Statements for Tables"));
     _options_box.add(&_generate_insert_check, false, false);
+    _no_FK_for_inserts.set_text(_("Disable FK checks for INSERTs"));
+    _options_box.add(&_no_FK_for_inserts, false, false);
+//    _triggers_after_inserts.set_text(_("Create triggers after INSERTs"));
+ //   _options_box.add(&_triggers_after_inserts, false, false);
 
     add(&_options, false, false);
     
@@ -116,8 +122,11 @@ public:
     _skip_FK_indexes_check.set_active(module->document_int_data("SkipFKIndexes", 0));
     _generate_show_warnings_check.set_active(module->document_int_data("GenerateWarnings", 0)); 
     _generate_create_index_check.set_active(module->document_int_data("GenerateCreateIndex", 0));
+    _no_view_placeholders.set_active(module->document_int_data("NoViewPlaceholders", 0));
     _skip_users_check.set_active(module->document_int_data("NoUsersJustPrivileges", 0));
     _generate_insert_check.set_active(module->document_int_data("GenerateInserts", 0));
+    _no_FK_for_inserts.set_active(module->document_int_data("NoFKForInserts", 0));
+//    _triggers_after_inserts.set_active(module->document_int_data("TriggersAfterInserts", 0));
     _omit_schema_qualifier_check.set_active(module->document_int_data("OmitSchemata", 0));
     _generate_use_check.set_active(module->document_int_data("GenerateUse", 0));
     _generate_use_check.set_enabled(_omit_schema_qualifier_check.get_active());
@@ -143,9 +152,13 @@ public:
     values().gset("GenerateWarnings", _generate_show_warnings_check.get_active());
     values().gset("GenerateCreateIndex", _generate_create_index_check.get_active());
     values().gset("NoUsersJustPrivileges", _skip_users_check.get_active());
+    values().gset("NoViewPlaceholders", _no_view_placeholders.get_active());
     values().gset("GenerateInserts", _generate_insert_check.get_active());
     values().gset("OmitSchemata", _omit_schema_qualifier_check.get_active());
     values().gset("GenerateUse", _generate_use_check.get_active());
+    values().gset("NoFKForInserts", _no_FK_for_inserts.get_active());
+//    values().gset("TriggersAfterInserts", _triggers_after_inserts.get_active());
+
     
     grt::Module *module= ((WizardPlugin*)_form)->module();
     module->set_document_data("GenerateDrops", _generate_drop_check.get_active());
@@ -155,9 +168,12 @@ public:
     module->set_document_data("GenerateWarnings", _generate_show_warnings_check.get_active());
     module->set_document_data("GenerateCreateIndex", _generate_create_index_check.get_active());
     module->set_document_data("NoUsersJustPrivileges", _skip_users_check.get_active());
+    module->set_document_data("NoViewPlaceholders", _no_view_placeholders.get_active());
     module->set_document_data("GenerateInserts", _generate_insert_check.get_active());
     module->set_document_data("OmitSchemata", _omit_schema_qualifier_check.get_active());
     module->set_document_data("GenerateUse", _generate_use_check.get_active());
+    module->set_document_data("NoFKForInserts", _no_FK_for_inserts.get_active());
+//    module->set_document_data("TriggersAfterInserts", _triggers_after_inserts.get_active());
   }
 
 protected:
@@ -172,7 +188,10 @@ protected:
   CheckBox _generate_create_index_check;
   CheckBox _generate_show_warnings_check;
   CheckBox _skip_users_check;
+  CheckBox _no_view_placeholders;
   CheckBox _generate_insert_check;
+  CheckBox _no_FK_for_inserts;
+//  CheckBox _triggers_after_inserts;
   CheckBox _omit_schema_qualifier_check;
 };
 
@@ -237,23 +256,23 @@ protected:
     
     _table_filter= add_filter(db_mysql_Table::static_class_name(),
                               _("Export %s Objects"),
-                              tables_model, tables_imodel);
+                              tables_model, tables_imodel, NULL);
     
     _view_filter= add_filter(db_mysql_View::static_class_name(),
                              _("Export %s Objects"),
-                             views_model, views_imodel);
+                             views_model, views_imodel, NULL);
     
     _routine_filter= add_filter(db_mysql_Routine::static_class_name(),
                                 _("Export %s Objects"),
-                                routines_model, routines_imodel);
+                                routines_model, routines_imodel, NULL);
 
     _trigger_filter= add_filter(db_mysql_Trigger::static_class_name(),
                                 _("Export %s Objects"),
-                                triggers_model, triggers_imodel);
+                                triggers_model, triggers_imodel, NULL);
 
     _user_filter= add_filter(db_User::static_class_name(),
                              _("Export %s Objects"),
-                             users_model, users_imodel);
+                             users_model, users_imodel, NULL);
   }
   
   virtual bool advance()
@@ -267,7 +286,10 @@ protected:
     _export_be->set_option("GenerateWarnings", values().get_int("GenerateWarnings"));
     _export_be->set_option("GenerateCreateIndex", values().get_int("GenerateCreateIndex"));
     _export_be->set_option("NoUsersJustPrivileges", values().get_int("NoUsersJustPrivileges"));
+    _export_be->set_option("NoViewPlaceholders", values().get_int("NoViewPlaceholders"));
     _export_be->set_option("GenerateInserts", values().get_int("GenerateInserts"));
+    _export_be->set_option("NoFKForInserts", values().get_int("NoFKForInserts"));
+//    _export_be->set_option("TriggersAfterInserts", values().get_int("TriggersAfterInserts"));
     _export_be->set_option("OmitSchemata", values().get_int("OmitSchemata"));
     _export_be->set_option("GenerateUse", values().get_int("GenerateUse"));
     
@@ -324,7 +346,7 @@ public:
 
   int export_task_finished()
   {
-    _text.set_value(((WbPluginDbExport*)_form)->be()->export_sql_script());
+    set_text(((WbPluginDbExport*)_form)->be()->export_sql_script());
     script_ready = true;
     _form->update_buttons();
     return 0;
@@ -406,7 +428,7 @@ class ExportProgressPage : public WizardProgressPage
   MyConnectionPage  *_conn_page;
 
 public:
-  ExportProgressPage(WizardForm *form) : WizardProgressPage(form, "progress")
+  ExportProgressPage(WizardForm *form) : WizardProgressPage(form, "progress", false)
                                        , _finished(false)
                                        , _conn_page(0)
   {
@@ -417,13 +439,21 @@ public:
              boost::bind(&ExportProgressPage::do_connect, this),
              _("Connecting to DBMS..."));
 
-    TaskRow *task= add_async_task(_("Execute Forward Engineered Script"),
+    add_async_task(_("Execute Forward Engineered Script"),
                    boost::bind(&ExportProgressPage::do_export, this),
                    _("Executing forward engineered SQL script in DBMS..."));
-    task->process_finish= boost::bind(&ExportProgressPage::export_finished, this, _1);
 
-    end_adding_tasks(false,
-                     _("Forward Engineer Finished Successfully"));
+    add_async_task(_("Read Back Changes Made by Server"),
+                   boost::bind(&ExportProgressPage::back_sync, this),
+                   _("Fetching back object definitions reformatted by server..."));
+
+    TaskRow *task= add_task(_("Save Synchronization State"),
+                            boost::bind(&ExportProgressPage::save_sync_profile, this),
+                            _("Storing state information to synchronization profile..."));
+
+    task->process_finish= boost::bind(&ExportProgressPage::export_finished, this, _1);
+    
+    end_adding_tasks(_("Forward Engineer Finished Successfully"));
 
     set_status_text("");
   }
@@ -465,6 +495,24 @@ public:
     return true;
   }
 
+  bool back_sync()
+  {
+    execute_grt_task(boost::bind(&ExportProgressPage::back_sync_, this), false);
+    return true;
+  }
+
+  grt::IntegerRef back_sync_()
+  {
+    ((WbPluginDbExport*)_form)->be()->read_back_view_ddl();
+    return grt::IntegerRef(0);
+  }
+
+  bool save_sync_profile()
+  {
+//#warning  TODO
+//    ((WbPluginDbExport*)_form)->be()->save_
+    return true;
+  }
 
   void export_finished(const grt::ValueRef &result)
   {
@@ -489,6 +537,12 @@ public:
 WbPluginDbExport::WbPluginDbExport(grt::Module *module)
     : WizardPlugin(module), _db_frw_eng(grtm())
 {
+
+  set_name("db_export_wizard");
+  if (CatalogValidationPage::has_modules(module->get_grt()))
+    _validation_page = new CatalogValidationPage(this);
+  else
+    _validation_page = 0;
   _input_page= new ExportInputPage(this);
   _connection_page= new MyConnectionPage(this);
   _connection_page->set_db_connection(be()->db_conn());
@@ -498,6 +552,8 @@ WbPluginDbExport::WbPluginDbExport(grt::Module *module)
   _progress_page= new ExportProgressPage(this);
   _progress_page->set_connection_page((MyConnectionPage*)_connection_page);
   add_page(mforms::manage(_connection_page));
+  if (_validation_page)
+    add_page(mforms::manage(_validation_page));
   add_page(mforms::manage(_input_page));
   add_page(mforms::manage(_selection_page));
   add_page(mforms::manage(_preview_page));
