@@ -1,4 +1,4 @@
-# Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -28,7 +28,7 @@ import mforms
 import wb_admin_config_file_be
 from wb_common import dprint_ex, debug_level, PermissionDeniedError, InvalidPasswordError, OperationCancelledError, Users
 from wb_admin_config_file_be import multi_separator
-from wb_admin_utils import no_remote_admin_warning_label
+from wb_admin_utils import no_remote_admin_warning_label, make_panel_header
 
 from workbench.utils import server_os_path
 
@@ -160,14 +160,21 @@ class Page(object):
 
 #===============================================================================
 class WbAdminConfigFileUI(mforms.Box):
+    @classmethod
+    def wba_register(cls, admin_context):
+        admin_context.register_page(cls, "wba_instance", "Options File", True)
+
+    @classmethod
+    def identifier(cls):
+        return "admin_option_file"
+
         #---------------------------------------------------------------------------
     def page_activated(self):
-        self.main_view.set_content_label(" Options File")
         if not self.ui_created:
             self.ui_created = True
-            self.suspend_layout()
+            #self.suspend_layout()
             self.create_ui()
-            self.resume_layout()
+            #self.resume_layout()
         else:
             on = self.server_profile.admin_enabled
             self.file_name_ctrl.set_enabled(on)
@@ -177,19 +184,29 @@ class WbAdminConfigFileUI(mforms.Box):
             self.search_panel.set_enabled(on)
 
     #---------------------------------------------------------------------------
-    def __init__(self, server_profile, ctrl_be, main_view, version="5.1"):
+    def __init__(self, ctrl_be, server_profile, main_view, version="5.1"):
         mforms.Box.__init__(self, False)
+        self.set_managed()
+        self.set_release_on_add()
+
+        self.set_padding(12)
+        self.set_spacing(8)
+
+        self.search_panel = self.create_search_panel()
+
+        self.heading = make_panel_header("title_options.png", server_profile.name, "Options File", self.search_panel)
+        self.add(self.heading, False, True)
+
         self.tab_view = newTabView()
         self.main_view = main_view
 
         self.myopts = None
         self.opt2ctrl_map = {}
+        self.not_multiline_options = set()
         self.loading = True
         self.section = None
         self.ui_created = False
 
-        self.set_managed()
-        self.set_release_on_add()
         self.version = version
         self.ctrl_be = ctrl_be
         self.server_profile = server_profile
@@ -198,10 +215,7 @@ class WbAdminConfigFileUI(mforms.Box):
         self.dir_dict = {}  # Will be filled later when loading default values
 
         self.pack_to_top()
-        container = self.main_view.add_content_page(self, "CONFIGURATION", "Options File", "admin_option_file_win", True)
 
-        self.search_panel = self.create_search_panel()
-        container.add_end(self.search_panel, False, False);
 
     #---------------------------------------------------------------------------
     def create_search_panel(self):
@@ -221,7 +235,6 @@ class WbAdminConfigFileUI(mforms.Box):
         search_box.add(search_btn, False, False)
         search_panel = newPanel(mforms.FilledPanel)
         search_panel.add(search_box)
-        self.main_view.ui_profile.apply_style(search_panel, 'option-search-panel')
 
         def lookup_option_wrapper():
             self.locate_option(self.option_lookup_entry.get_string_value())
@@ -252,7 +265,6 @@ class WbAdminConfigFileUI(mforms.Box):
 
         box = newBox(False)
         box.set_spacing(8)
-        box.suspend_layout()
 
         # Actual option values from config file parsed by cfg_be, in form of list of tuples (<name>, <value>)
         options = self.cfg_be.get_options(self.section_ctrl.get_string_value())
@@ -276,14 +288,8 @@ class WbAdminConfigFileUI(mforms.Box):
             table.set_row_spacing(10)
             table.set_column_spacing(20)
             table.set_padding(5)
-            table.suspend_layout()
             table.set_homogeneous(False)
 
-            panel = newPanel(TitledBoxPanel)
-            panel.add(table)
-            panel.set_title(group['caption'])
-            box.add(panel, False, True)
-
             table.set_row_count(number_of_controls)
             table.set_column_count(3)
 
@@ -340,11 +346,16 @@ class WbAdminConfigFileUI(mforms.Box):
 
             # Remove empty rows
             table.set_row_count(table_row+1)#number_of_controls - (number_of_controls - table_row))
-            table.resume_layout()
+
+            # Important for good performance: add the table *after* all content was added to its container. So we have only one layout call!
+            panel = newPanel(TitledBoxPanel)
+            panel.add(table)
+            panel.set_title(group['caption'])
+            box.add(panel, False, True)
+
 
         page.panel.add(box)
         page.created = True
-        box.resume_layout()
         self.loading = False
 
     #---------------------------------------------------------------------------
@@ -446,7 +457,7 @@ class WbAdminConfigFileUI(mforms.Box):
         te = newTextEntry()
         te.set_enabled(False)
         te.add_changed_callback(lambda: self.control_action(name))
-        te.set_tooltip("To convert option to a multi-line one, use " + multi_separator + " to separate values. The symbol " + multi_separator + " should not be the first char")
+        te.set_tooltip("To convert option to a multi-line one, use " + multi_separator + " to separate values. The symbol " + multi_separator + " should not be at the start or end of the value")
         return te
 
     #---------------------------------------------------------------------------
@@ -455,7 +466,7 @@ class WbAdminConfigFileUI(mforms.Box):
         dir_box.set_spacing(4)
 
         te = newTextEntry()
-        te.set_tooltip("To convert option to a multi-line one, use " + multi_separator + " to separate values. The symbol " + multi_separator + " should not be the first char")
+        te.set_tooltip("To convert option to a multi-line one, use " + multi_separator + " to separate values. The symbol " + multi_separator + " should not be at the start or end of the value")
         btn = newButton()
         btn.set_text("...")
         btn.enable_internal_padding(False)
@@ -897,7 +908,7 @@ class WbAdminConfigFileUI(mforms.Box):
             if not self.loading:
                 # Notify config BE about change
                 if enabled:
-                    self.cfg_be.option_added(name, self.get_string_value_from_control(ctrl), self.section)
+                    self.cfg_be.option_added(name, self.get_string_value_from_control(name, ctrl), self.section)
                 else:
                     self.cfg_be.option_removed(name, self.section)
 
@@ -910,10 +921,10 @@ class WbAdminConfigFileUI(mforms.Box):
             ctrl = self.opt2ctrl_map[name]
 
             if not self.loading:
-                self.cfg_be.option_changed(name, self.get_string_value_from_control(ctrl), self.section)
+                self.cfg_be.option_changed(name, self.get_string_value_from_control(name, ctrl), self.section)
 
     #---------------------------------------------------------------------------
-    def get_string_value_from_control(self, ctrl):
+    def get_string_value_from_control(self, option_name, ctrl):
         #ctrl is a tupple from map
         value = ""
 
@@ -939,11 +950,11 @@ class WbAdminConfigFileUI(mforms.Box):
         elif tag == "dir":
             value = control(1).get_string_value()
             if is_multiple:
-                value = value.split(';')
+                value = value.split(multi_separator)
         elif tag == "fed":
             value = control(1).get_string_value()
             if is_multiple:
-                value = value.split(';')
+                value = value.split(multi_separator)
         elif tag == "chk":
             value = (control(0).get_active(),)
 
@@ -962,16 +973,18 @@ class WbAdminConfigFileUI(mforms.Box):
             is_string = type(value) is str or type(value) is unicode
             has_separator = value.find(multi_separator) > 0
 
-        if is_multiple == False and is_string and has_separator and not self.loading:
-            answer = Utilities.show_message("Confirm"
-                ,"Multi-line option format entered. Would you like to convert option to multi-line?"
-                , "Convert", "Skip", "")
+        if is_multiple == False and is_string and has_separator and not self.loading and option_name not in self.not_multiline_options:
+            answer = Utilities.show_message("Multi Line Option"
+                ,"The %s sequence was detected in the string, which is used for separating multi-line options. Would you like to split the value to multiple options?" % multi_separator
+                , "Convert", "Keep as Is", "")
             if answer == mforms.ResultOk:
                 control(1).set_name("Multiple")
+            else:
+                self.not_multiline_options.add(option_name)
 
         # some controls return values in form of one-item tuples
         # so we need to extract that item for processing below
-        if has_separator and is_string:
+        if has_separator and is_string and not option_name in self.not_multiline_options:
             if type(value) is tuple:
                 if len(value) == 1:     # Only extract values from one-item tuples
                     value = value[0]      # If tuple has more items it already has been converted to multi-line
@@ -1039,7 +1052,11 @@ class WbAdminConfigFileUI(mforms.Box):
                 # TODO: Add config file error message
                 value = ""
             value = value.strip(" \r\n\t")
-            control(1).set_value(value % self.dir_dict)
+            try:
+                value = value.format(**self.dir_dict)
+            except Exception:
+                pass
+            control(1).set_value(value)
         elif tag == "spn":
             if value is None:
                 value = ""
@@ -1090,6 +1107,7 @@ class WbAdminConfigFileUI(mforms.Box):
 
     #---------------------------------------------------------------------------
     def clear_and_load(self):
+        self.not_multiline_options = set()
         if self.loading == False:
             self.load_defaults()
             self.load_options_from_cfg(self.section_ctrl.get_string_value())
@@ -1177,8 +1195,6 @@ class WbAdminConfigFileUI(mforms.Box):
 
     #-------------------------------------------------------------------------------
     def pack_to_top(self):
-        self.suspend_layout()
-        self.main_view.ui_profile.apply_style(self, 'page')
 
         #if self.server_profile.admin_enabled:
         self.file_name_ctrl = newLabel("")
@@ -1192,20 +1208,16 @@ class WbAdminConfigFileUI(mforms.Box):
         self.section_ctrl = newSelector()
         self.section_ctrl.set_size(150, -1)
 
-        #spacer = newPanel(mforms.TransparentPanel)
-        #spacer.set_size(100, 10)
-
         self.bottom_box = newBox(True)
 
         accept_btn = newButton()
-        accept_btn.set_text("Apply ...")
+        accept_btn.set_text("Apply...")
         accept_btn.add_clicked_callback(self.config_apply_changes_clicked)
 
         discard_btn = newButton()
         discard_btn.set_text("Discard")
         discard_btn.add_clicked_callback(self.config_discard_changes_clicked)
 
-        #self.add(self.search_panel, False, True)
         self.add(self.tab_view, True, True)
         self.add(self.bottom_box, False, False)
 
@@ -1218,12 +1230,10 @@ class WbAdminConfigFileUI(mforms.Box):
         self.bottom_box.set_spacing(8)
         self.bottom_box.set_padding(12)
 
-
-        self.resume_layout()
-        
+    #-------------------------------------------------------------------------------
     def shutdown(self):
         if hasattr(self.cfg_be, 'changeset') and self.cfg_be.changeset:
-            res = Utilities.show_warning('Configuration file', '''Configuration file from instance '%s' may contain unsaved changed.
+            res = Utilities.show_warning('Configuration file', '''Configuration file from instance '%s' may contain unsaved changes.
 Would you like to review them?''' % self.server_profile.name, 'Review', 'Cancel', 'Discard')
             if res == mforms.ResultOk:
                 self.config_apply_changes_clicked()
