@@ -22,7 +22,6 @@
 #include "wb_editor_storednote.h"
 #include "grts/structs.workbench.physical.h"
 #include "grt/exceptions.h"
-#include "grtui/file_charset_dialog.h"
 #include "base/string_utilities.h"
 
 #include "mforms/code_editor.h"
@@ -53,7 +52,10 @@ Sql_editor::Ref StoredNoteEditorBE::get_sql_editor()
 
     // Remove syntax highlighting if this is a note.
     if (!is_script())
+    {
       _sql_editor->get_editor_control()->set_language(mforms::LanguageNone);
+      _sql_editor->set_sql_check_enabled(false);
+    }
   }
   return _sql_editor;
 }
@@ -131,54 +133,6 @@ std::string StoredNoteEditorBE::get_name()
   return _note->name();
 }
 
-
-bool StoredNoteEditorBE::load_file(const std::string &name)
-{
-  gchar *contents;
-  gsize length;
-
-  if (g_file_get_contents(name.c_str(), &contents, &length, NULL))
-  {
-    std::string converted;
-    
-    Sql_editor::Ref editor = get_sql_editor();
-    mforms::CodeEditor* code_editor = editor->get_editor_control();
-    if (FileCharsetDialog::ensure_filedata_utf8(contents, length, "", name, converted))
-    {
-      g_free(contents);
-
-      set_text(converted); // Text to GRT.
-    
-      code_editor->set_text_keeping_state(converted.c_str());
-      code_editor->reset_dirty();
-  
-      return true;
-    }
-    else
-      code_editor->set_text(_("Data is not UTF8 encoded and cannot be displayed."));
-
-    g_free(contents);
-  }
-  return false;
-}
-
-
-bool StoredNoteEditorBE::save_file(const std::string &name)
-{
-  grt::Module *module = get_grt()->get_module("Workbench");
-  if (!module)
-    throw std::runtime_error("Workbench module not found");
-
-  grt::BaseListRef args(get_grt());
-
-  args.ginsert(_note->filename());
-  args.ginsert(grt::StringRef(name));
-
-  grt::IntegerRef value(grt::IntegerRef::cast_from(module->call_function("exportAttachedFileContents", args)));
-
-  return *value == 0;
-}
-
 //--------------------------------------------------------------------------------------------------
 
 std::string StoredNoteEditorBE::get_title()
